Package: Xplortext
Type: Package
Encoding: UTF-8
Title: Statistical Analysis of Textual Data
Version: 1.6
Date: 2026-05-14
Authors@R: c(
            person(given="Ramón",family="Alvarez-Esteban", email="ramon.alvarez@unileon.es", role = c("aut", "cre"), comment = c(ORCID="0000-0002-4751-2797")),
            person(given="Mónica",family="Bécue-Bertaut", email="monica.becue@upc.edu", role = "aut", comment = c(ORCID = "0000-0002-6027-3655")),
            person(given="Josep-Anton",family="Sánchez-Espigares", email="josep.a.sanchez@upc.edu",role="ctb",  comment = c(ORCID = "0000-0001-8195-1913")),
            person(given="Belchin Adriyanov", family="Kostov", email="belchin.adriyanov.kostov@upc.edu",role="ctb", comment = c(ORCID ="0000-0002-2126-3892")))
Maintainer: Ramón Alvarez-Esteban <ramon.alvarez@unileon.es>
Author: Ramón Alvarez-Esteban [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4751-2797>),
  Mónica Bécue-Bertaut [aut] (ORCID:
    <https://orcid.org/0000-0002-6027-3655>),
  Josep-Anton Sánchez-Espigares [ctb] (ORCID:
    <https://orcid.org/0000-0001-8195-1913>),
  Belchin Adriyanov Kostov [ctb] (ORCID:
    <https://orcid.org/0000-0002-2126-3892>)
Description: Provides a set of functions devoted to multivariate exploratory statistics on textual data. Classical methods such as correspondence analysis and agglomerative hierarchical clustering are available. Chronologically constrained agglomerative hierarchical clustering enriched with labelled-by-words trees is offered. Given a division of the corpus into parts, their characteristic words and documents are identified. Further, accessing to 'FactoMineR' functions is very easy. Two of them are relevant in textual domain. MFA() addresses multiple lexical table allowing applications such as dealing with multilingual corpora as well as simultaneously analyzing both open-ended and closed questions in surveys. See <http://xplortext.unileon.es> for examples.
License: GPL (>= 2.0)
Depends: R (>= 4.5), FactoMineR(>= 2.11), ggplot2(>= 3.5.1), tm(>=
        0.7-16)
Imports: ape, cluster, dendextend, flexclust, flashClust, ggdendro,
        ggforce, ggpubr, ggrepel, graphics, gridExtra, MASS, methods,
        patchwork, plotly, stringi, stringr, slam, stats, utils, vegan
URL: https://xplortext.unileon.es
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-14 17:04:42 UTC; DEEPGAMING
Repository: CRAN
Date/Publication: 2026-05-14 20:30:02 UTC
Built: R 4.5.2; ; 2026-05-14 21:19:55 UTC; unix

Package: Xplortext
Type: Package
Encoding: UTF-8
Title: Statistical Analysis of Textual Data
Version: 1.6.1
Date: 2026-06-15
Authors@R: c(
            person(given="Ramón",family="Alvarez-Esteban", email="ramon.alvarez@unileon.es", role = c("aut", "cre"), comment = c(ORCID="0000-0002-4751-2797")),
            person(given="Mónica",family="Bécue-Bertaut", email="monica.becue@upc.edu", role = "aut", comment = c(ORCID = "0000-0002-6027-3655")),
            person(given="Josep-Anton",family="Sánchez-Espigares", email="josep.a.sanchez@upc.edu",role="ctb",  comment = c(ORCID = "0000-0001-8195-1913")),
            person(given="Belchin Adriyanov", family="Kostov", email="belchin.adriyanov.kostov@upc.edu",role="ctb", comment = c(ORCID ="0000-0002-2126-3892")))
Maintainer: Ramón Alvarez-Esteban <ramon.alvarez@unileon.es>
Author: Ramón Alvarez-Esteban [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4751-2797>),
  Mónica Bécue-Bertaut [aut] (ORCID:
    <https://orcid.org/0000-0002-6027-3655>),
  Josep-Anton Sánchez-Espigares [ctb] (ORCID:
    <https://orcid.org/0000-0001-8195-1913>),
  Belchin Adriyanov Kostov [ctb] (ORCID:
    <https://orcid.org/0000-0002-2126-3892>)
Description: Provides a set of functions devoted to multivariate exploratory statistics on textual data. Classical methods such as correspondence analysis and agglomerative hierarchical clustering are available. Chronologically constrained agglomerative hierarchical clustering enriched with labelled-by-words trees is offered. Given a division of the corpus into parts, their characteristic words and documents are identified. Further, accessing to 'FactoMineR' functions is very easy. Two of them are relevant in textual domain. MFA() addresses multiple lexical table allowing applications such as dealing with multilingual corpora as well as simultaneously analyzing both open-ended and closed questions in surveys. See <http://xplortext.unileon.es> for examples.
License: GPL (>= 2.0)
Depends: R (>= 4.6), FactoMineR(>= 2.15), ggplot2(>= 4.0.3), tm(>=
        0.7-18)
Imports: ape, cluster, dendextend, flexclust, flashClust, ggdendro,
        ggforce, ggpubr, ggrepel, graphics, gridExtra, MASS, methods,
        patchwork, plotly, stringi, stringr, slam, stats, utils, vegan
URL: https://xplortext.unileon.es
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-15 17:23:58 UTC; ralve
Repository: CRAN
Date/Publication: 2026-06-15 18:00:09 UTC
Built: R 4.6.0; ; 2026-06-15 20:00:02 UTC; unix

S3method(plot,LexCA)
S3method(plot,LexCHCca)
S3method(plot,LexChar)
S3method(plot,LexGalt)
S3method(plot,LexHCca)
S3method(plot,TextData)
S3method(print,LexCA)
S3method(print,LexChar)
S3method(print,TextData)
S3method(summary,LexCA)
S3method(summary,LexChar)
S3method(summary,TextData)
export(LabelTree)
export(LexCA)
export(LexCHCca)
export(LexChar)
export(LexGalt)
export(LexHCca)
export(TextData)
export(ellipseLexCA)
export(summary.LexCA)
export(summary.LexChar)
export(summary.TextData)
export(plot.TextData)
export(plot.LexCA)
export(plot.LexHCca)
export(plot.LexCHCca)
export(plot.LexGalt)
export(plot.LexChar)
import(FactoMineR)
import(ape, except = c(rotate, ladderize))
import(cluster)
import(dendextend, except=c(cutree))
import(flashClust)
import(flexclust, except = c(barplot, boxplot, predict, plot, pairs, image))
import(ggdendro, except = c(theme_dendro))
import(ggforce)
import(ggplot2)
import(ggpubr, except = c(rotate))
import(ggrepel)
import(graphics)
import(gridExtra)
import(patchwork)
import(slam)
import(stats, except = c(hclust))
import(stringi)
import(stringr)
import(tm)
import(utils)
importFrom(MASS,ginv)
importFrom(ggforce,geom_mark_ellipse)
importFrom(ggforce,geom_mark_hull)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,adjustcolor)
importFrom("grDevices", "dev.cur", "dev.off", "graphics.off")
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.interactive)
importFrom(grDevices,dev.new)
importFrom(grDevices,palette)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,barplot)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,locator)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(methods,hasArg)
importFrom(plotly,plotly_build)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,packageDescription)
importFrom(utils,write.table)
importFrom(vegan,mantel)

#' @import ggplot2 
#' @importFrom grDevices dev.new dev.interactive
#' @importFrom utils head write.table
#' @importFrom plotly plotly_build
#' @export
plot.TextData <- function (x, ndoc=25, nword=25, nseg=25, sel=NULL, ordFreq=TRUE,
  stop.word.tm=FALSE, idiom= "en", stop.word.user=NULL, theme=theme_bw(), title=NULL, 
  xtitle=NULL, col.fill="grey", col.lines="black", text.size=12, freq=NULL, vline=NULL, 
  interact=FALSE, round.dec=4,...)
{

  if(!is.null(vline))
  if(vline=="median")
  {
    warning("median is deprecated, it is changed by the mean")
    vline <- "mean"
  }

  options(stringsAsFactors = FALSE)
if (!inherits(x, "TextData")) 
 stop("Object x should be TextData class")
  pdoc <- pword <- pseg <- NULL 
 if(!is.null(vline)) if(vline==FALSE) vline <-NULL
  
idiom <- x$info$idiom[[1]]
theme$text$size <- text.size
words<-iFreq<-docnames<-rsegment<-frequency<-DocName<-y.var<-NULL

if(!is.null(freq)) if(freq=="YES"| freq==TRUE) freq <- 5


###############################################################################################################
## sel. will have the selected words/docs/segments, sel.tot. all the possible elements
#  1) bsel.seg <- bsel.word <- bsel.doc <- FALSE
bsel.word <- bsel.doc <- bsel.seg <- FALSE

n.Init.word <-length(x$DocTerm$dimnames$Terms)
n.Init.doc <- length(x$DocTerm$dimnames$Docs)


# 2) if("indexS" %in% names(x)) bSegment <- TRUE else bSegment <- FALSE
# 3) if(bSegment) n.Init.seg <- length(x$indexS$segOrderlist$segment)

if("indexS" %in% names(x)) bSegment <- TRUE else bSegment <- FALSE
if(bSegment) n.Init.seg <- length(x$indexS$segOrderlist$segment)


########################################################################################

if(is.null(sel)) {
  if(interact==TRUE) stop("seg=NULL is not allowed with interact=TRUE. \n Select word, doc or seg in sel argumment")
  bsel.seg <- bsel.word <- bsel.doc <- TRUE
  
  if(bSegment) s.init.seg <- s.sel.seg <- x$indexS$segOrderlist$segment
  s.init.word <- s.sel.word <- x$DocTerm$dimnames$Terms
  s.init.doc <- s.sel.doc <- x$DocTerm$dimnames$Docs

} else { # No null sel
  
  if(length(sel)==1){
    
    if(bSegment) if(sel=="seg") {ndoc<-0; nword<-0;  s.sel.seg <- s.init.seg <- x$indexS$segOrderlist$segment
    n.Init.seg <- length(s.init.seg); bseg <- TRUE
    }

    if(sel=="word") {ndoc<-0; nseg<-0;  s.sel.word <- s.init.word <- x$DocTerm$dimnames$Terms
    n.Init.word <- length(s.init.word); bword <- TRUE
    }
    
    if(sel=="doc") {nword<-0; nseg<-0;  s.sel.doc <- s.init.doc <- x$DocTerm$dimnames$Docs
    n.Init.word <- length(s.init.doc); bdoc <- TRUE
    } 
    
    
  } else {   # no length(sel)==1
    
    if(bSegment) if(sel$type=="seg") { 
      ndoc <- nword <- 0
      s.init.seg <- x$indexS$segOrderlist$segment
      
      if(is.numeric(sel$select)) s.sel.seg <- s.init.seg[sel$select]
      else {
        if(length(sel$select)==1) {
          s.sel.seg<- s.init.seg[grepl(sel$select, s.init.seg)]
        } else
          s.sel.seg <- s.init.seg[s.init.seg %in% sel$select] 
      }
      
    }
    
    if(sel$type=="word"){ 
      ndoc <- nseg <- 0
      s.init.word <- x$DocTerm$dimnames$Terms

      if(is.numeric(sel$select)) s.sel.word <- s.init.word[sel$select]
      else  s.sel.word <- s.init.word[s.init.word %in% sel$select] }
    
    if(sel$type=="doc"){ 
      nword <- nseg <- 0
      s.init.doc <- x$DocTerm$dimnames$Docs
      if(is.numeric(sel$select)) s.sel.doc <- s.init.doc[sel$select]
      else  s.sel.doc <- s.init.doc[s.init.doc %in% sel$select] }
  } # End   length(sel)
} # End sel
#################################



########################  Segments   ####################################
if(bSegment) if(nseg>0){
  df <- data.frame(x$indexS$segOrderlist)  # All the segments from TextData
  colnames(df)[1] <- "rsegment"
  #  nseg.TD <- nrow(df)
  
  df.rs <- df
  df.rs$rank <- rank(-df.rs$frequency, ties.method = "average")
  df.rs <- df.rs[df.rs$rsegment %in%  s.sel.seg,]
  df.rs <- df.rs[with(df.rs, order(-frequency,rsegment)), ]
  
  bsel <- ifelse(nrow(df.rs) == n.Init.seg, FALSE, TRUE)  # For titles
  nseg <- min(nseg,nrow(df.rs))
  # Order by freq
  

  if(nseg<nrow(df.rs))  df.rs <- df.rs[c(1:nseg),]
  
  if(is.null(title)){
    if(bsel) ttitle.s <- paste0( nseg," selected segments" )
    else ttitle.s <- paste0( nseg," most frequent segments" )
  } else  ttitle.s <- title
  
  ifelse(is.null(xtitle), txtitle.s <- "Repeated segment frequency" ,txtitle.s <- xtitle)
  
  #  df <- merge(df, df.rs, by="rsegment")
  #  df <- df[, c(1,2,6)]
  colnames(df.rs)[2] <- "iFreq"
  
  df.rs <- df.rs[order(-df.rs$iFreq,df.rs$rsegment), ]
  

  if(ordFreq) level_order <- rev(df.rs$rsegment)
  else level_order <-  df[rev(order(df$rsegment)),]$rsegment
  
  pseg <- ggplot(df.rs)+ geom_bar(aes(x=factor(rsegment, levels=level_order), y=iFreq),stat = "identity", color = col.lines, fill = col.fill) + 
    coord_flip() +
    ylab(txtitle.s) + xlab("") + ggtitle(ttitle.s) + theme + theme(plot.title = element_text(hjust = 0.5))
  
  if(!is.null(freq))
    pseg <- pseg + geom_text(aes(x=df.rs$rsegment, y=iFreq+ freq, label = iFreq))
  
  if(!is.null(vline)) if(is.numeric(vline)) 
    pseg<- pseg +  geom_hline(yintercept=vline, linetype="dashed", color = "red")
  

  if(interact==TRUE) {
    my.text <-  paste("Rank", pseg$data$rank, "", pseg$data$rsegment, "<br>", "Freq: ", pseg$data$iFreq, "<br>")
    g <- plotly::plotly_build(pseg)
    g$x$data[[1]]$text <- my.text
    pseg <- g
  }
}


##################################################  End Segments ####################################



##############################  stop words tm and users  #######################################

if(ndoc>0 | nword > 0) {
  
  # It is not a segment (it is word or doc)
  df <- data.frame(x$indexW[,1], rownames(x$indexW))
  colnames(df) <- c("iFreq", "words") 
  # nword.TD <- nrow(df)
  
  
  # Remove stopwords
  if(stop.word.tm!=FALSE) {
    stop.word <- tm::stopwords(idiom)
    pos <- which(!df$words%in%stop.word)
    df <-  df[pos,]
  }
  
  # Remove user words
  if(!is.null(stop.word.user)) {
    pos <- which(!df$words %in% stop.word.user)
    df <-  df[pos,]}
  
  final.words <- rownames(df)
  #  nword <- min(nword,nrow(df))
  
}
################################  End stop words tm and users ######################################


if(nword>0){
  ##################################
  # Selection of words by number and name
  df.rw <- df
  df.rw$rank <- rank(-df.rw$iFreq, ties.method = "average")
  df.rw <- df.rw[df.rw$words %in%  s.sel.word,]
  df.rw <- df.rw[with(df.rw, order(-iFreq,words)), ]

  
  # Mean words before
  vlineword.Init.w <- x$summGen[2,2] / x$summGen[3,2]
  
  b.sel.word <- ifelse(nrow(df.rw) == n.Init.word, FALSE, TRUE)  # For titles
  
  nword <- min(nword,nrow(df.rw))
  # Order by freq
  
  if(nword<nrow(df.rw))  df.rw <- df.rw[c(1:nword),]
  

  if(is.null(title)){
    if(b.sel.word) ttitle.w <- paste0(nword," selected words" )
    else ttitle.w <- paste0(nword," most frequent words" )
  } else  ttitle.w <- title
  
  
  ifelse(is.null(xtitle), txtitle.w <- "Word frequency" ,txtitle.w <- xtitle)

  
  
  # Mean  for indexW
  if(!is.null(vline)) {
    if(is.numeric(vline)) {vlineword.Final.w <-  vlineword.Init.w <- vline}
    if(vline=="YES"|vline==TRUE|vline=="mean" ) {
      vlineword.Final.w <- mean(df$iFreq)
    } }
  
  df.rw <- df.rw[with(df.rw, order(-iFreq,words)), ]
  
  
  if(ordFreq) level_order.w <- rev(df.rw$words)
  else level_order.w <-  df.rw[rev(order(df.rw$words)),]$words
  

  pword <- ggplot(df.rw)+ geom_col(aes(x=factor(words, levels=level_order.w), y=iFreq),color = col.lines, fill = col.fill) + 
     coord_flip() +
    ylab(txtitle.w) + xlab("") + ggtitle(ttitle.w) + theme + theme(plot.title = element_text(hjust = 0.5))

  if(!is.null(freq))
    pword <- pword + geom_text(aes(x=df.rw$words, y=iFreq+ freq, label = iFreq))
  

  if(!is.null(vline)) {
    pword <- pword+ geom_hline(yintercept=vlineword.Final.w, linetype="dashed", color = "blue")
    if(vlineword.Final.w != vlineword.Init.w)
     pword <- pword+ geom_hline(yintercept=vlineword.Init.w, linetype="dashed", color = "red")

  }

  


    if(interact==TRUE) {
   my.text <-  paste("Rank", pword$data$rank, "Word: ", pword$data$words, "<br>", "Freq: ", pword$data$iFreq, "<br>",
                      "%Before " , round(100*pword$data$iFreq/ x$summGen[2,1],round.dec), "<br>",
                      "%After " , round(100*pword$data$iFreq/ x$summGen[2,2],round.dec))
    

    suppressWarnings(g <- plotly::plotly_build(pword))
    g$x$data[[1]]$text <- my.text
    pword <- suppressWarnings(g) 
  }
} # End Words



####################################################   Docs   #################################################

if(ndoc>0){
  # If aggegate documents by qualitative variable mean length is shown. In summary and print both mean and total are shown
  var.agg <- ifelse(is.null(x$var.agg), FALSE, TRUE)
  
  mean.before.d <- x$summGen[2,2]/x$summGen[1,1]

  if(var.agg==FALSE) {
    df.doc <- as.matrix(x$DocTerm)[,df$words]
    df.doc <- data.frame("DocName"=rownames(df.doc), "iFreq"=rowSums(df.doc))
    mean.after.d <- sum(df.doc$iFreq)/ x$summGen[1,1]
   
    a <- suppressWarnings(sum(as.numeric(df.doc$DocName)))  # If a is NA, non numeric rownames
    if(!is.na(a)) df.doc$DocName <- as.numeric(df.doc$DocName)
    df.doc <- df.doc[with(df.doc, order(-iFreq, DocName)), ]
    colnames(df.doc) <- c("DocName", "Number.Words")
    # Ranks
    df.rd <- df.doc
    df.rd$rank <- rank(-df.rd$Number.Words, ties.method = "average")
    df.rd <- df.rd[df.rd$DocName %in% s.sel.doc,]
    mean.after.d <- sum(df.rd$Number.Words) / nrow(df.rd)
  }
  

  
   if(var.agg) {
    df.before <- x$summDoc[, c("DocName", "Occurrences.after", "NumberDocs")]
    df.after <- as.matrix(x$DocTerm[,final.words])
    df.doc <- data.frame("DocName"=rownames(df.after), "iFreq"=rowSums(df.after))
    df.doc <- merge(df.doc, df.before, by=c("DocName"))
    df.doc$MeanWords <- df.doc$iFreq / df.doc$NumberDocs
    df.doc <- df.doc[, c("DocName",  "iFreq", "NumberDocs", "MeanWords")]
    colnames(df.doc) <- c("DocName",  "Number.Words", "Number.Docs", "Mean.Words")
    df.rd <- df.doc
    
    df.rd<- df.rd[order(-df.rd$Mean.Words, df.rd$DocName),]
    
    df.rd$rank <- rank(-df.rd$Mean.Words, ties.method = "average")
    df.rd <- df.rd[df.rd$DocName %in% s.sel.doc,]
    df.rd$Mean.Words <-  as.numeric(df.rd$Mean.Words)
    df.rd$Number.Docs <- as.integer(df.rd$Number.Docs)
    mean.after.d <- sum(df.rd$Number.Words) / sum(df.rd$Number.Docs)
  }

  ndoc <- min(ndoc,nrow(df.rd))

  
  if(is.null(title))
    if(var.agg){
      ttitle.d<- ifelse(nrow(df.rd)==ndoc, paste0("Mean length of each category"), 
                        paste0("Mean length of ", ndoc, " longest categories"))
    } else {  # no varagg
      ttitle.d<- ifelse(nrow(df.rd)==ndoc, paste0("Mean length of each document"), 
                        paste0("Mean length of ", ndoc, " longest documents"))
    } else ttitle.d <- title
  
  if(is.null(xtitle)) {
    txtitle.d <- ifelse(var.agg, "Mean words of the category", "Number of words") 
  } else txtitle.d <- xtitle
  
  
  df.rd <- df.rd[1:ndoc,]
  
  df.rd$DocName <- as.character(df.rd$DocName)
  
  if(ordFreq==FALSE) {
    if(is.numeric(df.rd$DocName))  level_order <- as.character(sort(df.rd$DocName, decreasing=TRUE))
    else  level_order.d <- sort(df.rd$DocName, decreasing=TRUE)
  } else level_order.d <- rev(df.rd$DocName)
  

  if(var.agg) {
    df.rd$y.var <- df.rd$Mean.Words
    
  } else {
    df.rd$y.var <- df.rd$Number.Words
    df.rd$DocName <- factor(df.rd$DocName, levels = df.rd$DocName[order(df.rd$DocName)])
  }
 
  
   pdoc <- ggplot(df.rd)+ geom_bar(aes(x=factor(DocName, levels=level_order.d), y=y.var),stat = "identity",
                                  color = col.lines, fill = col.fill) +   coord_flip() +
    ggtitle(ttitle.d) + theme + theme(plot.title = element_text(hjust = 0.5)) + ylab(txtitle.d) + xlab("") 


  if(!is.null(freq))
    pdoc <- pdoc + geom_text(aes(x=df.rd$DocName, y=df.rd$y.var + freq, label = round(df.rd$y.var,round.dec)))  
  

  
  if(!is.null(vline)) {
    if(is.numeric(vline)) mean.before.d <- mean.after.d  <- vline
    pdoc <- pdoc + geom_hline(yintercept=mean.before.d, linetype="dashed", color = "red") +
     geom_hline(yintercept=mean.after.d, inetype="dashed", color = "blue")
  } 

  
  if(interact==TRUE) {
    if(var.agg) 
      my.text <-  paste("Cat: ",  pdoc$data$DocName," Number.Docs: ", pdoc$data$Number.Docs,  
                        "<br>", "Rank", pdoc$data$rank,  " Mean.Words: ", round(pdoc$data$Mean.Words,round.dec)  )
    else my.text <-  paste("Doc: ",  pdoc$data$DocName," Words: ", pdoc$data$y.var,  
                           "<br>", "Rank", pdoc$data$rank)
  #  g <- plotly::plotly_build(pdoc)

    g <- plotly::ggplotly(pdoc)
    g$x$data[[1]]$text <- my.text
    pdoc <- g

  }
} # End ndoc


if(!is.null(pseg)) {
  print(pseg)
  if(interact==TRUE) return(pseg) } 

if(!is.null(pword)) {
  suppressWarnings(print(pword))
  if(interact==TRUE) suppressWarnings(return(pword))
  }

if(!is.null(pdoc)) {
  print(pdoc)
  if(interact==TRUE) return(pdoc)}
}

#' @export
summary.LexChar <- function (object, CharWord=TRUE, stats=TRUE, CharDoc=TRUE, Vocab=TRUE,  file = NULL, ...) 
{
if (!inherits(object, "LexChar")) stop("object mut be LexChar class")
  options(stringsAsFactors = FALSE)
 

  fCharWord<- function(x)  {
  #  nlev <- length(object$CharWord)
    nlev <- length(x)
     
    for (ilev in 1:nlev)
    {			
      # Intern % glob % Intern freq Glob freq   p.value v.test
      cat(paste("\n",names(x)[ilev],sep="", "\n"))
      cat(paste(rep("-",40)))
      

      if(is.null(x[[ilev]])) { cat(paste("\n"))} else { 
        bposit <- TRUE
        cat("\n      Word          Intern %  glob % Intern freq Glob freq  p.value    v.test \n")
        cat(paste(rep("-",40)))
        for(j in 1:nrow(x[[ilev]]))
        {
          if(bposit & x[[ilev]][j,6] <0) {cat(paste("\n")) ; bposit<-FALSE }
          cat(paste("\n",stringr::str_pad(rownames(x[[ilev]])[j],width=20, side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(format(round(x[[ilev]][j,1],3),nsmall=3),width=6,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(format(round(x[[ilev]][j,2],3),nsmall=3),width=6,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(x[[ilev]][j,3],width=9,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(x[[ilev]][j,4],width=9,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(format(round(x[[ilev]][j,5],5),nsmall=5,digits=5, format="fg", scientific = FALSE)
                                     ,width=7,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(format(round(x[[ilev]][j,6],6),nsmall=6, format="fg"),width=10)))
        }
        cat("\n")			
      }	}		
    
  } # End fCharWord
    
  
  
  
  fCharWord2 <- function(x,i)  {
        nlev <- length(x)
        nwords <- nrow(x[[i]])


      if(is.null(x[[i]])) { cat(paste("\n"))} else { 
        bposit <- TRUE
        cat("\n      Word          Intern %  glob % Intern freq Glob freq  p.value    v.test \n")
        cat(paste(rep("-",40)))

        for(j in 1:nwords)
        {

          if(bposit & x[[i]][j,6] <0) {cat(paste("\n")) ; bposit<-FALSE }
        
          cat(paste("\n",stringr::str_pad(rownames(x[[i]])[j],width=20, side="left",pad=" "),"  ",sep=""))
          
          
          cat(paste(stringr::str_pad(format(round(x[[i]][j,1],3),nsmall=3),width=6,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(format(round(x[[i]][j,2],3),nsmall=3),width=6,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(x[[i]][j,3],width=9,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(x[[i]][j,4],width=9,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(format(round(x[[i]][j,5],5),nsmall=5,digits=5, format="fg", scientific = FALSE)
                                     ,width=7,side="left",pad=" "),"  ",sep=""))
          cat(paste(stringr::str_pad(format(round(x[[i]][j,6],6),nsmall=6, format="fg"),width=10)))
        }
        cat("\n")			
      }
   #   }		
    
  } # End fCharWord2
  
  sink.reset <- function(){
    for(i in seq_len(sink.number())){sink()}}
  sink.reset
  
  if(!is.null(file)) sink(file) else file=""
  

  
  #################  Check CharWord
  if("CharWord" %in% names(object)) if(CharWord==TRUE)   {
  ###### Printing characteristic words on the screen      	
  cat("\n\nCHARACTERISTIC WORDS OF EACH DOCUMENT\n(DETAILED INFORMATION)\n")
  fCharWord(object$CharWord) 
  } # end charword
  
  
  #################  Check if stats exists  
  if("stats" %in% names(object)) if(stats==TRUE) {
    cat("\n\n\nSTATISTIC ASSOCIATION LEXICAL TABLE DOCUMENTS (OR AGGREGATED DOCS,) AND VOCABULARY\n\n")
    print(object$stats)
  }
  
  if("quali" %in% names(object)) if(Vocab==TRUE)   {
    if(!is.null(object$quali))  {
      cat("\n\nCHARACTERISTIC QUALITATIVE VARIABLES\n")
      cat("\nStatistics for qualitative variables\n")
      cat("\n-------------------------------------\n")
      print(object$quali$stats)
      
      
      
      for(i in 1:length(object$quali$CharWord)) {
        cat("\n\nCharacteristic words for qualitative variable: ", names(object$quali$CharWord[i]), "\n")
        #    cat("\n\nCharacteristic words for qualitative variable:\n")
        cat("=====================================================================================\n")
        fCharWord2(object$quali$CharWord,i) 
      }
    } # End quali
    
    
    if(!is.null(object$quanti))  {
      cat("\n\nCHARACTERISTIC QUANTITATIVE VARIABLES\n")
      cat("\nStatistics for quantitative variables\n")
      cat("\n-------------------------------------\n")
      print(object$quanti$stats)
      
      cat("\n\nCharacteristic quantitative variables for each word\n")
      cat("\n-----------------------------------------------------\n")
      print(object$quanti$CharWord)
      
      
      
      
      cat("\n\nCharacteristic words for each quantitative variable\n ")
      cat("\n---------------------------------------------------\n")
      
      fCharQuanti<- function(x)  {
        tac <- NULL
        strcolnames<- c("GlobalAverage", "AverageWord","Differ.", "pvalue", "Word", "Variable")
        for(i in 1:length(x)) {
          t1<- as.data.frame(x[i,drop=FALSE])
          t2 <- data.frame(t1,rep(names(x[i]),length(x[i])), rownames(t1))
          colnames(t2) <- strcolnames
          if(i==1) tac <- t2 else tac <- rbind(tac,t2)
          SP <- split(tac,f=tac$Variable, drop=FALSE)
          str.colnames<- c("Word", "GlobalAverage", "AverageWord","Differ.", "pvalue")
          empty_list = structure(vector(mode = "list", length = length(SP)), names = names(SP))			
          for(i in 1:length(SP)) {
            t1<- as.data.frame(SP[i,drop=FALSE])
            t2.pos <- t1[t1[,3]>0, ,drop=FALSE]
            t2.neg <- t1[t1[,3]<0, ,drop=FALSE]
            if(nrow(t2.pos)>0) {
              t2.pos <- t2.pos[order(t2.pos[,4]),,drop=FALSE]
              rownames(t2.pos) <- paste0("P", c(1:nrow(t2.pos)))
              t3.pos <- t2.pos[,c(5,1:4)]
              colnames(t3.pos) <- str.colnames
              empty_list[[i]]$posit <- t3.pos
            }
            if(nrow(t2.neg)>0) {
              t2.neg <- t2.neg[order(t2.neg[,4]),,drop=FALSE]
              rownames(t2.neg) <- paste0("N", c(1:nrow(t2.neg)))
              t3.neg <- t2.neg[,c(5,1:4)]
              colnames(t3.neg) <- str.colnames
              #    print(t3.neg)
              empty_list[[i]]$negat <- t3.neg
            }
          } # End for
        }
        return(empty_list)
      } # End fCharQuanti
      
      res <- fCharQuanti(object$quanti$CharWord)
      print(res)
      
      
      if(!is.null(object$quanti.aggr)) {
        cat("\n\n\nQUANTITATIVE STATISTICS AND WORDS FOR AGGREGATED CONTEXTUAL VARIABLES\n\n")
        cat(names(object$quanti$CharWord))
        cat("\nStatistics for quantitative variables\n")
        cat("\n-------------------------------------\n")
        print(object$quanti$stats)
        
      }
      
      
    } # End quanti
  } 
  

  if(file!=""){
    sink()
    cat("\nAll the results are in file: ",file,"\n")
  }


}

#' @rawNamespace import(flexclust, except = c(barplot, boxplot, predict, plot, pairs, image))
#' @import flashClust
#' @import cluster
#' @rawNamespace import(dendextend, except=c(cutree))
#' @export
LexHCca <- function(x, cluster.CA="docs", type="agnes",  ncp=5, nb.clust="click",  min=2, max=NULL,
                    kk=Inf, consol=FALSE, iter.max=500, graph=TRUE, description=TRUE, 
                    proba=0.05, nb.desc=5, size.desc=80, marg.doc="before", seed=12345,...)
  
{
  #### Change method.sel to "Ward.D2"
 # agnes function (x, diss = inherits(x, "dist"), metric = "euclidean", 
 #            stand = FALSE, method = "average", par.method, keep.diss = n < 
 #              100, keep.data = !diss, trace.lev = 0) 

  dots <- list(...)
  if(is.null(dots$method)) method.sel <- "ward"
    else method.sel <-  dots$method
    str.type <- c("agnes","diana")
    if(!type %in% str.type) stop("Argument type must be agnes or diana")

######################  New. Modification of x removing no factors used ######################
    ncp.max <- ncol(x$row$coord)
    if(ncp>ncp.max)   {
      warning(paste0("Number of components ncp= ", ncp ," is bigger than number of components in ",
                     deparse(substitute(x)), "= ", ncp.max),"\nncp is changed to ",ncp.max)	
     ncp <- ncp.max
    } 
    x$row$coord <- x$row$coord[, 1:ncp, drop=FALSE]
    x$row$contrib <- x$row$contrib[, 1:ncp, drop=FALSE]
    x$row$cos2 <- x$row$cos2[, 1:ncp, drop=FALSE]
    x$row$inertia <- x$row$inertia[1:ncp, drop=FALSE]

    x$col$coord <- x$col$coord[, 1:ncp, drop=FALSE]
    x$col$contrib <- x$col$contrib[, 1:ncp, drop=FALSE]
    x$col$cos2 <- x$col$cos2[, 1:ncp, drop=FALSE]
    x$col$inertia <- x$col$inertia[1:ncp, drop=FALSE]
    
    if(!is.null(x$quanti.sup$coord)) {
      x$quanti.sup$coord <- x$quanti.sup$coord[, 1:ncp, drop=FALSE]
      x$quanti.sup$cos2 <- x$quanti.sup$cos2[, 1:ncp, drop=FALSE]   }
    
    
    if(!is.null(x$quali.sup$coord)) {
      x$quali.sup$coord <- x$quali.sup$coord[, 1:ncp, drop=FALSE]
      x$quali.sup$cos2 <- x$quali.sup$cos2[, 1:ncp, drop=FALSE] 
      x$quali.sup$v.test <- x$quali.sup$v.test[, 1:ncp, drop=FALSE] 
      x$quali.sup$eta2 <- x$quali.sup$eta2[, 1:ncp, drop=FALSE] }

    if(!is.null(x$meta$Word)) x$meta$Word <- x$meta$Word[as.numeric(x$meta$Word$Dim)<(ncp+1),, drop=FALSE]
    if(!is.null(x$meta$Doc)) x$meta$Doc <- x$meta$Doc[as.numeric(x$meta$Doc$Dim)<(ncp+1),, drop=FALSE]
################################# End removing not used factors #############################
  
  object <- x
if (!inherits(object, "LexCA")) stop("object should be LexCA class")
  options(stringsAsFactors = FALSE)
  
#   marg.doc<-"before"
 if(description==FALSE)  nb.desc <-0
  
  


#---- Initial checks -----------------------------------------
  ## Type selection number of clusters
  ### nb.clust can be a number; 0 is click; -1 is automatic
if(is.character(nb.clust))
  nb.clust <- ifelse(nb.clust == "auto", -1,0)
 # if(nb.clust=="click") nb.clust<- 0
 # if(nb.clust=="auto") nb.clust<- -1
  
  graph.scale="inertia"


 
  metric = "euclidean"
  if (cluster.CA=="rows" ) cluster.CA <- "docs"
  if (cluster.CA=="columns" ) cluster.CA <- "words"
 #---- End Initial checks -----------------------------------------


  
#---- F1. auto.cut.tree function -----
  auto.cut.tree <- function(res, min, max, metric, method=method.sel, weight = NULL, cla = NULL, ...)
  {
    set.seed(seed)
    X <- as.data.frame(res$ind$coord, drop=FALSE)
    do <- dist(X, method = metric)^2	
    eff <- outer(weight, weight, FUN = function(x, y, n) {x * y/n/(x + y)}, n = sum(weight))
    dissi <- do * eff[lower.tri(eff)]
    if(type=="agnes") hc <- flashClust::hclust(dissi, method = method, members = weight)

    if(type=="diana") hc <- cluster::diana(dissi)
    inert.gain <- rev(hc$height)
    intra <- rev(cumsum(rev(inert.gain)))
    quot <- intra[min:(max)]/intra[(min-1):(max-1)]
  

    nb.clust.new <- which.min(quot) + min-1     # Cluster proposal
    rm(.Random.seed,envir=globalenv()) # Retrieve set.seed
    if(kk==nb.clust)  nb.clust.new <- kk
    return(list(res = res, tree = hc, nb.clust = nb.clust.new,
                within = intra, inert.gain = inert.gain, quot = quot, dissi=dissi))
  }	  # End Function auto.cut.tree  
  
  

  
#---- F2. print.inertia function ----
  print.inertia <- function(title) {
    if (!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) 
      dev.new()
    old.mar <- par()$mar
    par(mar = c(0.5, 2, 0.75, 0))
    lay = matrix(ncol = 5, nrow = 5, c(2, 4, 4, 4, 4, 
                                       2, 4, 4, 4, 4, 2, 4, 4, 4, 4, 2, 4, 4, 4, 4,1, 3, 3, 3, 3))
    layout(lay, respect = TRUE)
     barplot(rev(t$tree$height)[1:max(15, max)], 
           col = c(rep("black",t$nb.clust),
           rep("grey", max(max, 15) - t$nb.clust + 1)), rep(0.1, max(max, 15)), space = 0.9)
    plot(x = 1, xlab = "", ylab = "", main = "", col = "white",axes = FALSE)
    text(1,1,title,cex=2)
    plot(x = 1, xlab = "", ylab = "", main = "", col = "white",axes = FALSE)
    legend("top", "Aggreg. Crit", box.lty = NULL, cex = 1)
  }    # End function print.inertia
  
  


#---- F3. sum of squares ----
  ssquares <- function(coord, weight_,clust_){ # }, weight_, clust_) {
    dfw <- data.frame(coord,"weight_"=weight_, "clust_"=clust_)
        sum.weights <- sum(dfw$weight_)
        pct.weight <- dfw$weight_/ sum.weights  
        dfclust <- as.data.frame(table(clust_))
        dimen <- ncol(coord)
    mediadim <- apply(dfw[,c(1:dimen)]*pct.weight,2,FUN=sum)
    dfw$sstot_ <- apply(sweep(dfw[,c(1:dimen)],2,mediadim,"-")^2  *pct.weight,1,FUN="sum")
    cl.weight  <- aggregate(pct.weight, by=list(dfw$clust_), FUN=sum)$x
    coord_centers <- aggregate(dfw[,c(1:dimen)]*pct.weight/cl.weight[dfw$clust_],by=list(dfw$clust_), FUN=sum) 
    dfw$ssintra_ <- apply(((dfw[,c(1:dimen)] - coord_centers[clust_, c(2:ncol(coord_centers))])^2),1,sum) *pct.weight
    dfw$ssinter_ <- dfw$sstot_-dfw$ssintra_
    dfw$ssexplain_ <- 100*dfw$ssinter_ / dfw$sstot_ 
    dfclust$weight <- cl.weight*sum.weights 
    dfclust$sstot <- aggregate( dfw$sstot_,by=list(dfw$clust_), FUN=sum)$x
    dfclust$ssintra <- aggregate( dfw$ssintra_,by=list(dfw$clust_), FUN=sum)$x
    dfclust$ssinter <- dfclust$sstot-dfclust$ssintra
    dfclust$ssexplain <- 100*dfclust$ssinter / dfclust$sstot 
    ss.tot <- sum(dfclust$sstot)  
    ss.intra <- sum(dfclust$ssintra)
    ss.inter <- sum(dfclust$ssinter)
    pct.explained <- 100* ss.inter / ss.tot

       if (cluster.CA == "docs") {							
      colnames(dfclust) <- c("Cluster", "Docs", "Occurrences", "ss.tot", "ss.intra", "ss.between", "%Explained")							
    } else {							
      colnames(dfclust) <- c("Cluster", "Words", "Occurrences", "ss.tot", "ss.intra", "ss.between", "%Explained")							
    }	
     dfw <- dfw[,c((ncol(dfw)-5):ncol(dfw))]
     colnames(dfw) <- c("Weight","Cluster", "ss.tot", "ss.intra", "ss.between","%Explained")
    
    res.ss <- list("ss.tot"=ss.tot, "ss.intra"=ss.intra,"ss.inter"= ss.inter, "coord.centers"=coord_centers, "dfw"=dfw,
                   "dfclust"=dfclust)
    return(res.ss)
  }  # End function F3. sum of squares
  
  
  
  #---- F4. Consolidation function ----
  consolidationW <- function(coordinates, cluster_i,  weights =NULL, method="hardcl", iter.max = iter.max, ...) {		
    # X son las coordenadas documents x dims
    # iter.max <- iter.max+1
     # init_centers <- as.matrix(centers[, c(2:ncol(centers))])
    iter <- 0
    cluster_iminus1 <- 1

    while(identical(cluster_i, cluster_iminus1) == F && iter < iter.max){
      iter <- iter + 1			 # update iteration	
      suppressWarnings(
        ck_flex <- flexclust::cclust(coordinates,k=cluster_i, method="hardcl",weights= weights))
       # w_centers <- as.matrix(ck_flex@centers)
      cluster_iminus1 <- cluster_i
      cluster_i <- ck_flex@cluster
    } # End while
    return(ck_flex@cluster)
  } 
  #---- F4 End Consolidation function ----
  #---- F5 descfreq_New ----
  descfreq_New <- function (donnee, by.quali = NULL, proba = 0.05, row.INIT, col.INIT)
  {
    lab <- colnames(donnee)
    for (i in 1:length(lab)) lab[i] = gsub(" ", ".", lab[i])
    

    if (!is.null(by.quali)) {
      donnee <- as.data.frame(matrix(unlist(by(donnee, by.quali,apply, 2, sum)), ncol = ncol(donnee), byrow = TRUE))				
     rownames(donnee) <- levels(by.quali)
      marge.li <- aggregate(row.INIT, by=list(by.quali), FUN="sum")
      marge.li <- as.vector(marge.li$x)
    }
    
    colnames(donnee) <- lab
    old.warn = options("warn")
    options(warn = -1)				
    nom <- tri<- structure(vector(mode = "list", length = nrow(donnee)), names = rownames(donnee))	
    
    marge.col <- col.INIT
    SumTot <- sum(marge.li)
    nr <- nrow(donnee)
    nc <- ncol(donnee)
    for (j in 1:nr) {
      aux3 = marge.li[j]/SumTot
      for (k in 1:nc) {
        aux2 = donnee[j, k]/marge.col[k]
        
        
        if (aux2 > aux3) # Over representation
          aux4 <- phyper(donnee[j, k] - 1, marge.col[k],SumTot - marge.col[k], marge.li[j],lower.tail=FALSE)*2
        else aux4 = phyper(donnee[j, k], marge.col[k], SumTot - marge.col[k], marge.li[j])*2
        
        if (aux4 > 1)
          aux4 <- 2 - aux4
        if (aux4 < proba) {
          aux5 <- (1-2 * as.integer(aux2 > aux3)) * qnorm(aux4/2)
          aux1 = donnee[j, k]/marge.li[j]
          tri[[j]] = rbind(tri[[j]], c(aux1 * 100, sum(marge.col[k])/SumTot *100, donnee[j, k], marge.col[k], aux4, aux5))
          nom[[j]] = rbind(nom[[j]], c(colnames(donnee)[k], colnames(donnee)))
        }				
      }		
    }			
    
    for (j in 1:nrow(donnee)) {
      if (!is.null(tri[[j]])) {
        oo = rev(order(tri[[j]][, 6]))
        tri[[j]] = tri[[j]][oo, ]
        nom[[j]] = nom[[j]][oo, ]
        if (nrow(matrix(tri[[j]], ncol = 6)) > 1)
          rownames(tri[[j]]) = nom[[j]][, 1]
        else {
          tri[[j]] = matrix(tri[[j]], ncol = 6)
          rownames(tri[[j]]) = nom[[j]][1]
        }
        colnames(tri[[j]]) = c("Intern %", "glob %", "Intern freq","Glob freq ", "p.value", "v.test")
      }
    }
    names(tri) <-paste("cluster",1:length(tri),sep="_")
    res <- tri
    options(old.warn)
    class(res) <- c("descfreq", "list ")
    return(res)				
  }	
  #---- End descfreq_New function -------------------------------
  
  

  ########################################################################################  
  # Starting analysis ------------------------------------------------------------  
  ######################################################################################## 


   #---- 1. Weighting documents (after/before) and words ----
  # marg.doc is used to characterization
  # For cluster analysis, the weighting is always after. Named vector
  doc.w.after <- apply(object$call$X,1,sum)    # Initial weighting for documents after selection
  bBefore <- ifelse(marg.doc=="before", TRUE, FALSE)

  if(bBefore) {
    doc.w.before <- object$rowINIT[which(rownames(object$rowINIT) %in%  names(doc.w.after)), ,drop=FALSE]
    # Convert the matrix or data frame to a named vector
    doc.w.before <- setNames(as.vector(doc.w.before[,1]), rownames(doc.w.before))
  } 
    # doc.w.before before word's selection
    # Remove documents before selection when they have zero frequencies after selection
    # named vector
    word.w <- apply(object$call$X, 2, sum) 
 



  #---- 2. Selecting words or documents to cluster  ----
  if (cluster.CA == "docs") {
    weight.dw <- doc.w.after # 
    coord.dw <- object$row$coord
  } else {
    weight.dw <- word.w
    coord.dw <- object$col$coord
  }   
  
    
    
    
    

# Maximum number of clusters
n.max.clust <- nrow(coord.dw)-1

if (min < 2) {
  warning("The minimum number of clusters must be 2. min argument is changed to 2")
  min <- 2
}



if (!is.null(max)) if(min > max) stop("The minimum number of clusters min = ", min, " can't be higher than the maximum max = ", max)
if (min > n.max.clust) stop("The minimum number of clusters min = ", min, " can't be higher than the number of cases - 2 = ",
                            n.max.clust-1,"\n", "Please change to min <= ", n.max.clust-1)                   
                            



if(kk!=Inf) {   # k-means
  if(n.max.clust < kk)
   stop("Number of clusters in k-means = ", kk, " can't be higher than the number of cases - 2 = ", n.max.clust-1,"\n",
            "Please change to kk <= ", n.max.clust-1)
  if (kk < 2) stop("The minimum for kk is 2. Please, change kk value")
  if (is.null(max)) max <- kk
  
  if (max < kk)   stop("The maximum max = ", max, " is lower than kk = ", kk, " Please, change max value")
  if (kk < nb.clust & nb.clust > 0) {
    warning("nb.clust = ", nb.clust, " can't be higher than kk = ", kk, " Automatic change nb.clust = kk.")
    nb.clust <- kk
  }                     
  if (consol == TRUE) {
    warning("No consolidation has been done after the hierarchical clustering since kk is different from Inf")				
    consol <- FALSE
  }
  if(type == "diana") 
    stop("No divisive method (diana) with k-means (kk argument) is allowed.")
  if(nb.clust > kk) {
    warning("K-means builds kk= ", kk, " groups, nb.clust= ", nb.clust, " can't be higher than kk.", "\n",
            "  nb.clust= ", nb.clust, " is changed to nb.clust= ", kk)
    nb.clust <- kk     
  }
}



    
  #---- 3. Building PCA object ----
  # Check if kmeans will be performed
  # Operations with kk
  #  If kk = Inf => hierarchical cluster
  #  If kk is not Inf => k-means cluster. If k-means coordinates * sqrt(eigenvalue)   
  ################# Selecting cluster of rows (documents) or columns (words) 
  cla <- res <- NULL	
  


  #########################################################  Kmeans    ###########################
  if (kk < nrow(coord.dw)) {  # Kmeans
    res <- as.data.frame(sweep(coord.dw, 2, sqrt(object$eig[1:ncol(coord.dw), 1]), FUN = "*"))
    kk <- min(kk, nrow(unique(coord.dw)))
  } else 
  { if(kk!=Inf)
    stop("Number of elements to cluster using K-means= ", kk, " can't be higher than ", nrow(coord.dw), " cases.")
  }
    

    
   if (!is.null(res)) {	# It is k-means. res is a dataframe coordinates active documents (or words) x dimensions	
     
    if (kk < nrow(res)) kk <- min(kk, nrow(unique(res)))	
  
    if (kk <= nrow(res)) {	
      # Weighted k-means (no with Factoclass) Do it with flexclust   
      set.seed(seed)
      cla <- flexclust::cclust(res,k=kk, method="hardcl",weights= weight.dw)
      # cla <- flexclust::cclust(res,k=kk, method="kmeans",weights= weight.dw)
      cla <- data.frame(cla@cluster) # documents/words win names x 1 column whith number of cluster	
      cla <- cbind(res, weight.dw ,cla[rownames(res),,drop=FALSE])
      cla1 <- cbind(cla[,c(1:(ncol(cla)-2))]*cla[,(ncol(cla)-1)],cla[,c((ncol(cla)-1):ncol(cla))])
      cla.agg <- aggregate(cla1[,c(1:(ncol(cla1)-1))], by=list(cla.cluster=cla$cla.cluster), FUN=sum)	
      cla.centers <-  cla.agg[,c(2:(ncol(cla.agg)-1))] /cla.agg[,ncol(cla.agg)]  
      res <- FactoMineR::PCA(cla.centers, row.w = cla.agg[,ncol(cla.agg)], scale.unit = FALSE, ncp = Inf, graph = FALSE)
      rm(.Random.seed,envir=globalenv()) # Retrieve set.seed
    }          # end      if (kk <= nrow(res)) 
  } # Final of k-means
  
    

    
  ################  Hiearchical ascendent cluster ############################
  # If kk >= nrow(word) now a hierarchical cluster is performed, res don't exist
  if(kk==Inf){ ## It is hierarchical cluster, no k-means
    aux <- object$eig			
    res <- FactoMineR::PCA(coord.dw, scale.unit = FALSE, ncp = Inf, graph = FALSE, row.w = weight.dw)	
    res$eig <- aux			
  }  # Now, all res object are PCA objects type;  End Building PCA object



  #---- 4. Checking the minimum and maximum number of clusters ----
  # max min  "ind" is documents if cluster.CA="docs", words if cluster.CA="words"
  nb.ind <- nrow(coord.dw)			## Cases (docs o words) 


    

  if (is.null(max))    max <- min(10, round(nb.ind/2))
  
  if(max > (nb.ind-1)){
    warning("The number of selected clusters is bigger than the number of cases - 1.
              Automatic change: max=", nb.ind-1)	
    max <- nb.ind-1
  }  
    

  if (nb.clust > 1 && nb.clust > max) {
    warning("The number of selected clusters is bigger than the number of maximum clusters.
              Automatic change: max=", min(nb.clust, nrow(res$ind$coord) - 1))				
    max <- min(nb.clust,nrow(res$ind$coord) - 1)
    nb.clust <- max  # New instruction
  }    
  

  
  
  if(max<2) {
    stop("The maximum number of clusters (",max,") can not be lower than 2")
  }
  
  
  if(max < min) {
    warning("The minimum number of clusters (",min,") can not be bigger than the maximum (",max,"). 
              Automatic change: min=max")				
    min<- max
  }   # End max min
  

  
  
  #---- 5. Building the tree to select number clusters -----   
  # Retrieve the tree with cases if hierarchical or groups if k-means	
  # row.w.init has the weighting from rows from res object of PCA
  # method=ward, metric=euclidean , res$call$row.w.init is absolute frequency after
  

  ####### stop("repasar cla y order) en el original. ###############################
  # res$call$row.w.init has the frequencies after selection
  

    t <- auto.cut.tree(res, min = min, max = max, metric = metric, method.sel = method.sel,
                     weight = res$call$row.w.init, cla = cla, order = order, ...)	
   # tree can be plotted using plot(t$tree)


  

  # We need to select the number of clusters  only if necessary if nb.clust = click (0 value)
  if (graph.scale == "inertia") {
    nb.ind.clI <- nrow(t$res$ind$coord)
    inertia.height <- rep(0, nb.ind.clI - 1)							
    for (i in 1:(nb.ind.clI - 1)) inertia.height[i] <- t$inert.gain[(nb.ind.clI -i)]							
    inertia.height <- sort(inertia.height, decreasing = FALSE)	
    t$tree$height <- inertia.height							
  }			# end of graph.scale =="inertia"
  


  #---- 6. Check number clusters (max,min) before selection ----
  options(warn=0)
  if (nb.clust == 1)  #
  {
    warning("The number of clusters cannot be 1. Automatic change: nb.clust=",-1)				
    nb.clust <- -1
  }  
  
  
  
  
  #-------------------------------
  kmeans.only <- FALSE
  if(nb.clust==kk)  kmeans.only <- TRUE




    
    


  y <- 0
  #---- 7. Compute Height value  for proposed number of clusters, necessary to plot for selection ----
  if(!kmeans.only) {
    auto.haut <- ((t$tree$height[length(t$tree$height) - t$nb.clust + 2]) +		
                  (t$tree$height[length(t$tree$height) - t$nb.clust + 1]))/2 
    #---- 8. Print Inertia gain graph for selection number of groups ----
  if (nb.clust==0) {
    # New 1.5.6 

    graphics.off()
    dev.new(noRStudioGD = TRUE)
    
    # End new 1.5.6
    par(mfrow=c(1,1))			
    print.inertia("Hierarchical Clustering")
    print("Click on the graph to cut the tree")	
    flush.console()		

    par(mar = c(6, 4, 4, 2) + 0.1)
     dend <- as.dendrogram(as.hclust(t$tree))
    plot(dend, main = "Click to cut the tree", xlab = "", sub = "",
         ylim = c(0, max(t$tree$height)), yaxs = "i")
    
    text(1, 1, "Hierarchical Clustering", "cex" = 2)	
    abline(h = auto.haut, col = "black", lwd = 3)	
    # graphics.off()
    # windows() (for Windows) / quartz() (for Mac) / x11() (for UNIX)
    coupe <- locator(n = 1)		

      while(coupe$y < 0 | coupe$y > max(t$tree$height)) {
      cat("No class \n")							
      coupe <- locator(n = 1)							
    }						    # End while	
    y <- coupe$y							
    # y is a value of height, value of xy axis							
    # If it is not manual selections inertia gain graph is not plotted
    # Next added 1.5.6
    dev.off(which = dev.cur())
  } else {
    
    if (nb.clust < 0) 	
      y <- auto.haut	
    else     y <- (t$tree$height[length(t$tree$height) - nb.clust + 2] + 
                t$tree$height[length(t$tree$height) - nb.clust + 1])/2	
  }


  # y has the point of cut
  # clust is a vector with composition of clusters (if kmeans not the case)
  # integer with the number of the cluster
  # The name is the label of the doc/word 
  
  }  # End   if(!kmeans.only) 
 

  
  clust <- stats::cutree(as.hclust(t$tree), h = y)	
  nb.clust <- max(clust)	

  

  #---- End Inertia gain graph for selection case ----
  # Order of clusters in dendrogram (duplicates are removed)
  ordColo <- unique(clust[t$tree$order])   # Need to plot rectangles when number of clusters known
  X <- as.data.frame(t$res$ind$coord)	   # Coordinates Docs or words x dimensions, If kmeans groups x dimensions
 


  

  

  #---- 9. df.coord.cla docs x  (Dim.1;...;Dim k; clusterweight numbercluster ----
  if(kk==Inf) {  # Hierarchical
    df.coord.cla <- data.frame(X, weight= res$call$row.w.init[rownames(X)], cluster=clust[rownames(X)])
  }
  if(kk!=Inf) {   # K.means
    df.coord.cla <-   data.frame(coord.dw, weight= cla[rownames(coord.dw),"weight.dw"], cluster=clust[cla$cla.cluster])                               #cluster=cla[rownames(coord.dw),"cluster"])
  }






  #---- 10 Print dendrogram - Selection of the number of the clusters has been done, cla ----
  if(graph) if(kk!=Inf) {
    warning("Warning: Dendrogram is not plotted for kmeans method")
   # graph <- FALSE
  }
  if(graph) if(consol) {
    warning("Warning: Dendrogram is not plotted for consolidation method")
  #  graph <- FALSE
  }


  if(graph & kk==Inf) { 
    par(mfrow=c(1,1))
    if(type=="agnes") str.title <- "Aglomerative Hierarchical Clustering" else
      str.title <- "Divisive Hierarchical Clustering"  
    if(type=="agnes") plot(t$tree, hang = -1, main = str.title, xlab = "", sub = "")
    if(type=="diana") plot(t$tree, which.plots=2, hang = -1, main = str.title, xlab = "", sub = "")
    text(1, 1, str.title, "cex" = 2)
    rect <- rect.hclust(t$tree, h = y, border = ordColo)
    clust <- NULL
    
    for (j in 1:nb.clust) clust <- c(clust, rep(j, length(rect[[j]])))
    clust <- as.factor(clust)
    belong <- cbind.data.frame(t$tree$order, clust)
    belong <- belong[do.call("order", belong), ]
    clust <- as.factor(belong$clust)
    names(clust) <- rownames(X)
    if (nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")))				
      layout(matrix(nrow = 1, ncol = 1, 1), respect = TRUE)
  }

  
  

  #---- 11. Consolidation if hierarchical ascendent ----			
 if(consol==TRUE){
   clust.afterconsol <- consolidationW(df.coord.cla[,c(1:(ncol(df.coord.cla)-2))], cluster_i=df.coord.cla[,"cluster"],
                                               weights=df.coord.cla[,"weight"] , iter.max = iter.max)
   df.after <- as.data.frame( clust.afterconsol ) 
   dftemp<- data.frame(df.coord.cla[,c(1:(ncol(coord.dw)))],  weight=df.coord.cla[,"weight"], cluster=df.coord.cla[,"cluster"])
   dftemp$cluster <- df.after[rownames(dftemp),"clust.afterconsol"]
   df.coord.cla <- dftemp
}


  #---- 12. Computing centers. Building RDO object  ----			
  RDO <-   ssquares(df.coord.cla[,c(1:(ncol(coord.dw)))],  df.coord.cla[,"weight"], df.coord.cla[,"cluster"]) 
  RDO$CA <- object


  
  RDO <- list("data.clust"=data.frame(df.coord.cla[,c(1:(ncol(coord.dw)))], "clust"=df.coord.cla[,"cluster"]),   
              "centers"=RDO$coord.centers, "clust.count"=RDO$dfclust, "clust.content"=RDO$dfw,
              "global"= list("ss.tot"=RDO$ss.tot, "ss.intra"=RDO$ss.intra,"ss.inter"= RDO$ss.inter,			
                             "%Explained"= 100*RDO$ss.inter/RDO$ss.tot), cluster.CA=cluster.CA)
 
  class(RDO) <- "LexHCca"
    # End 12. Computing centers. Building RDO object 
 
  #---- 13. Weighting documents (after/before) and words ----
  if(bBefore) doc.w <- doc.w.before else doc.w <- doc.w.after # before word's selection

  
  #---- 14. tabInd coordinates + cluster if active docs/words
  if (cluster.CA == "docs"){ 
    descr.data <- object$call$X   # after docs x words in columns
    descr.cl <- data.frame("cluster_"=df.coord.cla[rownames(descr.data),"cluster"])  # drop=FALSE
    rownames(descr.cl) <- rownames(descr.data)

    tabInd <- cbind.data.frame(object$row$coord, clust= as.factor(df.coord.cla[rownames(descr.data),"cluster"]))
  }else{ 
    descr.data <- t(object$call$X)
    descr.cl <- data.frame("cluster_"=df.coord.cla[rownames(descr.data),"cluster"] ) # drop=FALSE
    rownames(descr.cl) <- rownames(descr.data)
    tabInd <- cbind.data.frame(object$col$coord, clust=as.factor(df.coord.cla[rownames(descr.data),"cluster"]))
  }
  #---- Final 14 



  #---- 15. descword, Cluster description by words and descdoc ---- 
  desc.ind <- descaxes <- descword <- descwordsup <- descdoc <- descquali <- descquanti <- NULL
  if(description) { 
    if(cluster.CA=="docs") {
      if(bBefore)   descr.w.rows <- doc.w.before else descr.w.rows <- doc.w
        descr.w.cols <- word.w[colnames(descr.data)]
      # Cluster Description by active words
      descword <- descfreq_New(descr.data, descr.cl$cluster_,prob=proba, 
                               row.INIT= descr.w.rows, col.INIT=descr.w.cols) 
          } else { 
            # Cluster of words
      descr.w.rows <- word.w[rownames(descr.data)]
       # Cluster Description by active documents # Using doc.w.after
      descdoc <- descfreq_New(descr.data, descr.cl$cluster_ ,prob=proba, 
                              row.INIT= descr.w.rows, col.INIT=doc.w) 
    } 
    #----  15. End descword, Cluster description by words and descdoc ----    

    


 
    #---- 16. Document description by qualitative supplementary variables ----
    if(!is.null(object$quali.sup$coord)) {
      if(cluster.CA=="docs") {
        a1 <- object$call$Xtot[rownames(descr.data), rownames(object$quali.sup$eta2), drop=FALSE]
        a2 <- cbind(a1, "clust_"=as.factor(descr.cl[rownames(a1),"cluster_"]))
         descquali <- FactoMineR::catdes(a2, num.var=ncol(a2), proba = proba, row.w=doc.w[rownames(a2)])
         names(descquali$category) <-paste("cluster",1:length(descquali$category),sep="_")
      }      }
    #---- 16. End Document description by qualitative supplementary variables ----    
 


    #---- 17. Document description by quantitative supplementary variables ----
    if(!is.null(object$quanti.sup$coord)) {
      if(cluster.CA=="docs") {
        a1 <- object$call$Xtot[rownames(descr.data), rownames(object$quanti.sup$coord), drop=FALSE]
        a2<- cbind(a1, "clust_"=as.factor(descr.cl[rownames(a1),"cluster_"]))
        descquanti <- FactoMineR::catdes(a2, num.var=ncol(a2), proba = proba, row.w=doc.w[rownames(a2)])
        if(!is.null(descquanti$quanti)>0)  
          names(descquanti$quanti) <-paste("cluster",1:length(descquanti$quanti),sep="_")
      }}
    #---- 17. End Document description by quantitative supplementary variables ----

    #---- Description of axes ----
    if(cluster.CA=="docs") {
      descaxes <- data.frame(cbind(object$row$coord,clust_=as.factor(descr.cl[rownames(object$row$coord),"cluster_"])))
      descaxes$clust_ <- as.factor(descaxes$clust_)
      descaxes <- FactoMineR::catdes(descaxes, num.var=ncol(descaxes), proba = proba, row.w=doc.w[rownames(descaxes)])  
    } else { # columns
      descaxes <- data.frame(cbind(object$col$coord,clust_=as.factor(clust[rownames(object$col$coord)])))
      descaxes$clust_ <- as.factor(descaxes$clust_)
      descaxes <- FactoMineR::catdes(descaxes, num.var=ncol(descaxes), proba = proba, row.w=word.w[rownames(descaxes)]) 
    }
    

  
    if(nb.desc>0){
      # Computing centers
      # list.centers <- by(tabInd[, -ncol(tabInd), drop = FALSE], tabInd[, ncol(tabInd)], colMeans)
      # centers <- matrix(unlist(list.centers), ncol = ncol(tabInd) - 1, byrow = TRUE)
      #---- Function select --------------------------------
      select <- function(Y, default.size, method, coord.centers) {
        clust <- as.numeric(Y[1, ncol(Y)])
        Y <- Y[, -ncol(Y), drop = FALSE]
        colnames(coord.centers) <- colnames(Y)
        Z <- rbind(Y, coord.centers)

        if (nrow(Y) == 1) {  #No elements
          distance <- data.frame(0, row.names = "")
          colnames(distance) <- rownames(Z[1, ])
        }
        else {
          distance <- as.matrix(dist(Z, method = method))
          distance <- distance[(nrow(Y) + 1):nrow(distance), 
                               -((nrow(Y) + 1):ncol(distance))]
          distance <- sort(distance[clust, ], decreasing = FALSE)
        }
        if (length(distance) > default.size) 
          distance <- distance[1:default.size]
        else distance <- distance
      }

            #---- Function distinctivness  ---------------------
      distinctivness <- function(Y, default.size, method, coord.centers) {
        clust <- as.numeric(Y[1, ncol(Y)])
        Y <- Y[, -ncol(Y), drop = FALSE]	
        colnames(coord.centers) <- colnames(Y)
        Z <- rbind(Y, coord.centers)
        if (nrow(Y) == 1) {
          distance <- as.matrix(dist(Z, method = method))
          ind.car <- vector(length = 1, mode = "numeric")
          ind.car <- min(distance[-c(1, (clust + 1)), 1])
          names(ind.car) <- rownames(Z[1, ])		
        }
        else {
          distance <- as.matrix(dist(Z, method = method))
          distance <- distance[(nrow(Y) + 1):nrow(distance),-((nrow(Y) + 1):ncol(distance))]
          if (nrow(distance) == 2)
            center.min <- distance[-clust, ]
          else center.min <- apply(distance[-clust, ], 2, min)
          ind.car <- sort(center.min, decreasing = TRUE)		
        }	
        if (length(ind.car) > default.size)
          ind.car = ind.car[1:default.size]
        else ind.car = ind.car
      } # Final distinctivness
      #---- End Function distinctivness  ---------------------
      
      m_centers <- data.matrix(RDO$centers)[,-1]
      Cluster <- tabInd[,ncol(tabInd), drop=FALSE]
      para <- by(tabInd, Cluster, simplify = FALSE, select, 
                 default.size = nb.desc, method = metric, coord.centers = m_centers)
      dist <- by(tabInd, Cluster, simplify = FALSE, distinctivness, 
                 default.size = nb.desc, method = metric, coord.centers = m_centers)
      desc.ind <- list(para = para, dist = dist)

    } # End nb.par
  }  
  #---- Final Descriptions ----



  newcall <- list(t = t, min = min, max = max, X = tabInd[,-ncol(tabInd)], vec = FALSE, call = match.call(), 
                  cluster.CA=cluster.CA)  
  
  df.tmp <- cbind(descr.data,descr.cl)
  colnames(df.tmp)[ncol(df.tmp)] <- "Clust_"
  colnames(RDO$centers)[1] <- "Clust_" 
  newres <- list(data.clust = df.tmp, centers =as.data.frame(RDO$centers), clust.count = RDO$clust.count, 
                 clust.content = RDO$clust.content, ss=RDO$global, call = newcall)
  
  if (!is.null(descaxes)) {
    class(descaxes) <- NULL
    newres$description$desc.axes <- descaxes
  }
  if (cluster.CA == "docs") {
    if (!is.null(descword)) 
      newres$description$desc.cluster.doc <- list(words = descword, 
                                                  wordsup = descwordsup, qualisup = descquali, 
                                                  quantisup = descquanti)
  }
  else {
    if (!is.null(descdoc)) 
      newres$description$desc.cluster.word <- list(docs = descdoc)
  }
  


  

  if (!is.null(object$info)) {
    if (nb.desc > 0) {
      if (is.null(desc.ind$para)) 
        stop("Please, use description=TRUE \n  to obtain a description of the clusters by the characteristic documents ")
   
      if(!is.null(object$var.agg)) vbaggr <- TRUE else vbaggr <- FALSE

      if(vbaggr==FALSE) {
        base.new <- object$info$base[[1]]
        var.text <- unlist(object$info$var.text[1:(length(object$info$var.text)-1)])   #  Important  Relaunch
        corpus <- as.character(base.new[, var.text[1]])
        corpus[is.na(corpus)] <- ""

        if(length(var.text) > 1) {	
          for (i in 2:length(var.text)){	
            corpus2 <- as.character(base.new[, var.text[i]])
            corpus2[is.na(corpus2)] <- ""
            dos <- which(corpus!="" & corpus2!="")
            corpus[dos] <- paste(corpus[dos], corpus2[dos], sep=". ")
            uno <-which(corpus=="" & corpus2!="")
            corpus[uno] <- corpus2[uno]
          }
          rm(corpus2)
        }	
       corpus <- data.frame(corpus, stringsAsFactors = FALSE)	
       rownames(corpus) <- rownames(base.new)
      }
      

      lispara <- vector(mode = "list")
   
      for (iclus in 1:nb.clust) {
        doctot <- length(desc.ind$para[[iclus]])
        ntdoc <- min(nb.desc, doctot)
        lispara[[iclus]] <- data.frame()
        for (i in 1:ntdoc) {
          lispara[[iclus]][i, 1] <- names(desc.ind$para[[iclus]][i])
          lispara[[iclus]][i, 2] <- desc.ind$para[[iclus]][i]

          if (cluster.CA == "docs") { if(vbaggr==FALSE)
            lispara[[iclus]][i, 3] <- strtrim(corpus[lispara[[iclus]][i,1], 1], size.desc)
            else lispara[[iclus]][i, 3] <- ""
          }
        }

        if (cluster.CA == "docs") {
          colnames(lispara[[iclus]]) <- c("DOCUMENT", "CRITERION", "TEXT") 
        }
        else colnames(lispara[[iclus]]) <- c("WORD", "CRITERION")
      }
      names(lispara) <- paste("cluster", 1:nb.clust, sep = "_")
      if (cluster.CA == "docs") 
        newres$description$desc.cluster.doc$para <- lispara
      else newres$description$desc.cluster.word$para <- lispara
      lisdist <- vector(mode = "list")
      for (iclus in 1:nb.clust) {
        doctot <- length(desc.ind$dist[[iclus]])
        ntdoc <- min(nb.desc, doctot)
        lisdist[[iclus]] <- data.frame()
        for (i in 1:ntdoc) {
          lisdist[[iclus]][i, 1] <- names(desc.ind$dist[[iclus]][i])
          lisdist[[iclus]][i, 2] <- desc.ind$dist[[iclus]][i]
          if (cluster.CA == "docs") 
          { if(vbaggr==FALSE)
            lisdist[[iclus]][i, 3] <- strtrim(corpus[lisdist[[iclus]][i, 1], 1], size.desc)
          else lisdist[[iclus]][i, 3] <-""
        }}
        if (cluster.CA == "docs") 
          colnames(lisdist[[iclus]]) <- c("DOCUMENT","CRITERION", "TEXT")
        else colnames(lisdist[[iclus]]) <- c("WORD", "CRITERION")
      }
      names(lisdist) <- paste("cluster", 1:nb.clust, sep = "_")
      if (cluster.CA == "docs") 
        newres$description$desc.cluster.doc$dist <- lisdist
      else newres$description$desc.cluster.word$dist <- lisdist
    }
  }
  newres$call$CA <- object
  newres$coord.clust <- cbind(newres$call$X, "Clust_"=newres$data.clust[rownames(newres$call$X),"Clust_"])
  newres$type <- type

  #---- 13. Plot Factorial Plane with centers ----
  if(graph){
      p<- plot.LexHCca(newres, type="map")
    plot(p)
  } 
  # 13. Plot Factorial Plane with centers
 
  # Agglomeraive or divisive coefficient
  # Computes the “agglomerative coefficient” (aka “divisive coefficient” for diana), 
  #  measuring the clustering structure of the dataset.
  # 
  # For each observation i, denote by # its dissimilarity to the first cluster it is merged with, 
  # divided by the dissimilarity of the merger in the final step of the algorithm. 
  # The agglomerative coefficient is the average of all . 
  # It can also be seen as the average width (or the percentage filled) of the banner plot.
  # 
  # coefHier() directly interfaces to the underlying C code, and “proves” that only object$heights is needed to compute the coefficient.
  # 
  # Because it grows with the number of observations, this measure should not be used to compare datasets of very different sizes.
 if(!is.null(t$tree)) {
   newres$coef.hclust <- coef(as.hclust(t$tree))# Compute divisive coefficient
   newres$t.tree <- t$tree }

  ## Guardar el tipo, consolidado...
  class(newres) <- c("LexHCca")

return(newres)

}

#' @rawNamespace import(ape, except = c(rotate, ladderize))
#' @rawNamespace import(ggdendro, except = c(theme_dendro))
#' @import ggplot2
#' @import ggrepel
#' @import ggforce
#' @importFrom ggforce geom_mark_ellipse geom_mark_hull
#' @importFrom ggrepel geom_label_repel geom_text_repel
#' @importFrom grDevices rainbow
#' @rawNamespace import(dendextend, except=c(cutree))
#' @export
plot.LexHCca <- function(x, type=c("map", "tree", "phylo", "clado", "radial", "fan"), 
                         plot=c("points","labels","centers"), selClust="ALL", selInd="ALL", axes=c(1,2),
                         theme=theme_bw(), palette=NULL, title=NULL, axis.title=NULL, axis.text=NULL, 
                         xlim=NULL,ylim=NULL, hvline=NULL, points=NULL, labels=NULL, centers=NULL, 
                         traject=NULL, hull=NULL, rotate=FALSE, branches=NULL, mar.value=NULL,...)
{
  

  # dots <- list(...) 
  # if("max.overlaps" %in% names(dots)) max.overlaps <- dots$max.overlaps else max.overlaps <- 20
  object <- x
  
  # original_mar <- par(no.readonly = TRUE)
  # original_mar$new <- FALSE 
  # original_mar$mfg <- NULL
  # 
  # on.exit({
  #   layout(1)  # Crash window divisions
  #   par(original_mar)
  # })
  
  # Guardamos ÚNICAMENTE los márgenes originales
  # old_mar <- par("mar") 
  # 
  # # Nos aseguramos de restaurarlos al salir de la función, pase lo que pase
  # on.exit({
  #   par(mar = old_mar)
  # })
  # 
  
  
  



# Reading information  -------------------------
  dots <- list(...)
   #  isnothing <- function(x) {is.null(x) || is.na(x) || is.nan(x) }
   # isnothing <- function(x) {is.null(x) || is.na(x) || is.nan(x) || length(x)=0}
   
  # Selection type
  str.type <- c("map","tree", "phylo", "clado", "radial", "fan")
  str.type.2 <- c("phylo", "clado", "radial", "fan")
  # Selection for type error

  
  if(!all(type %in% str.type))
    stop(paste0("type must be one of: ", str.type))
  type <- match.arg(type)
  if(type %in% str.type.2) bphylo <- TRUE else bphylo <- FALSE 
  
  nclust <- nrow(object$clust.count)
  ncases <- nrow(object$data.clust)
  xlim.seg <- xlim
  ylim.seg <- ylim
  
  if("horiz" %in% names(dots)) 
  {
    warning("horiz argument is deprecated for tree type. Please, use rotate argument")
    horiz <- dots$horiz
    rotate <- horiz
  } 
  
  if(is.null(hvline)){
    if(type=="map") hvline <- TRUE else hvline <- FALSE
  }



# Selection --------------------------
  selection <- function(sel1,xobj,bType,axx,axy) {
    xx<-""
    if(length(sel1)==1) if(sel1=="ALL") sel1 <- c(1:dim(xobj$coord)[1])
    if(length(sel1)==1) {
      if(sel1=="meta") sel1 <- "meta 3"
      if(sel1=="char") sel1 <- "char 0.05"
      xx <- gregexpr(pattern =' ',sel1)[[1]][1]-1
      xx <- substr(sel1, 1, xx)
    }
    
    
    if(xx=="coord" | xx=="cos2" | xx=="contrib" | xx=="meta" | xx=="char") 
    { 
      nc <- nchar(sel1)
      if(xx=="coord") {
        # Selection by coordinates"
        sel1 <- as.numeric(substr(sel1, 7, nc))
        # dft Coordinates for elements x two selected dimensions
        dft <- data.frame(xobj$coord[,c(axx,axy),drop=FALSE])
        # fval maximum value of element from two dimensions
        fval <- apply(dft, 1, function(z) max(abs(z)))
        ordmax <- rank(fval)
        posic <- which(ordmax > (length(fval)-sel1))
        sel1 <- rownames(xobj$coord)[posic]
      }
      
      if(xx=="contrib") {
        # Selection by contrib
        sel1 <- as.numeric(substr(sel1, 9, nc))
        dft <- data.frame(xobj$contrib[,c(axx,axy),drop=FALSE])
        fval <- apply(dft, 1, function(z) max(abs(z)))
        posic <- which(fval>= sel1)
        sel1 <- rownames(xobj$contrib)[posic]
      }
      if(xx=="cos2") {
        # Selection by cos2
        sel1 <- as.numeric(substr(sel1, 5, nc))
        dft <- data.frame(xobj$cos2[,c(axx,axy),drop=FALSE])
        fval <- apply(dft, 1, function(z) sum(z))
        posic <- which(fval>= sel1)
        sel1 <- rownames(xobj$cos2)[posic]
      }
      
      if(xx=="meta") {
        sel1 <- as.numeric(substr(sel1, 5, nc))
        sMeta <-  rownames(xobj$coord)[which(xobj$contrib[,axx]>mean(xobj$contrib[,axx])*sel1)]
        sMeta <-  c(sMeta,rownames(xobj$coord)[which(xobj$contrib[,axy]>mean(xobj$contrib[,axy])*sel1)])
        sel1 <- unique(sMeta)
      }
    } # End if(xx=="coord" ....
    if(is.character(sel1)) sel1 <- which(rownames(xobj$coord) %in% sel1)
    sel1 <- rownames(xobj$coord)[sel1]
    sel1  <- sel1[!is.na(sel1)]  
    return(sel1)
  }
  
  
  
  
# palette definition ----------
  if(is.null(palette))  palette <- rainbow(nclust)
  if(length(palette)==1) palette<- rep(palette, times=nclust)
  if(length(palette)<nclust) palette <- c(palette, rep("black", nclust-length(palette)))
  if(length(palette)>nclust) palette <- palette[1:nclust]


# Type no map -------------  
  if(type!="map") 
  {
    # if(type=="tree" || type=="phylo"|| type=="clado"|| type=="radial"|| type=="fan") 
    if(is.null(object$call$t)) stop("No dendrogram is built for kmeans or consolidation")
  palette.branches <- palette
  
  # Title for the dendrogram -------------
  if(is.null(title)) {
    if(type=="tree") title <- paste0("Cluster dendrogram. ", toupper(x$type))
  }

  if(is.null(mar.value)) mar.value <- c(4,3,1,1)

  # Building Dendrogram -----------
  hc <- as.hclust(x$call$t$tree)  # diana or agnes dendrogram
  groups <- stats::cutree(hc, k = nclust)  # Cut the dendrogram into k groups
  # Convert hclust object to dendrogram
  dend <- as.dendrogram(hc)  # "dendrogram"
  dend.D <- dend
  
  grL <- FALSE  # And not labels in group labels
  # Add numbers or roman letters to each cluster
  # If groupLabels=TRUE then numeric group labels will be added to each cluster. 
  # If a vector is supplied then these entries will be used as the group labels. 
  # If a function is supplied then it will be passed a numeric vector of groups (e.g. 1:5) 
  # and must return the formatted group labels. 
  

  # Height. To remove
  # if(is.null(axis.text)) axis.text <- FALSE
  

   #### Branches   -----
  if(!is.null(branches)) if(!is.list(branches)) stop("Please, use branches argument with a list structure") 
  
  if(length(branches[["color"]])>0) {
    palette.branches <- branches[["color"]]
   

    if(length(palette.branches)==1) palette.branches <- rep(palette.branches , times=nclust)
    

    if(length(palette.branches)<nclust) palette.branches  <- c(palette.branches, rep("black", nclust-length(palette.branches )))
    if(length(palette.branches)>nclust) {
      palette.branches <- palette.branches[1:nclust] 
    }
  }

  
  if(length(branches[["linesize"]])>0)
    dend.D<-  dendextend::assign_values_to_branches_edgePar(dend.D,k=nclust, value = branches[["linesize"]], edgePar = "lwd")
  if(length(branches[["linetype"]])>0)
    dend.D<-  dendextend::assign_values_to_branches_edgePar(dend.D,k=nclust, value = branches[["linetype"]], edgePar = "lty")
  
 
 
   #### Labels ---------------------------------
  if(!is.null(labels)) if(!is.list(labels)) stop("Please, use labels argument with a list structure") 
  
  grL <- NULL
  if(!is.null(labels)) {
    if(!is.null(labels[["groupLabels"]])) {
      # If number of elements equal to 1
      if(length(labels[["groupLabels"]])==1){
        if(is.logical(labels[["groupLabels"]])) grL <- labels[["groupLabels"]]
        if(labels[["groupLabels"]]=="as.roman")  grL <- as.roman
        if(labels[["groupLabels"]]=="letters")  grL <- letters[1:nclust]
        if(labels[["groupLabels"]]=="LETTERS")  grL <- LETTERS[1:nclust]
      } else {
        grL <- unname(labels[["groupLabels"]])  
        # For two labels in the same box:
        # labels=list(groupLabels=c(paste0("A1","\n", "Word 2"), "B2", "C3", "D4")))

        if(length(grL)>nclust) grL <- grL[1:nclust]
        if(length(grL)<nclust) grL <- c(grL, rep("", (nclust-length(grL))))
  
      }
    } 
  } # End null(labels)
  

  

  if(!is.null(labels[["color"]])) {
    if(length(labels[["color"]])>0) {
      palette.labels <- labels[["color"]]
      if(length(palette.labels)==1) palette.labels <- rep(palette.labels , times=ncases)
      
      if(length(palette.labels)<nclust) palette.labels  <- c(palette.labels, rep("black", ncases-length(palette.labels)))
      if(length(palette.labels)>nclust) palette.labels  <- palette.labels[1:ncases]
    }
  } 
  else  palette.labels <- palette
  

  

  # Size of the labels
  nlcex <- 1
  if(!is.null(labels[["cex"]])) {
    if(length(labels[["cex"]])==1)  {
      if(labels[["cex"]]==0) palette.labels  <- rep("transparent",length(palette.labels)) 
        else nlcex <- labels[["cex"]]
      } else {
        nlcex <- labels[["cex"]]
        }

  }


  dend.D <- dendextend::set(dend.D, "labels_cex", nlcex)  # Adjust the size of labels  

  
  ### 
  if(length(branches[["color"]])==1)
    dend.D <- dendextend::set(dend.D, "branches_col", branches[["color"]])
    dend.D <- dendextend::color_labels(dend.D, k=nclust, col = palette.labels)
    dend.D <- dendextend::color_branches(dend.D, k=nclust, col = palette.branches, groupLabels = grL)
  

  #-----------------  Cluster selection ---------------------  
  if(selClust[1]!="ALL") {
    dend.D.sel <- dend.D
    dend_list <- dendextend::get_subdendrograms(dend.D, nclust) 
    str.dend <- lapply(dend_list, labels)
    remov.labels <- labels(dend.D)[!labels(dend.D) %in% unlist(str.dend[selClust])]
    dend.D <- dendextend::prune(dend= dend.D, leaves=remov.labels)
  } else dend.D.sel <- NULL
  
  if(is.null(axis.title)) axis.title <- TRUE 
  

    

# type tree -------  
  if(type=="tree") { 
    # Save the original par settings
    # original_mar <- par("mar")
    # Modify the margins (bottom, left, top, right)
    par(mfrow = c(1, 1))  # <----------  NEW
    
    
    
    
    
    par(mar=c(mar.value))

    if(rotate==FALSE) rotate<-0 ; if(rotate==TRUE) rotate<- 90
    if(rotate==0) horiz <- FALSE else horiz <- TRUE
    if(rotate==0 | rotate==90)  plot(dend.D , axes=axis.title,  horiz=horiz, main=title)
    if(rotate==-90) dendextend::plot_horiz.dendrogram(dend.D, side = TRUE, main=title)
    
    ### Rectangles hull ------------------
    if(!is.null(hull)) {
      
      if(is.null(hull[["which"]]))  str.which <- 1:nclust
      else str.which <- unlist(hull[["which"]])
      
      if(is.null(hull[["color"]])) hull[["color"]] <- palette.branches[str.which]
      
      # Color selection
      str.col <- hull[["color"]]
      
      # prop_k_height  a (scalar) value (should be between 0 to 1), 
      # indicating what proportion of the height our rect will be between the height needed
      # for k and k+1 clustering. By default 0.5
      if(is.null(hull[["prop_k_height"]])) prop_k_height <- 0.5 else 
        prop_k_height <- hull[["prop_k_height"]]
      
      if(is.null(hull[["lower_rect"]])) lower_rect<- 0 else
        lower_rect <- hull[["lower_rect"]]
      
      if(is.null(hull[["upper_rect"]])) upper_rect<- 0 else
        upper_rect <- hull[["upper_rect"]]
      
      dendextend::rect.dendrogram(dend.D, k=nclust,which =str.which, border=str.col, horiz=horiz,
                                  prop_k_height= prop_k_height, lower_rect=lower_rect,
                                  upper_rect=upper_rect)
    } # End hull
    
    
    
 
    #### Add line ----------------------
    if(selClust[1]!="ALL") hvline <- NULL
    
    if(!is.null(hvline)) if(hvline[1]!=FALSE) {
      hv.lwd <- 2 ; hv.lty <- 2 ; hv.col <- "black"
      hv.pos <- dendextend::heights_per_k.dendrogram(dend.D)[nclust]
      
      if(!is.logical(hvline[1])) {
        if(!is.null(hvline[["linesize"]])) hv.lwd <- hvline[["linesize"]]
        if(!is.null(hvline[["linetype"]])) hv.lty <- hvline[["linetype"]]
        if(!is.null(hvline[["color"]])) hv.col <- hvline[["color"]]
        
        if(!is.null(hvline[["pos"]])) hv.pos <- hvline[["pos"]]
      }
      if(horiz==TRUE) abline(v = hv.pos, lwd = hv.lwd, lty = hv.lty, col = hv.col)
      else abline(h = hv.pos, lwd = hv.lwd, lty = hv.lty, col = hv.col)
    }
 
    # Modify the margins (bottom, left, top, right)
   # par("mar" = original_mar)  # Adjust as needed for your plot  
  } # End type tree
  
  

    
    
    

#   #### ape required ------------
#   if(bphylo) {
# 
#     if(is.null(mar.value)) mar.value <- c(4,3,1,1)
#     par(mar = mar.value)
#     
#     if(system.file(package='ape')=="") install.packages("ape")
#     
#     if(is.null(dend.D.sel))
#       dend.DP <- ape::as.phylo(dend.D)
#     else
#       dend.DP <- ape::as.phylo(dend.D.sel)
#     
#     # Computing color for branches in phylo graph
#     df <- data.frame(dend.DP$edge)
#     colnames(df) <- c("node", "case")
#     
#     # Adding names and groups
#     df$names <- dend.DP$tip.label[df$case]
#     df$gr <- x$clust.content[df$names, "Cluster"] # node case names gr
#     
#     # Selecting possible nodes to join
#     df$sel <- TRUE
#     df$sel[is.na(df$gr)] <- FALSE
#     
#     
#     ##############################
#     df$gr2 <- df$gr
#     df$order <- 1:nrow(df)
#     
#     # Number of groups
#     ngr <- sum(!is.na(unique(df$gr)))
#     
#     for(i in 1:ngr) {
#       # Only duplicated nodes can be joined
#       if(sum(df$sel & !is.na(df$gr))>ngr) {
#         
#         df.sel <- df[df$sel==TRUE,]
#         dupl <- rownames(df.sel[duplicated(df.sel[c("node")],fromLast = F), ])
#         dupl <- c(dupl,rownames(df.sel[duplicated(df.sel[c("node")],fromLast = T), ]))
#         df.sel <- df[dupl,]
#         u.sel <- unique(df.sel[c("node","gr")])
#         
#         # Put in gr.y the groups of the new nodes
#         # rownames case node names gr.x   sel gr.y
#         T1 <- merge(df, u.sel, by.x="case", by.y="node", all.x=TRUE)
#         T1 <- T1[order(T1$order),]
#         # Put in gr.x the new nodes
#         T1[is.na(T1$gr.x),"gr.x"] <- T1[is.na(T1$gr.x),"gr.y"]
#         # sel variable for joined cases must be FALSE now
#         # Put TRUE in sel variable for not NA values in gr.y
#         # u.sel$node
#         
#         # If case is in u.sel node sel TRUE.
#         T1[T1$case %in% u.sel$node, "sel"] <- TRUE
#         # If node está en u.sel node sel FALSE
#         T1[T1$node %in% u.sel$node, "sel"] <- FALSE
#         T1$gr.y <- NULL
#         colnames(T1)[4] <- "gr"
#         df <- T1
#       }
#     } # End i
#     
#     df$gr[is.na(df$gr)] <- ngr+1
#     if(length(palette.branches) < ngr+1)
#       palette.branches <- c(palette.branches, rep("black", (ngr+1-length(palette.branches))))
#     ecn <-df$gr
#     
#     if(!is.null(dend.D.sel)) { # selecting cases in phylo
#       # dend_list <- dendextend::get_subdendrograms(dend.D.sel, nclust)
#       # str.dend <- lapply(dend_list, labels)
#       # select.labels <- labels(dend.D.sel)[!labels(dend.D.sel) %in% unlist(str.dend[selClust])]
#       palette.branches[!c(1:length(palette.branches)) %in% selClust] <- "transparent"
#       palette.labels[!c(1:length(palette.labels)) %in% selClust] <- "transparent"
#     }
#     
#     if(type=="phylo") ptype <- "u"
#     if(type=="radial") ptype <- "r"
#     if(type=="clado") ptype <- "c"
#     if(type=="fan") ptype <- "fan"
#     
#     # rotate.tree
#     # for "fan", "unrooted", or "radial" trees: the rotation of the whole tree in degrees (negative values are accepted).
#     if(rotate==FALSE) rotate <- 0
#     if(rotate==TRUE) rotate <- 90
#     
#     ape::plot.phylo(dend.DP, type=ptype, main=title,
#                     use.edge.length = FALSE,
#                     tip.col=palette.labels[groups],
#                     edge.color=palette.branches[ecn], cex=nlcex,
#                     rotate.tree=rotate,
#                     edge.width=branches[["linesize"]],
#                     edge.lty= branches[["linetype"]],
#                     font=1)
#     # nodelabels() # blue
#     # tiplabels() # yellow
#     # edgelabels() # green
#   } # End bphylo
# par("mar"=original_mar)
# } # End no  map
#   
#   

    #### ape required ------------
    if(bphylo) {
      par(mfrow = c(1, 1)) # <-------- new
      
      
      
      if(is.null(mar.value)) mar.value <- c(4,3,1,1)
      par(mar = mar.value)
      
      if(is.null(dend.D.sel))
        dend.DP <- ape::as.phylo(dend.D)
      else
        dend.DP <- ape::as.phylo(dend.D.sel)
      
      # 1. Inicializamos todas las ramas en color "negro" (o el último de la paleta)
      num_edges <- nrow(dend.DP$edge)
      num_groups <- length(unique(x$clust.content$Cluster))
      edge_colors_idx <- rep(num_groups + 1, num_edges) 
      
      # 2. Mapeo REAL: Para cada clúster, buscamos sus ramas correspondientes
      # 'groups' contiene el clúster de cada tip en el orden del dendrograma
      unique_clusters <- sort(unique(x$clust.content$Cluster))
      
      for(g_idx in seq_along(unique_clusters)) {
        current_cluster <- unique_clusters[g_idx]
        
        # Obtenemos los nombres de las etiquetas que pertenecen a este clúster
        tips_in_cluster <- rownames(x$clust.content[x$clust.content$Cluster == current_cluster, ])
        
        # Buscamos qué ramas (edges) conectan a estos tips
        # which.edge nos da los índices exactos que plot.phylo espera
        edges_to_color <- ape::which.edge(dend.DP, tips_in_cluster)
        
        # Asignamos el índice de la paleta a esas ramas
        edge_colors_idx[edges_to_color] <- g_idx
      }
      
      # 3. Ajuste de paleta (asegurar que sea lo bastante larga)
      if(length(palette.branches) < (num_groups + 1)) {
        palette.branches <- c(palette.branches, rep("black", (num_groups + 1 - length(palette.branches))))
      }
      
      # 4. Configuración de tipos
      ptype <- switch(type, "phylo"="u", "radial"="r", "clado"="c", "fan"="fan", "u")
      if(rotate==FALSE) rotate <- 0
      if(rotate==TRUE) rotate <- 90
      
      # 5. El PLOT definitivo
      ape::plot.phylo(dend.DP, type=ptype, main=title,
                      use.edge.length = FALSE,
                      tip.col = palette.labels[groups],      # Color de etiquetas (Tips)
                      edge.color = palette.branches[edge_colors_idx], # Color de ramas (Edges)
                      cex = nlcex,
                      rotate.tree = rotate,
                      edge.width = branches[["linesize"]],
                      edge.lty = branches[["linetype"]],
                      font = 1)
    }
  
}
  
  
  
  
  
  
#### Type map
  if(type=="map") {
    if(is.null(mar.value)) mar.value<- c(1,1,1,1)

    #---- Check what is plotted and limits ----
    if(plot[1]=="ALL") bplot <- TRUE else bplot <-FALSE
    bPoints <- bLabels <- bCenters <- bplot 
    if("points" %in% plot) bPoints <- TRUE
    if("labels" %in% plot) bLabels <- TRUE
    if("centers" %in% plot) bCenters <- TRUE
    if("traject" %in% plot) bTrajectory <- TRUE else bTrajectory <- FALSE
    if("hull" %in% plot) bHull <- TRUE else bHull <- FALSE

    if(!is.null(xlim))
      xlim <- c(min(object$coord.clust[axes[1]]), max(object$coord.clust[axes[1]]))
    if(!is.null(ylim)) 
      ylim <- c(min(object$coord.clust[axes[2]]), max(object$coord.clust[axes[2]]))
    

    #---- Main title ----
    if(!is.null(title)) if(!is.list(title)) stop("Please, use title argument with a list structure") 
    
    if(!is.list(title)) {
      s.title <- title
      title <- list(text=NULL, color=NULL, size= NULL, family=NULL,face=NULL,just=NULL)
      if(!is.null(s.title)) title[["text"]] <- s.title
    } 
 
    if(is.null(title[["text"]])) title.text <- "Clusters on the CA map" else title.text <- title[["text"]]
    
    if(is.null(title[["color"]])) title.color <- "black" else title.color <- title[["color"]]
    if(is.null(title[["size"]])) title.size <- 18 else title.size <- title[["size"]]
    if(is.null(title[["family"]])) title.family <- 'serif' else title.family <- title[["family"]]
    if(is.null(title[["face"]])) title.face <- 'plain' else title.face <- title[["face"]]  
    if(is.null(title[["just"]])) title.hjust <- "c" else title.hjust <- title[["just"]] 
    if(title.hjust=="c"| title.hjust=="centered") title.hjust <- 0.5
    if(title.hjust=="l"| title.hjust=="left") title.hjust <- 0
    if(title.hjust=="r"| title.hjust=="right") title.hjust <- 1
    

    #---- Axestitle ----
   # if(!is.list(axis.title)) axis.title <- list(text.x= NULL, text.y=NULL, color=NULL, size= NULL, family=NULL,
   #                                        face=NULL,just=NULL)
    

   if(!is.list(axis.title)) axis.title <- list()
    # 2. Extracción segura de etiquetas
    # if(is.null(axis.title[["text.x"]]))
    #   labx <- paste0("Dim ", axes[1], " (", round(object$call$CA$eig[axes[1],2],2), "%)")
    # else labx <- axis.title[["text.x"]]
    # 
    # if(is.null(axis.title[["text.y"]]))
    #   laby <- paste0("Dim ", axes[2], " (", round(object$call$CA$eig[axes[2],2],2), "%)")
    # else laby <- axis.title["text.y"]
    labx <- if (!is.null(axis.title[["text.x"]])) axis.title[["text.x"]] 
                 else paste0("Dim ", axes[1], " (", round(object$call$CA$eig[axes[1],2],2), "%)")
    laby <- if (!is.null(axis.title[["text.y"]])) axis.title[["text.y"]] 
                 else paste0("Dim ", axes[2], " (", round(object$call$CA$eig[axes[2],2],2), "%)")

    
    # if("color" %in% names(axis.title)) axis.title.color <- axis.title[["color"]] else axis.title.color <- "black" 
       axis.title.color  <- if (!is.null(axis.title[["color"]]))  axis.title[["color"]]  else "black"
    # if("size" %in% names(axis.title)) axis.title.size <- axis.title[["size"]] else axis.title.size <- 12 
       axis.title.size <- if (!is.null(axis.title[["size"]]))   axis.title[["size"]]   else 12
    # if("family" %in% names(axis.title)) axis.title.family <- axis.title[["family"]] else axis.title.family <- 'serif' 
       axis.title.family <- if (!is.null(axis.title[["family"]])) axis.title[["family"]] else "serif"
     # if("face" %in% names(axis.title)) axis.title.face <- axis.title[["face"]] else axis.title.face <- 'plain' 
       axis.title.face   <- if (!is.null(axis.title[["face"]]))   axis.title[["face"]]   else "plain"

     # if("just" %in% names(axis.title)) axis.title.hjust <- axis.title[["just"]] else axis.title.hjust <- 'c' 
    # if(axis.title.hjust=="c"| axis.title.hjust=="centered") axis.title.hjust <- 0.5
    # if(axis.title.hjust=="l"| axis.title.hjust=="left") axis.title.hjust <- 0
    # if(axis.title.hjust=="r"| axis.title.hjust=="right") axis.title.hjust <- 1

       if (is.null(axis.title) || !is.list(axis.title)) {
         axis.title <- list()
       }
       at_just_raw <- if (!is.null(axis.title[["just"]])) axis.title[["just"]] else "c"
  
       # 2. Lógica de asignación
       if (is.numeric(at_just_raw)) {
         # Si es número (ej: 0), se asigna directamente
         axis.title.hjust <- at_just_raw
       } else {
         # Si es texto, comparamos siempre contra 'at_just_raw'
         if (at_just_raw %in% c("c", "centered", "center")) {
           axis.title.hjust <- 0.5
         } else if (at_just_raw %in% c("l", "left")) {
           axis.title.hjust <- 0
         } else if (at_just_raw %in% c("r", "right")) {
           axis.title.hjust <- 1
         } else {
           # Si pasan un texto que no conocemos (ej: "top"), volvemos al centro
           axis.title.hjust <- 0.5 
         }
       }

     axis.title <- ggplot2::element_text(color = axis.title.color, family=axis.title.family, 
                                           face=axis.title.face,
                                  hjust =axis.title.hjust, size=axis.title.size)


    
    
    # ---- Axes text / numbers (Etiquetas de los ejes) ----
    
    if (!exists("axis.text") || !is.list(axis.text)) axis.text <- list()
    
    axis.text.color  <- if (is.null(axis.text[["color"]]))  "black" else axis.text[["color"]]
    axis.text.size   <- if (is.null(axis.text[["size"]]))   8       else as.numeric(axis.text[["size"]])
    axis.text.family <- if (is.null(axis.text[["family"]])) "serif" else axis.text[["family"]]
    axis.text.face   <- if (is.null(axis.text[["face"]]))   "plain" else axis.text[["face"]]
    
    axis_text_element <- ggplot2::element_text(
      color  = axis.text.color,
      family = axis.text.family,
      face   = axis.text.face,
      size   = axis.text.size
    )
    

    # Axes text/numbers
    if(is.null(axis.text[["color"]])) axis.text.color <- "black" else axis.text.color <- axis.text[["color"]]
    if(is.null(axis.text[["size"]])) axis.text.size <- 8 else axis.text.size <- as.numeric(axis.text[["size"]])
    if(is.null(axis.text[["family"]])) axis.text.family <- 'serif' else axis.text.family <- axis.text[["family"]]
    if(is.null(axis.text[["face"]])) axis.text.face <- 'plain' else axis.text.face <- axis.text[["face"]]
    

 
    #---- Starting plot --------------
    
    nclust <- nrow(object$clust.count)
    ncases <-  nrow(object$data.clust)
    if(max(axes[1],axes[2]) > ncol(object$centers)-1) stop("The number of axes is bigger than saved dimensions")
    
    dataT  <- object$coord.clust                # Total data
    dataT$ID_ <- rownames(dataT)
    dataC  <-  object$centers               # Centers
    dataC$ID_ <- paste0("cl.",dataC$Clust_)
    dataC$Clust_C <- dataC$Clust_
    dataC <- dataC[,-1] 
    
    # if(is.null(palette)) palette<- rainbow(nclust)
    if(selClust[1]!="ALL") palette[!(1:nclust) %in% selClust] <- "transparent"
    
    if(deparse(substitute(theme))=="theme_bw()") 
      theme <- theme+ theme(panel.grid.major = element_blank(),
                            panel.grid.minor = element_blank(),
                            panel.background = element_blank())
    
    
    
   
    
    theme <- theme +
      theme(axis.text= element_text(color=axis.text.color, size=axis.text.size,family=axis.text.family,
                                    face=axis.text.face),
            axis.title.x = axis.title,
            axis.title.y = axis.title,
            plot.title = element_text(hjust = title.hjust, colour=title.color, size=title.size, 
                                      family=title.family, face=title.face)
      )
    p <- ggplot(data=dataT, aes(x=dataT[,axes[1]], y=dataT[,axes[2]])) +  theme +
      coord_equal() + labs(x=labx)+labs(y=laby) + ggtitle(title.text)
    
    


    #---- hvline -----------------
    if(!is.null(hvline)) if(hvline[[1]]!=FALSE) {
      if(!is.list(hvline)) {
        hvline <- list(intercept.x=NULL, intercept.y=NULL, linetype=NULL, color=NULL, linesize= NULL,
                       family=NULL,face=NULL,just=NULL, alpha.t=NULL)
        intercept.y <- intercept.x <- 0
        intercept.linetype <- 2 # Dashed
        intercept.color <- "grey"
        intercept.size <- .5
        intercept.alpha <- 1
      } else {  # Is list
        if(is.null(hvline[["intercept.y"]])) intercept.y<-0 else  intercept.y <- as.numeric(hvline[["intercept.y"]])
        if(is.null(hvline[["intercept.x"]])) intercept.x<-0 else  intercept.x <- as.numeric(hvline[["intercept.x"]])
        if(is.null(hvline[["linetype"]])) intercept.linetype<-"dashed" else  intercept.linetype <- hvline[["linetype"]]
        if(is.null(hvline[["color"]])) intercept.color <-"gray" else  intercept.color <- hvline[["color"]]
        if(is.null(hvline[["linesize"]])) intercept.size <- .5 else  intercept.size <- as.numeric(hvline[["linesize"]])
        if(is.null(hvline[["alpha.t"]])) intercept.alpha <- 1 else  intercept.alpha <- as.numeric(hvline[["alpha.t"]])
      }
      
      p <- p + geom_hline(yintercept=intercept.y, linetype=intercept.linetype, color = intercept.color,
                          linewidth=intercept.size, alpha=intercept.alpha)
      p <- p + geom_vline(xintercept=intercept.x, linetype=intercept.linetype, color = intercept.color,
                          linewidth=intercept.size, alpha=intercept.alpha)
    }
    
    

    
    #---- Selection of cases. by cos2, contrib,... ----    
    if(is.null(selInd[1])) {
      bPoints <- bLabels <- FALSE
    } else {
      if(object$call$cluster.CA=="docs")  selInd <-  selection(selInd, object$call$CA$row,"Doc", axes[1],axes[2])
      else selInd <-  selection(selInd, object$call$CA$col,"Word", axes[1],axes[2])
    } # End if(is.null(selInd[1]))
    
    ncases.sel <- length(selInd)    
    if(ncases.sel==0) stop("No selected cases")
    # df.sel <- dataK$data.clust[selInd,] 
    df.sel <- dataT[selInd,] 
    
    
    

    #---- Only print the selected cases, do not fix them transparent  ---------------
    
    points.shape  <- if(is.null(points[["shape"]])) 21 else as.numeric(points[["shape"]])
    points.size   <- if(is.null(points[["size"]]))  2  else as.numeric(points[["size"]])
    points.alpha  <- if(is.null(points[["alpha.t"]])) 1 else as.numeric(points[["alpha.t"]])
    
    # # Shape between 0 to 20 do not fill the symbol    
    # if(is.null(points[["fill"]])) points.fill <- palette[df.sel$Clust_] else points.fill<- points[["fill"]]
    # if(is.null(points[["stroke"]])) points.stroke<-0 else points.stroke<- as.numeric(points[["stroke"]])
    # 
    # 
    # 
    # if(is.null(points[["border"]])) points.border <-"transparent" # palette[df.sel$Clust_] 
    # else 
    # {
    #   points.border<- points[["border"]]
    #   if(points.stroke==0) points.stroke<-1
    # }
    # 
    # if(is.null(points.size) | points.size==0 )  bPoints <- FALSE
    # if(bPoints==FALSE) points.size <- NA
    # --- Configuración de Color y Relleno ---
    # Primero el relleno
    if(is.null(points[["fill"]])) {
      points.fill <- palette[df.sel$Clust_] 
    } else {
      points.fill <- points[["fill"]]
    }
    
    # --- Manejo del Borde (Evita la "X") ---
    if(is.null(points[["border"]])) {
      # Si el usuario no pide borde, usamos el mismo color del relleno
      # Esto es MUCHO más seguro que "transparent"
      points.border <- points.fill 
      points.stroke <- 0.1 # Stroke casi invisible pero presente
    } else {
      points.border <- points[["border"]]
      # Si el usuario definió un borde, nos aseguramos de que el stroke no sea 0
      points.stroke <- if(is.null(points[["stroke"]]) || points[["stroke"]] == 0) 1 else as.numeric(points[["stroke"]])
    }
    
    if(is.na(points.size) || points.size == 0) bPoints <- FALSE
    
    # if(bPoints)
    #   p <- p + geom_point(data=df.sel, aes(x=df.sel[,axes[1]], y=df.sel[,axes[2]]),
    #                       shape=points.shape, size=points.size, fill=points.fill , color=points.border,
    #                       stroke=points.stroke, alpha=points.alpha, show.legend=FALSE)
    # 
    # 
    if(bPoints) {
      p <- p + geom_point(
        data = df.sel, 
        aes(x = df.sel[,axes[1]], y = df.sel[,axes[2]]),
        shape  = points.shape, 
        size   = points.size, 
        fill   = points.fill, 
        color  = points.border, # Ahora nunca es "transparent" problemático
        stroke = points.stroke, 
        alpha  = points.alpha, 
        show.legend = FALSE
      )
    }
    
    
    
    
    #---- Labels  ----  
    if(!is.null(labels)) if(!is.list(labels)) stop("Please, use labels argument with a list structure") 
    
    if(bLabels) {
      if(is.null(labels[["numbers"]])) bNumbers <- FALSE else bNumbers=labels[["numbers"]]
      if(is.null(labels[["size"]])) labels.size <- 4 else labels.size<- as.numeric(labels[["size"]])
      if(labels.size==0) bLabels<-FALSE
      if(is.null(labels[["family"]])) labels.family <- 'serif' else labels.family <- labels[["family"]]
      if(is.null(labels[["face"]])) labels.face <- 'plain' else labels.face <- labels[["face"]]
      if(is.null(labels[["hjust"]])) labels.hjust <-1  else labels.hjust <- labels[["hjust"]]
      if(is.null(labels[["vjust"]])) labels.vjust <-1  else labels.vjust <- labels[["vjust"]]
      if(is.null(labels[["max.overlaps"]])) max.overlaps <-10  else max.overlaps <- labels[["max.overlaps"]]
      
      if(is.null(labels[["color"]])) labels.color.text <- points.fill else labels.color.text<-labels[["color"]]
      if(is.null(labels[["alpha.t.text"]])) labels.alpha.text<-1 else labels.alpha.text <- as.numeric(labels[["alpha.t.text"]])
      if(labels.alpha.text!=1) {
        labels.color.text <- apply(sapply(labels.color.text, col2rgb)/255, 2,  function(x) 
          rgb(x[1], x[2], x[3], alpha=labels.alpha.text))} 
      
      if(is.null(labels[["color.fill"]])) labels.color.fill <- "transparent" else labels.color.fill=labels[["color.fill"]]
      if(labels.color.fill[1]==FALSE)  labels.color.fill <- "transparent"
      if(labels.color.fill[1]==TRUE)  labels.color.fill <- points.fill
      
      if(is.null(labels[["alpha.t.fill"]])) labels.alpha.fill<-1 else labels.alpha.fill <- as.numeric(labels[["alpha.t.fill"]])
      if(labels.alpha.fill!=1) {
        labels.color.fill <- apply(sapply(labels.color.fill, col2rgb)/255, 2,  function(x) 
          rgb(x[1], x[2], x[3], alpha=labels.alpha.fill))} 
      
      if(is.null(labels[["rect"]])) labels.rect <- FALSE else labels.rect<- labels[["rect"]]
      if(is.null(labels[["force"]])) labels.force <-1  else labels.force <- as.numeric(labels[["force"]])
      if(labels.force==FALSE) labels.force <-0
      if(labels.force>0) { labels.hjust <- NULL ; labels.vjust <- NULL}
      if(is.null(labels[["set.seed"]])) labels.seed <- set.seed(Sys.time())  else labels.seed <- labels[["set.seed"]]
    } # End bLabels
 
    if(bLabels) {

      if(is.null(labels[["max.overlaps"]])) max.overlaps <- 20 else max.overlaps <-labels[["max.overlaps"]]

      
      
      if(bNumbers) IDL <- df.sel$Clust_ else IDL <- df.sel$ID_
      if(labels.force!=0)
        if(labels.rect) {
          
          p <- p + ggrepel::geom_label_repel(data=df.sel, aes(x=df.sel[,axes[1]], y=df.sel[,axes[2]],label = IDL,
                                                              hjust=labels.hjust, vjust=labels.vjust, ),
                                             size=labels.size, family = labels.family, fontface = labels.face,
                                             colour=labels.color.text, fill= labels.color.fill,force=labels.force,
                                             max.overlaps = max.overlaps,
                                             seed=labels.seed) 
        } else {
          # Label without border
          p <- p + ggrepel::geom_text_repel(data=df.sel, aes(x=df.sel[,axes[1]], y=df.sel[,axes[2]],label = IDL,
                                                             hjust=labels.hjust, vjust=labels.vjust),
                                            size=labels.size, family = labels.family, fontface = labels.face,
                                            force=labels.force, colour=labels.color.text,
                                            max.overlaps = max.overlaps,
                                            seed=labels.seed)
        }
      else { # No repel
        if(labels.rect) {
          p <- p + geom_label(data=df.sel, aes(x=df.sel[,axes[1]], y=df.sel[,axes[2]],label = IDL,
                                               hjust=labels.hjust, vjust=labels.vjust, ),
                              size=labels.size, family = labels.family, fontface = labels.face,
                              colour=labels.color.text, fill= labels.color.fill) 
          
        } else {
          p <- p + geom_text(data=df.sel, aes(x=df.sel[,axes[1]], y=df.sel[,axes[2]],label = IDL,
                                              hjust=labels.hjust, vjust=labels.vjust),
                             size=labels.size, family = labels.family, fontface = labels.face,
                             colour=labels.color.text)
        }
        
      }
    } # End bLabels  
    
    
    
    
  

    
    #---- trajectory ------------
    if(bTrajectory) {

      if ("color" %in% names(traject)) traject.color <- traject[["color"]] else traject.color <- "blue"
      if ("space" %in% names(traject)) traject.space <- as.numeric(traject[["space"]]) else traject.space <- 0
      if ("size" %in% names(traject)) traject.size <- as.numeric(traject[["size"]]) else traject.size <- 1
      if ("linetype" %in% names(traject)) traject.linetype <- traject[["linetype"]] else traject.linetype <- 'solid'
      if ("arrow.length" %in% names(traject)) traject.arrow.length <- as.numeric(traject[["arrow.length"]]) else traject.arrow.length <- .3
      if ("arrow.type" %in% names(traject)) traject.arrow.type <- traject[["arrow.type"]] else traject.arrow.type <- 'closed'
      if ("arrow.angle" %in% names(traject)) traject.arrow.angle <- as.numeric(traject[["arrow.angle"]]) else traject.arrow.angle<- 30
      if ("alpha.t" %in% names(traject)) traject.alpha <- as.numeric(traject[["alpha.t"]]) else traject.alpha<- 1


       ntr <- nrow(df.sel)
      dftr <- data.frame("x1"=df.sel[-ntr,axes[1]],"y1"=df.sel[-ntr,axes[2]],
                         "xend"=df.sel[-1,axes[1]],"yend"=df.sel[-1,axes[2]] )
      sc.x <-traject.space * max(df.sel[,axes[1]])- min(df.sel[,axes[1]])/10000
      sc.y <-traject.space * max(df.sel[,axes[2]])- min(df.sel[,axes[2]])/10000
      modul <- sqrt((dftr$xend-dftr$x1)^2+(dftr$yend-dftr$y1)^2)
      
      dftrj <- data.frame("x1"=dftr$x1 +sc.x* (dftr$xend-dftr$x1)/modul)
      dftrj$y1 <- dftr$y1 +sc.y* (dftr$yend-dftr$y1)/modul
      v.xend <- dftr$xend -sc.x*(dftr$xend-dftr$x1)/modul
      v.yend <- dftr$yend -sc.y* (dftr$yend-dftr$y1)/modul
      
      if(traject.alpha!=1) {
        traject.color <- apply(sapply(traject.color, col2rgb)/255, 2,  function(x) 
          rgb(x[1], x[2], x[3], alpha=traject.alpha ))  
      } 
      # dftrj <- fortify(dftrj)
      xinit <- dftr$x1
      yinit <- dftr$y1
      

      p<- p+ ggplot2::geom_segment(data=dftrj, aes(x=xinit, y=yinit, xend=v.xend, yend=v.yend), 
                                   arrow=arrow(length=unit(traject.arrow.length,"cm"),
                                               type=traject.arrow.type, 
                                               angle=traject.arrow.angle), 
                                               arrow.fill =traject.color,
                                   linewidth=traject.size, colour=traject.color,
                                   linetype=traject.linetype, linejoin = "mitre") 
      }
    
    
    
    #---- Centers ----
    if(bCenters) {
      if(!is.null(centers)) if(!is.list(centers)) stop("Please, use centers argument with a list structure") 
      
      centers.labels <- dataC$ID_
      if(!is.null(centers[["labels"]])) {
        nc  <- length(centers[["labels"]])
        nl  <- length(centers.labels)
        if(nl > nc) centers.labels[1:nc] <-  centers[["labels"]] else
          centers.labels <- centers[["labels"]][1:nl] }
      
      centers.fill <- rep("transparent", nclust)
      if(!is.null(centers[["fill"]])) {
        nc  <- length(centers[["fill"]])
        if(nc==1) { centers.fill[1:nclust] <- centers[["fill"]][1] ; nc <- nclust } else
          centers.fill[1:nclust] <- centers[["fill"]][1:nclust]} 
      

      centers.color <- NULL
      if(!is.null(centers[["color"]])) {
          nc  <- length(centers[["color"]])
         if(nc==1) centers.color[1:nclust] <- centers[["color"]]
        else
        { centers.color[1:nclust] <- centers[["color"]][1:nclust]
        centers.color[is.na(centers.color)] <- "black"}
      } else centers.color <- palette
 

      if(is.null(centers[["size"]])) centers.size <- 5 else 
      {
        centers.size <- centers[["size"]]  
        
      }

      
      if(!is.null(centers[["alpha.t"]])) {
        centers.alpha <- centers[["alpha.t"]]
        centers.fill <- apply(sapply(centers.fill, col2rgb)/255, 2,  function(x) 
          rgb(x[1], x[2], x[3], alpha=centers.alpha)) 
      } 
      
      if(is.null(centers[["family"]])) centers.family <- 'serif' else centers.family <- centers[["family"]]
      
      if(is.null(centers[["face"]])) centers.face <- 3 else centers.face <- centers[["face"]]
      if(centers.face=="italic") centers.face <- 3
      
      p <- p +  ggplot2::geom_label(data=dataC,
                                    aes(x=dataC[,axes[1]], y=dataC[,axes[2]]), 
                                    fill=centers.fill,
                                    size = centers.size,
                                    family = centers.family,
                                    fontface = centers.face,
                                    colour= centers.color,
                                    label = centers.labels)
      
    }  # End centers
    
     
    
    
    #---- Hulls ----
    if(bHull) {
       if(is.null(hull[["type"]])) hull.type <- "ellipse" else hull.type <-hull["type"]
      if(is.null(hull[["alpha.t"]])) hull.alpha <- .1 else hull.alpha <- as.numeric(hull[["alpha.t"]])
      if(is.null(hull[["color"]])) hull.color <- "black"  else hull.color<-hull[["color"]]  
      if(is.null(hull[["linetype"]])) hull.linetype <- "dotted"  else hull.linetype<- hull[["linetype"]]
      
      
      v.Clust_ <- as.factor(dataT$Clust_)
      if (hull.type =="ellipse") 
        p <- p + ggforce::geom_mark_ellipse(data=dataT, aes(x= dataT[,axes[1]], y= dataT[,axes[2]],
                                                            fill=v.Clust_), alpha= hull.alpha, color=hull.color, linetype=hull.linetype,
                                            show.legend =FALSE)
      if (hull.type =="hull")   
        
        p <- p + ggforce::geom_mark_hull(data=dataT, aes(x= dataT[,axes[1]], y= dataT[,axes[2]],
                                                         fill=v.Clust_), alpha= hull.alpha, color=hull.color, linetype=hull.linetype,
                                         show.legend =FALSE)   
      if (is.null(xlim))  xlim <-ggplot_build(p)$layout$panel_scales_x[[1]]$range$range
      if (!is.null(ylim)) ylim(ylim) else ylim <- ggplot_build(p)$layout$panel_scales_y[[1]]$range$range
    } # End hull
    
    # Margins of map in cm
    p <- p + theme(plot.margin=unit(c(mar.value),'cm'))
  }#  End map ---------------
    

  # ---- return
  if(type=="map") {
  suppressMessages(p <- p + ggplot2::coord_cartesian(xlim = xlim.seg, ylim = ylim.seg))
  p <- p + theme(legend.position = "none") 
   return(p)
  }
  

} # End function
  
  
 
  
  
  
  

#' @export
summary.TextData <- function(object,ndoc=10, nword=50, nseg=50, ordFreq = TRUE, file = NULL, sep=";", dec=".",
     info=TRUE,...) 
{
  options(stringsAsFactors = FALSE)
 if (!inherits(object, "TextData")) 
 stop("Object should be TextData class")
  

if(is.null(ndoc)) ndoc<-0
if(is.null(nword)) nword<-0


if(ndoc=="ALL") ndoc <- nrow(object$summDoc)

if(nword=="ALL") nword <- object$info$Nword[[1]]

 nword <- min(nword, object$info$Nword[[1]])
 ndoc <- min(ndoc, nrow(object$summDoc))

 var.agg <- as.character(object$info$name.var.agg[[1]])

if(!is.null(object$DocSeg)) {
 if(nseg=="ALL") nseg <- ncol(object$DocSeg)
 if(is.null(nseg)) nseg<-0
 nseg <- min(nseg, ncol(object$DocSeg))
} else { nseg <-0}

if(!is.null(file)) sink(file)
cat("\nTextData summary\n\n")


if(is.null(file)) print(object$summGen)  else {
  # out1 <- t(c("","Before", "After",sep="\t"))
  out1 <- t(c("","Before", "After",sep=sep))
  write.table(out1, quote=FALSE, sep = sep,col.names=FALSE, row.names=FALSE)
  write.table(object$summGen, quote=FALSE, sep = sep, col.names=FALSE, dec=dec)
}

if(var.agg=="") {
  name1 <- c("Occurrences", "DistinctWords", "PctLength", "Mean Length100")
  nameF <- c("DocName", name1, name1)
  name2 <- c("", rep("before",4), rep("after",4))
} else {
  nameF <- c("DocName", "Occurrences", "DistinctWords", "NumberDocs","PctLength", "MeanLength100", 
             "Occurrences", "DistinctWords", "PctLength", "MeanLength100")
  name2 <- c("", rep("before",5), rep("after",4))
}

if(ordFreq==TRUE) {
  datDoc <- object$summDoc[with(object$summDoc, order(-object$summDoc$Occurrences.after)), ]
} else {datDoc <-object$summDoc  }

A1 <- data.frame(nameF,name2)
colnames(A1) <- NULL
A2 <- cbind(A1,t(datDoc[c(1:ndoc),]))
colnames(A2) <- NULL
A3 <- data.frame(t(A2)) 
colnames(A3) <- NULL
rownames(A3) <- c(""," ",c(1:(nrow(A3)-2)))
A4 <- lapply(A3[3:(ndoc+2), ], function(x) as.numeric(as.character(x)))

if(is.null(file)) print(A3) else {
  cat("\n")
write.table(A3[1:2,], col.names=FALSE, quote=FALSE, sep = sep)
write.table(A4, col.names=FALSE, quote=FALSE, sep = sep, dec=dec)}


if(ndoc>0){
  if(ndoc== object$info$Ndoc[[1]]) cat("\nStatistics for the documents\n")
  else cat("\nStatistics for the ", ndoc, " first docs\n")
  
  if(is.null(file))  print(A3[1:(ndoc+2),],na.print = "") else
  { 
    write.table(A3[1:2,], col.names=FALSE, quote=FALSE, sep = sep)
    write.table(A4, col.names=FALSE, quote=FALSE, sep = sep, dec=dec)
  }
     }




if(nword>0){
  if (ordFreq) { 
    if(nword== object$info$Nword[[1]]) cat("\nIndex of the words\n")
    else cat("\nIndex of the ", nword, " most frequent words\n")
    
    A1 <- cbind(format(as.data.frame(rownames(object$indexW[c(1:nword), ])), justify = "l"),
                object$indexW[c(1:nword), ])
    colnames(A1) <- c(format("Word",justify = "l"), "Frequency", "N.Documents")
    

    if(is.null(file))  print(A1) else {
      # out1 <- t(c("","Word", "Frequency", "N.Documents",sep="\t"))
      out1 <- t(c("","Word", "Frequency", "N.Documents",sep=sep))
      write.table(out1, quote=FALSE, col.names=FALSE, row.names=FALSE,sep=sep, dec=dec)
      write.table(A1, quote=FALSE, col.names=FALSE,sep=sep, dec=dec)}
  } else { 
    # out1 <- t(c("","Word", "Frequency", "N.Documents",sep="\t"))
    out1 <- t(c("","Word", "Frequency", "N.Documents",sep=sep))
    Tabfq  <- object$indexW[(order(rownames(object$indexW)))[c(1:nword)],]
    
    if(nword== object$info$Nword[[1]]) cat("\nIndex of the words in alphabetical order\n")
    else cat("\nIndex of the first ", nword, " words in alphabetical order\n")
    if(is.null(file)){  
      print(Tabfq) 
    }else {
      # cat("\nIndex of the words in alphabetical order\n")
      write.table(Tabfq, quote=FALSE, sep = sep, row.names=TRUE,col.names=FALSE, dec=dec)
    }
  } 
}




if(nseg>0) { 
  cat("\nNumber of repeated segments\n")
  cat(" ", ncol(object$DocSeg),"\n") 
  if (ordFreq) {
    df <- data.frame(object$indexS$segOrderFreq[,])
    df <- df[with(df, order(-frequency,segment)), ]
    row.has.na <- apply(df, 1, function(z){any(is.na(z))})
    A1 <- df[!row.has.na,]
    A1 <- A1[c(1:nseg),]
    nseg <- min(nseg, nrow(A1))
    colnames(A1) <- c(format("Segment",justify = "l"), "Frequency", "Long")
    if(nseg== ncol(object$DocSeg)) cat("\nIndex of the repeated segments\n")
    else cat("\nIndex of the ", nseg, " most frequent repeated segments\n")
    if(is.null(file))  print(A1) else {
      # out1 <- t(c("Number","Segment", "Frequency", "Long",sep="\t"))
      out1 <- t(c("Number","Segment", "Frequency", "Long",sep=sep))
      write.table(out1, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE)
      write.table(object$indexS$segOrderFreq[1:nseg,], quote=FALSE, sep = sep, col.names=FALSE)}
  } else {
    if(is.null(file)) {
      if(nseg== ncol(object$DocSeg)) cat("\nIndex of the repeated segments in alphabetical order\n")
      else cat("\nIndex of the first ", nseg, " repeated segments in alphabetical order\n")
      colnames(object$indexS$segOrderlist) <- c("Segment", "Frequency", "Long")
      print(object$indexS$segOrderlist[1:nseg,]) 
    } else {
      cat("\nIndex of the repeated segments in alphabetical order\n")
      # out1 <- t(c("Number","Segment", "Frequency", "Long",sep="\t"))
      out1 <- t(c("Number","Segment", "Frequency", "Long",sep=sep))
      write.table(out1, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE)
      write.table(object$indexS$segOrderlist[1:nseg,], quote=FALSE, sep = sep, col.names=FALSE)}
  }} 

if(var.agg=="") {
  if(!is.null(object$context$quali))
    if(dim(object$context$quali)[[2]]>0)
    {
      cat("\nSummary of the contextual categorical variables\n")
      print(summary(object$context$quali))}
  
  if(!is.null(object$context$quanti))
    if(ncol(object$context$quanti)>0){
      cat("\nSummary of the contextual quantitative variables\n")
      print(summary(object$context$quanti))}
  
} else {
  if(!is.null(object$context$quanti)){
    cat("\nSummary of the contextual quantitative variables\n")
    Tabfq <- cbind(object$context$quanti, object$summDoc[,4])
    sname <- colnames(object$context$quanti)
    colnames(Tabfq) <- c(sname,"Ndocs")
    print(Tabfq)}
  
  if(!is.null(object$context$quali$qualitable))
  {
    cat("\nSummary of the contextual categorical variables\n")
    cat("Supplementary tables in: $context$quali$qualitable\n")
  }}


if(info){
  cat("\n\nSummary of used argumments\n" )
  cat("==========================" )
  
if(object$info$name.var.agg[[1]]!="") {
  cat("\nThis is an aggregated TextData object built using the aggregated variable: ", object$info$name.var.agg[[1]]," \n")
}

  
cat("* ", object$info$var.text[[2]], " :",object$info$var.text[[1]],"\n" )
cat("* ", object$info$idiom[[2]], " :",object$info$idiom[[1]],"\n" )
cat("* ", object$info$lminword[[2]], " :",object$info$lminword[[1]],"\n" )
cat("* ", object$info$lower[[2]], " :",object$info$lower[[1]] ,"\n")
cat("* ", object$info$remov.number[[2]], " :",object$info$remov.number[[1]] ,"\n")
cat("* ", object$info$Fmin[[2]], " :",object$info$Fmin[[1]] ,"\n")
cat("* ", object$info$Fmax[[2]], " :",object$info$Fmax[[1]] ,"\n")
cat("* ", object$info$Dmin[[2]], " :",object$info$Dmin[[1]] ,"\n")
cat("* ", object$info$Ndoc[[2]], " :",object$info$Ndoc[[1]] ,"\n")
cat("* ", object$info$LengthW[[2]], " :",object$info$LengthW[[1]] ,"\n")
cat("* ", object$info$Nword[[2]], " :",object$info$Nword[[1]] ,"\n\n")
cat("* ", object$info$stop.word.tm[[2]], " :",sort(object$info$stop.word.tm[[1]]) ,"\n")
cat("* ", object$info$stop.word.user[[2]], " :",sort(object$info$stop.word.user[[1]]) ,"\n")



} # End info TRUE
if(!is.null(file)) sink()
}

#' @import FactoMineR
#' @importFrom methods hasArg
#' @export
LexCA <- function(object,ncp=5, context.sup="ALL", doc.sup=NULL, word.sup=NULL, 
                  segment=FALSE, graph=TRUE, axes=c(1, 2), lmd=3, lmw=3)
{
  if(is.null(object)) stop("Missing argument for object")

  # if (!inherits(object,"TextData")) stop("Object should be TextData class")
  if (!inherits(object,"TextData") & !inherits(object,"DocumentTermMatrix")) stop("Object should be TextData class or DocumentTermMatrix")
  bDT <- ifelse (inherits(object,"DocumentTermMatrix"), TRUE,FALSE)
  options(stringsAsFactors = FALSE)
  
  
  # Starting CA_New  
  CA_New <- function (x, ncp = 5, row.sup = NULL, col.sup = NULL, quanti.sup = NULL, 
                      quali.sup = NULL, graph = TRUE, axes = c(1, 2), row.w = NULL, 
                      excl = NULL) 
  {
  
    X <-x
      
    fct.eta2 <- function(vec, x, weights) {
      VB <- function(xx) {
        return(sum((colSums((tt * xx) * weights)^2)/ni))
      }
      tt <- tab.disjonctif(vec)
      ni <- colSums(tt * weights)
      unlist(lapply(as.data.frame(x), VB))/colSums(x * x *  weights)}
    # End fct.eta2
    
    if (is.table(X)) X <- matrix(as.vector(X), nrow(X), dimnames = dimnames(X))
    if (is.null(rownames(X))) rownames(X) <- 1:nrow(X)
    if (is.null(colnames(X))) colnames(X) <- colnames(X, do.NULL = FALSE, prefix = "V")
    if(bDT) X <- as.data.frame(as.matrix(X))  else  X <- as.data.frame(X)
    
    is.quali <- which(!unlist(lapply(X, is.numeric)))
    X[, is.quali] <- lapply(X[, is.quali, drop = FALSE], as.factor)
    for (i in is.quali) X[, i] = as.factor(X[, i])
    X <- droplevels(X)
    Xtot <- X

    if (any(!sapply(X, is.numeric))) {
      auxi = NULL
      for (j in (1:ncol(X))[!((1:ncol(X)) %in% quali.sup)]) if (!is.numeric(X[, j])) 
        auxi = c(auxi, colnames(X)[j])
      if (!is.null(auxi)) 
        stop(paste("\nThe following variables are not quantitative: ", auxi))
    }
    
    if (!inherits(X, "data.frame")) stop("X is not a data.frame")
    if (!is.null(row.sup)) X <- as.data.frame(X[-row.sup, ])
    if ((!is.null(col.sup)) || (!is.null(quanti.sup)) || (!is.null(quali.sup))) 
      X <- as.data.frame(X[, -c(col.sup, quanti.sup, quali.sup)])
    
    if (any(apply(X, 1, sum) == 0)) {
      warning(paste0("The rows ", paste(rownames(X)[which(apply(X, 1, sum) == 0)], collapse = ", "),
                     " sum at 0. They were suppressed from the analysis"))
      X <- X[-which(apply(X, 1, sum) == 0), , drop = FALSE]
    }
    
    
    if (any(apply(X, 2, sum) == 0)) {
      warning(paste0("The columns ", paste(colnames(X)[which(apply(X,2, sum) == 0)], collapse = ", "), 
                     " sum at 0. They were suppressed from the analysis"))
      X <- X[, -which(apply(X, 2, sum) == 0), drop = FALSE]
    }
    
    
    if (is.null(row.w)) row.w = rep(1, nrow(X))
    row.w.init <- row.w
    
    if (length(row.w) != nrow(X)) stop("length of vector row.w should be the number of active rows")
    total <- sum(X * row.w)
    
    F <- as.matrix(X) * (row.w/total)
    marge.col <- colSums(F)
    marge.row <- rowSums(F)
    ncp <- min(ncp, (nrow(X) - 1), (ncol(X) - 1))
    Tc <- t(t(F/marge.row)/marge.col) - 1
    if (!is.null(excl)) marge.col[excl] <- 1e-15
    tmp <- svd.triplet(Tc, row.w = marge.row, col.w = marge.col, ncp = ncp)
    
    if (!is.null(excl))  marge.col[excl] <- 0
    eig <- tmp$vs^2
    vp <- matrix(NA, length(eig), 3)
    rownames(vp) <- paste("dim", 1:length(eig))
    colnames(vp) <- c("eigenvalue", "percentage of variance", 
                      "cumulative percentage of variance")
    vp[, "eigenvalue"] <- eig
    vp[, "percentage of variance"] <- (eig/sum(eig)) * 100
    vp[, "cumulative percentage of variance"] <- cumsum(vp[,"percentage of variance"])
    V <- tmp$V
    U <- tmp$U
    

    eig <- eig[1:ncol(U)]
    coord.col <- t(t(V) * sqrt(eig))
    coord.row <- t(t(U) * sqrt(eig))
    dist2.col <- colSums(Tc^2 * marge.row)
    contrib.col <- t(t(coord.col^2 * marge.col)/eig)
    cos2.col <- coord.col^2/dist2.col
    colnames(coord.col) <- colnames(contrib.col) <- colnames(cos2.col) <- paste("Dim",1:length(eig))
    rownames(coord.col) <- rownames(contrib.col) <- rownames(cos2.col) <- attributes(X)$names
    dist2.row <- rowSums(t(t(Tc^2) * marge.col))
    contrib.row <- t(t(coord.row^2 * marge.row)/eig)
    cos2.row <- coord.row^2/dist2.row
    colnames(coord.row) <- colnames(contrib.row) <- colnames(cos2.row) <- paste("Dim",1:length(eig))
    rownames(coord.row) <- rownames(contrib.row) <- rownames(cos2.row) <- attributes(X)$row.names
    
    
    inertia.row = marge.row * dist2.row
    inertia.col = marge.col * dist2.col
    names(inertia.col) <- attributes(coord.col)$row.names
    names(inertia.row) <- attributes(coord.row)$row.names
    res.call <- list(X = X, marge.col = marge.col, marge.row = marge.row, 
                     ncp = ncp, row.w = row.w, excl = excl, call = match.call(), 
                     Xtot = Xtot, N = sum(row.w * rowSums(X)))
    
    res.col <- list(coord = as.matrix(coord.col[, 1:ncp]), contrib = as.matrix(contrib.col[, 1:ncp] * 100), 
                    cos2 = as.matrix(cos2.col[, 1:ncp]), inertia = inertia.col)
    res.row <- list(coord = coord.row[, 1:ncp], contrib = contrib.row[, 1:ncp] * 100,
                    cos2 = cos2.row[, 1:ncp], inertia = inertia.row)
    res <- list(eig = vp[1:min(nrow(X) - 1, ncol(X) - 1), , drop = FALSE], 
                call = res.call, row = res.row, col = res.col, svd = tmp)
    

    
    if (!is.null(row.sup)) {
      X.row.sup <- as.data.frame(Xtot[row.sup, ])
      
    if ((!is.null(col.sup)) || (!is.null(quanti.sup)) || (!is.null(quali.sup))) 
      X.row.sup <- as.data.frame(X.row.sup[, -c(col.sup, quanti.sup, quali.sup)])
      somme.row <- rowSums(X.row.sup)
      X.row.sup <- X.row.sup/somme.row
      coord.row.sup <- crossprod(t(as.matrix(X.row.sup)), V)
      dist2.row <- rowSums(t((t(X.row.sup) - marge.col)^2/marge.col))
      cos2.row.sup <- coord.row.sup^2/dist2.row
      coord.row.sup <- coord.row.sup[, 1:ncp, drop = FALSE]
      cos2.row.sup <- cos2.row.sup[, 1:ncp, drop = FALSE]
      colnames(coord.row.sup) <- colnames(cos2.row.sup) <- paste("Dim", 1:ncp)
      rownames(coord.row.sup) <- rownames(cos2.row.sup) <- rownames(X.row.sup)
      res.row.sup <- list(coord = coord.row.sup, cos2 = cos2.row.sup)
      res$row.sup <- res.row.sup
      res$call$row.sup <- row.sup
    }
    
    if (!is.null(col.sup)) {
      X.col.sup <- as.data.frame(Xtot[, col.sup])
      if (!is.null(row.sup)) 
        X.col.sup <- as.data.frame(X.col.sup[-row.sup, ])
      X.col.sup <- X.col.sup * row.w
      colnames(X.col.sup) <- colnames(Xtot)[col.sup]
      somme.col <- colSums(X.col.sup)
      X.col.sup <- t(t(X.col.sup)/somme.col)
      coord.col.sup <- crossprod(as.matrix(X.col.sup), U)
      dist2.col <- colSums((X.col.sup - marge.row)^2/marge.row)   
      coord.col.sup <- as.matrix(coord.col.sup[, 1:ncp, drop = FALSE])
      cos2.col.sup <- coord.col.sup^2/dist2.col
      cos2.col.sup <- cos2.col.sup[, 1:ncp, drop = FALSE]
      colnames(coord.col.sup) <- colnames(cos2.col.sup) <- paste("Dim",1:ncp)   
      rownames(coord.col.sup) <- rownames(cos2.col.sup) <- colnames(X.col.sup)
      res.col.sup <- list(coord = coord.col.sup, cos2 = cos2.col.sup)
      res$col.sup <- res.col.sup
      res$call$col.sup <- col.sup
    }
    
    

    
    if (!is.null(quanti.sup)) {
      coord.quanti.sup <- matrix(NA, length(quanti.sup), ncp)
      
      if (is.null(row.sup)) 
        coord.quanti.sup <- cov.wt(cbind.data.frame(res$row$coord, 
                                                    Xtot[, quanti.sup, drop = FALSE]), cor = TRUE, 
                                   wt = marge.row, method = "ML")$cor[-(1:ncp), 1:ncp, drop = FALSE]
      else coord.quanti.sup <- cov.wt(cbind.data.frame(res$row$coord, 
                                                       Xtot[-row.sup, quanti.sup, drop = FALSE]), wt = marge.row, 
                                      cor = TRUE, method = "ML")$cor[-(1:ncp), 1:ncp, drop = FALSE]
      dimnames(coord.quanti.sup) <- list(colnames(Xtot)[quanti.sup], paste("Dim", 1:ncp, sep = "."))   
      res$quanti.sup$coord <- coord.quanti.sup
      res$quanti.sup$cos2 <- coord.quanti.sup^2
      res$call$quanti.sup <- quanti.sup
    }
    
    
    if (!is.null(quali.sup)) {
      if (!is.null(row.sup)) 
        X.del <- as.data.frame(Xtot[-row.sup, -c(col.sup,quanti.sup, quali.sup)])
      else X.del <- Xtot[, -c(col.sup, quanti.sup, quali.sup)]
      
      X.quali.sup <- NULL
      Xtot2 <- Xtot
      if (!is.null(row.sup))  Xtot2 <- Xtot[-row.sup,] 
      
      for (j in 1:length(quali.sup)) {
        Xtot2[,quali.sup[j]] <- droplevels(Xtot2[,quali.sup[j]] , reorder=FALSE)  
        X.quali.sup <- rbind(X.quali.sup, matrix(unlist(by(X.del, 
                                                           Xtot2[, quali.sup[j]], colSums)), ncol = ncol(X.del), byrow = T))
      }
      
      somme.quali <- rowSums(X.quali.sup)
      X.quali.sup <- X.quali.sup/somme.quali
      coord.quali.sup <- crossprod(t(as.matrix(X.quali.sup)), V)
      dist2.quali <- rowSums(t((t(X.quali.sup) - marge.col)^2/marge.col))
      cos2.quali.sup <- coord.quali.sup^2/dist2.quali
      coord.quali.sup <- coord.quali.sup[, 1:ncp, drop = FALSE]
      cos2.quali.sup <- cos2.quali.sup[, 1:ncp, drop = FALSE]
      
      rownames(coord.quali.sup) <- rownames(cos2.quali.sup) <- paste(rep(colnames(Xtot2)[quali.sup], 
                                                                         lapply(Xtot2[, quali.sup, drop = FALSE], nlevels)), 
                                                                     unlist(lapply(Xtot2[, quali.sup, drop = FALSE], levels)),sep = ".") 
      
      colnames(coord.quali.sup) <- colnames(cos2.quali.sup) <- paste("Dim", 1:ncp)
      res$quali.sup <- list(coord = coord.quali.sup, cos2 = cos2.quali.sup)
      
      Zqs <- tab.disjonctif(Xtot2[, quali.sup])
      
      Nj <- colSums(Zqs * row.w)
      Nj <- colSums(Zqs * marge.row) * total
      if (total > 1) 
        coef <- sqrt(Nj * ((total - 1)/(total - Nj)))
      else coef <- sqrt(Nj)
      res$quali.sup$v.test <- res$quali.sup$coord * coef
      
      eta2 = matrix(NA, length(quali.sup), ncp)
      eta2 <- sapply(as.data.frame(Xtot2[, quali.sup, drop = FALSE]), 
                     fct.eta2, res$row$coord, weights = marge.row)
      
      eta2 <- t(as.matrix(eta2, ncol = ncp))
      colnames(eta2) = paste("Dim", 1:ncp)
      rownames(eta2) = colnames(Xtot)[quali.sup]
      res$quali.sup$eta2 <- eta2
      res$call$quali.sup <- quali.sup
    }
 
    
    class(res) <- c("CA", "list")
    
    if (graph & (ncp > 1)) {
      print(plot(res, axes = axes))
      if (!is.null(quanti.sup)) 
        print(plot(res, choix = "quanti.sup", axes = axes, 
                   new.plot = TRUE))
    }
    return(res)
  }
  

  ##### Final CA_New 
  
  
  # Functions -------------------
  plotLexCA <- function()
  {
    # if(dev.interactive()) dev.new()
    plot.LexCA(res, selDoc=NULL, selWord=NULL, eigen=TRUE, axes=axes, graph.type = "classic", new.plot=TRUE)
    
    
    if(!is.null(res$quanti.sup$coord)) {
      #  if(dev.interactive()) dev.new()
      plot.LexCA(res, selDoc=NULL, selWord=NULL, quanti.sup=rownames(res$quanti.sup$coord), eigen=FALSE, axes=axes, 
                 graph.type = "ggplot", new.plot=TRUE)
    }
    
    #if(dev.interactive()) dev.new()
    plot.LexCA(res, selDoc= paste("meta ", lmd), selWord= paste("meta ", lmw), eigen=FALSE, axes=axes, graph.type = "classic",
               new.plot=TRUE)
    
    if(!is.null(rownames(res$segment$coord))& segment==TRUE){
      #  if(dev.interactive()) dev.new()
      plot.LexCA(res, selDoc= NULL, selWord= NULL, selSeg="ALL", axes=axes, graph.type = "classic",
                 new.plot=TRUE)}
    
    if(!is.null(res$quali.sup)) {
      # if(dev.interactive()) dev.new()
      plot.LexCA(res, selDoc= NULL, selWord= NULL, quali.sup="ALL", axes=axes, graph.type = "classic",
                 new.plot=TRUE)
    }
    
    if(!is.null(res$row.sup$coord)){
      ident <- identical(dimnames(res$quali.sup$coord)[1], dimnames(res$row.sup$coord)[1])
      if(ident==FALSE) {
        #  if(dev.interactive()) dev.new()
        plot.LexCA(res, selDoc= NULL, selWord= NULL, selDocSup="ALL", axes=axes, graph.type = "classic",
                   new.plot=TRUE)
      }
    }
    
    if(!is.null(res$col.sup$coord)){
      ident <- identical(dimnames(res$col.sup$coord)[1], dimnames(res$segment$coord)[1])
      if(ident==FALSE) {
        #  if(dev.interactive()) dev.new()
        plot.LexCA(res, selDoc= NULL, selWord= NULL, selWordSup="ALL", selSeg=NULL, axes=axes, graph.type = "classic",
                   new.plot=TRUE) }
    }
  }
  


  # Put documents at the end
  remdocs <- function(z,doc_sup){
    df <- data.frame(z[,, drop=FALSE])
    tmprow <- rownames(df)[doc_sup]
    rowqq <- data.frame(df[doc_sup,, drop=FALSE])	
    df <- rbind(df,rowqq)	
    df <- data.frame(df[-doc_sup,,drop=FALSE])	
    rownames(df)[(ndoc-ndocsup+1):ndoc] <- tmprow	
    z <- data.frame(df[,, drop=FALSE])
    return(z)}
  
  # --------------- Check context quanti-quali
  checkcontext <- function(context_, TDContext) {
    if (is.character(context_)) 	
      context_ <- which(colnames(TDContext) %in% context_)	
    context_ <- colnames(TDContext)[context_]	
    context_ <- context_[!is.na(context_)]	
    context_ <- which(colnames(TDContext) %in% context_)	
    if(length(context_ )==0) context_ <- NULL
    return(context_) }
  
  
  # --------------- Keys function
  Keys <- function(res,lmd,lmw,naxes, axes) {			
    title <- "Metakeys & Dockeys"
    col.word <- "red"; col.doc<-"blue"; cex=1
    axx <- axes[1]
    axy <- axes[2]			
    Ccontr <- res$col$contrib			
    Ccoor <- res$col$coord			
    Rcontr <-res$row$contrib			
    Rcoor <- res$row$coord			
    eigen <- res$eig			
    if(naxes>=min(nrow(Rcontr),nrow(Ccontr)))			
      naxes <- min((nrow(Rcontr)-1),(nrow(Ccontr)-1))			
    # Computing the metakeys: words with contributions over lmw*average_contribution of words  		
    

    Metakeys <-vector(mode="list",length=naxes)		
    for (i in 1:naxes){Metakeys[[i]]=vector(mode="list",length=2)}	
    
    for(i in 1:naxes){		
      lmw_ <- lmw*mean(Ccontr[,i])
      Metakeys[[i]][[1]]<-sort(Ccontr[which(Ccontr[,i]>lmw_&Ccoor[,i]>0),i],decreasing=TRUE)	
      if (length(Metakeys[[i]][[1]])==1) 	
        names(Metakeys[[i]][[1]])<-rownames(Ccontr)[which(Ccontr[,i]%in%sort(Ccontr[which(Ccontr[,i]>lmw_&Ccoor[,i]>0),i],decreasing=TRUE))]		
      Metakeys[[i]][[2]]<-sort(Ccontr[which(Ccontr[,i]>lmw*mean(Ccontr[,i])&Ccoor[,i]<0),i],decreasing=TRUE)	
      if (length(Metakeys[[i]][[2]])==1) 	
        names(Metakeys[[i]][[2]])<-rownames(Ccontr)[which(Ccontr[,i]%in%sort(Ccontr[which(Ccontr[,i]>lmw_&Ccoor[,i]<0),i],decreasing=TRUE))]		
    }
    
    
    # Computing the Keydocs : documents/answers with contributions over lmd*average_contribution of documents 			
    #lmd <- 3; lmw<-3

    
    Keydocs<-vector(mode="list",length=naxes)			
    for (i in 1:naxes){			
      Keydocs[[i]]=vector(mode="list",length=2)}		
    for(i in 1:naxes){			
      Keydocs[[i]][[1]]<-sort(Rcontr[which(Rcontr[,i]>lmd*mean(Rcontr[,i])&Rcoor[,i]>0),i],decreasing=TRUE)		
      if (length(Keydocs[[i]][[1]])==1) 		
        names(Keydocs[[i]][[1]])<-rownames(Rcontr)[which(Rcontr[,i]%in%sort(Rcontr[which(Rcontr[,i]>lmd*mean(Rcontr[,i])&Rcoor[,i]>0),i],decreasing=TRUE))]			
      Keydocs[[i]][[2]]<-sort(Rcontr[which(Rcontr[,i]>lmd*mean(Rcontr[,i])&Rcoor[,i]<0),i],decreasing=TRUE)		
      if (length(Keydocs[[i]][[2]])==1) 		
        names(Keydocs[[i]][[2]])<-rownames(Rcontr)[which(Rcontr[,i]%in%sort(Rcontr[which(Rcontr[,i]>lmd*mean(Rcontr[,i])&Rcoor[,i]<0),i],decreasing=TRUE))]			
    }			
    
    # metakeys and Keydocs				
    Metakeys.Keydocs <-vector(mode="list",naxes)				
    names(Metakeys.Keydocs)<-paste("DIM",1:naxes,sep="")				
    for (i in 1:naxes){			
      X <-list(names(Metakeys[[i]][[1]]),names(Keydocs[[i]][[1]]),				
               names(Metakeys[[i]][[2]]),names(Keydocs[[i]][[2]]))				
      names(X)<-c("Metakeys+","Keydocs+","Metakeys-","Keydocs-")				
      Metakeys.Keydocs[[i]]<-X       }				
    #Dimension of the words	
    
    

    mkeys<-numeric()				
    for (i in 1:naxes){			
      for (j in 1:2){		
        mkeys<-c(mkeys,names(Metakeys[[i]][[j]]))	}}
    
    mkeys<-as.factor(mkeys)	
  nlevels <- length(levels(mkeys))
  
  # matmkeys<-t(rbind(summary(mkeys,maxsum=100000),rep(naxes,length(levels(mkeys))),
   #                  round(summary(mkeys,maxsum=100000)*100/naxes,2)))	
  matmkeys<-t(rbind(summary(mkeys,maxsum=nlevels),rep(naxes,nlevels),
                   round(summary(mkeys,maxsum=nlevels)*100/naxes,2)))	
    colnames(matmkeys)<-c("Dim","Total.Dim","%Dim")	
   DimensionWord<-matmkeys[order(matmkeys[,1],decreasing = T),]	

   
   # Graph of metakeys and Keydocs on the "axes" of the CA
   axes<-c(axx,axy)
   # Identify metakeys of the dimensions "axes"
   ClusPal1<-which(Ccontr[,axes[1]]%in%Metakeys[[axes[1]]][[1]])
   ClusPal2<-which(Ccontr[,axes[1]]%in%Metakeys[[axes[1]]][[2]])
   ClusPal3<-which(Ccontr[,axes[2]]%in%Metakeys[[axes[2]]][[1]])
   ClusPal4<-which(Ccontr[,axes[2]]%in%Metakeys[[axes[2]]][[2]])
 ClusPal<-c(ClusPal1,ClusPal2,ClusPal3,ClusPal4)
 ClusPal<-ClusPal[!duplicated(ClusPal)]

 
 
 # Identify Keydocs of the dimensions "axes"
 ClusDoc1<-which(Rcontr[,axes[1]]%in%Keydocs[[axes[1]]][[1]])
 ClusDoc2<-which(Rcontr[,axes[1]]%in%Keydocs[[axes[1]]][[2]])
 ClusDoc3<-which(Rcontr[,axes[2]]%in%Keydocs[[axes[2]]][[1]])
 ClusDoc4<-which(Rcontr[,axes[2]]%in%Keydocs[[axes[2]]][[2]])
 ClusDoc<-c(ClusDoc1,ClusDoc2,ClusDoc3,ClusDoc4)
 ClusDoc<-ClusDoc[!duplicated(ClusDoc)]

 if(length(ClusPal)>0&length(ClusDoc)>0){
 xl=c(min(Ccoor[ClusPal,axes[1]],Rcoor[ClusDoc,axes[1]]),max(Ccoor[ClusPal,axes[1]],Rcoor[ClusDoc,axes[1]]))
 yl=c(min(Ccoor[ClusPal,axes[2]],Rcoor[ClusDoc,axes[2]]),max(Ccoor[ClusPal,axes[2]],Rcoor[ClusDoc,axes[2]]))}

 if(length(ClusPal)>0&length(ClusDoc)==0){
 xl=c(min(Ccoor[ClusPal,axes[1]]),max(Ccoor[ClusPal,axes[1]]))
 yl=c(min(Ccoor[ClusPal,axes[2]]),max(Ccoor[ClusPal,axes[2]]))}

if(length(ClusPal)==0&length(ClusDoc)>0){
 xl=c(min(Rcoor[ClusDoc,axes[1]]),max(Rcoor[ClusDoc,axes[1]]))
 yl=c(min(Rcoor[ClusDoc,axes[2]]),max(Rcoor[ClusDoc,axes[2]]))}

 

if(length(ClusPal)>0|length(ClusDoc)>0){
dfW<-dfD<-NULL
if(length(ClusPal)>0) {
 dfW <- data.frame(Words=character(),Dim=integer(), Coordinates=integer(), stringsAsFactors = FALSE)
for(i in 1:naxes) {
for(k in 1:2){
Q <- ifelse(k==1,1,-1)
if(length(Metakeys[[i]][[k]])>0)  {
   for(j in 1: length(Metakeys[[i]][[k]]))  {
      dfW[nrow(dfW)+1,] <- c(names(Metakeys[[i]][[k]])[j], i, Q*Metakeys[[i]][[k]][j])}
}}}


if(length(ClusDoc)>0) {
 dfD <- data.frame(Docs=character(),Dim=integer(), Coordinates=integer(), stringsAsFactors = FALSE)
for(i in 1:naxes) {
 for(k in 1:2){
Q <- ifelse(k==1,1,-1)
if(length(Keydocs[[i]][[k]])>0) {
  for(j in 1: length(Keydocs[[i]][[k]]))  {
 dfD[nrow(dfD)+1,] <- c(names(Keydocs[[i]][[k]])[j], i, Q* Keydocs[[i]][[k]][j])}
}}}
}
keysR <- list(Word=dfW, Doc = dfD, lmd=lmd, lmw=lmw)
 return(keysR)
}

}else{
 cat("\nThere are no elements to plot, lower lmd and/or lmw values\n")
}
}

 Xtab.disjonctif <- function(tab)
 {
   tab <- as.data.frame(tab,  stringsAsFactors = TRUE)
   modalite.disjonctif <- function(i) {
     moda <- as.factor(tab[, i])
     n <- length(moda)
     x <- matrix(0L, n, nlevels(moda))
     x[(1:n) + n * (unclass(moda) - 1L)] <- 1L
     return(x)
   }
   if (ncol(tab) == 1) {
     res <- modalite.disjonctif(1)
     dimnames(res) <- list(attributes(tab)$row.names, levels(tab[,1]))
   }
   else {
     variable <- rep(attributes(tab)$names, lapply(tab, nlevels))
     listModa <- unlist(lapply(tab, levels))
     wlistModa <- which((listModa) %in% c("y", "n", 
                                          "Y", "N"))
     if (!is.null(wlistModa)) 
       listModa[wlistModa] <- paste(variable[wlistModa], 
                                    listModa[wlistModa], sep = ".")
     numlistModa <- which(unlist(lapply(listModa, is.numeric)))
     if (!is.null(numlistModa)) 
       listModa[numlistModa] <- paste(variable[numlistModa], 
                                      listModa[numlistModa], sep = ".")
     res <- lapply(1:ncol(tab), modalite.disjonctif)
     res <- as.matrix(data.frame(res, check.names = FALSE))
     dimnames(res) <- list(attributes(tab)$row.names, listModa)
   }
   return(res)
 }
 
 
 # End Functions -------------------------

 if (bDT) {

   context.sup=NULL; doc.sup=NULL; word.sup=NULL; segment=FALSE
  res <-  CA_New(object, ncp = 5, row.sup = NULL, col.sup = NULL, quanti.sup = NULL, 
                       quali.sup = NULL, graph = graph, axes = c(1, 2), row.w = NULL, 
                       excl = NULL)  

  naxes <- ncp
  minrowcol <- min(ncol(object),nrow(object))-1
  res$VCr <- round(sqrt(sum(res$eig[, 1])/ minrowcol ), 4)
  res$Inertia <- round(sum(res$eig[, 1]), 4)
  res$meta <- Keys(res,lmd,lmw,naxes, axes)
  
  X <- as.data.frame(as.matrix(object)) 
  res$rowINIT <- data.frame(rowSums(X))
  rownames(res$rowINIT) <- rownames(X)
  res$row.sup <- NULL
  res$col.sup <- NULL
  res$quanti.sup <- NULL
  res$quali.sup <- NULL
  res$segment <- NULL
  res$var.agg <- NULL
  class(res) <- c("LexCA","CA", "list")
  return(res)
  
 } # End DocumentTermMatrix
   
 
 
 
 var.text <- object$info$var.text[[1]]
 str.base <- object$info$base[[1]]
 str.envir <- object$info$menvir[[1]]
 
 

info <- list(var.text= object$info$var.text, base=object$info$base,
  menvir= object$info$menvir, catquali = object$context$quali)

naxes <- ncp
context <- context.sup
var.agg <- object$info$name.var.agg[[1]]
quali.sup <- NULL; quanti.sup<-NULL	
ndocsup <-0; nsegm <-0	
nwordsup <- 0; nquali<-0; nquanti<-0	



if(segment==TRUE)	
 if(is.null(object$DocSeg))stop("No segment selected in TextData object")


if(var.agg=="") {
context.quanti<-context.quali<-NULL

if(!is.null(context)) {
 if(length(context)==1)
  if(context=="ALL") {
  context<- NULL
 if(!is.null(object$context$quali)) context <- as.character(colnames(object$context$quali))
 if(!is.null(object$context$quanti)) context <- c(context,colnames(object$context$quanti))
   }
  if(!is.null(object$context$quanti)){
   context.quanti <- checkcontext(context,object$context$quanti)
   context.quanti <- colnames(object$context$quanti)[context.quanti]

  fval <- apply(object$context$quanti[context.quanti], 2, function(x) max(x)-min(x))
  fval <- context.quanti[which(fval==0)]
  if(length(fval)>0){
   warning("Quantitative variables with the same value are removed: ",fval)
   context.quanti <- context.quanti[- which(fval==0)]
  }
  if(length(context.quanti)==0) context.quanti<-NULL
    } else {context.quanti<-NULL}

  if(!is.null(as.character(colnames(object$context$quali)))){
   context.quali <- checkcontext(context,object$context$quali)
   context.quali <- colnames(object$context$quali)[context.quali]

   if(length(context.quali)==0) context.quali<-NULL

} else { context.quali<- NULL}
} # Final if(!is.null(context))



# ========== Check quali variables $context$quali no aggregate
if(!is.null(context.quali))	
object$context$quali <- data.frame(object$context$quali[,context.quali,drop=FALSE] )

# =======  Check quanti variables $context$quanti no aggregate
if(!is.null(context.quanti)) 	
object$context$quanti <- data.frame(object$context$quanti[,context.quanti,drop=FALSE])	

LT <- as.matrix(object$DocTerm)	
ndoc <- nrow(LT)	
if(segment==TRUE){  LTS <- as.matrix(object$DocSeg)
 rownames(LTS) <- rownames(LT)}	

# =================== Selecting supplementary documents docs by rownumber or rowname	
if(!is.null(doc.sup)) {	
      if (is.character(doc.sup)) 	
         doc.sup <- which(rownames(LT) %in% doc.sup)	
         doc.sup <- rownames(LT)[doc.sup]	 
         doc.sup <- doc.sup[!is.na(doc.sup)]	
         doc.sup <- which(rownames(LT)%in% doc.sup)	
         ndocsup <- length(doc.sup)	
 if(ndocsup==0) doc.sup <- NULL	
}	
ndocact <- ndoc - ndocsup


# Put supplementary docs at the end	
if(ndocsup>0) {	
  tmpdf <- LT[doc.sup,,drop=FALSE]
  LT <- LT[-doc.sup,,drop=FALSE] 
  LT <- rbind(LT,tmpdf)
 if(!is.null(context.quali)) if(length(context.quali)>0)  	
     object$context$quali<- remdocs(object$context$quali,doc.sup)	
 if(!is.null(context.quanti)) if(length(context.quanti)>0)	
  object$context$quanti<- remdocs(object$context$quanti,doc.sup)

  #_____________________________________________
if(segment==TRUE){	
 tmpdf <- LTS[doc.sup,,drop=FALSE]
  LTS <- LTS[-doc.sup,,drop=FALSE] 
  LTS <- rbind(LTS,tmpdf)
}} # final de if(!is.null(doc.sup))	


#_____________________________________________	
nword <- ncol(LT)
nwordact <- nword
# Put supplementary words at the right of LT	
if(!is.null(word.sup)) {	
      if (!is.character(word.sup)) 					
      word.sup <- colnames(LT)[word.sup]
      word.sup <- which(colnames(LT) %in% word.sup)	
      nwordsup <- length(word.sup)

   if(nwordsup==0){ word.sup <- NULL} else {
    LT <- cbind(LT,LT[,word.sup])	
    wcolnames <- colnames(LT)[word.sup]
     LT <- LT[,-word.sup]	
   colnames(LT)[(ncol(LT)-nwordsup+1):ncol(LT)] <- wcolnames
   nwordact <- ncol(LT) - nwordsup}	}


#_____________________________________________	
# Remove supplementary words with zero sum	
if(!is.null(word.sup)) if(nwordsup>0) {
  pos.elim<-which(colSums(LT[c(1:ndocact),c((nwordact+1):nword),drop=FALSE])==0)	
if(length(pos.elim)>0)	
    LT <- LT[,-pos.elim] 
  nwordsup <- nwordsup - length(pos.elim)
   nword <- ncol(LT)}

#_____________________________________________	
# Remove words with zero sum	
  pos.elim<-which(colSums(LT[c(1:ndocact),,drop=FALSE])==0)	
  if(length(pos.elim)>0)	
    LT <- LT[,-pos.elim] 	
  nword <- ncol(LT)	
  nwordact <- nword - nwordsup

#_____________________________________________	
if(segment==TRUE){	
# Remove segments with zero sum	
pos.elim<-which(colSums(LTS[c(1:ndocact),,drop=FALSE])==0)	
  if(length(pos.elim)>0)	
    LTS <- LTS[,-pos.elim] 	
  nsegm <- ncol(LTS)	
   LT <- cbind(LT,LTS)	}	

  
# ==== Yuxtapostition of qualitative and quantitative variables 	
 if(length(context.quali)>0) {
      yQL <- data.frame(object$context$quali[rownames(LT),context.quali])
      colnames(yQL) <- colnames(object$context$quali)
	LT <- cbind(LT,yQL)
	nquali <- ncol(yQL)}
 if(length(context.quanti)>0) {	
      yQ <- data.frame(object$context$quanti[rownames(LT),context.quanti])
      colnames(yQ) <- colnames(object$context$quanti)
	LT <- cbind(LT,yQ)
      nquanti <- ncol(yQ)}	

  
#_____________________________________________
# ==== Remove supplementary empty docs 
if(ndocsup>0) {
pos.elim<-which(rowSums(LT[c((ndocact+1):ndoc),c(1:nwordact),drop=FALSE])==0)
  if(length(pos.elim)>0){
  LT <- LT[-pos.elim,] 
  ndocsup <- ndocsup - length(pos.elim)
}}


#_____________________________________________
# ==== Remove empty docs 
  pos.elim<-which(rowSums(LT[c(1:nrow(LT)),c(1:nwordact),drop=FALSE])==0)
if(length(pos.elim)>0)
  LT <- LT[-pos.elim,] 
 ndoc <- nrow(LT)
 ndocact <- ndoc-ndocsup

 

 
 
#_____________________________________________
# Fill NA quantitative values with the average
if(nquanti >0){
posquanti <- ncol(LT)-nquanti+1
for(i in (posquanti:ncol(LT))){
if(any(is.na(LT[,i]))) warning("\n", names(LT)[i], " variable has missing values. They will be replaced by the mean\n") 
  LT[is.na(LT[,i]), i] <- mean(LT[,i], na.rm = TRUE)
}}


if(ndocact <3)
    stop("At least three active documents have to be selected")
if(nwordact <3)
    stop("At least three active words have to be selected")
minrowcol <- min((ndocact-1), (nwordact-1))
ncp <- min(ncp, minrowcol)


if(ndocsup >0){ row.sup <- c(c(ndocact+1):ndoc)} else {row.sup <-NULL}

pos.word.sup <- pos.seg.sup <- NULL
ncoltemp <- nwordact
if(nwordsup >0){ 
  pos.word.sup <- c((ncoltemp +1):(ncoltemp+nwordsup))   
  ncoltemp <- ncoltemp + nwordsup
}

if(nsegm >0){
  pos.seg.sup <- c(c(ncoltemp +1):c(ncoltemp+nsegm))       # 1081 al 2713  ; Hay 1487 segmentos
  ncoltemp <- ncoltemp + nsegm
}
pos.col.sup <- c(pos.word.sup,pos.seg.sup)


if(nquali >0){ quali.sup <- c((ncoltemp+1):(ncoltemp+nquali)); ncoltemp <- ncoltemp+nquali}
if(nquanti >0){ quanti.sup <- c((ncoltemp+1):(ncoltemp+nquanti)); ncoltemp <- ncoltemp+nquanti}


# res <- FactoMineR::CA(LT, ncp, row.sup=row.sup, col.sup=word.seg.sup, quali.sup=quali.sup, quanti.sup=quanti.sup, graph=FALSE)
# res <- FactoMineR::CA(LT, ncp, row.sup=row.sup, col.sup=pos.col.sup, quali.sup=quali.sup, quanti.sup=quanti.sup, graph=FALSE)
res <- CA_New(LT, ncp, row.sup=row.sup, col.sup=pos.col.sup, quali.sup=quali.sup, quanti.sup=quanti.sup, graph=FALSE)
res$meta <- Keys(res,lmd,lmw,naxes, axes)



#################################
############ Modificada la siguiente:
##### res$VCr <- round(sqrt(sum(res$eig[, 1])/ (minrowcol-1) ), 4)
 res$VCr <- round(sqrt(sum(res$eig[, 1])/ minrowcol ), 4)
 res$Inertia <- round(sum(res$eig[, 1]), 4)
 res$info <- info

 
 if(segment==TRUE) if(ncol(LTS)==0) segment<-NULL
 
 if(segment==TRUE) {
  nsegm <- ncol(LTS)	

 if(nwordsup>0){
 res$segment <- res$col.sup
 res$segment$coord <- res$segment$coord[-c(1:nwordsup),,drop=FALSE]
 res$segment$cos2 <- res$segment$cos2[-c(1:nwordsup),,drop=FALSE]
 res$col.sup$coord <- res$col.sup$coord[c(1:nwordsup),,drop=FALSE]
 res$col.sup$cos2 <- res$col.sup$cos2[c(1:nwordsup),,drop=FALSE] 
} else {
 res$segment <- ""
 suppressWarnings(res$segment$coord <-res$col.sup$coord[(1:nsegm),,drop=FALSE]) 
 suppressWarnings(res$segment$cos2 <- res$col.sup$cos2[(1:nsegm),,drop=FALSE])
}
  }


if(!is.null(res$quali.sup$coord)) {
  res$quali.sup$cos2 <- res$quali.sup$coord[,,drop=FALSE]
  if(is.null(row.sup))
    LTnosup <- LT
  else LTnosup <- LT[-row.sup,, drop=FALSE]

  
   Xact <- as.matrix(LTnosup[,1:nwordact, drop=FALSE]) 
   PJ <- colSums(Xact)/sum(Xact)
   tt <-as.matrix( t(Xtab.disjonctif(as.matrix(LTnosup[,quali.sup,drop=FALSE]))))
   PIJ <- tt %*% (Xact/sum(Xact))
  disto2 <- ((t(PIJ/rowSums(PIJ)) -PJ)^2)/PJ
  disto2 <- colSums(disto2)
  cos2 <- res$quali.sup$coord^2 
  res$quali.sup$cos2 <- cos2 / disto2
}

res$rowINIT <- object$rowINIT 
 class(res) <- c("LexCA","CA", "list")
 
if(graph==TRUE) plotLexCA()
return(res)

 
 
 
 
 } else {


## Aggregate analysis
context.quanti<-context.quali<-NULL
if(!is.null(context)) {
ctmp <- NULL
if(length(context)==1)
if(context=="ALL") {
 context<- NULL
 if(!is.null(object$context$quali$qualivar[,1])) context <- as.character(object$context$quali$qualivar[,1])
 if(!is.null(object$context$quanti)) context <- c(context,colnames(object$context$quanti))
}

  if(!is.null(object$context$quanti)){
   context.quanti <- checkcontext(context,object$context$quanti)
   context.quanti <- colnames(object$context$quanti)[context.quanti]
  if(length(context.quanti)==0) context.quanti<-NULL
    } else {context.quanti<-NULL}

  if(!is.null(as.character(object$context$quali$qualivar[,1]))){
   context.quali <- as.character(object$context$quali$qualivar[,1])


if(is.null(context)) context<-""
 if(is.character(context)) 
        context_ <- which(context %in% context.quali)
        context_ <- as.character(object$context$quali$qualivar[context_,1])
        context_ <-  context_[!is.na(context_)]
        context.quali <- which(context_ %in% context.quali)
 if(length(context.quali )==0) context.quali <- NULL
} else { context.quali<- NULL}


} # Final if(!is.null(context))


# =======  Check quanti variables $context$quanti aggregate	
if(!is.null(context.quanti))  
        data.frame(object$context$quanti[,context.quanti,drop=FALSE])

# Aggregate lexical table	
LT <- as.matrix(object$DocTerm)	
ndoc <- nrow(LT)	
if(segment==TRUE) {
 LTS <- data.frame(as.matrix(object$DocSeg)[,,drop=FALSE])
 rownames(LTS) <- rownames(LT)}

# =================== Selecting supplementary documents docs by rownumber or rowname					
if(!is.null(doc.sup)) {					
      if (is.character(doc.sup)) 					
         doc.sup <- which(rownames(LT) %in% doc.sup)					
         doc.sup <- rownames(LT)[doc.sup]		
         doc.sup <- doc.sup[!is.na(doc.sup)]					
         doc.sup <- which(rownames(LT)%in% doc.sup)					
 ndocsup <- length(doc.sup)					
 if(ndocsup==0) doc.sup <- NULL					
}					
ndocact <- ndoc - ndocsup

LTQ <- NULL; LTQL<- NULL
if(!is.null(context.quanti)){
 LTQ <- data.frame(object$context$quanti[,context.quanti,drop=FALSE])
 nquanti <- length(LTQ)}

# Put supplementary docs at the end
if(ndocsup>0) {
  tmpdf <- LT[doc.sup,,drop=FALSE]
  LT <- LT[-doc.sup,,drop=FALSE] 
  LT <- rbind(LT,tmpdf)		

  if(segment==TRUE){	
  tmpdf <- LTS[doc.sup,,drop=FALSE]
  LTS <- LTS[-doc.sup,,drop=FALSE] 
  LTS <- rbind(LTS,tmpdf)	
}

 if(!is.null(context.quanti)){ 
  tmpdf <- LTQ[doc.sup,,drop=FALSE]
  LTQ <- LTQ[-doc.sup,,drop=FALSE] 
  LTQ <- rbind(LTQ,tmpdf)
}

ndocact <- ndoc-ndocsup
if(ndocact <3)
    stop("At least three active documents have to be selected")
} # Final if(ndocsup>0) 

 
if(!is.null(context.quali))
   LTQL <- data.frame(object$context$quali$qualitable[,,drop=FALSE])

nwordact <- ncol(LT); nword<- nwordact	



#==================================================================
# Put supplementary words at the right word.sup	
if(!is.null(word.sup)) {	
   if (!is.character(word.sup)) 					
    word.sup <- colnames(LT)[word.sup]
    word.sup <- which(colnames(LT) %in% word.sup)
    nwordsup <- length(word.sup)	

 if(nwordsup==0){ word.sup <- NULL} else {
    LTSUP <- data.frame(LT[,word.sup,drop=FALSE])	
    LT <- cbind(LT,LTSUP)	
    LT <- data.frame(LT[,-word.sup,drop=FALSE])	
 if(!is.null(context.quali)){ 	
    LTQLSUP <- data.frame(LTQL[,word.sup,drop=FALSE])	
    LTQL <- cbind(LTQL,LTQLSUP)	
    LTQL <- data.frame(LTQL[,-word.sup,drop=FALSE])  	
}}	
nwordact <- nword - nwordsup	
}	


########### Remove supplementary words with zero sum			
if(nwordsup>0) {			
  pos.elim<-which(colSums(LT[c(1:ndocact),c((nwordact+1):nword),drop=FALSE])==0)			
if(length(pos.elim)>0) {			
 if(!is.null(LTQL))  LTQL <- LTQL[,-pos.elim] 			
  LT <- LT[,-pos.elim] 			
nword <- ncol(LT)			
nwordsup <- nword-nwordact			
}}			

if(nwordact <3)
    stop("At least three active words have to be selected")
		
# Removing active words with zero margins			
if(ndocact>0) {			
  pos.elim<-which(colSums(LT[c(1:ndocact),c(1:nwordact),drop=FALSE])==0)			
  if(length(pos.elim)>0) {			
  LT <- LT[,-pos.elim, drop=FALSE] 			
 if(!is.null(LTQL))			
  LTQL <- LTQL[,-pos.elim, drop=FALSE] 			
nword <- ncol(LT)			
nwordact <- nword-nwordsup 			
}}			


# Fill NA quantitative values with the average	
if(!is.null(LTQ)){	
for(i in (1:ncol(LTQ))){
if(any(is.na(LTQ[,i]))) warning("\n", names(LTQ)[i], " variable has missing values. They will be replaced by the mean\n") 
  LTQ[is.na(LTQ[,i]), i] <- mean(LTQ[,i], na.rm = TRUE)	
}}	

if(ndocact <3)		
    stop("At least three active documents have to be selected")	
if(nwordact <3)		
    stop("At least three active words have to be selected")
	
minrowcol <- min((ndocact-1), (nwordact-1))		
ncp <- min(ncp, minrowcol)		

if(segment==FALSE) nsegm<-0 else nsegm <- ncol(LTS)
if(ndocsup >0){ row.sup <- c((ndocact+1):(ndocact+ndocsup))} else {row.sup <-NULL}
word.sup <-NULL
ncoltemp <- nwordact
if(nwordsup >0){ word.sup <- c((ncoltemp+1):(nwordact+nwordsup)); ncoltemp <- ncoltemp+nwordsup}
if(nsegm >0){ word.sup <- c(word.sup,c((ncoltemp+1):(ncoltemp+nsegm))); ncoltemp <- ncoltemp+nsegm
LT <- cbind(LT,LTS)}


if(!is.null(LTQL)) nquali <- nrow(LTQL) else nquali<-0

if(nquali >0){
  doc.sup.q <- c((ndocact+ndocsup+1):(ndocact+ndocsup+nquali))
  ndifcol <- ncol(LT)-ncol(LTQL) 
if(ndifcol>0) {
  tempcol <- matrix(c(rep.int(NA,(nquali*ndifcol))),nrow=nquali,ncol=ndifcol)
  newcol <- data.frame(tempcol)
  colnames(newcol) <- colnames(LT)[(ncol(LTQL)+1):ncol(LT)]
  LTQL <- cbind(LTQL,newcol)
 }
  colnames(LTQL) <- colnames(LT)
  LT <- rbind(LT,LTQL)
}

quanti.sup.q <- NULL
if(nquanti >0){ 
 quanti.sup.q <- c((ncol(LT)+1):(ncoltemp+nquanti))
 ndif <- nrow(LT)-nrow(LTQ)
 if(ndif >0) {
 temprow <- matrix(c(rep.int(NA,(ndif*ncol(LTQ)))),nrow=ndif,ncol=ncol(LTQ))
 newrow <- data.frame(temprow)[,,drop=FALSE]
 rownames(newrow) <- rownames(LT)[(nrow(LT)-ndif+1):nrow(LT)]
 colnames(newrow) <- colnames(LTQ)
 LTQ <- rbind(LTQ,newrow)
 } 
 LT <- cbind(LT,LTQ)
}



if((ndocsup+nquali)>0) doc.sup <- c((ndocact+1):(ndocact+ndocsup+nquali))
res <- CA_New(LT, ncp, row.sup=doc.sup, col.sup=word.sup, quanti.sup=quanti.sup.q, graph=FALSE)
res$var.agg <- var.agg
res$meta <- Keys(res,lmd,lmw,naxes, axes)
####### Modificar lo siguiente
############################
######res$VCr <- round(sqrt(sum(res$eig[, 1])/ (minrowcol-1) ), 4)
res$VCr <- round(sqrt(sum(res$eig[, 1])/ minrowcol ), 4)
res$Inertia <- round(sum(res$eig[, 1]), 4)
res$info <- info



if(segment==TRUE){
if(nwordsup>0){
res$segment <- res$col.sup
res$segment$coord <- res$segment$coord[-(1:nwordsup),]
res$segment$cos2 <- res$segment$cos2[-(1:nwordsup),]
} else {
res$segment <- ""
 suppressWarnings(res$segment$coord <-res$col.sup$coord[(1:nsegm),]) 
suppressWarnings(res$segment$cos2 <- res$col.sup$cos2[(1:nsegm),])
}
 if(!is.null(var.agg))
{
   rownames(res$segment$coord)<- substring(rownames(res$segment$coord),2)
   aa <- sub('\\.', ':', rownames(res$segment$coord)) 
   rownames(res$segment$coord) <- gsub("\\."," ",aa)
   rownames(res$segment$cos2) <-  rownames(res$segment$coord)
   rownames(res$col.sup$coord)[(nwordsup+1):nrow(res$col.sup$coord)] <- rownames(res$segment$coord)
   rownames(res$col.sup$cos2)<-  rownames(res$col.sup$coord)
}
}


if(nquali >0) {
 res$quali.sup <- res$row.sup
 if(ndocsup>0){
 res$quali.sup$coord <- res$quali.sup$coord[-(1:ndocsup),,drop=FALSE]
 res$quali.sup$cos2 <- res$quali.sup$cos2[-(1:ndocsup),,drop=FALSE]
 res$row.sup$coord  <- res$row.sup$coord[(1:ndocsup),,drop=FALSE]
 res$row.sup$cos2  <- res$row.sup$cos2[(1:ndocsup),,drop=FALSE]}
}
res$rowINIT <- object$rowINIT 
 class(res) <- c("LexCA","CA", "list")

 if(graph==TRUE) plotLexCA()
 return(res)
}
}

#' @export
print.TextData <- function (x, file = NULL, sep = ";", dec=".", ...) 
{
  options(stringsAsFactors = FALSE)
  
    res.TextData <- x
  if (!inherits(res.TextData, "TextData")) 
        stop("non convenient data")

sink.reset <- function(){
    for(i in seq_len(sink.number())){sink()}}
sink.reset

    cat("*The results are available in the following objects:\n\n")
addindex <- 0 
if(is.null(x$DocSeg)) indexp <- 7 else indexp <- 9 
if(!is.null(x$SourceTerm)) addindex <- 1 
if(!is.null(x$FullDocTerm)) addindex <- addindex +1
index <- indexp+addindex
    res <- array("", c(index, 2), list(1:index, c("name", "description")))
    res[1, ] <- c("$summGen", "General summary")
    res[2, ] <- c("$summDoc", "Documents summary")
    res[3, ] <- c("$indexW", "Index of words")
    res[4, ] <- c("$DocTerm", "Documents by Words table (*1)")
    res[5, ] <- c("$context", "Contextual variables")
    res[6, ] <- c("$info", "Information about selection of words")
    res[7, ] <- c("$remov.docs", "Removed empty documents")
if(!is.null(x$DocSeg)){
     res[8, ] <- c("$indexS", "Index of segments")
     res[9, ] <- c("$DocSeg", "Documents by Segments table (*2)")}
if(!is.null(x$SourceTerm)) {
    res[indexp+1, ] <- c("$SourceTerm", "Sequency Documents by Words table (*3)")
}

    print(res[c(1:index),])
    print(paste("(*1) $DocTerm has a compressed format"))
    print(paste("Use print(as.matrix(x$DocTerm))"))
 
if(!is.null(x$DocSeg)) {
    print(paste("(*2) $DocSeg has a compressed format"))
    print(paste("Use print(as.matrix(x$DocSeg))"))
}

if(!is.null(x$SourceTerm)) {
    print(paste("(*3) $SourceTerm has a compressed format"))
    print(paste("Use print(as.matrix(x$SourceTerm))"))
}

if(!is.null(x$FullDocTerm)) {
    print(paste("(*4) $FullDocTerm has a compressed format"))
    print(paste("Use print(as.matrix(x$FullDocTerm))"))
}

    if (!is.null(file)) {
for(i in seq_len(sink.number())){sink(NULL)}
 sink(file)
 ndoc <- x$info$Ndoc[[1]]
 var.agg <- as.character(x$info$name.var.agg[[1]])

cat("\nGeneral summary (summGen)\n")
if(is.null(file)) print(x$summGen)  else {
  # out1 <- t(c("","Before", "After",sep="\t"))
  out1 <- t(c("","Before", "After",sep=sep))
  write.table(out1, quote=FALSE, sep = sep,col.names=FALSE, row.names=FALSE, dec=dec)
  write.table(x$summGen, quote=FALSE, sep = sep, col.names=FALSE, dec=dec)}


cat("\nDocuments summary (summDoc)\n")  
if(var.agg=="") {
 name1 <- c("Occurrences", "DistinctWords", "PctLength", "Mean Length100")
 nameF <- c("DocName", name1, name1)
 name2 <- c("", rep("before",4), rep("after",4))
} else {
 nameF <- c("DocName", "Occurrences", "DistinctWords", "NumberDocs","PctLength", "MeanLength100", 
            "Occurrences", "DistinctWords", "PctLength", "MeanLength100")
 name2 <- c("", rep("before",5), rep("after",4))
}

A1 <- data.frame(nameF,name2)
colnames(A1) <- NULL
A2 <- cbind(A1,t(x$summDoc))
colnames(A2) <- NULL
A3 <- data.frame(t(A2)) 
colnames(A3) <- NULL
rownames(A3) <- c(""," ",c(1:(nrow(A3)-2)))
write.table(A3[1:2,], col.names=FALSE, quote=FALSE, sep = sep)
A4 <- lapply(A3[3:(ndoc+2), ], function(x) as.numeric(as.character(x)))
write.table(A4, col.names=FALSE, quote=FALSE, sep = sep, dec=dec)


cat("\nIndex of words (indexW)\n")
   cat("\nIndex of the most frequent words\n")
   A1 <- cbind(format(as.data.frame(rownames(x$indexW)), justify = "l"), x$indexW)
   colnames(A1) <- c(format("Word",justify = "l"), "Frequency", "N.Documents")
   # out1 <- t(c("","Word", "Frequency", "N.Documents",sep="\t"))
   out1 <- t(c("","Word", "Frequency", "N.Documents",sep=sep))
   write.table(out1, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE, dec=dec)
   write.table(A1, quote=FALSE, sep = sep, col.names=FALSE, dec=dec)
   # out1 <- t(c("","Word", "Frequency", "N.Documents",sep="\t"))
   out1 <- t(c("","Word", "Frequency", "N.Documents",sep=sep))
   Tabfq  <- x$indexW[order(rownames(x$indexW)),]
   cat("\nIndex of the words in alphabetical order\n")
   write.table(Tabfq, quote=FALSE, sep = sep, col.names=FALSE, dec=dec)

   

cat("\nDocuments by Words table (DocTerm)\n")
 # out1 <- t(c("", colnames(x$DocTerm),sep="\t"))
 out1 <- t(c("", colnames(x$DocTerm),sep=sep))
 write.table(out1, quote=FALSE, sep = sep,col.names=FALSE, row.names=FALSE, dec=dec)
 write.table(as.matrix(x$DocTerm), quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE, dec=dec)

# Repeated segments
if(!is.null(x$DocSeg)) { 
 cat("\nNumber of repeated segments\n")
 cat(" ", ncol(x$DocSeg),"\n")
 cat("\nIndex of segments (indexS)\n")
 cat("\nSegments ordered by frequency (indexS$segOrderFreq)\n")
  A1 <- cbind(format(as.data.frame(rownames(x$indexS$segOrderFreq)), justify = "l"),
     x$indexS$segOrderFreq)
     colnames(A1) <- c("Number", format("Segment",justify = "l"), "Frequency", "Long")
     # out1 <- t(c("Number","Segment", "Frequency", "Long",sep="\t"))
     out1 <- t(c("Number","Segment", "Frequency", "Long",sep=sep))
     write.table(out1, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE, dec=dec)
     write.table(x$indexS$segOrderFreq, quote=FALSE, sep = sep, col.names=FALSE, dec=dec)
 cat("\nSegments in alphabetical order (indexS$segOrderlist)\n")
 # out1 <- t(c("Number","Segment", "Frequency", "Long",sep="\t"))
 out1 <- t(c("Number","Segment", "Frequency", "Long",sep=sep))
    write.table(out1, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE, dec=dec)
    write.table(x$indexS$segOrderlist, quote=FALSE, sep = sep, col.names=FALSE, dec=dec)
 cat("\nDocuments by Repeated segments table (DocSeg)\n")
   # out1 <- t(c("", colnames(x$DocSeg),sep="\t"))
   out1 <- t(c("", colnames(x$DocSeg),sep=sep))
  write.table(out1, quote=FALSE, sep = sep,col.names=FALSE, row.names=FALSE, dec=dec)
  write.table(as.matrix(x$DocSeg), quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE, dec=dec)
}

# Supplementary variables
if(var.agg=="") {
 if(!is.null(x$context$quali))
 if(length(x$context$quali)>0) {
  cat("\nSummary of contextual qualitative variables\n")
  print(summary(x$context$quali))}

if(!is.null(x$context$quanti))
if(ncol(x$context$quanti)>0){
 cat("\nSummary of contextual quantitative variables\n")
  print(summary(x$context$quanti))}
} else {
  if(!is.null(x$context$quanti)){
  cat("\nSummary of contextual quantitative variables\n")
 Tabfq <- cbind(x$context$quanti, x$summDoc[,4])
 sname <- colnames(x$context$quanti)
 colnames(Tabfq) <- c(sname,"Ndocs")
  out1 <- t(c("",colnames(Tabfq)))
  write.table(out1, quote=FALSE, sep = sep,col.names=FALSE, row.names=FALSE)
  write.table(Tabfq, quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE)
}
if(!is.null(x$context$quali$qualivar))
 if(ncol(x$context$quali$qualivar)>0){
  cat("\nSummary of contextual qualitative variables\n")
  out1 <- t(c("",colnames(x$context$quali$qualitable)))
  write.table(out1, quote=FALSE, sep = sep,col.names=FALSE, row.names=FALSE)
  write.table(x$context$quali$qualitable, quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE)
 }
}

if(!is.null(x$SourceTerm)) {
  cat("\nFirst 5 documents in the source documents by words table\n")
  cat("For print all documents to a file use  write.table(as.matrix(x$SourceTerm),file=''...)\n")
  write.table(head(as.matrix(x$SourceTerm)))}

 cat("\nInformation about selected options in TextData\n")
 writeLines(unlist(lapply(x$info, paste, collapse=" ")))
  sink()	
}

if (!is.null(file)) {	
        	print(paste("All the results are in the file", file))
    	}
}

#' @export
print.LexChar <- function (x,file = NULL, sep = ";", dec=".",...) 
{
  
  if (!inherits(x, "LexChar")) 
    stop("x object must be LexChar class")

  df.res <- data.frame(name = character(),
                       description = character(),
                       stringsAsFactors = FALSE)

  if("CharWord" %in% names(x)) df.res[nrow(df.res)+1,] <- c("$CharWord", "Words characterizing the documents")
  if("stats" %in% names(x)) df.res[nrow(df.res)+1,] <- c("$stats", "Association statistics of the lexical table")
  if("CharDoc" %in% names(x)) df.res[nrow(df.res)+1,] <- c("$CharDoc", "Documents characterizing categories of aggregate table")
  if("Proba" %in% names(x)) df.res[nrow(df.res)+1,] <- c("$Proba", "Threshold on the p-value used when selecting the characteristic words
  (by default 0.05")
                                                                         
  if(!is.null(x$quali$CharWord)) df.res[nrow(df.res)+1,] <-c("$quali$CharWord", "Words characterizing the qualitative variables and their categories")
  if(!is.null(x$quali$stats)) df.res[nrow(df.res)+1,] <- c("$quali$stats", "Association statistics for vocabulary and qualitative variables")
  if(!is.null(x$quali$CharDoc)) df.res[nrow(df.res)+1,] <-c("$quali$CharDoc", "Documents characterizing categories of aggregate table from qualitative variables and their categories")
 
  if(!is.null(x$quanti$CharWord)) df.res[nrow(df.res)+1,] <- c("$quanti$CharWord", "Provides characteristic quantitative variables for each word") 
  if(!is.null(x$quanti$stats)) df.res[nrow(df.res)+1,] <- c("$quanti$stats", "Association statistics for vocabulary and quantitative variables")
  
  df.res <- as.matrix(df.res) 
  index <- nrow(df.res)
  res <- array("", c(index, 2), list(1:index, c("name", "description")))
  for(i in 1:index) res[i,] <-df.res[i,]
  cat("\n*The results are available in the following objects:\n\n")
  print(res[c(1:index),])

  sink.reset <- function(){
    for(i in seq_len(sink.number())){sink()}}
  sink.reset
  
  if (is.null(file))
    cat("(*) To obtain more detailed information use print to file (print.LexChar)")
  else 
    {
      sink(file)
  
  fCharWord <- function(z)  {
    nobjects <- length(z) 

    cat("\n\nCHARACTERISTIC WORDS FOR EACH DOCUMENT\n(DETAILED INFORMATION)\n\n")
    cat("\nOver_used_words are ordered from the most characteristic word to the less one\n")
    cat("Infra_used_words are ordered from the most anti-characteristic word to the less one\n\n")
    stit <- t(c("Word","Intern %","glob %","Intern freq","Glob freq","p.value", "v.test"))
  
    
    for(i in 1:nobjects){	
      out1 <- c(names(z[i]), "Over_used_words")	
      out2 <- c(names(z[i]), "Infra_used_words")	
      
      #    write.table(out1, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
      temp <- z[[i]]	
      posic <- temp[,"v.test"]>0	
      
      out.over <- data.frame(temp[posic,,drop=FALSE])	
      out.infra <- data.frame(temp[!posic,,drop=FALSE])	

      if(nrow(out.over)>0){	
        write.table(out1, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
        write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
        write.table(out.over, quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE,dec=dec) 
        cat("\n")	
      }
      
      if(nrow(out.infra)>0){	
        write.table(out2, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
        write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
        write.table(out.infra, quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE,dec=dec) 
        cat("\n")	
      }
    }
  }
  
  fCharQuanti<- function(x)  {
    tac <- NULL
    strcolnames<- c("GlobalAverage", "AverageWord","Differ.", "pvalue", "Word", "Variable")
    for(i in 1:length(x)) {
      t1<- as.data.frame(x[i,drop=FALSE])
      t2 <- data.frame(t1,rep(names(x[i]),length(x[i])), rownames(t1))
      colnames(t2) <- strcolnames

      if(i==1) tac <- t2 else tac <- rbind(tac,t2)
      SP <- split(tac,f=tac$Variable, drop=FALSE)
      str.colnames<- c("Word", "GlobalAverage", "AverageWord","Differ.", "pvalue")
      empty_list = structure(vector(mode = "list", length = length(SP)), names = names(SP))		
      

      for(i in 1:length(SP)) {
        t1<- as.data.frame(SP[i,drop=FALSE])
        t2.pos <- t1[t1[,3]>0, ,drop=FALSE]
        t2.neg <- t1[t1[,3]<0, ,drop=FALSE]
        if(nrow(t2.pos)>0) {
          t2.pos <- t2.pos[order(t2.pos[,4]),,drop=FALSE]
          rownames(t2.pos) <- paste0("P", c(1:nrow(t2.pos)))
          t3.pos <- t2.pos[,c(5,1:4)]
          colnames(t3.pos) <- str.colnames
          empty_list[[i]]$posit <- t3.pos
        }
        if(nrow(t2.neg)>0) {
          t2.neg <- t2.neg[order(t2.neg[,4]),,drop=FALSE]
          rownames(t2.neg) <- paste0("N", c(1:nrow(t2.neg)))
          t3.neg <- t2.neg[,c(5,1:4)]
          colnames(t3.neg) <- str.colnames
          #    print(t3.neg)
          empty_list[[i]]$negat <- t3.neg
        }
      } # End for
    }
    return(empty_list)
  } # End fCharQuanti
  
  
  #################  Check CharWord
  if(!is.null(x$CharWord)) {
    
  fCharWord(x$CharWord)
  nobjects <- length(x$CharWor)
  
  cat("\n\nDOCUMENTS (OR AGGREGATE DOCS) RELATED WITH A WORD\n")	
  tw <- NULL	
  for(i in 1:nobjects){	
    tw <- c(tw,rownames(x$CharWord[[i]]))	
  }	
  tw <- paste(unique(tw))	
  tw <- sort(tw, decreasing = FALSE)	
  ntw <- length(tw)	
  
  
  for(i in 1:ntw){	
    vsearch <- tw[i]	
    vt_ <- NULL; posit <- NULL; negat <- NULL  	
    for(j in 1:nobjects){	
      cat_ <- which(rownames(x$CharWord[[j]]) %in% vsearch)	
      if(length(cat_)>0){	
        vt  <- x$CharWord[[j]][cat_,6]	
        if(vt>0) posit <- c(posit,sep,names(x$CharWord[j])) else	
          negat <- c(negat,sep,names(x$CharWord[j]))	
      }	
    }	
    cat("\n",vsearch)	
    if(length(posit)>0)  cat("\n",sep, "Over", posit) 	
    if(length(negat)>0)  cat("\n",sep, "Infra", negat) 	
    cat("\n")	
  }	
    }
  
  
  fCharWord2 <- function(z,i)  {
    nobjects <- length(z) 
    
  #  cat("\n\nCHARACTERISTIC WORDS FOR EACH DOCUMENT\n(DETAILED INFORMATION)\n\n")
  #  cat("\nOver_used_words are ordered from the most characteristic word to the less one\n")
  #  cat("Infra_used_words are ordered from the most anti-characteristic word to the less one\n\n")
    stit <- t(c("Word","Intern %","glob %","Intern freq","Glob freq","p.value", "v.test"))
    
    
  #  for(i in 1:nobjects){	
      out1 <- c(names(z[i]), "Over_used_words")	
      out2 <- c(names(z[i]), "Infra_used_words")	
      
      #    write.table(out1, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
      temp <- z[[i]]	
      posic <- temp[,"v.test"]>0	
      
      out.over <- data.frame(temp[posic,,drop=FALSE])	
      out.infra <- data.frame(temp[!posic,,drop=FALSE])	
      
      if(nrow(out.over)>0){	
        write.table(out1, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
        write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
        write.table(out.over, quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE,dec=dec) 
        cat("\n")	
      }
      
      if(nrow(out.infra)>0){	
        write.table(out2, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
        write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)	
        write.table(out.infra, quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE,dec=dec) 
        cat("\n")	
      }
   # }
  }
  

  
  
    
    #################  Check of stats exists  
  if(!is.null(x$stats)) {
    cat("\n\n\nSTATISTIC ASSOCIATION LEXICAL TABLE DOCUMENTS (OR AGGREGATE DOCS) AND VOCABULARY\n\n")
      stit <- t(c("",colnames(x$stat)))
      write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
      write.table(x$stats, quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE,dec=dec)
    }

  
  
  #################  Check CharDoc
  if(!is.null(x$CharDoc)) {
    cat("\n\n\nCHARACTERISTIC SOURCE-DOCUMENTS\n")
    #     print(object$CharDoc)
    stit <- t(names(x$CharDoc[[1]]))
    for(i in 1:length(x$CharDoc)){	
      cat("\n")
      write.table(names(x$CharDoc[i]), quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
      write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
      write.table(x$CharDoc[[i]][1:3], quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
    }	
  }  
  
  ###########  Qualitative contextual variables
  if(!is.null(x$quali$stats)){	
    cat("\n\nASSOCIATION STATISTICS OF AGGREGATE CONTEXTUAL QUALITATIVE TABLE\n")
    stit <- t(c("",colnames(x$quali$stats)))
    write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
    write.table(x$quali$stats, quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE,dec=dec)
  } # End Vocab$quali$CharWord$stats
  
  if(!is.null(x$quali$CharWord)){	
    for(i in 1:length(x$quali$CharWord)) {
      cat("\n\nCharacteristic of aggregate-documents for qualitative variable: ", names(x$quali$CharWord[i]), "\n")
      cat("=====================================================================================")
      fCharWord2(x$quali$CharWord,i)
    }
    
    #################  Check CharDoc
    if(!is.null(x$quali$CharDoc)) {
      cat("\n\n\nCHARACTERISTIC SOURCE-DOCUMENTS\n")
      #     print(object$CharDoc)
      stit <- t(names(x$quali$CharDoc[[1]]))
      for(i in 1:length(x$quali$CharDoc)){	
        cat("\n")
        write.table(names(x$quali$CharDoc[i]), quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
        write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
        write.table(x$quali$CharDoc[[i]][1:3], quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
      }	
    }   
  } # End CharWord

  

  ###########  Quantitative contextual variables
  if(!is.null(x$quanti$stats)){	
    cat("\nRELATIONSHIPS BETWEEN VOCABULARY AND CONTEXTUAL QUANTITATIVE VARIABLES\n")
    cat("\nStatistics for quantitative variables")
    cat("\n-------------------------------------\n")
    stit <- t(c("",colnames(x$quanti$stats)))
    write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
    write.table(x$quanti$stats, quote=FALSE, sep = sep, col.names=FALSE, row.names=TRUE,dec=dec)
    

  #  print(object$quanti$CharWord)
    res.quanti <- fCharQuanti(x$quanti$CharWord)
    num.var.quanti <- length(res.quanti)
    stit<- t(c("Word", "GlobalAverage", "AverageWord","Differ.", "pvalue"))
    
       for(i in 1:num.var.quanti) {
      str.name.var <- rownames(x$quanti$stats)[i]
      cat("\n\nCharacteristic quantitative variables for each word. Variable:", str.name.var)
      cat("\n----------------------------------------------------------------------------------------\n")
      
      
      if(length(res.quanti[[str.name.var]]$posit)>0) {
        cat(paste0("\n$",str.name.var, "$posit\n"))
        write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
        write.table(res.quanti[[str.name.var]]$posit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
      }
   
      if(length(res.quanti[[str.name.var]]$negat)>0) {
        cat(paste0("\n$",str.name.var, "$negat\n"))
        write.table(stit, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
        write.table(res.quanti[[str.name.var]]$negat, quote=FALSE, sep = sep, col.names=FALSE, row.names=FALSE,dec=dec)
      }    
      
    }
    
  } # End quanti
  
  cat("\n------- Proba threshold -----------\n")
  cat(paste("Proba=", x$Proba))
  
  
  sink()
    }  # End sink file    
  if (!is.null(file)) cat(paste("(*) All the results are in file", file))
}

#' @import FactoMineR slam stringr tm graphics gridExtra utils stringi
#' @rawNamespace import(stats, except = c(hclust))
#' @importFrom utils packageDescription
#' @export
TextData <- function (base, var.text=NULL, var.agg=NULL, context.quali=NULL, context.quanti= NULL, 
                      selDoc="ALL", lower=TRUE, remov.number=TRUE, lminword=1, Fmin=Dmin, Dmin=1, Fmax=Inf,
                      stop.word.tm=FALSE, idiom="en", stop.word.user=NULL, segment=FALSE, 
                      sep.weak="default",
                      sep.strong="\u005B()\u00BF?./:\u00A1!=;{}\u005D\u2026", 
                      seg.nfreq=10, seg.nfreq2=10, seg.nfreq3=10,
                      graph=FALSE)
{
  # Version 1.5.4. Allow use tibble as base
  str.type <- class(base)
  if("data.frame" %in% str.type) class(base) <- "data.frame" else
    base<- as.data.frame(base)
  
 if(sep.weak=="default")
   sep.weak = ("[%`:*$&#/^|<=>;'+@.,~?(){}|[[:space:]]|\u2014|\u002D|\u00A1|\u0021|\u00BF|\u00AB|\u00BB|\u2026|\u0022|\u005D|\u0097")
  dfold <- base # deparse(substitute(base))

  filt=sep.weak

  
  if(!is.null(var.agg)) {
    if(is.numeric(base[,var.agg])) base[,var.agg] <- as.character(base[,var.agg] ) # version 1.4.2
    if(is.character(base[,var.agg])) base[,var.agg] <- as.factor(base[,var.agg]) # version 1.3.1	
    base[,var.agg] <- droplevels(base[,var.agg])
    var.agg.seg <- data.frame(base[,var.agg,drop=FALSE])
  } 	
  


  #---------------------------------------------------
  plotTextData <- function()
  {
    # if(dev.interactive()) dev.new()
    plot.TextData(y)
  }
  
  #---------------------------------------------------
  # Count occurrences		
  occurrFunc <- function(z,title, dOcc, bagg){		
    DT <- data.frame(z$i,z$j,z$v)	
    colnames(DT) <- c("i","j","v") 	
    a1 <- data.frame(sapply(split(DT, DT$i, drop=FALSE), function(w) sum(w$v)))	
    q <-4			
    if(bagg==TRUE) if(!is.null(dOcc )){q <- q+1 }		
    if(is.null(dOcc)){ dOcc <- cbind(data.frame(z$dimnames$Docs,0)); q<-0}		
    dOcc[,(q+2)] <- 0	
    dOcc[rownames(a1),(q+2)] <- a1[1]	
    a2 <- data.frame(sapply(split(DT, DT$i, drop=FALSE), function(w) length(w$v))) 	
    dOcc <- cbind(dOcc,0)		
    dOcc[,(q+3)] <- 0		
    dOcc[rownames(a1),(q+3)] <- a2[1]		
    if(q==0) colnames(dOcc)[1] <- c("DocName")	
    colnames(dOcc)[q+2]<- c( paste0("Occurrences.",title,sep="" ))		
    colnames(dOcc)[q+3]<- c( paste0("DistinctWords.",title,sep=""))		
    return(dOcc)}		
  
  #---------------------------------------------------			
  aggreg <- function(text.var, grouping.var){			
    G <- as.character(substitute(grouping.var))			
    grouping <- unlist(grouping.var)			
    y <- rle(as.character(as.vector(grouping)))			
    lens <- y$lengths			
    group <- y$values			
    x <- cumsum(lens)			
    st <- c(1, x[-length(x)] + 1)			
    end <- c(x)			
    L1 <- invisible(lapply(seq_along(st), function(i) {			
      pasteagg(text.var[st[i]:end[i]], sep = " . ")}))			
    names(L1) <- group			
    DF <- data.frame(x = names(L1), text.var = unlist(L1), row.names = NULL)			
    colnames(DF)[1] <- "Group"			
    return(DF)}			
  
  #---------------------------------------------------
  pasteagg <- function(mc,sep = ".") 
  {
    mc <- lapply(mc, function(x) {
      gsub("^\\s+|\\s+$", "", x)})
    mc <- do.call("cbind", mc)
    m <- { apply(mc, 1, function(x) {
      if (any(is.na(x))) { NA }
      else {
        paste(x, collapse = sep)}})}
    names(m) <- NULL
    return(m)
  }
  
  #---------------------------------------------------		
  # Function to recodify the position of the words		
  recoderFunc <- function(z, from, to) return(to[match(z, from)])		
  
  #---------------------------------------------------		
  #Function to select words		
  selectFunc <- function(z, selwords) {		
    pos<- which(z$j%in%sel.words)		
    z$j<- z$j[pos]; z$i<- z$i[pos]; z$v<- z$v[pos]		
    z$j<-recoderFunc(z$j,selwords,1:length(selwords))		
    z$j<- as.numeric(factor(z$j,labels=c(1:length(selwords))))		
    z$dimnames$Terms<-z$dimnames$Terms[selwords]		
    z$ncol<-length(z$dimnames$Terms)		
    z$nrow<-length(z$dimnames$Docs)		
    return(z)}		
  
  #---------------------------------------------------					
  infoNew <- function(){
    mbase <- list(dfold, "name of input R data.frame")
    menvir <- list(globalenv(),"name of environment")
    mvartext <- list(var.text, "names of textual columns")
    midiom <- list(idiom, "idiom of the corpus, (by default en)")
    mlminword <- list(lminword, "minimum length of a word (by default 1)")					
    mlower <- list(lower, "converting the corpus into lowercase (by default TRUE)")					
    mremnum <- list(remov.number, "removing the numbers (by default TRUE)")            					
    mFmin <- list(Fmin, "minimum frequency of a word (by default Dmin)")					
    if(is.null(Fmax)) Fmax <- Inf					
    mFmax <- list(Fmax, "maximum frequency of a word (by default Inf)")					
    mDmin <-  list(Dmin, "minimum number of documents using a word (by default 1)")		
    mndoc <- list(dtm$nrow, "number of non-empty non-aggregate documents") 	
    mnlength <- list(sum(Nfreqword), "corpus size (total number of occurences)")	
    mnWord <- list(dtm$ncol, "vocabulary size (total number of words)")
    if(is.null(var.agg)) var.agg <- "" 	
    mndocsagg <- list(var.agg, "name of the aggregation variable")
    stop_word_tm <- stop.word.tm
    if(stop.word.tm==TRUE) 			
      stop_word_tm <- stopwords(kind=idiom)
    mSwtm <-  list(stop_word_tm, "stopword list provided by tm package")
    mSwuser <-  list(stop.word.user, "stopword list provided by the user")
    mbsegm <- list(segment, "searching repeated segments (by default FALSE)")
    zinfo <- list(base=mbase,menvir=menvir, var.text=mvartext, idiom = midiom, lminword=mlminword, lower=mlower,remov.number=mremnum, Fmin=mFmin,
                  Fmax=mFmax,Dmin=mDmin,Ndoc=mndoc, LengthW= mnlength, Nword=mnWord,
                  name.var.agg=mndocsagg ,stop.word.tm=mSwtm, stop.word.user =mSwuser, segment.searched = mbsegm)
    
    
    if(segment==TRUE) { 
      mNseg <- list( nbseg, "number of segments")
      mseg.nfreq <- list(nfreq, "minimum frequency of a more-than-3-words repeated segment (by default 10)")			
      mseg.nfreq2 <- list(nfreq2, "minimum frequency of a length-two repeated segment (by default 10)")					
      mseg.nfreq3 <- list(nfreq3, "minimum frequency of a length-three repeated segment (by default 10)")					
      segments <- list(Nseg=mNseg, seg.nfreq=mseg.nfreq, seg.nfreq2=mseg.nfreq2 ,seg.nfreq3=mseg.nfreq3) 
      zinfo<- c(zinfo, segments=list(segments))
    }

    return(zinfo)
  }
  
  nxlon<-20
  nfreq <- seg.nfreq; nfreq2<- seg.nfreq2; nfreq3<-seg.nfreq3 					
  if(segment==FALSE) { nfreq <- ""; nfreq2<-""; nfreq3<-""}	
  
  blongErr <- FALSE				
  
  if(segment==TRUE) {					
    # auxiliary functions
    REPWEAK <-function(chaine,sep.weak) res<-stringr::str_replace_all(chaine,sep.weak, " ")					
    REPSTRONG <-function(chaine,sep.strong) res<-stringr::str_replace_all(chaine,sep.strong, " zzwwxxyystr ")					
    PROCHE <-function(ideb,ifin,ITEX,ITDR,ITRE,nfreq,nfreq2,nfreq3,long,nxlon,nbseg)					
    {   					
      # the function proche detects the first sublist of adresses in ITDR corresponding a same successor					
      # if this successor is not "end of answer" or "strong separator", we have located an admissible segment					
      list.segment<-list()				
      ad.segment<-vector()					
      te.segment<-NULL					
      rep.segment<-vector()					
      mfrec<-0					
      ipunt<-ideb-1					
      isucc<-ITEX[ITDR[ideb]+long-1] 					
      while(  (ITEX[ITDR[ipunt+1]+long-1]==isucc) &  (ipunt < ifin) )					
      {					
        if (!( (isucc=="zzwwxxyystr") | (isucc=="zzwwxxyyendrep")))					
        {					
          mfrec<-mfrec+1					
          ad.segment[mfrec]<-ITDR[ipunt+1]					
          rep.segment[mfrec]<-min(which(ITRE>ad.segment[mfrec]))					
        }					
        ipunt<-ipunt + 1					
      }									
      ifin<-ipunt					
      nfreq.threshold<-nfreq					
      if (long==1)  nfreq.threshold<-999999999999					
      if (long==2)  nfreq.threshold<-nfreq2					
      if (long==3)  nfreq.threshold<-nfreq3					
      ltrou<-( !( (isucc=="zzwwxxyystr") | (isucc=="zzwwxxyyendrep"))  & (mfrec >= nfreq)) 					
      ltrouseg<-( ltrou & (mfrec >= nfreq.threshold))  					
      
      ### here we have to control that it is not a "constrained segment", either at the left part or at the right part					
      if (ltrouseg) 					
      { 					
        contraintG<-TRUE  					
        contraintD<-TRUE								
        IGAUC<-ad.segment-1					
        special<-c("zzwwxxyystr", "zzwwxxyyendrep")					
        if (0 %in% IGAUC) contraintG<-FALSE 					
        if (max(special %in% ITEX[IGAUC])) contraintG<-FALSE    					
        if (contraintG) {					
          SUCC<-as.factor(ITEX[IGAUC])					
          if (nlevels(SUCC) > 1) contraintG <- FALSE 					
        }					
        IDROI<-ad.segment+long					
        if (max(special %in% ITEX[IDROI])) contraintD<-FALSE    					
        if (contraintD) {					
          SUCC<-as.factor(ITEX[IDROI])					
          if (nlevels(SUCC) > 1) contraintD <- FALSE 					
        }					
        if (contraintD | contraintG) ltrouseg<-FALSE					
        if (contraintG) ltrou<-FALSE      					
      }					
      ###  final contrained segments removing					
      if (ltrouseg) 					
      { 					
        for (i in 1:long){					
          te.segment<-paste(te.segment,ITEX[ITDR[ideb]+(i-1)],sep=" ")				
          te.segment<-stringr::str_trim(te.segment)}					
        lo.segment<-long				
        fr.segment<-mfrec
        nr.segment<-nbseg+1					
        list.segment<-list(te.segment,fr.segment,ad.segment,rep.segment,lo.segment,nr.segment)					
        names(list.segment)<-c("text","frequency","adresses","documents","length","nr.seg")					
      }       					
      return(list(ifin=ifin,ltrou=ltrou,ltrouseg=ltrouseg,list.segment=list.segment))					
    }                                                                                                                                                                                                                                                                                                                                                                                                                                                					
    
    ORD.EXT<-function(ICRIT,ADR,long1)					
    {					
      # Ordering adresses from successors in text					
      ICRIT_ord<-order(ICRIT)					
      ADR<-ADR[c(ICRIT_ord)]					
      return(list(ADR=ADR))					
    }					
  }					
  ##### Final internal functions
#  
#  if(!is.null(var.agg)){
#    if(is.numeric(base[,var.agg])) base[,var.agg] <- as.factor(base[,var.agg])
#    # Añadida la siguiente el 10/01/2019 para eliminar niveles de factores no utilizados en variable de agrupación
#    base[,var.agg] <- factor(base[,var.agg])
#    var.agg.seg <- data.frame(base[,var.agg,drop=FALSE])
#  }
  remov.docs <- rownames(base)
  

  #--------- Selecting docs by rownumber or rowname -------------------					
  if(selDoc[1]!="ALL") {					
    if (!is.character(selDoc)) 					
    selDoc <- rownames(base)[selDoc]					
    selDoc <- which(rownames(base) %in% selDoc)					
    base <- base[selDoc,]}		

  #--------- Save corpus var.text  -------------------					
  if(!is.character(var.text)) var.text <- colnames(base)[var.text]
  var.text <- var.text[which(var.text %in% colnames(base))]
  if(min(var.text)<1) stop("Error in var.text")					
  if(length(var.text) == 0) stop("You must define var.text")					
  
  # NA in texts is replaced by ""
  for (i in 1:length(var.text)){		
    base[, var.text[i]] <- as.character(base[, var.text[i]])
    base[is.na(base[var.text[i]]), var.text[i]] <- ""
    }
    corpus <- base[, var.text[1]]
  if(length(var.text) > 1) {					
    for (i in 2:length(var.text)){	
      corpus2 <- as.character(base[, var.text[i]])
      dos <-which(corpus!="" & corpus2!="")
      corpus[dos] <- paste(corpus[dos], corpus2[dos], sep=". ")
      uno <-which(corpus=="" & corpus2!="")
      corpus[uno] <- corpus2[uno]
    }
    rm(corpus2)
    }					
  corpus <- data.frame(corpus, stringsAsFactors = FALSE)					
  rownames(corpus) <- rownames(base)					


  #--------- Save context.quanti  -------------------
  data.context.quanti <- NULL
  if(!is.null(context.quanti)){ 
    if(length(context.quanti)==1) {
      if(context.quanti=="ALL") 
        context.quanti <- names(which(sapply(base,is.numeric)))  }
    if (!is.character(context.quanti)) 
      context.quanti <- colnames(base[context.quanti])
    context.quanti <- context.quanti[which(context.quanti %in% colnames(base))]
    tq <- names(which(sapply(base,is.numeric)))
    pos <- which(context.quanti %in% tq)
    data.context.quanti <- data.frame(base[,context.quanti[pos]])
    colnames(data.context.quanti) <- context.quanti[pos]
    qf <- context.quanti[-pos]
    if(!is.null(qf))
      if(length(qf)>0)
      { # There are quantitative variables as factors
        for (i in 1:length(qf)) {
          levels(base[qf[i]]) <- levels(droplevels(base[qf[i]]))  
          rdo <- suppressWarnings(as.numeric(levels(base[,qf[i]])))
          valrdo <- rdo[!is.na(rdo)]
          if(length(valrdo)>0) {
            rdo <- suppressWarnings(as.numeric(as.character(base[,context.quanti[i]])))
            if(is.null(data.context.quanti)) {	
              data.context.quanti <- data.frame(rdo) 	
              colnames(data.context.quanti) <- context.quanti[i]	
            } else { 	
              data.context.quanti <- cbind(data.context.quanti, rdo) 	
              colnames(data.context.quanti)[ncol(data.context.quanti)] <- context.quanti[i]	
            }}}} 
    if(!is.null(data.context.quanti)) rownames(data.context.quanti) <- rownames(base)
    SourceTerm.quanti <- data.context.quanti
  }
  
  var.check <- NULL
  
  #--------- Save context.quali  -------------------
  tmpquali <- NULL
  if(!is.null(context.quali)) {
    if(length(context.quali)==1) {
      if(context.quali=="ALL") 
        context.quali <- names(which(sapply(base,is.factor)))
    }
    if (!is.character(context.quali)) 	
      context.quali <- colnames(base[context.quali])	
    context.quali <- context.quali[which(context.quali %in% colnames(base))]
    tq <- names(which(sapply(base,is.numeric)))
    pos <- which(context.quali %in% tq)
    if(length(pos)>0)  
      base[,context.quali[pos]] <- as.factor(as.character(base[,context.quali[pos]]))
    
    # Remove selected text variables, var.text from context.quali
    pos <- which(context.quali %in% var.text)
    if(length(pos)>0) context.quali <- context.quali[-pos]
  }
 
  
   
  # Remove var.agg from context.quali
  if(!is.null(var.agg)){
  #  if(is.numeric(base[,var.agg])) base[,var.agg] <- as.factor(base[,var.agg])
    if(!is.null(context.quali)) {  		
      # Remove var.agg from qualitative context		
      pos <- which(context.quali %in% var.agg)		
      if(length(pos)>0) context.quali <- context.quali[-pos]		
      var.check <- c(context.quali, var.agg)			
    }}		
  
  if(!is.null(context.quali)) var.check <- context.quali
  if(!is.null(var.agg)) var.check <- c(context.quali, var.agg)
  if(length(var.agg)>1) stop("Only one variable for aggregation")

  
  
    
  #--------- Rename NA levels in factor var.context and var.agg variables To Missing
  nvcheck <- length(context.quali)
  for(i in 1:nvcheck) {
    labi <- levels(base[,context.quali[i]])  # Levels of factors
    # If any value is NA but there is not a level NA
    if(any(is.na(base[,context.quali[i]]))){
      levels(base[,context.quali[i]]) <- c(labi,"Missing")
      pos <- which(is.na(base[,context.quali[i]]))
      base[pos,context.quali[i]] <- "Missing"}
    # If any value is '' but there is not a level NA
    pos <- which(labi %in% '')
    if(!is.null(pos)>0) levels(base[,context.quali[i]])[pos] <- "Missing"
    # If any value is <NA> but there is not a level <NA>
    pos <- which(labi %in% '<NA>')
    if(length(pos)>0) levels(base[,context.quali[i]])[pos] <- "Missing"
  }

  
  if(!is.null(var.agg)){
    if(anyNA((base[,var.agg]))) { # There some NA in var.agg
      if(!"Missing" %in% levels(base[,var.agg])) levels(base[,var.agg])[length(base[,var.agg])+1] <- "Missing"
       base[is.na(base[,var.agg]),var.agg] <- "Missing"
    }
  }
  

  
  #--------- Rename repeated levels in factor var.context and
  if(nvcheck >1){
    for(i in 1:(nvcheck -1)) {
      strnamei <- var.check[i]
      levi <- levels(base[,strnamei])
      for(j in (i+1):nvcheck) {
        strnamej <- var.check[j]
        levj <- levels(base[,var.check[j]])
        repetij <- levi[(which(levi %in% levj))]
        nrep <- length(repetij)
        if(nrep>0){
          missrep <- which("Missing" %in% repetij)
          if(missrep==1) nrep <- nrep-1
          if(nrep>0){
            levels(base[,strnamei]) <- paste0(strnamei,"_",levi)
            levels(base[,strnamej]) <- paste0(strnamej,"_",levj)
          }}}}}
  
  
  # Rename "Missing" to "Missing" & variable name
  for(i in 1:(nvcheck)) {
    strnamei <- var.check[i]
    levi <- levels(base[,strnamei])
    miss <- which(levi %in% "Missing")
    levels(base[,strnamei])[miss] <- paste0("Missing","_",strnamei)
  }

  if(!is.null(var.agg)){
    dfvaragg <- data.frame(base[,var.agg,drop=FALSE])
  }
  #if(packageDescription("tm")$Version >"0.7-1") {
  colnames(corpus)[1] <- "text"
  corpus$doc_id <- rownames(corpus)
  #}
  #--------- Read texts from tm -------------------
  dtmCorpus <- VCorpus(DataframeSource(corpus), readerControl = list(language = idiom)) 
  dtmCorpus <- tm_map(dtmCorpus, content_transformer(function(x) gsub(filt, " ", x, fixed=FALSE)))
  dtmCorpus <- tm_map(dtmCorpus, stripWhitespace)
#  dtm <- DocumentTermMatrix(dtmCorpus, control = list(tolower = lower, wordLengths = c(lminword, Inf)))
  dtm <- DocumentTermMatrix(dtmCorpus, control = list(tolower = lower, wordLengths = c(1, Inf)))
  rownames(dtm) <- rownames(base)
 
  if(!is.null(var.agg)) SourceTerm.seg <- SourceTerm <- dtm

  
  
  # ---------------- If aggregation ---------
  if(!is.null(var.agg)){
    # To build a data frame with 3 columns (rows, columns and frequency)
    # This is a compressed table
    agg <- data.frame(base[dtm$i,var.agg], dtm$j,dtm$v)
    agg <-aggregate(agg[,3], by=list(agg[,1], agg[,2]),FUN=sum, na.rm=TRUE)
    agg <- agg[order(agg[,1],agg[,2]),]
   dtmagg <- dtm
    dtmagg$nrow <- length(levels(agg[,1]))
    dtmagg$i <- as.numeric(agg[,1])
    dtmagg$j <- agg[,2]
    dtmagg$v <- agg[,3]
    dtmagg$dimnames$Docs <- levels(agg[,1])
    detOccAgg <- occurrFunc(dtmagg, "before",NULL, TRUE) 
    # return(detOccAgg)
  }
  

  
  #--------- ------------------			
  Nfreqword <-tapply(dtm$v,dtm$j,sum)			
  Ndocword  <-tapply(dtm$v>0,dtm$j,sum)	
  Table <- cbind(Nfreqword,Ndocword)			
  rownames(Table) <- dtm$dimnames$Terms			
  colnames(Table) <- c("Frequency", "N.Documents")		
  TFreq <- Table[order(Nfreqword, Ndocword, decreasing = TRUE), ]		
  detOcc <- occurrFunc(dtm, "before",NULL, FALSE) 	
  ndocIni <- nrow(detOcc)		
  ndocIniEmpty <- ndocIni - length(unique(dtm$i))			
  rownamesdocs.no.empty <- rownames(base)[unique(dtm$i)]	

  
  
  
  rownamesdocs.empty <- rownames(base)[-unique(dtm$i)]			

  N <- ndocIni
  detOcc$PctLength.before <- 100*detOcc[,2]/sum(detOcc[,2])
  detOcc$MeanLength100.before <- round(N*100*detOcc[,2]/sum(detOcc[,2]),2)
  detOcc$PctLength.before <- round(detOcc$PctLength.before,2)				
  wordsafter <- dtm$ncol			
  
  Docs.before <- detOcc[,c("DocName", "Occurrences.before")]


  if(!is.null(var.agg)){			
    numberdocs <- table(base[,var.agg])			
    posic <- which(names(numberdocs) %in% dtmagg$dimnames$Docs) 			
    numberdocs <- numberdocs[posic]			
    detOccAgg$NumberDocs <- numberdocs			
    detOccAgg$PctLength.before <- 100*detOccAgg[,2]/sum(detOccAgg[,2])			
    detOccAgg$MeanLength100.before <- round((detOccAgg[,2]*100/numberdocs)/(sum(detOccAgg[,2])/sum(numberdocs)),2)			
    detOccAgg$PctLength.before <- round(detOccAgg$PctLength.before,2)			
  }			
  
  if(!is.null(var.agg)) {	
    if(!is.null(corpus$doc_id)) corpus$doc_id <- NULL
    corpusSeg <- corpus
    corpus <- cbind(corpus,base[,var.agg]) 		
    corpus <- corpus[order(corpus[,2]), ]		
    names(corpus)[1] <- "my_text"		
    names(corpus)[2] <- "my_agg"	
    corpus <- aggreg(text.var=corpus$my_text, grouping.var = corpus$my_agg)		
    rownames(corpus) <- corpus[,1]		
    names(corpus)[2] <- "corpus"		
    corpus[,1] <- NULL 
  }		


  
  if(segment==TRUE) {						
    maj.in.min = lower # y$info[3,2]						
    sep.weak ="([\u0027\u02BC\u0060]|[,;'?\n\u202F\u2009\u0028]|[[:punct:]]|[[:space:]]|[[:cntrl:]])+"						
    if (nfreq2<nfreq) nfreq2<-nfreq		
    if (nfreq3<nfreq) nfreq3<-nfreq		
    if (nfreq3>nfreq2)nfreq3<-nfreq2	
    text1<-apply(as.matrix(apply(as.matrix(corpus),1,FUN=REPSTRONG,sep.strong)),1,FUN=REPWEAK,sep.weak)						
    text3<-apply(matrix(text1),1,(stringr::str_c),"zzwwxxyyendrep",sep=" ") 						
    nrep<-NROW(text3)						
    listrep<-strsplit(as.character(text3),split=" ") 						
    ITEX <- unlist(listrep)						
    # ITEX is a vector of occurrences of words 						
    if (maj.in.min == TRUE)  ITEX <- tolower(ITEX)						
    if (remov.number == TRUE) ITEX<- removeNumbers(ITEX)						
    # ITEX is a vector of occurrences of words, but with fictitious "empty" words because of the multiple spaces						
    # these fictitious words have to be eliminated						
    sel <- which(ITEX=="") 					
    if (length(sel)!=0){	 				
      ITEX <- ITEX[-sel]                					
    }					
    # The text is in the form of a vector of occurrences  						
    ITEX.f<-as.factor(ITEX)						
    FREQ.mots<-table(ITEX.f)						
    FREQ.cum<-cumsum(FREQ.mots)						
    Vplus<-dim(FREQ.mots)										
    # To conserve the addresses when ordering ITEX						
    ITDR<-order(ITEX)
    # adress of the answers (=adress of the first word corresponding to the answer) in ITEX						
    ITRE<-vector()						
    ITRE<-which(ITEX=="zzwwxxyyendrep")											
    #######  the data structures are built						
    Nplus<-length(ITEX)						
    ITDR<-seq(1,Nplus,1)						
    lpil<-vector()						
    list.tot.segment<-list()						
    # global initialisations						
    ideb<-1						
    ifin<-Nplus						
    long<-0						
    nbseg<-0						
    # for all the distinct words, we have to detect the segments beginning with this word						
    ltrou<-((ifin-ideb+1) >= nfreq)  											
    while(ltrou)      						
    {                						
      while (ltrou & (long<=nxlon))       #exploration of the possible segments issued from word_in_course	
      {						
        if(long>nxlon) blongErr <- TRUE	
        long1<-long						
        long<-long+1					
        lpil[long]<-ifin						
        res.ORD.EXT<-ORD.EXT(ITEX[ITDR[ideb:ifin]+long1],ITDR[ideb:ifin],long1)						
        ITDR[ideb:ifin]<-res.ORD.EXT$ADR						
        ltrou<-FALSE						
        res.proch<-PROCHE(ideb,ifin,ITEX,ITDR,ITRE,nfreq,nfreq2,nfreq3,long,nxlon,nbseg)						
        ifin<-res.proch$ifin						
        ltrou<-res.proch$ltrou						
        ltrouseg<-res.proch$ltrouseg						
        if (ltrouseg)						
        {						
          nbseg<-res.proch$list.segment[[6]]						
          list.segment<-res.proch$list.segment						
          list.tot.segment[[nbseg]]<-list.segment 						
        }         						
      }				
      ltrou<-FALSE						
      while (!ltrou & (long>=1) & (ifin<Nplus))						
      {						
        ideb=ifin+1						
        ifin=lpil[long]						
        while (  ( (ideb+nfreq)>ifin) & (long > 1) )						
        {						
          ideb=ifin+1						
          long<-long-1						
          ifin=lpil[long]						
        }						
        
        if (long>=1)						
        {						
          ltrou<-FALSE						
          res.proch<-PROCHE(ideb,ifin,ITEX,ITDR,ITRE,nfreq,nfreq2,nfreq3,long,nxlon,nbseg)						
          ifin<-res.proch$ifin						
          ltrou<-res.proch$ltrou						
          ltrouseg<-res.proch$ltrouseg						
          if (ltrouseg)						
          {						
            nbseg<-res.proch$list.segment[[6]]						
            list.segment<-res.proch$list.segment						
            list.tot.segment[[nbseg]]<-list.segment						
          }                						
        }     						
      }						
    }   						
    # all the segments have been detected and the doc_segments (tab.seg) will be created						
    
    tab.seg<-matrix(0,nrow=nrep,ncol=nbseg)						
    rownames(tab.seg)<-rownames(dtm$DocTerm)						
    if (nbseg==0) cat("\nNo segments fullfil the conditions\n")						
    
    if (nbseg>0)						
    {						
      for (iseg in 1:nbseg)						
      {						
        list.segment<-list.tot.segment[[iseg]]						
        mfreq<-list.segment[[2]]						
        long.seg<-list.segment[[5]]						
        nseg<-list.segment[[6]]						
        for (i in 1:mfreq) 						
        {						
          rep<-list.segment[[4]][i]						
          tab.seg[rep,nseg]<-tab.seg[rep,nseg]+1						
        }						
      }						
      row.names(tab.seg)<-row.names(dtm$DocTerm$dimnames$Docs)						
      nom.col<-vector()						
      for (iseg in 1:nbseg) nom.col[iseg]<-(list.tot.segment[[iseg]]$text)						
      colnames(tab.seg)<-nom.col						
    }						
    
    impri.segment<-data.frame(ncol=3)						
    # Segment list in alphabetical ordre						
    for (iseg in 1:nbseg) 						
    {						
      impri.segment[iseg,1]<-list.tot.segment[[iseg]]$text						
      impri.segment[iseg,2]<-list.tot.segment[[iseg]]$frequency						
      impri.segment[iseg,3]<-list.tot.segment[[iseg]]$length						
    }						
    colnames(impri.segment)<-c("segment","frequency","long")						
    segOrderFreq<-with(impri.segment,impri.segment[order(frequency,long,decreasing=TRUE),])						
    segOrderlist<-impri.segment						
    Index.segments<-list(segOrderFreq=segOrderFreq, segOrderlist=segOrderlist)						
    namesSeg<-colnames(tab.seg)						
    numSeg<-rep(1:ncol(tab.seg),1)						
    colnames(tab.seg) = paste(numSeg, namesSeg, sep=":")						
    rownames(tab.seg)<-rownames(dtm$DocTerm)						
  }  # Final segments						
  
 
  #--------- Remove the numbers  ------------------						
  # To Detect if the colname is a number and remov.number=TRUE we must remove the column						
  if(remov.number == TRUE) {						
    sel.words <- dtm$dimnames$Terms[suppressWarnings(is.na(as.numeric(dtm$dimnames$Terms)))]						
    sel.words <- which(dtm$dimnames$Terms%in%sel.words)						
    if(length(sel.words)>0){ 						
      dtm <- selectFunc(dtm,sel.words)						
      Nfreqword <- Nfreqword[sel.words]}}						
  
  #--------- Removing words with low length lminword ------------------						
  if (lminword > 1) {
    #  sel.words <- which(nchar(dtm$dimnames$Terms) > (lminword-1)) 										
    sel.words <- which(stringi::stri_length(dtm$dimnames$Terms) > (lminword-1)) 
    if(length(sel.words)>0){						
      dtm <- selectFunc(dtm,sel.words)						
      Nfreqword <- Nfreqword[sel.words] }}						

  
  #--------- Removing words with low frequency "Fmin" times ------------------						
  if (Fmin > 1) {						
    sel.words <- which(Nfreqword > (Fmin-1)) 						
    if(length(sel.words)>0){						
      dtm <- selectFunc(dtm,sel.words)						
      Nfreqword <- Nfreqword[sel.words] }}						

  

  #--------- Selecting words appearing with a minimum frequency of "Fmin" times 						
  #--------- in a minimum of "Dmin" documents											
  Ndocword <-tapply(dtm$v>0,dtm$j,sum)						
  if(Fmin>1 | Dmin>1) {						
    sel.words <- which(Nfreqword >= Fmin & Ndocword >= Dmin)		
      if(length(sel.words)>0) dtm <- selectFunc(dtm,sel.words)						
  }						
  Nfreqword<-tapply(dtm$v,dtm$j,sum)											


  
  #--------- Removing words appearing a maximum frecuency of Fmax times						
  if(!is.null(Fmax)) {						
    sel.words <- which(Nfreqword < (Fmax+1))						
    if(length(sel.words)>0) {						
      dtm <- selectFunc(dtm,sel.words)						
      Nfreqword <- Nfreqword[sel.words]}}
    
  #--------- Removing stopwords tm (defined in tm package)only if not previously removed						
  if(stop.word.tm==TRUE){						
    stop.word <- tm::stopwords(idiom)	

    sel.words <- which(!dtm$dimnames$Terms%in%stop.word)	

    if(length(sel.words)>0) {						
      dtm <- selectFunc(dtm,sel.words)						
      Nfreqword <- Nfreqword[sel.words]}}						
  
  #--------- Removing user stopwords						
  if(!is.null(stop.word.user)) {						
    if(is.data.frame(stop.word.user)) stop.word.user <- t(stop.word.user)						
    stop.word.user <- stop.word.user[order(stop.word.user)] 						
    sel.words <- which(!dtm$dimnames$Terms%in%stop.word.user)						
    if(length(sel.words)>0){						
      dtm <- selectFunc(dtm,sel.words)						
      Nfreqword <- Nfreqword[sel.words]}}										
  
  docsbefore <- dtm$dimnames$Docs
  docsafter <- docsbefore[unique(dtm$i)]
  dseldoc <- which(docsbefore %in% docsafter) 
  remov.docs <- docsbefore[-dseldoc]
  
  


  # ---------------- If aggregation ---------				
  if(!is.null(var.agg)){				
    agg <- data.frame(base[dtm$i,var.agg], dtm$j,dtm$v)				
    agg <- aggregate(agg[,3], by=list(agg[,1], agg[,2]),FUN=sum, na.rm=TRUE)				
    agg <- agg[order(agg[,1],agg[,2]),]				
    dtmagg <- dtm				
    
    
    # Changed 10/Jan/2019. When aggregated documents are wide, showed
    #  agg[,1] <- droplevels(agg[,1])				
    dtmagg$nrow <- length(levels(agg[,1]))				
    dtmagg$i <- as.numeric(agg[,1])				
    dtmagg$j <- agg[,2]				
    dtmagg$v <- agg[,3]				
    dtmagg$dimnames$Docs <- levels(agg[,1])				
    detOccAgg <- occurrFunc(dtmagg, "after",detOccAgg, TRUE) 	
  } else {  detOcc <- occurrFunc(dtm, "after",detOcc, FALSE)}				
 
  
  
  #--------- If there is aggregation with supplementary variables			
  if (!is.null(var.agg)) {  			
    qualivar <- NULL; qualitable <- NULL; qualincat <-NULL			
    T2 <- as.matrix(dtm)			
    if(!is.null(context.quali))			
      for (i in 1:length(context.quali)) {			
        dis.X <- tab.disjonctif(base[,context.quali[i]])			
        T1 <- t(T2)%*% dis.X			
        sumcateg <- which(colSums(T1)==0)			
        if (length(sumcateg)>0) T1 <- T1[,-sumcateg]			
        acpos <- ncol(T1)			
        qualitable <- cbind(qualitable, T1)			
        qualivar <- rbind(qualivar,context.quali[i])			
        qualincat <- rbind(qualincat,acpos)			
      }			
    # Numerical variables		
    nnum <- ncol(data.context.quanti)		
    quantivar <- NULL; qcolname <- NULL		
    if(!is.null(nnum)) {		
      for (i in 1:nnum) {		
        qcolname <- c(qcolname,colnames(data.context.quanti[i]))	
        if(any(is.na(data.context.quanti[,i])))
          warning("\n", colnames(data.context.quanti[i]), 
                  " variable has missing values.\n They will be replaced by the mean of the category\n
         \n Consider to use missMDA R package") 
        qcateg <- aggregate(data.context.quanti[,i], by=list(base[,var.agg]), FUN=mean, na.rm=TRUE)	
#        no.empty.documents <- rowSums(as.matrix(dtm))
#        return(no.empty.documents)
        
        qcateg <- aggregate(data.context.quanti[,i], by=list(base[,var.agg]), FUN=mean, na.rm=TRUE)		
        
        acpos <- which(qcateg[,1]%in% dtmagg$dimnames$Docs)		
        qcateg <- qcateg[acpos,]		
        acpos <- qcateg[,1]		
        qcateg <- data.frame(qcateg[,-1])		
        rownames(qcateg) <- acpos ; colnames(qcateg) <- colnames(data.context.quanti[i])		
        quantivar <- cbind(quantivar,qcateg[,1])		
        rownames(quantivar) <- acpos 		
        colnames(quantivar) <- qcolname		
      }}		
    dtm <- dtmagg 		
  } # Final aggregation		
  
  
  # -------------  Computing final frequencies and tables				
  Nfreqword<-tapply(dtm$v,dtm$j,sum)				
  Ndocword<-tapply(dtm$v>0,dtm$j,sum)				
  Table <- cbind(Nfreqword,Ndocword)				
  rownames(Table) <- dtm$dimnames$Terms				
  colnames(Table) <- c("Frequency", "N.Documents")				
  TFreq <- Table[order(Nfreqword, Ndocword, decreasing = TRUE), ]					
  
  if(is.null(var.agg)) {
    rownamesdocs.no.empty.seg <- rownamesdocs.no.empty
    dsel <- unique(dtm$i)
    dtm$nrow <- length(dsel)	
    rownamesdocs.no.empty <- rownames(base)[dsel]
    dsel1 <- which(dtm$dimnames$Docs %in% rownamesdocs.no.empty) 
    dtm$dimnames$Docs <- dtm$dimnames$Docs[dsel1]			
    dtm$i<- as.numeric(factor(dtm$i,labels=c(1:length(dtm$dimnames$Docs))))
    if(segment==TRUE) {
      tcoln <-   colnames(tab.seg)
      tab.seg <- data.frame(tab.seg[dsel,,drop=FALSE])
      rownames(tab.seg) <- dtm$dimnames$Docs
      colnames(tab.seg) <- tcoln
    }
  } 
  
 
  
  if(!is.null(var.agg)) { 
    # Remove words in SourceTerm supressed in DocTerm
    wordsafteragg <- dtm$dimnames$Terms
    sel.words <- which(SourceTerm$dimnames$Terms%in%wordsafteragg)	
    if(length(sel.words)>0) SourceTerm <- selectFunc(SourceTerm,sel.words)	
    SourceTerm$j <- as.integer(SourceTerm$j)
    SourceTerm$i <- as.integer(SourceTerm$i)
    sumwdocs <- slam::row_sums(SourceTerm)
    pos.noempty <- which(sumwdocs>0)
    SourceTerm$nrow <- length(pos.noempty)
    SourceTerm$dimnames$Docs <- SourceTerm$dimnames$Docs[pos.noempty]	
    dsel <- which(SourceTerm$dimnames$Docs %in% rownamesdocs.no.empty) 		
    SourceTerm$dimnames$Docs <- SourceTerm$dimnames$Docs[dsel]		
    SourceTerm$i<- as.numeric(factor(SourceTerm$i,labels=c(1:length(SourceTerm$dimnames$Docs))))		
    dfvaragg <- dfvaragg[rownamesdocs.no.empty,,drop=FALSE]
    SourceTerm.freq <- Docs.before[Docs.before$DocName %in% rownames(SourceTerm),]
  }
  
  
  
  if(!is.null(var.agg)) { 	
    if(!is.null(qualincat)){	
      qualincat <- data.frame(qualincat, row.names=NULL)	
      qualivar <- data.frame(qualivar)
      rownames(qualincat) <- rownames(qualivar)		
      qualivar <- cbind(qualivar, qualincat)
      coltmp <- colnames(qualitable) 
      qualitable <- t(qualitable)
      rownames(qualitable) <- coltmp	
    } else {qualitable <- NULL; qualivar <- NULL}
    quali <- list(qualitable=qualitable, qualivar=qualivar)	
    context <- list(quali=quali ,quanti=quantivar)		
  } else {	
    context <- list(quali=data.frame(base[,context.quali,drop=FALSE]), quanti=data.context.quanti)		
  }		
  
  
  
  
  #--------- Compute results for the total of docs  ------------------	
  # detOcc has the results for not aggregated documments)
  seqDoc <- c(N, sum(detOcc[,2]), wordsafter, round(sum(detOcc[,2])/N,2))				
  rwnDoc <- c("Documents","Occurrences","Words","Mean-length")
  
  if(!is.null(var.agg)) detOcc <- detOccAgg
  
  
  detOcc$PctLength.after <- 100*detOcc[,"Occurrences.after"]/sum(detOcc[,"Occurrences.after"])				

  if(is.null(var.agg)){				
    detOcc$MeanLength100.after <- round(N*100*detOcc[,"Occurrences.after"]/sum(detOcc[,"Occurrences.after"]),2)				
  } else {				
    detOcc$MeanLength100.after <- round((100*detOcc[,"Occurrences.after"]/detOcc[,"NumberDocs"])				
                                        /(sum(detOcc[,"Occurrences.after"])/sum(detOcc[,"NumberDocs"])),2)				
  }				
  detOcc$PctLength.after <- round(detOcc$PctLength.after,2)				
  
  
  # ------------------------   Print summary for Tfreqdoc	
  seqDocAf <- c(dtm$nrow, sum(detOcc[,"Occurrences.after"]), dtm$ncol, round(sum(detOcc[,"Occurrences.after"])
                                                                             /dtm$nrow,2))

  ndocAft <- length(unique(dtm$i))
  ndocFEmpty <- ndocIni - ndocAft		
  if(ndocFEmpty>0) {		
    rwnDoc <-c(rwnDoc, "NonEmpty.Docs", "NonEmpty.Mean-length")		
    seqDoc <- c(seqDoc, (ndocIni-ndocIniEmpty),round(sum(detOcc[,"Occurrences.before"])/(ndocIni-ndocIniEmpty),2))		
    seqDocAf <- c(seqDocAf, ndocAft,round(sum(detOcc[,"Occurrences.after"])/(ndocAft),2))		
  }

  seqDoc <- c(seqDoc, seqDocAf) 		
  mTfreqdoc <- matrix(seqDoc, ncol=2, byrow=FALSE)		
  rownames(mTfreqdoc) <- rwnDoc		
  colnames(mTfreqdoc) <- c("Before", "After")	
  info <- infoNew()		

  attr(dtm, "language") <- info$idiom[[1]] # Version 1.3.1
  
  df.TFreq <- as.data.frame(TFreq)
  df.TFreq$rownames <- rownames(df.TFreq)
  df.TFreq <- df.TFreq[order(-df.TFreq[,1], df.TFreq[,3]), ]
  df.TFreq$rownames <- NULL
  y <- list(summGen=mTfreqdoc,summDoc=detOcc, indexW = as.matrix(df.TFreq), DocTerm =dtm) 
  
  if(segment==TRUE) {
    y$indexS <- Index.segments
    y$DocSeg <- slam::as.simple_triplet_matrix(tab.seg)
  }
  
  y$info <- info
  
  if(!is.null(var.agg)) { 
    y$SourceTerm <- SourceTerm 
    y$SourceTerm.freq <- SourceTerm.freq
    y$SourceTerm.var.agg <- base[,var.agg,drop=FALSE]
    y$SourceTerm.docs.no.empty <- rownamesdocs.no.empty
    y$SourceTerm.dtm <- SourceTerm.seg
    if(!is.null(context.quali)) y$SourceTerm.qual <- base[,context.quali,drop=FALSE]
    if(!is.null(context.quanti)) y$SourceTerm.quant <- base[,context.quanti,drop=FALSE]
    y$var.agg <- var.agg.seg[rownames(y$SourceTerm),,drop=FALSE]
    if(!is.null(context$quali)) y$context$quali <- context$quali
    if(!is.null(context$quanti)) y$context$quanti <- context$quanti
  }
  y$remov.docs <- remov.docs
  
  if(is.null(var.agg)) { 
    if(!is.null(context$quanti))  {
      y$SourceTerm.quant <- base[,context.quanti,drop=FALSE]
      y$SourceTerm.docs.no.empty <- rownamesdocs.no.empty.seg
      if(length(remov.docs)==0) {y$context$quanti <- context$quanti}
      else {
        pos <- which(rownames(context$quanti) %in% remov.docs)
        y$context$quanti <- context$quanti[-pos,,drop=FALSE] 
      } 
    } # Final !is.null(context$quanti)
    
    
    if(!is.null(context$quali))  {
      y$SourceTerm.qual <- base[,context.quali,drop=FALSE]
      if(length(remov.docs)==0) {y$context$quali <- context$quali}
      else {
        pos <- which(rownames(context$quali) %in% remov.docs)
        y$context$quali <- context$quali[-pos,,drop=FALSE] 
      } 
      i <- sapply(y$context$quali, is.character)
      if(length(i)>0) y$context$quali[i] <- lapply(y$context$quali[i], as.factor) # version 1.3.1
      
    } # Final !is.null(context$quali)  
  } # Final if(is.null(var.agg))
  
  
  df <- y$summDoc[,2,drop=FALSE]
  rownames(df) <- y$summDoc[,1]
  y$rowINIT <- df
  
  class(y) <- c("TextData", "list")
  if(blongErr==TRUE) warning("Only repeated segments < 20 words have been computed")	

 # V Cramer DTM
  Chi.sq <- suppressWarnings(chisq.test(as.matrix(y$DocTerm), correct = FALSE)$statistic)
  Phi <- Chi.sq/sum(y$DocTerm)
  dimDTM <- dim(y$DocTerm)
  y$VCr <- as.numeric(sqrt(Phi/min(dimDTM[1] - 1, dimDTM[2] - 1)))
  
  # Inertia of DTM
  I.A <- y$DocTerm/sum(y$DocTerm)
  Inertia <-  sum((as.matrix(I.A) - as.matrix(rowSums(as.matrix(I.A))) %*% 
                   t(as.matrix(colSums(as.matrix(I.A)))))^2)
  y$Inertia <- Inertia
  if(graph==TRUE) plotTextData() 
  return(y)
}

#' @importFrom methods hasArg
#' @importFrom graphics barplot
#' @export
plot.LexCA <- function(x, selDoc="ALL", selWord="ALL", selSeg=NULL,
  selDocSup=NULL, selWordSup=NULL, quanti.sup=NULL, quali.sup=NULL, maxDocs=20,  
  eigen=FALSE, title=NULL, axes=c(1,2), col.doc="blue", col.word="red",
  col.doc.sup="darkblue", col.word.sup="darkred", col.quanti.sup = "blue",
  col.quali.sup="darkgreen", col.seg="cyan4",col="grey", cex=1, 
  xlim=NULL, ylim=NULL, shadowtext=FALSE, habillage="none", unselect=1,
  label="ALL", autoLab=c("auto", "yes", "no"), new.plot=TRUE, 
  graph.type = c("classic", "ggplot"),...) 
{
  
 if (!inherits(x, "LexCA"))  stop("x object should be LexCA class")
  options(stringsAsFactors = FALSE)
  if(is.null(label)) label <- "none"
  if(label=="ALL") label<-"all"
  if(length(graph.type)>1) graph.type <- "classic"
  
  if(length(selDoc)==1)  if(selDoc==0) selDoc <- NULL 
  if(length(selWord)==1)  if(selWord==0) selWord <- NULL  
  
  

if(eigen==TRUE) {selDoc<-selWord<-NULL}
if(!is.null(quali.sup)) {
 if(!is.null(x$var.agg)) { 
 if(length(quali.sup)==1) if(quali.sup=="ALL") 
 quali.sup <- as.character(x$info$catquali$qualivar[,1])
 df1 <- data.frame(x$info$catquali$qualivar)
 df1$inic <- cumsum(df1$qualincat) 
 df1$inic <- df1$inic - df1$qualincat+1 
 posic <- which(df1[,1] %in% quali.sup)
 df2 <- df1[posic,]  
 if(nrow(df2)>0)
  for(i in 1:nrow(df2)) {
  quali.sup <- c(quali.sup,dimnames(x$info$catquali$qualitable)[[1]][df2$inic[i]: (df2[i,"inic"]+df2[i,"qualincat"]-1)])
  if(length(quali.sup)==0) quali.sup <- NULL}
} else {
# Not aggregate
 if(length(quali.sup)==1) if(quali.sup=="ALL") 
  quali.sup <- as.character(rownames(x$quali.sup$coord))
  namevar <- colnames(x$info$catquali)
  posic <- which(namevar %in% quali.sup)
  if(length(posic)>0) {  # There are variables
    temp1 <- sapply(x$info$catquali, levels)[posic]
    temp2 <- unlist(temp1, use.names=FALSE)
    temp3 <- rep(names(temp1), lengths(temp1))
    temp3 <- paste0(temp3,".",temp2)
    quali.sup <- c(quali.sup, unlist(temp1),unlist(temp3))
    posic <- which(rownames(x$quali.sup$coord) %in% quali.sup)
    quali.sup <- rownames(x$quali.sup$coord)[posic]}
} # Final not aggregate
}

selection <- function(sel1, xobj, bType, axx, axy)
{
if(is.null(sel1)) return(sel1)
 xx <- ""
  if(length(sel1)==1){

     if(sel1=="ALL") sel1 <- c(1:dim(xobj$coord)[1])
  }
  if(length(sel1)==1) {
   if(sel1=="meta") sel1 <- "meta 3"
   if(sel1=="char") sel1 <- "char 0.05"
   xx <- gregexpr(pattern =' ',sel1)[[1]][1]-1
   xx <- substr(sel1, 1, xx)
  }
 

if(xx=="coord" | xx=="cos2" | xx=="contrib" | xx=="meta" | xx=="char") 
 { 
 nc <- nchar(sel1)
   if(xx=="coord") {
 # Selection by coordinates"
 sel1 <- as.numeric(substr(sel1, 7, nc))
 # dft Coordinates for elements x two selected dimensions
 dft <- data.frame(xobj$coord[,c(axx,axy),drop=FALSE])
 # fval maximum value of element from two dimensions
 fval <- apply(dft, 1, function(z) max(abs(z)))
 ordmax <- rank(fval)
 posic <- which(ordmax > (length(fval)-sel1))
 sel1 <- rownames(xobj$coord)[posic]
  }
   if(xx=="cos2") {
 # Selection by cos2
 sel1 <- as.numeric(substr(sel1, 5, nc))
 dft <- data.frame(xobj$cos2[,c(axx,axy),drop=FALSE])
 fval <- apply(dft, 1, function(z) sum(z))
 posic <- which(fval>= sel1)
 sel1 <- rownames(xobj$cos2)[posic]
   }
   if(xx=="contrib") {
 # Selection by contrib
 if(bType=="Seg") stop("Segments can not be selected by contribution")
 if(bType=="Quali") stop("Contextual categorical variables can not be selected by contribution")
 if(bType=="Quanti") stop("Contextual quantitative variables can not be selected by contribution")
 if(bType=="Dsup") stop("Supplementary documents can not be selected by contribution")
 if(bType=="Wsup") stop("Supplementary words can not be selected by contribution")
 sel1 <- as.numeric(substr(sel1, 9, nc))
 dft <- data.frame(xobj$contrib[,c(axx,axy),drop=FALSE])
 fval <- apply(dft, 1, function(z) max(abs(z)))
 posic <- which(fval>= sel1)
 sel1 <- rownames(xobj$contrib)[posic]
  }
 if(xx=="meta") {
 if(bType=="Seg") stop("Segments can not be selected by meta")
 if(bType=="Quali") stop("Contextual categorical variables can not be selected by meta")
 if(bType=="Quanti") stop("Contextual quantitative variables can not be selected by meta")
 if(bType=="Dsup") stop("Supplementary documents can not be selected by meta")
 if(bType=="Wsup") stop("Supplementary words can not be selected by meta")
 sel1 <- as.numeric(substr(sel1, 5, nc))
 sMeta <-  rownames(xobj$coord)[which(xobj$contrib[,axx]>mean(xobj$contrib[,axx])*sel1)]
 sMeta <-  c(sMeta,rownames(xobj$coord)[which(xobj$contrib[,axy]>mean(xobj$contrib[,axy])*sel1)])
 sel1 <- unique(sMeta)
}
 if(xx=="char") {
 if(bType!="Word") stop("char only can be used to select active words (selWord)")
  proba <- as.numeric(substr(sel1, 5, nc))
 if(proba<0|proba>1) stop("proba should be between 0 and 1")
 if(is.null(selDoc)) stop("selDoc can not be NULL when words are selected by char")
 ntdoc <- nrow(x$call$X) 
 if(ntdoc>maxDocs) stop("Computing the characteristic words when more than ", maxDocs, " documents is not allowed")
 resCharWord <- descfreq(x$call$X,proba=proba)
 sW <- NULL
 for (idoc in 1:ntdoc)
    {
 if(rownames(x$row$coord)[idoc] %in% selDoc){
     df <- data.frame(resCharWord[idoc])
     df <-rownames(df[df[6]>0,])
     sW <- c(sW,df)}
     }
 sel1 <- unique(sW)
 }
 } else {
 if(is.character(sel1)) sel1 <- which(rownames(xobj$coord) %in% sel1)
 sel1 <- rownames(xobj$coord)[sel1]
 sel1  <- sel1[!is.na(sel1)]
}
  return(sel1)
}
# Final functions




if(eigen) {

 if(is.null(title)) titleE <- "Eigenvalues" else titleE <- title
  args <- list(...)
  exist <- "names.arg" %in% names(args)

 if(hasArg(names.arg)) {
     valarg <- args$names.arg
     if(length(valarg)==1) {
      if(valarg=="") names.arg <- rep(valarg,nrow(x$eig))
      else names.arg <- paste0(valarg,1:nrow(x$eig))
       }
          else names.arg <- args$names.arg
 } else  names.arg <- paste("dim",1:nrow(x$eig))

  

 if(new.plot & graph.type!="interact") dev.new()
 if(graph.type == "classic") { 
   barplot(x$eig[, 1], main = titleE, col=col, names.arg = names.arg) 
      }  else  {   # No es classic
       y <- x$eig[,1]
       x1 <- names.arg 
       df <- data.frame(x1,y)
      # print(str(df))
      # return(df)
     # df <- data.frame(x=names.arg, y=x$eig[, 1])
      df$x1 <- factor(df$x1,levels=unique(df$x1))
      bplot<- ggplot(data=df, aes(x=x1, y=y)) + ggtitle(titleE)+ 
        labs(x = NULL)+ labs(y = NULL)+
        geom_bar(stat="identity", width=0.5, fill=col)+  
        theme_minimal()+
        theme(plot.title = element_text(hjust = 0.5))
      if(graph.type=="ggplot") {print(bplot); return(bplot)}

   #   if(graph.type == "interact") {
       my.text <-  paste(rownames(x$eig), "<br>", "Eigenvalue",round(x$eig[,1],4),
                    "<br>", "Pct.Variance", round(x$eig[,2],4),
                    "<br>", "AcumPct.Variance", round(x$eig[,3],4))
       bplot <- plotly::plotly_build(bplot)
       bplot$x$data[[1]]$text <- my.text
       
       fplot <- function(x) {
        #  default_opts <- callr::r(function(){options()}); options(default_opts)
         oo.viewer <- getOption("viewer")
         on.exit({ # print("Restoring viewer...")
           Sys.sleep(0)
           options(viewer = oo.viewer)
         }, add = TRUE)
         options(viewer = NULL)
         print(x)
       }
       if(new.plot) fplot(bplot) else print(bplot)
return(bplot)
  }

  
 stemp <- c(selDoc, selWord,selSeg,selDocSup,selWordSup,quanti.sup,quali.sup)
 stemp <- unique(stemp)
 if(!is.null(stemp)) {
   cat("When eigen=TRUE other elements cannot be plotted at the same time")
 return("")
                     }
} else {
invisib <- c("quali.sup")


if(!is.null(quanti.sup))
 {
  if (is.null(x$quanti.sup)) 
    stop("No quantitative supplementary variables in LexCA")
  qsn <- rownames(x$quanti.sup$coord)
  
  if(is.numeric(quanti.sup))   quanti.sup <-  na.omit(qsn[quanti.sup])
  sel1 <- selection(quanti.sup,x$quanti.sup,"Quanti",axes[1],axes[2])
 #  sel1  <- which(sel1 %in% qsn)  Changed version  1.5.4
  sel1  <-  which(qsn %in% sel1)

  
if(length(sel1)==0)
    stop("No quantitative supplementary variables in plot.LexCA")
 
objqs <- x$quanti.sup
drow <- unlist(dimnames(objqs[[1]])[1])
dcol <- unlist(dimnames(objqs[[1]])[2])
coord.qs <- matrix(x$quanti.sup[[1]],length(drow),length(dcol))
rownames(coord.qs) <- drow
colnames(coord.qs) <- dcol


coord.qs <- coord.qs[sel1,,drop=FALSE]
x$quanti.sup$coord <- coord.qs
 if(is.null(title)) titleS <- "Supplementary quantitative variables on the CA map"
 else titleS <- title


 res.quanti <-  FactoMineR::plot.CA(x, axes=axes, choix = c("quanti.sup"), col.quanti.sup =col.quanti.sup, title= titleS, 
         cex=cex,graph.type =graph.type)
 stemp <- c(selDoc, selWord,selSeg,selDocSup,selWordSup,quali.sup)

 stemp <- unique(stemp)
 if(!is.null(stemp)) {
   cat("When quanti.sup is not NULL other elements cannot be plotted at the same time")
 return("")
 }
 return(res.quanti)
} else {

if(!is.null(selDoc)) { 
 selDoc <- selection(selDoc,x$row,"Doc",axes[1],axes[2])
 if(length(selDoc)==0) selDoc <- NULL 
}

if(!is.null(selWord)) {
 selWord <- selection(selWord, x$col,"Word", axes[1],axes[2])
 if(length(selWord)==0) selWord <- NULL 
}


#====================================
if(!is.null(quali.sup)) {
  if(!is.null(x$quali.sup)) 
    quali.sup <- selection(quali.sup,x$quali.sup,"Quali", axes[1],axes[2])
    else stop("No categorical supplementary variables in LexCA object")}
if(!is.null(quali.sup)) {
  if(!is.null(x$var.agg)) 
   x$quali.sup$coord <- data.frame(x$quali.sup$coord[,,drop=FALSE])
  posic <- which(rownames(x$quali.sup$coord) %in% quali.sup)
   x$quali.sup$coord <- x$quali.sup$coord[posic,,drop=FALSE]
   x$quali.sup$coord <- x$quali.sup$coord[apply(!is.na(x$quali.sup$coord), 1, any),,drop=FALSE ] 

}

# Supplementary documents
if(!is.null(selDocSup)) 
  if(!is.null(x$row.sup)) 
    {selDocSup <- selection(selDocSup,x$row.sup,"Dsup", axes[1],axes[2])
     if(length(selDocSup)==0) selDocSup <- NULL
    } else {stop("No supplementary documents in LexCA object")}

# Qualitative variables
if(!is.null(quali.sup)) 
if(!is.null(x$quali.sup$coord)) {
 rdo <- which(rownames(x$quali.sup$coord) %in% rownames(x$row$coord))
 if(length(rdo)>0) rownames(x$quali.sup$coord)[rdo] <- paste0("_",rownames(x$quali.sup$coord)[rdo])

 if(!is.null(x$row.sup$coord)) {
 rdo <- which(rownames(x$row.sup$coord) %in% rownames(x$row$coord))
 if(length(rdo)>0) rownames(x$row.sup$coord)[rdo] <- paste0("_",rownames(x$row.sup$coord)[rdo])}
 ifelse(!is.null(x$row.sup), nrowsup <- nrow(x$row.sup$coord),nrowsup <-0)

if(is.null(selDocSup)){ 
    x$row.sup$coord <- x$quali.sup$coord
     selDocSup <- rownames(x$quali.sup$coord)
    col.doc.sup <- col.quali.sup
  } else {
 colnames(x$quali.sup$coord) <- colnames(x$row$coord)
 x$row.sup$coord <- rbind(x$row.sup$coord, x$quali.sup$coord)
 selDocSup <- c(selDocSup, rownames(x$quali.sup$coord))
 col.doc.sup <- c(rep(col.doc.sup,nrowsup), rep(col.quali.sup,nrow(x$quali.sup$coord)))
}

 if(length(selDocSup)==0) selDocSup <- NULL
}
if(is.null(selDocSup)) invisib <- c(invisib,"row.sup")
else selDoc <- c(selDoc,selDocSup)


#============================================================
# Supplementary Segments & words
#====================================
nsegm <-0; nwordsup <-0
 if(!is.null(x$segment$coord)) nsegm <- nrow(x$segment$coord)
 if(!is.null(x$col.sup$coord)) nwordsup <- (nrow(x$col.sup$coord))-nsegm


if(!is.null(selSeg)) {
  if(nsegm==0) stop("No segments in LexCA object")
   selSeg <- selection(selSeg,x$segment,"Seg", axes[1],axes[2])
   nsegm <- length(selSeg)
   if(nsegm==0) selSeg <- NULL } else {nsegm<-0}


# Supplementary words
if(!is.null(selWordSup)) {
  if(nwordsup==0) stop("No supplementary words in LexCA object")
    tmp <- x$col.sup
    tmp$coord <- tmp$coord[1:nwordsup,,drop=FALSE]
    tmp$cos2 <- tmp$cos2[1:nwordsup,,drop=FALSE]
    selWordSup <- selection(selWordSup,tmp,"Wsup", axes[1],axes[2])
    nwordsup <- length(selWordSup)
    if(nwordsup==0) selWordSup <- NULL} else {nwordsup<-0} 

if(!is.null(x$col.sup$coord)) x$col.sup$coord <- x$col.sup$coord[selWordSup,,drop=FALSE]
if(!is.null(x$segment$coord)) x$segment$coord <- x$segment$coord[selSeg,,drop=FALSE]

if(nsegm>0) {
 if(nwordsup==0) x$col.sup$coord <- x$segment$coord 
  else x$col.sup$coord <- rbind(x$col.sup$coord,x$segment$coord)
}

 col.word.sup <- c(rep(col.word.sup,nwordsup), rep(col.seg,nsegm))

 if(length(col.word.sup)==0) { col.sup <- NULL; invisib <- c(invisib,"col.sup")}
 invisib <- c(invisib,"quanti.sup")
 if(is.null(selDoc))  invisib <- c(invisib,"row")
 if(is.null(selWord))  invisib <- c(invisib,"col")

if(!is.null(x$row.sup$coord)) colnames(x$row.sup$coord) <- colnames(x$row$coord)
if(!is.null(selWord)){
 if(!is.null(selWordSup)) selWord <- c(selWord, selWordSup)
 if(!is.null(selSeg)) selWord <- c(selWord, selSeg)
}


invisib <- unique(invisib)
if(length(invisib)==6) stop("No selected elements to plot")


if(eigen!=TRUE)
if(new.plot==TRUE){if(dev.interactive()) dev.new()}
 plot.CA(x, axes=axes, invisible=invisib,
    choix=c("CA"), title= title, cex=cex, selectCol=selWord, selectRow=selDoc, 
    xlim=xlim, ylim=ylim, shadowtext=shadowtext,habillage=habillage, unselect=unselect,
    autoLab=autoLab, col.row=col.doc, col.row.sup=col.doc.sup, label=label, 
    col.col=col.word, col.col.sup=col.word.sup, graph.type =graph.type)
}}}

#' @export
print.LexCA <- function (x, file = NULL, sep = ";", ...) 
{

  options(stringsAsFactors = FALSE)
sink.reset <- function(){
    for(i in seq_len(sink.number())){
        sink()}}
sink.reset
res.LexCA <- x

	if (!inherits(res.LexCA, "LexCA")) stop("x object should be LexCA class")
sink.reset <- function(){
  for(i in seq_len(sink.number())){sink()}}
sink.reset


	cat("**Results for CA and Aggregate Lexical Table (LexCA)**\n")
    	cat("*The results are available in the following objects:\n\n")
       index <- 12
    	res <- array("", c(index, 2), list(1:index, c("name", "description")))
   	res[1, ] <- c("$eig", "Eigenvalues and % of variance")
   	res[2, ] <- c("$row", "CA results for the active documents/aggregate documents")
   	res[3, ] <- c("$col", "CA results for active words")
   	res[4, ] <- c("$row.sup", "CA results for the supplementary documents/aggregate documents")
   	res[5, ] <- c("$col.sup", "CA results for supplementary words")
   	res[6, ] <- c("$quanti.sup", "CA results for the supplementary continuous variables")
   	res[7, ] <- c("$quali.sup", "CA results for the supplementary categorical variables")
   	res[8, ] <- c("$meta", "list of Keydocs selected documents and Metakeys selected words")
   	res[9, ] <- c("$VCr", "Cramer's V coefficient")
   	res[10, ] <- c("$Inertia", "total inertia")
   	res[11, ] <- c("$segment", "CA results for repeated segments")
   	res[12, ] <- c("$var.agg","name of the aggregate variable used")
 
   	print(res[c(1:index),])
   	
   	if (!is.null(file)) {
 sink(file)

  cat("\nEigenvalues and percentage of explained variance\n")
  out1 <- t(c("",colnames(x$eig)))
  write.table(out1, quote=FALSE, sep = sep,col.names=FALSE, row.names=FALSE)
  x$eig <- format(x$eig, digits=6, scientific = FALSE)
  write.table(x$eig, quote=FALSE, sep = sep, col.names=FALSE)

  
  
  
  if(is.null(x$var.agg)) {
    msg1 <- " for active documents\n" 
    msg4 <- " for supplementary documents\n"
  } else{  
    msg1 <- " for active aggregate documents\n"
    msg4 <- " for supplementary aggregate documents\n"
  }
  msg3 <- " for active words\n"
  cat("\n\nCA results", msg1)
  cat("\nCoordinates ", msg1)
  msg2 <- t(c("",colnames(x$row$coord)))
  write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
  write.table(x$row$coord, quote=FALSE, sep = sep, col.names=FALSE)
  
  
  cat("\nContributions ", msg1)
  write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
  write.table(x$row$contrib, quote=FALSE, sep = sep, col.names=FALSE)
  
  cat("\nSquared Cosinus ", msg1)
  write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
  write.table(x$row$cos2, quote=FALSE, sep = sep, col.names=FALSE)
  
  cat("\nInertia ", msg1)
  x$row$inertia <- data.frame(x$row$inertia)
  rownames(x$row$inertia) <- rownames(x$row$coord) 
  write.table(x$row$inertia, quote=FALSE, sep = sep,row.names=TRUE, col.names=FALSE)
  
  cat("\nCoordinates ", msg3)
  msg2 <- t(c("",colnames(x$col$coord)))
  write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
  write.table(x$col$coord, quote=FALSE, sep = sep, col.names=FALSE)
  
  cat("\nContributions ", msg3)
  write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
  write.table(x$col$contrib, quote=FALSE, sep = sep, col.names=FALSE)
  
  cat("\nSquared Cosinus ", msg3)
  write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
  write.table(x$col$contrib, quote=FALSE, sep = sep, col.names=FALSE)
  
  cat("\nInertia ", msg3)
  x$col$inertia <- data.frame(x$col$inertia)
  rownames(x$col$inertia) <- rownames(x$col$coord) 
  write.table(x$col$inertia, quote=FALSE, sep = sep,row.names=TRUE, col.names=FALSE)
  
  # row.sup
  if(!is.null(x$row.sup)) {
    cat("\nCoordinates of ", msg4)
    write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$row.sup$coord, quote=FALSE, sep = sep, col.names=FALSE)
    
    cat("\nSquared Cosinus ", msg4)
    write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$row.sup$cos2,quote=FALSE, sep = sep, col.names=FALSE)
  }
  
  # col.sup
  if(!is.null(x$col.sup)) {
    ncsup <- nrow(x$col.sup$coord)
    
    if(!is.null(x$segment)) ncsup <- ncsup - nrow(x$segment$coord)
    cat("\nCoordinates of the supplementary words\n")
    write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$col.sup$coord[(1:ncsup),], quote=FALSE, sep = sep, col.names=FALSE)
    
    cat("\nSquared Cosinus of the supplementary words\n")
    write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$col.sup$cos2[1:ncsup,],quote=FALSE, sep = sep, col.names=FALSE)
  }
  
  # quanti.sup
  if(!is.null(x$quanti.sup)){
    cat("\nCoordinates of the supplementary quantitative variables\n")
    write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$quanti.sup$coord, quote=FALSE, sep = sep, col.names=FALSE)
    
    cat("\nSquared Cosinus of quantitative variables\n")
    write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$quanti.sup$cos2,quote=FALSE, sep = sep, col.names=FALSE)
  }
  
  # quali.sup
  if(is.null(x$var.agg)){
    if(!is.null(x$quali.sup)){
      cat("\nCoordinates of the supplementary categories\n")
      write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
      write.table(x$quali.sup$coord, quote=FALSE, sep = sep, col.names=FALSE)
      
      cat("\nv-test of the supplementary categories\n")
      write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
      write.table(x$quali.sup$v.test,quote=FALSE, sep = sep, col.names=FALSE)
      
      cat("\nEta Squared of the supplementary categories\n")
      write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
      write.table(x$quali.sup$eta2,quote=FALSE, sep = sep, col.names=FALSE)
    }}
  
  
  
  # keys
  nEig <- max(x$meta$Doc$Dim, x$meta$Word$Dim)
  
  if(!is.null(x$meta)){
    cat("\nlist of Keydocs documents\n")
    msg5 <- t(c("","Docs", "Dim", "Coordinate"))
    write.table(msg5, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$meta$Doc, quote=FALSE, sep = sep,row.names=TRUE, col.names=FALSE)
    
    cat("\n\nDocuments whose contribution is over ", x$meta$lmd, " times the average document contribution, axis by axis\n")
    for(i in 1:nEig) {
      cat("\nDimension ", i, "+\n")
      cat(x$meta$Doc$Docs[which(x$meta$Doc$Dim==i & x$meta$Doc$Coordinates>0)])
      cat("\nDimension ", i, "-\n")
      cat(x$meta$Doc$Docs[which(x$meta$Doc$Dim==i & x$meta$Doc$Coordinates<0)],"\n")
    }
    
    dfD <- x$meta$Doc
    dfD$Coordinates <- ifelse(dfD$Coordinates>0, as.numeric(dfD$Dim), -1* as.numeric(dfD$Dim))
    TDsign <- table(dfD$Docs, dfD$Coordinates)
    colnames(TDsign) <- paste("DIM_",colnames(TDsign))
    TDsign <- addmargins(TDsign)
    cat("\n\nDocuments whose contribution is over ", x$meta$lmd, " times the average document contribution, axis by axis\n")
    write.table(t(c("",colnames(TDsign))), quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(TDsign, quote=FALSE, sep = sep,row.names=TRUE, col.names=FALSE)
    
    cat("\nlist of Metakeys selected words\n")
    msg6 <- t(c("","Words", "Dim", "Coordinate"))
    write.table(msg6, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$meta$Word, quote=FALSE, sep = sep,row.names=TRUE, col.names=FALSE)
    
    
    cat("\n\nWords whose contribution is over ", x$meta$lmw , " times the average word contribution, axis by axis\n")
    for(i in 1:nEig) {
      cat("\nDimension ", i, "+\n")
      cat(x$meta$Word$Words[which(x$meta$Word$Dim==i & x$meta$Word$Coordinates>0)])
      cat("\nDimension ", i, "-\n")
      cat(x$meta$Word$Words[which(x$meta$Word$Dim==i & x$meta$Word$Coordinates<0)],"\n")
    }
    dfW <- x$meta$Word
    dfW$Coordinates <- ifelse(dfW$Coordinates>0, as.numeric(dfW$Dim), -1* as.numeric(dfW$Dim))
    TDsign <- table(dfW$Words, dfW$Coordinates)
    colnames(TDsign) <- paste("DIM_",colnames(TDsign))
    TDsign <- addmargins(TDsign)
    cat("\n\nWords whose contribution is over ", x$meta$lmw, " times the average word contribution, axis by axis\n")
    write.table(t(c("",colnames(TDsign))), quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(TDsign, quote=FALSE, sep = sep,row.names=TRUE, col.names=FALSE)
  }
  
  # VCr
  cat("\n\nCramer's V coefficient\n")
  cat(x$VCr)
  
  # Inertia
  cat("\n\nInertia\n")
  cat(x$Inertia)
  
  # segment
  if(!is.null(x$segment$coord)){
    cat("\nRepeated segments\n")
    cat("\nCoordinates of the repeated segments\n")
    write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$segment$coord, quote=FALSE, sep = sep, col.names=FALSE)
    
    cat("\nSquared Cosinus of of the repeated segments\n")
    write.table(msg2, quote=FALSE, sep=sep, col.names=FALSE, row.names=FALSE)
    write.table(x$segment$cos2, quote=FALSE, sep = sep, col.names=FALSE)
  }
  
  


 sink()}
       	print(paste("All the results are in file", file))
    }

#' @importFrom graphics abline layout legend locator par plot text
#' @importFrom grDevices palette
#' @importFrom vegan mantel
#' @export


LexCHCca <- function (object, ncp=5, nb.clust=0, min=2, max=NULL, 
                      graph=TRUE, cut.test = FALSE, alpha.test =0.05,
                      description=FALSE, proba=0.05, nb.desc=5, size.desc=80,...) 
{
  x <- object
  
# nb.par is deprecated
  dots <- list(...)
  if(!is.null(dots$nb.par)) {
    warning("nb.par is deprecated, it is replaced by nb.desc")
    nb.desc <- dots$nb.par
  }
  
  old.par <- par(no.readonly = TRUE)

  ######################  New. Modification of x removing no factors used ######################
  ncp.max <- ncol(x$row$coord)
  if(ncp>ncp.max)   {
    warning(paste0("Number of components ncp= ", ncp ," is bigger than number of components in ",
                   deparse(substitute(object)), "= ", ncp.max),"\nncp is changed to ",ncp.max)			
    ncp <- ncp.max  } 

  #----------------  Copy only the number of required factors
  x$row$coord <- x$row$coord[, 1:ncp,drop=FALSE]
  x$row$contrib <- x$row$contrib[, 1:ncp,drop=FALSE]
  x$row$cos2 <- x$row$cos2[, 1:ncp,drop=FALSE]
  x$row$inertia <- x$row$inertia[1:ncp]
  
  x$col$coord <- x$col$coord[, 1:ncp]
  x$col$contrib <- x$col$contrib[, 1:ncp]
  x$col$cos2 <- x$col$cos2[, 1:ncp]
  x$col$inertia <- x$col$inertia[1:ncp]
  
  if(!is.null(x$quanti.sup$coord)) {
    x$quanti.sup$coord <- x$quanti.sup$coord[, 1:ncp]
    x$quanti.sup$cos2 <- x$quanti.sup$cos2[, 1:ncp]   }
  
  
  if(!is.null(x$quali.sup$coord)) {
    x$quali.sup$coord <- x$quali.sup$coord[, 1:ncp]
    x$quali.sup$cos2 <- x$quali.sup$cos2[, 1:ncp] 
    x$quali.sup$v.test <- x$quali.sup$v.test[, 1:ncp] 
    x$quali.sup$eta2 <- x$quali.sup$eta2[, 1:ncp] }
  
  if(!is.null(x$meta$Word)) x$meta$Word <- x$meta$Word[as.numeric(x$meta$Word$Dim)<(ncp+1),]
  if(!is.null(x$meta$Doc)) x$meta$Doc <- x$meta$Doc[as.numeric(x$meta$Doc$Dim)<(ncp+1),]
  #----------------  End Copy only the number of required factors
  ##############################################################################################
  
  # dots <- list(...)
  # if(is.null(dots$method)) method.sel <- "ward"
  # else method.sel <-  dots$method
  # str.type <- c("agnes","diana")
  # if(!type %in% str.type) stop("Argument type must be agnes or diana")
  
  
  ###  <------ PENDIENTE 2 ............................
  # cut.test    FALSE y Legendere test not performed . NULL, percent, median
  if(cut.test == FALSE) cut.stat <- NULL else cut.stat <- "percent"
  
  
  options(warn=0)
  ###  <------ PENDIENTE 3 ............................
  marg.doc<- "after"
  ###  <------ PENDIENTE 4 ............................
  cor.stat <- "pearson"                      # Or spearman, not included in the function
  
  if (!inherits(x,"LexCA")) stop("Object x should be LexCA class")
  options(stringsAsFactors = FALSE)
  
  if(proba<0 | proba >1) stop("No valid values for proba argument")
  
 # Check if Legendre Test is applied
  if(is.null(cut.stat)) bTest <- FALSE
  else {
    if(!is.null(alpha.test)) {
      if(alpha.test<0 | alpha.test >1) stop("No valid values for alpha.test argument")
    }
    bTest<- TRUE
    if(cut.stat!="percent" & cut.stat!="median")
      warning(paste0("Not allowed value ", cut.stat, " for cut.stat argument, changed to 'percent'"))
      cut.stat <- "percent"
  }
  bAddTest <- FALSE


#----------- PHASE A
  ## Type selection number of clusters
  # If Legendre test is applied, the number of clusters is the number obtained by the test.
  if(is.character(nb.clust)) {
    if(nb.clust == "auto") nb.clust <- -1
    if(nb.clust == "click") nb.clust <- 0
  }

  if(nb.clust==0) graph <-TRUE  # Show the graph to manually select the graph
  
  if (nb.clust == 1)  stop("The number of clusters can't be 1. Automatic change: nb.clust= 'auto'")				

  # Define empty variable
  nb.cluster.test <- 0  
  
  
  
  
  
  
  
  if(bTest==TRUE & nb.clust != -1) {
    warning("Only automatic results are allowed using Test in cut.stat.\n nb.clust is changed to 'auto' value")
    nb.clust < -1
  }
  

  
  #xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx 
  # Starting functions
   #---- F1. sum of squares ----
  ssquares <- function(coord, weight_,clust_){ # }, weight_, clust_) {
    dfw <- data.frame(coord,"weight_"=weight_, "clust_"=clust_)
    sum.weights <- sum(dfw$weight_)
    pct.weight <- dfw$weight_/ sum.weights  
    dfclust <- as.data.frame(table(clust_))
    dimen <- ncol(coord)
    mediadim <- apply(dfw[,c(1:dimen)]*pct.weight,2,FUN=sum)
    dfw$sstot_ <- apply(sweep(dfw[,c(1:dimen)],2,mediadim,"-")^2  *pct.weight,1,FUN="sum")
    cl.weight  <- aggregate(pct.weight, by=list(dfw$clust_), FUN=sum)$x
    coord_centers <- aggregate(dfw[,c(1:dimen)]*pct.weight/cl.weight[dfw$clust_],by=list(dfw$clust_), FUN=sum) 
    dfw$ssintra_ <- apply(((dfw[,c(1:dimen)] - coord_centers[clust_, c(2:ncol(coord_centers))])^2),1,sum) *pct.weight
    dfw$ssinter_ <- dfw$sstot_-dfw$ssintra_
    dfw$ssexplain_ <- 100*dfw$ssinter_ / dfw$sstot_ 
    dfclust$weight <- cl.weight*sum.weights 
    dfclust$sstot <- aggregate( dfw$sstot_,by=list(dfw$clust_), FUN=sum)$x
    dfclust$ssintra <- aggregate( dfw$ssintra_,by=list(dfw$clust_), FUN=sum)$x
    dfclust$ssinter <- dfclust$sstot-dfclust$ssintra
    dfclust$ssexplain <- 100*dfclust$ssinter / dfclust$sstot 
    ss.tot <- sum(dfclust$sstot)  
    ss.intra <- sum(dfclust$ssintra)
    ss.inter <- sum(dfclust$ssinter)
    pct.explained <- 100* ss.inter / ss.tot
    ###############################  REVISAR
      colnames(dfclust) <- c("Cluster", "Docs", "Occurrences", "ss.tot", "ss.intra", "ss.between", "%Explained")							
    dfw <- dfw[,c((ncol(dfw)-5):ncol(dfw))]
    colnames(dfw) <- c("Weight","Cluster", "ss.tot", "ss.intra", "ss.between","%Explained")
    
    res.ss <- list("ss.tot"=ss.tot, "ss.intra"=ss.intra,"ss.inter"= ss.inter, "coord.centers"=coord_centers, "dfw"=dfw,
                   "dfclust"=dfclust)
    return(res.ss)
  }  # End function F1. sum of squares
  
  
  ### function F2 select
  select <- function(Y, default.size, method, coord.centers) {
    clust <- Y[1, ncol(Y)]
    Y <- Y[, -ncol(Y), drop = FALSE]
    Z <- rbind(Y, coord.centers)
    if (nrow(Y) == 1) {
      distance <- data.frame(0, row.names = "")
      colnames(distance) <- rownames(Z[1, ])
    }
    else {
      distance <- as.matrix(dist(Z, method = method))
      distance <- distance[(nrow(Y) + 1):nrow(distance),-((nrow(Y) + 1):ncol(distance))]
      distance <- sort(distance[clust, ], decreasing = FALSE)
    }
    if (length(distance) > default.size) 
      distance <- distance[1:default.size]
    else distance <- distance
  }
  
  ### function F3 distinctivness
  distinctivness = function(Y, default.size, method, coord.centers) {
    clust <- as.numeric(Y[1, ncol(Y)])
    Y <- Y[, -ncol(Y), drop = FALSE]
    Z <- rbind(Y, coord.centers)
    if (nrow(Y) == 1) {
      distance <- as.matrix(dist(Z, method = method))
      ind.car <- vector(length = 1, mode = "numeric")
      ind.car <- min(distance[-c(1, (clust + 1)), 1])
      names(ind.car) <- rownames(Z[1, ])
    }
    else {
      distance <- as.matrix(dist(Z, method = method))
      distance <- distance[(nrow(Y) + 1):nrow(distance), 
                           -((nrow(Y) + 1):ncol(distance))]
      if (nrow(distance) == 2) 
        center.min <- distance[-clust, ]
      else center.min <- apply(distance[-clust, ], 2, min)
      ind.car <- sort(center.min, decreasing = TRUE)
    }
    if (length(ind.car) > default.size) 
      ind.car = ind.car[1:default.size]
    else ind.car = ind.car
  }
  
  
  

  ### function F4 auto.cut.constree
  auto.cut.constree <- function(res, min, max, alpha.test) 
  {
    X <- as.data.frame(res$ind$coord)  # Documents x Dimensions
    
    hcc <- hcclust(X, alpha.test)
    height <- hcc$height
    inv.height <-rev(height)
    quot<-inv.height[(min-1):(max-1)]/inv.height[(min):(max)]
    
    if(is.null(cut.stat))
    { nb.clust <- which.max(quot) + min - 1 } else nb.clust <-  nb.cluster.test
    #  stop(nb.clust)
    return(list(res = res, tree = hcc, nb.clust = nb.clust,height=height,quot = quot))
  }  # End auto.cut.constree function
  
  
  
  #---- F2. print.inertia function ----
  print.inertia <- function(title) {
    if (!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) 
      dev.new()
    # old.mar <- par()$mar
    par(mar = c(0.5, 2, 0.75, 0))
    lay = matrix(ncol = 5, nrow = 5, c(2, 4, 4, 4, 4, 
                                       2, 4, 4, 4, 4, 2, 4, 4, 4, 4, 2, 4, 4, 4, 4,1, 3, 3, 3, 3))
    layout(lay, respect = TRUE)
    barplot(rev(t$tree$height)[1:max(15, max)], col = c(rep("black",t$nb.clust),
                                                        rep("grey", max(max, 15) - t$nb.clust + 1)), rep(0.1, max(max, 15)), space = 0.9)
    plot(x = 1, xlab = "", ylab = "", main = "", col = "white",axes = FALSE)
    text(1,1,title,cex=2)
    plot(x = 1, xlab = "", ylab = "", main = "", col = "white",axes = FALSE)
    legend("top", "Aggreg. Crit", box.lty = NULL, cex = 1)
  }    # End function print.inertia
  
  
  
  ### function F5 hcclust
  hcclust <- function(X, alpha.test)  
  {
    options(warn=-1)
    ## PHASE 1 ##
    ndoc<-nrow(X)                  # Number of documents to cluster
    d<-dist(X)                     # Euclidean distance of LexCA coordinates of documents as matrix format. Mdist
    mDist.0 <- mDist<-as.matrix(d)

    # mSim is the original similarity matrix 
    maxd<-max(mDist)+1e-10
    mSim<-(maxd-mDist)
    mCont<-rep(0,ndoc*ndoc)
    attr(mCont,"dim") <- c(ndoc, ndoc)   # matrix, array
    # If only restricted contiguity clusters
    for(i in 1:(nrow(mCont)-1)) mCont[i,i+1] <- 1
    for(i in 2:(nrow(mCont))) mCont[i,i-1] <- 1
    rownames(mCont) <- colnames(mCont) <- colnames(mSim)
    ###   Introduction of the constriction into the similarity and distance matrices
    mSimCont<-mSim*mCont
    mDistCont<-mDist*mCont
    maxMsim <- max(mSim)
    # detail. For explaining who is joined in each step
    detail <- data.frame(n.step=character(),joined=logical(),cluster.Left=character(), cluster.Right=character())
    ## END PHASE 1 ##
    

    
    ## PHASE 2 ##
    ###    Groups existing at Phase 0. Empty list with as many items as individuals
    groups <- list()                           
    for (i in 1:nrow(mSim)) groups[[i]]<-i
    ###    Critagreg will file the successive values of the agregation criterium
    Critagreg<-numeric()                     # value of the distance in the (n-1) fusions of nodes
    ###  clust will archive the successive pairs of nodes that are merged
    clust<-list()
    ###    Building the hierarchy: (n-1) intermediate nodes have to be created
    i<-1
    # indice<-nrow(Mdist)-1
    
  
    #    if(bTest) {
    ### ONLY WHEN alpha.test !=0. mSingletons save if one individual is a singleton
    #    mSingletons <- matrix(0, nrow = nrow(mCont), ncol = 2)
    #    colnames(mSingletons) <- c("Left", "Right")
    #    rownames(mSingletons) <- rownames(mCont)
    #    mSingletons[1,1] <- mSingletons[nrow(mDist),2] <-1
    #  write.csv2(mSingletons,file= "C:/LexCHCCA/mSingletons.csv")
    ### End ONLY WHEN alpha.test !=0
    #    } # End mSingletons
    
    ## END PHASE 2 ##
    
    ############ Pendiente esto, solamente si bTest 
    indice <- 1

    
    #####################################################################################################################################################
    # creating the n-1 nodes 
    #  while(indice>0) {	 
    while(sum(mCont)>0) {	                                 ### PHASE 3. DO UNTIL  AGGREGATION IS POSSIBLE
      #        Indentifying the maximum similarity
      maxsim<- max(mSimCont)  
      mat_pos <- which(mSimCont == maxsim,  arr.ind = TRUE)
      minpos <- min(mat_pos[1,])
      maxpos <- max(mat_pos[1,])
      # minpos is the row and maxps is the column to group with the similarities
      

      ##############  Checking Legendre test ib bTest=TRUE ##############
      if(bTest) {                                                 ### PHASE 4 ########
        # n.minpos is the number of cases in the cluster int the position minpos
        # n.maxpos is the number of cases in the cluster int the position maxpos
        n.minpos <- length(groups[[minpos]])
        n.maxpos <- length(groups[[maxpos]])
        

        
        if(n.minpos+n.maxpos==2) {   # Each one of the two checked clusters has only one case.
          bAddTest <- TRUE   # Aggregation of two single clusters
          detail <- rbind(detail,data.frame(n.step=indice,bAddTest, cluster.Left=colnames(mDist)[minpos], cluster.Right=colnames(mDist)[maxpos] ) )
                 }
        else{
          mDistExtrem <- cbind(matrix(0,nrow=n.minpos, ncol=n.minpos),
                               matrix(1,nrow=n.minpos, ncol=n.maxpos))
          mDistExtrem <- rbind(mDistExtrem, cbind(t(matrix(1,nrow=n.minpos, ncol=n.maxpos)),
                                                  matrix(0,nrow=n.maxpos, ncol=n.maxpos)))
          
          mDist.CL <- mDist.0[c(groups[[minpos]],groups[[maxpos]]),
                              c(groups[[minpos]],groups[[maxpos]])]
          
          if(cut.stat =="percent") {
            n.lower <-  ((( n.minpos+n.maxpos)^2)-( n.minpos+ n.maxpos))/2
            n.ones <- n.minpos*n.maxpos
            cut.stat.value <- quantile(mDist.CL[row(mDist.CL)>col(mDist.CL)],
                                       probs = 1-n.ones/n.lower)
          } else { # End "percent
            cut.stat.value <- median(mDist.CL[row(mDist.CL)!=col(mDist.CL)])
            
          } # End if(cut.stat =="percent")
          ind01 <- mDist.CL >= cut.stat.value 
          ind01 <- +ind01
          
          suppressMessages(Mantel.res <- vegan::mantel(mDistExtrem, ind01, method=cor.stat, permutations=9999))
          
          bAddTest <- ifelse(Mantel.res$signif < alpha.test,FALSE,TRUE)  ### Only one tail greater
          
          detail <- rbind(detail,data.frame(n.step=indice,bAddTest, cluster.Left=colnames(mDist)[minpos], 
                                            cluster.Right=colnames(mDist)[maxpos] ) ) 
          if(bAddTest==FALSE) 
            mCont[minpos, maxpos] <- mCont[maxpos, minpos] <- 0 
          
        } # End  if(n.minpos+n.maxpos==2) End  #### Hay test  
      } # End bTest is TRUE
      
      

      if(bTest==FALSE | bAddTest==TRUE) {   # Adding clusters to dendrogram
        Critagreg[i]<- mDistCont[minpos,maxpos]
        clust[[i]] <- vector(mode="list", length=2)
        clust[[i]][[1]] <- groups[[minpos]] 
        clust[[i]][[2]] <- groups[[maxpos]]

    
        
        colnames(mSim)[minpos]<-rownames(mSim)[minpos] <- colnames(mDist)[minpos]<-rownames(mDist)[minpos] <- 
        colnames(mCont)[minpos]<-rownames(mCont)[minpos] <- paste0(rownames(mSim)[minpos],"-",colnames(mSim)[maxpos])
        ###### Lance & Williams. Values for complete4 link (CLCA) alpha=0.5 ; gamma=.5
        # Actualization of Msim, Mdist and Mcont
        # When for distances alfa=0.5 and gamma  = 1/2 (Complete linkage). Joining of maximum distances
        # When for similarities alfa =0.5 and gamma = -1/2 (single linkage). Joining minimum similarities
        #   fila <- minpos; columna <- maxpos
        #  if (fila!=1) {
        #      Msim[fila,fila-1]<-Msim[fila-1,fila]<- 0.5*Msim[fila,fila-1]+0.5*Msim[columna,fila-1]-0.5*abs(Msim[fila,fila-1]-Msim[columna,fila-1])
        #      Mdist[fila,fila-1]<-Mdist[fila-1,fila]<-0.5*Mdist[fila,fila-1]+0.5*Mdist[columna,fila-1]+0.5*abs(Mdist[fila,fila-1]-Mdist[columna,fila-1])
        #      Mcont[fila-1,fila]<-Mcont[fila,fila-1]<-1
        #   }
        #    if (columna!=nrow(MSimCont)) {
        #      Msim[columna+1,fila]<-Msim[fila,columna+1]<-0.5*Msim[fila,columna+1]+0.5*Msim[columna,columna+1]-0.5*abs(Msim[fila,columna+1]-Msim[columna,columna+1])
        #     Mdist[columna+1,fila]<-Mdist[fila,columna+1]<-0.5*Mdist[fila,columna+1]+0.5*Mdist[columna,columna+1]+0.5*abs(Mdist[fila,columna+1]-Mdist[columna,columna+1])
        #      Mcont[columna+1,fila]<-Mcont[fila,columna+1]<-1
        #    }
        
        # New algorithm, only when method=complete is equal to max distance and minimum similarity
        # Compute the similarities and distances for all old clusters with the new cluster 
        mSim[minpos,]<- mSim[,minpos]<- apply(X=mSim[,c(minpos,maxpos)], MARGIN=1, FUN=min)
        mSim[minpos,minpos]<- maxMsim
        mDist[minpos,]<- mDist[,minpos]<- apply(X=mDist[,c(minpos,maxpos)], MARGIN=1, FUN=max)
        mDist[minpos,minpos]<- 0
   
        
 # Check if there is a cluster on the left and on the right       
        if(maxpos!=nrow(mCont)) mCont[maxpos+1,minpos] <- mCont[minpos,maxpos+1] <-1
        if(minpos!=1) mCont[minpos-1,minpos] <- mCont[minpos,minpos-1] <-1
        

        groups[[minpos]] <- c(groups[[minpos]], groups[[maxpos]])
        # Removing rows and columns not necessary
        groups <- groups[-maxpos]
        mSim <- mSim[-maxpos,-maxpos]
        mDist <- mDist[-maxpos,-maxpos]
        mCont <- mCont[-maxpos,-maxpos]
        i <- i + 1         
      }  # End  if(bTest==FALSE | bAddTest==TRUE)
      
      mSimCont <- mSim * mCont
      mDistCont <- mDist * mCont
      indice <- indice + 1
      

      if(bTest) if(sum(mCont)==0) {    # No more possible aggregations
        nb.cluster.test <<- ncol(mCont)
        max.Dist <- max(mDist)
        Critagreg.old <- Critagreg
        if(length(Critagreg.old)== ndoc-1) stop(paste0("No clusters for alpha.test level= ", alpha.test,
                                               "\nIncrease this value"))
        Critagreg[i:(ndoc-1)]<- max.Dist

        for(k in i:(ndoc-1)) {
          clust[[k]] <- vector(mode="list", length=2)
          clust[[k]][[1]] <- groups[[1]] 
          clust[[k]][[2]] <- groups[[2]]
          groups[[1]] <- c(groups[[1]], groups[[2]])
          groups <- groups[-2]
        }
        
      }
      
    } # End     while(sum(mCont)>0)  
    
    height <- Critagreg
    


    ######### Eliminar los singletons después #################
    order <- as.integer(1:nrow(X))
    ##################
    grups_blocs <- list()
    grups_blocs[[1]] <- rep(0, nrow(X))
    ############
    
    
    for (i in 1:(length(clust) - 1)) {
      grups_blocs[[i + 1]] <- grups_blocs[[i]]
      grups_blocs[[i + 1]][c(clust[[i]][[1]], clust[[i]][[2]])] <- i
    }
    merge <- matrix(nrow = (ndoc - 1), ncol = 2, 0)
    
    for (i in 1:(ndoc - 1)) {
      if (length(clust[[i]][[1]]) == 1 & length(clust[[i]][[2]]) == 1) {
        merge[i, 1] <- (-clust[[i]][[1]])
        merge[i, 2] <- (-clust[[i]][[2]])
      }
      else {
        if (length(clust[[i]][[1]]) == 1) {
          merge[i, 1] <- (-clust[[i]][[1]])
        }
        else {
          merge[i, 1] <- grups_blocs[[i]][clust[[i]][[1]][1]]
        }
        if (length(clust[[i]][[2]]) == 1) {
          merge[i, 2] <- (-clust[[i]][[2]])
        }
        else {
          merge[i, 2] <- grups_blocs[[i]][clust[[i]][[2]][1]]
        }
      }
    }
    
    #  if(bTest==FALSE) cut.names <- stats::cutree()
    #  else cut.names <- rownames(mDist)
    #  
    
    if(bTest==FALSE) detail <- NULL

    hcccall<-call("hcclust",match.call()$res)		
    hcc<-structure(list(merge = merge,height=height,
                        order=order, labels = attr(d, "Labels"), method = "complete", 
                        call = hcccall, dist.method = "euclidean",clust=clust), class="hclust")
    options(warn=0)
      return(hcc)

    # stop("FINAL DE OBTENCION DEL DENDROGRAMA")
    
    #    dhcc <- as.dendrogram(hcc)
    #    plot(cut(dhcc, h = Critagreg[[ndoc-1]]-.000000001)$lower[[1]])
    #    plot(dhcc)
  } # End hclust
  
  
  
  

  
  ### main part of the function
  res.ca<- x           ##################  <- Pendiente ¿Necesario?
  res.sauv <- res.ca            ##################  <- Pendiente ¿Necesario?
  metric="euclidean"
  method="complete"             ##################  <- Pendiente
  if (min <= 1) min<-2
  

  ### PCA equivalent to CA. It is necessary to know the max number of clusters
  aux <- res.ca$eig
  weight<- row.w <- res.ca$call$marge.row * sum(res.ca$call$X)
  res <- PCA(res.ca$row$coord, scale.unit = FALSE, ncp = Inf, graph = FALSE, row.w = row.w)
  res$eig <- aux
  

    if (is.null(max)) max <- min(10, round(nrow(res$ind$coord)/2))
     max <- min(max, nrow(res$ind$coord) - 1)
  ###


  ######## auto.cut.constree call to auto.cut.constree function
  ######## auto.cut.constree call to hcclust function
  t <- auto.cut.constree(res=res,min = min, max = max, alpha.test=alpha.test) 
  # It is possible to have inversions in t$tree
  # t$tree contains the constrained dendrogram. índependentely of nb.clust
  nb.ind <- nrow(t$res$ind$coord)
  
  auto.haut <- ((t$tree$height[length(t$tree$height) - 
                                 t$nb.clust + 2]) + (t$tree$height[length(t$tree$height) - t$nb.clust + 1]))/2

  graph.1 <- function() {
    if (!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) 
      dev.new()
    # old.mar <- par()$mar
    par(mar = c(0.5, 2, 0.75, 0))
    lay <- matrix(ncol = 5, nrow = 5, c(2, 4, 4, 4, 4, 
                                        2, 4, 4, 4, 4, 2, 4, 4, 4, 4, 2, 4, 4, 4, 4, 
                                        1, 3, 3, 3, 3))
    layout(lay, respect = TRUE)


    n.tot<- min(15, nrow(t$tree$data.clust), max)
    n.black<- min(n.tot, t$nb.clust)
    n.grey <- max(0, n.tot-n.black)
    
    barplot(rev(t$tree$height)[1:n.tot], col = c(rep("black", n.black), rep("grey", n.grey)),
            rep(0.1, max(n.tot, 15)), space = 0.9)
 ##############

    # barplot(rev(t$tree$height)[1:max(15, max)], col = c(rep("black", 
    #       t$nb.clust - 1), rep("grey", max(max, 15) - t$nb.clust +1)),
    #         rep(0.1, max(max, 15)), space = 0.9)

    plot(x = 1, xlab = "", ylab = "", main = "", col = "white", axes = FALSE)
    text(1, 1, "Constrained Hierarchical Clustering", cex = 2)
    plot(x = 1, xlab = "", ylab = "", main = "", col = "white", axes = FALSE)
    legend("top", "Aggreg. Crit", box.lty = NULL, cex = 1)
  }


  if(bTest==FALSE & nb.clust == 0) {  # click
    # # New 1.5.8
    graphics.off()
    dev.new(noRStudioGD = TRUE)
  
    par(mfrow=c(1,1))
        print.inertia("Constrained Hierarchical Clustering")
        plot(t$tree, hang = -1, main = "Click to cut the tree", xlab = "", sub = "")
        abline(h = auto.haut, col = "black", lwd = 3)	
        coupe <- locator(n = 1)	
        
        while(coupe$y < min(t$tree$height) | coupe$y > max(t$tree$height)) {					
          cat("No class \n")							
          coupe <- locator(n = 1)							
        }	
        y <- coupe$y							
        dev.off(which = dev.cur())        
  } 
  
  if(bTest==FALSE & nb.clust==-1) y <- auto.haut # No test, automatic clusters
  if(bTest==FALSE & nb.clust>1 )  # No test, fixed number clusters
    y <- (t$height[length(t$height)-nb.clust+1]+t$height[length(t$height)-nb.clust+2])/2

  if(bTest==TRUE) {
    # nb.clust <- nb.cluster.test
    #  y <- (t$height[length(t$height)]+t$height[length(t$height)-nb.clust+1])/2
      clust <- stats::cutree(as.hclust(t$tree), k = nb.cluster.test)
  } else {
    clust <- stats::cutree(as.hclust(t$tree), h = y)
  } 

  

  
  nb.clust <- max(clust)
  X <- as.data.frame(t$res$ind$coord)
  ordColo <- unique(clust[t$tree$order])


  
  if (graph) {
    graph.1()
    plot(t$tree, hang = -1, main = "",xlab = "", sub = "")

    if(bTest==TRUE) 
      rect <- rect.hclust(t$tree, h = t$tree$height[length(t$tree$height)], border = ordColo)
    
    else rect <- rect.hclust(t$tree, h = y, border = ordColo)
    clust <- NULL
    

    for (j in 1:nb.clust) clust <- c(clust, rep(j, length(rect[[j]])))
    
    clust <- as.factor(clust)
    belong <- cbind.data.frame(t$tree$order, clust)
    belong <- belong[do.call("order", belong), ]
    clust <- as.factor(belong$clust)
    if (nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) 
      layout(matrix(nrow = 1, ncol = 1, 1), respect = TRUE)
  }



  # Ver si aux no es necesario
  aux <- names(clust) <- rownames(X)
  # names(clust) <- aux
  clust <- as.factor(clust)
  X <- cbind.data.frame(X, clust)
  
  #  data.clust <- cbind.data.frame(res.sauv$call$Xtot[rownames(t$res$call$X),], clust)
  #  data.clust <- data.clust[rownames(res.sauv$row$coord), ]
  # return(data.clust)  # <-   PROBLEMA   Contiene vocabulario + variable cuantitativa + cluster
  # <----------- Xtot contiene las contextuales cuantitativas, lo cambio por S
  ###########   REVISAR EN LexHCca

  data.clust <- cbind.data.frame(res.sauv$call$X[rownames(t$res$call$X),], "_clust"=clust)
  data.clust <- data.clust[rownames(res.sauv$row$coord), ]
  #  return(data.clust)  # <-   YA NO PROBLEMA   Contiene vocabulario + cluster
  # 
  
  ################################################################
  # Checking description marg.doc before or after
  ################################################################
  doc.w.after <- apply(x$call$X,1,sum)    # Initial weighting for documents after selection
  nm <- names(doc.w.after)
  
  # doc.w.before before word's selection
  doc.w.before <- object$rowINIT[which(rownames(object$rowINIT) %in% nm) ,]
  names(doc.w.before) <- nm
  
  
  #@ Si marg.doc = before o no (after)
  
  # doc.w.before before word's selection
  doc.w.before <- x$rowINIT[which(rownames(x$rowINIT) %in% nm) ,]
  names(doc.w.before) <- nm
  
  if(marg.doc=="before") doc.w <- doc.w.before else doc.w <- doc.w.after # before word's selection
  doc.w <- as.data.frame(doc.w)
  
  
  #  stop("insertar weight en la siguiente")
  df.coord.cla<- data.frame(X[,c(1:(ncol(X)-1))], weight=doc.w[rownames(X),], cluster=X[,"clust"])
  

  
  #---- 12. Computing centers. Building RDO object  ----			
  
  RDO <-   ssquares(df.coord.cla[,c(1:(ncol(df.coord.cla)-2))],  df.coord.cla[,"weight"], df.coord.cla[,"cluster"])
  

  #  desc.var <- descfreq(data.clust[, -which(sapply(data.clust,is.factor))], 
  #                       data.clust[, ncol(data.clust)], proba = proba)

if(description) {
  desc.var <- descfreq(data.clust[,1:(ncol(data.clust)-1)], 
                       data.clust[, ncol(data.clust)], proba = proba)
  ### Modificar en el siguientes las salidas de los objetos:
  # Link between the cluster and the quantitative variables
  # Description of each cluster by quantitative variables
#   desc.axe <- FactoMineR::catdes(X, num.var=ncol(X), proba = proba)
} # End description

  
  tabInd <- cbind.data.frame(res.sauv$row$coord, data.clust[rownames(res.sauv$row$coord), 
                                                            ncol(data.clust)])
  colnames(tabInd)[ncol(tabInd)] <- "Cluster"
  
  list.centers <- by(tabInd[, -ncol(tabInd), drop = FALSE], 
                     tabInd[, ncol(tabInd)], colMeans)
  
  centers <- matrix(unlist(list.centers), ncol = ncol(tabInd) - 1, byrow = TRUE)
  colnames(centers) <- colnames(tabInd)[-ncol(tabInd)]
  cluster <- tabInd[, ncol(tabInd), drop = FALSE]
  
if(description) {
  para <- by(tabInd, cluster, simplify = FALSE, select, default.size = nb.desc, #  nb.par, 
             method = metric, coord.centers = centers)
  
  dist <- by(tabInd, cluster, simplify = FALSE, distinctivness, 
             default.size = nb.desc, method = metric, coord.centers = centers) #  nb.par, 
  desc.doc <- list(para = para, dist = dist)
} # End description  

  colnames(data.clust)[ncol(data.clust)] <- "Clust_"  
  colnames(df.coord.cla)[ncol(df.coord.cla)] <- "Clust_"  
  
if(description ==FALSE)   desc.var <- desc.axe <- desc.doc <- NULL
  call <- list(t = t, min = min, max = max, X = X, vec=FALSE, call = match.call())
  description <- list(des.word = desc.var, des.doc = desc.doc)
  res.LexCHCca <- list(data.clust = data.clust, coord.clust= df.coord.cla, centers=centers, description=description,  
                        call = call, dendro=t$tree, phases=t$tree$clust, sum.squares=RDO,
                       cut.test = cut.stat, alpha.test = alpha.test)
  class(res.LexCHCca) <- "LexCHCca"

 # if (graph) par(mar = old.mar)
  par(old.par)
  return(res.LexCHCca)
 
}

#' @export
summary.LexCA <- function (object, ncp=5, nb.dec = 3, ndoc=10, nword=10, nseg=10, 
  nsup=10, metaDocs=FALSE, metaWords=FALSE, file = NULL, ...) 
{
if (!inherits(object, "LexCA")) stop("non convenient object")
  options(stringsAsFactors = FALSE)
  
printE <- function(mat, file = "") {
            mat <- cbind(format(rownames(mat)), mat)
            mat <- rbind(colnames(mat), mat)
            mat2 <- cbind(format(mat[, 1], justify = "right"),format(mat[, 2], justify = "right"))
            for (k in 3:ncol(mat)) mat2 <- cbind(mat2, format(mat[,k], justify = "right"))
            mat2 <- cbind(mat2, "\n")
            for (i in 1:nrow(mat2)) cat(mat2[i, ])} 

printC <- function(obj, text1, nval, nDim, nIni) {
 cat(paste("\n",text1,"\n"))
 if(nrow(obj)> nval) cat(paste("Only the first ", (nval-nIni+1), "elements are shown\n"))
  print(round(obj[nIni:(nval+nIni-1),1:nDim,drop=FALSE],nb.dec), quote=FALSE)
}


printI <- function(obj, obj2, text1, nval) {
# cat(paste("\n",text1,"\n"))
 cat(paste("\n"))
 obj <- matrix(round(obj[1:nval],nb.dec),nrow=nval,ncol=1)
 rownames(obj) <- rownames(obj2)[1:nrow(obj)]; colnames(obj) <- "Inertia"
# if(nrow(obj2)> nval) cat(paste("Only the first ", nval, "elements are shown\n"))
 print(obj)
}

if(is.null(ndoc)) ndoc<-0
if(is.null(nword)) nword<-0
if(is.null(nseg)) nseg<-0
if(is.null(nsup)) nsup<-0
if(is.null(object$segment$coord)) nseg<-0



if(ndoc=="ALL") ndoc<- nrow(object$row$coord)
if(nword=="ALL") nword<- nrow(object$col$coord)
if(nseg=="ALL") nseg<- nrow(object$segment$coord)

ndoc <- min(nrow(object$row$coord),ndoc)
nword <- min(nrow(object$col$coord),nword)
nseg <- min(nseg,nrow(object$segment$coord))

# for(i in seq_len(sink.number())){sink(NULL)}
if(!is.null(file)) sink(file) else file=""

cat("Correspondence analysis summary\n")
cat("\nEigenvalues\n")

nEig <- min(ncp,ncol(object$row$coord))
eige <- format(t(round(object$eig[1:nEig, 1:3], nb.dec)), justify = "right")
   rownames(eige) <- c("Variance", "% of var.", "Cumulative % of var.")
    nEig <- min(nEig, nrow(object$eig))
    colnames(eige[, 1:nEig]) <- paste("Dim", 1:nEig, sep = ".")
    printE(t(eige[, 1:nEig]))
    cat(paste("\nCramer's V ", round(object$VCr,nb.dec) ,"   Inertia ", round(object$Inertia, nb.dec), "\n", sep=" "))

if(ndoc>0) {
cat("\n\nDOCUMENTS")
if(!is.null(object$var.agg)) cat("\nAll documents are aggregate documents\n")
printC(object$row$coord,"Coordinates", ndoc, nEig,1)
printC(object$row$contrib,"Contributions (by column total=100)", ndoc, nEig,1)
printC(object$row$cos2,"Square cosinus (by row total=1)", ndoc, nEig,1)
printI(object$row$inertia,object$row$coord, "", ndoc)}

if(metaDocs!=FALSE){
cat("\n\nDocuments whose contribution is over ", object$meta$lmd, " times the average document contribution\n")
for(i in 1:nEig) {
 cat("\nDimension ", i, "+\n")
 cat(object$meta$Doc$Docs[which(object$meta$Doc$Dim==i & object$meta$Doc$Coordinates>0)])
 cat("\nDimension ", i, "-\n")
 cat(object$meta$Doc$Docs[which(object$meta$Doc$Dim==i & object$meta$Doc$Coordinates<0)],"\n")
}}



if(nword>0) {
cat("\n\nWORDS\n")
printC(object$col$coord,"Coordinates", nword, nEig,1)
printC(object$col$contrib,"Contributions (by-column total=100)", nword, nEig,1)
printC(object$col$cos2,"Square cosinus (by-row total=1)", nword, nEig,1)
printI(object$col$inertia,object$col$coord, "", nword)}

if(metaWords!=FALSE){
cat("\n\nWords whose contribution is over ", object$meta$lmw , " times the average word contribution\n")
for(i in 1:nEig) {
 cat("\nDimension ", i, "+\n")
 cat(object$meta$Word$Words[which(object$meta$Word$Dim==i & object$meta$Word$Coordinates>0)])
 cat("\nDimension ", i, "-\n")
 cat(object$meta$Word$Words[which(object$meta$Word$Dim==i & object$meta$Word$Coordinates<0)],"\n")
}}


nSeg<- ifelse(is.null(object$segment$coord),0 ,nrow(object$segment$coord))
if(nseg>0) {
cat("\n\nREPEATED SEGMENTS\n")
nS <- min(nSeg,nseg)
   printC(object$segment$coord,"Coordinates", nS , nEig, 1)
   printC(object$segment$cos2,"Square cosinus", nS , nEig, 1)
 }



if(nsup!=0) {

if(!is.null(object$row.sup)) {
  if(nsup=="ALL") nDsup<-nrow(object$row.sup$coord) 
     else nDsup <- min(nrow(object$row.sup$coord),nsup)} else {nDsup<-0}

nWsup<- ifelse(is.null(object$col.sup$coord),0 ,nrow(object$col.sup$coord)-nSeg)
if(nWsup>0) {if(nsup!="ALL") nWsup<- min(nWsup,nsup)} 


if(!is.null(object$quali.sup)) {
  nQsup<- nrow(object$quali.sup$coord)
  if(nsup!="ALL") nQsup<- min(nQsup,nsup)}else {nQsup<-0}
if(!is.null(object$quanti.sup)) nQQsup<- nrow(object$quanti.sup$coord) else nQQsup<-0
if(nDsup + nWsup+ nQsup + nQQsup>0) {


if(nDsup>0){
 cat("\n\nSUPPLEMENTARY DOCUMENTS\n")
 printC(object$row.sup$coord,"Coordinates", nDsup, nEig,1)
 printC(object$row.sup$cos2,"Square cosinus", nDsup, nEig,1)}

if(nWsup>0) {
 cat("\n\nSUPPLEMENTARY WORDS\n")
 printC(object$col.sup$coord,"Coordinates", nWsup,nEig,1)
 printC(object$col.sup$cos2,"Square cosinus", nWsup, nEig,1)}


if(nQsup>0) {
 cat("\n\nSUPPLEMENTARY CATEGORIES\n")
 printC(object$quali.sup$coord,"Coordinates",nQsup, nEig,1)
 printC(object$quali.sup$cos2,"Square cosinus", 
  nQsup, nEig,1)
 if(!is.null(object$quali.sup$v.test)) {
 printC(object$quali.sup$v.test,"v.test of the supplementary categories", 
   nQsup, nEig,1)}}


if(nQQsup>0) {
cat("\n\nSUPPLEMENTARY QUANTITATIVE VARIABLES\n")
   cat("\nCoordinates\n")
   cat("\nEqual to the correlations of the variable with the axes\n")
   print(data.frame(round(object$quanti.sup$coord[,1:nEig,drop=FALSE],nb.dec),
      row.names=rownames(object$quanti.sup$coord)))
   cat("\nSquare cosinus\n")
   print(data.frame(round(object$quanti.sup$cos2[,1:nEig,drop=FALSE],nb.dec),
      row.names=rownames(object$quanti.sup$cos2)))}
}
}


if(file!=""){
  sink()
cat("\nAll the results are in file: ",file,"\n")
}
}

#' @importFrom graphics lines
#' @export
 ellipseLexCA <- function (object, selWord="ALL",selDoc="ALL", 
    nbsample = 100, level.conf = 0.95, axes = c(1, 2), ncp=NULL, xlim = NULL, ylim = NULL, title=NULL,
    col.doc = "blue", col.word = "red", col.doc.ell = col.doc, 
    col.word.ell = col.word, cex=1) 
{
    if (!inherits(object, "LexCA"))  stop("Object should be LexCA class")
   graph.type <- "classic"
   options(stringsAsFactors = FALSE)
   
if(is.null(ncp)) ncp <- ncol(object$col$coord) else
ncp <- min(ncol(object$col$coord),ncp)
if(max(axes)> ncp) stop("Axis=" , max(axes), " is bigger than ncp=", ncp, ". Change ncp value in LexCA" )
   
   
selectionDW <- function(sel1, xobj,bType, axx, axy)
{
if(is.null(sel1)) {  sel1<-c()
                     return(sel1)}
xx <- ""
if(length(sel1)==1){
     if(sel1=="ALL") sel1 <- c(1:dim(xobj$coord)[1])
                   }
if(length(sel1)==1) {
   if(sel1=="meta") sel1 <- "meta 3"
   xx <- gregexpr(pattern =' ',sel1)[[1]][1]-1
   xx <- substr(sel1, 1, xx)
                     }
if(xx=="coord" | xx=="cos2" | xx=="contrib" | xx=="meta" ) 
 { 
  nc <- nchar(sel1)
  if(xx=="coord") {
 # Selection by coordinates"
 sel1 <- as.numeric(substr(sel1, 7, nc))
 dft <- data.frame(xobj$coord[,c(axx,axy),drop=FALSE])
 fval <- apply(dft, 1, function(x) max(abs(x)))
 ordmax <- rank(fval)
 posic <- which(ordmax > (length(fval)-sel1))
 sel1 <- rownames(xobj$coord)[posic]
                   }
  if(xx=="cos2") {
 # Selection by cos2
 sel1 <- as.numeric(substr(sel1, 5, nc))
 dft <- data.frame(xobj$cos2[,c(axx,axy),drop=FALSE])
 fval <- apply(dft, 1, function(x) sum(x))
 posic <- which(fval>= sel1)
 sel1 <- rownames(xobj$cos2)[posic]
                  }
  if(xx=="contrib") {
 # Selection by contrib
 sel1 <- as.numeric(substr(sel1, 9, nc))
 dft <- data.frame(xobj$contrib[,c(axx,axy),drop=FALSE])
 fval <- apply(dft, 1, function(x) max(abs(x)))
 posic <- which(fval>= sel1)
 sel1 <- rownames(xobj$contrib)[posic]
  }
 if(xx=="meta") {
 sel1 <- as.numeric(substr(sel1, 5, nc))
 sKeys <-  rownames(xobj$coord)[which(xobj$contrib[,axx]>mean(xobj$contrib[,axx])*sel1)]
 sKeys <-  c(sKeys,rownames(xobj$coord)[which(xobj$contrib[,axy]>mean(xobj$contrib[,axy])*sel1)])
 sel1 <- unique(sKeys)
}
 } else {
 if(is.character(sel1)) sel1 <- which(rownames(xobj$coord) %in% sel1)
 sel1 <- rownames(xobj$coord)[sel1]
 sel1  <- sel1[!is.na(sel1)]
}
 return(sel1)
}
# Final functions

X <- object$call$X
if(!is.null(selDoc)) selDoc <- selectionDW(selDoc,object$row,"Doc",axes[1],axes[2])
if(!is.null(selWord)) selWord <- selectionDW(selWord, object$col,"Word",axes[1],axes[2])
if ( (length(selDoc) + length(selWord))  == 0)
    stop("No elements are selected to be plotted;", "\n",
         "  verify the expression of both arguments selWord and/or selDoc")

    concCol = X
    concRow = X
    sampcol=0
    samprow=0
    method <- "multinomial" 
    proba = unlist(c(X))
    N = sum(proba)
    proba = proba/N
    aa = rmultinom(nbsample, size = N, prob = proba)
    for (i in 1:nbsample) {
            aux = matrix(aa[, i, drop = FALSE], nrow = nrow(X))
            dimnames(aux) = dimnames(X)
            if(!is.null(selWord))
               {
               aaa<-apply(aux,2,sum)  
               if (!(0 %in% aaa)) 
                      {
                      concCol = cbind.data.frame(concCol, aux)
                      } else {
                      sampcol<-sampcol+1 
                      } 
               }
            if(!is.null(selDoc)) 
               {
               bbb<-apply(aux,1,sum)  
               if (!(0 %in% bbb)) 
                      {
                      concRow = rbind.data.frame(concRow, aux)
                      } else {
                      samprow<-samprow+1 
                      } 
                }
        }

    
  
  if ((!is.null(selWord)) &  (sampcol/nbsample)>0.1 )  {
    stop("over 10% of the replicated samples present words with null frequency","\n",
          "  it is advisable to increase the threshold on the word frequency in TextData") }

  if ( (!is.null(selDoc)) & (samprow/nbsample)>0.1 )   {
    stop("over 10% of the replicated samples present documents with null length","\n",
         "  it is advisable to increase the threshold on the word frequency in TextData") }

    vdvword<-rep("transparent",ncol(X))
    vdvword[which(colnames(X) %in% selWord)]<-col.word.ell
    vdvdoc<-rep("transparent",nrow(X))
    vdvdoc[which(rownames(X) %in% selDoc)]<-col.doc.ell

    
    
    
    
    Xcoord.ellipse <- function (coord.simul, centre = NULL, axes = c(1, 2), level.conf = 0.95, 
              npoint = 100, bary = FALSE) 
    {
  
      
      Xellipse <- function(x, scale = c(1, 1), centre = c(0, 0), 
                          level = 0.95, t = sqrt(qchisq(level, 2)), which = c(1, 2), npoints = 100) {
        names <- c("x", "y")
        if (is.matrix(x)) {
          xind <- which[1]
          yind <- which[2]
          r <- x[xind, yind]
          if (missing(scale)) {
            scale <- sqrt(c(x[xind, xind], x[yind, yind]))
            if (scale[1] > 0) 
              r <- r/scale[1]
            if (scale[2] > 0) 
              r <- r/scale[2]
          }
          if (!is.null(dimnames(x)[[1]])) 
            names <- dimnames(x)[[1]][c(xind, yind)]
        }
        else r <- x
        r <- min(max(r, -1), 1)
        d <- acos(r)
        a <- seq(0, 2 * pi, len = npoints)
        matrix(c(t * scale[1] * cos(a + d/2) + centre[1], t * 
                   scale[2] * cos(a - d/2) + centre[2]), npoints, 2, 
               dimnames = list(NULL, names))
      }
      nbre.fact <- nlevels(coord.simul[, 1])
      res <- label <- NULL
      lev <- levels(coord.simul[, 1])
    
      
      for (f in 1:nbre.fact) {
        x <- coord.simul[which(coord.simul[, 1] == lev[f]), axes[1] + 1]
        y <- coord.simul[which(coord.simul[, 1] == lev[f]), axes[2] + 1]
        if (is.null(centre)) 
          center <- c(mean(x, na.rm = TRUE), mean(y, na.rm = TRUE))
        else {
          if (ncol(coord.simul) != ncol(centre)) 
            stop("ncol de centre incorrect")
          if (!all.equal(lev, levels(centre[, 1]))) 
            stop("Levels of centre are not corrects")
          center <- as.numeric(centre[which(centre[, 1] ==levels(centre[, 1])[f]), 
                                      c(axes[1] + 1, axes[2] + 1)])
        }
        
       # tab <- data.frame(x = x, y = y)
        tab <- data.frame(x = x, y = y, stringsAsFactors = TRUE)
   
        if (nrow(tab) > 1) 
          mat.cov <- cov(tab)
        else mat.cov <- matrix(0, 4, ncol = 2)
        if (bary) 
          mat.cov = mat.cov/nrow(tab)
      elli.tmp <- Xellipse(mat.cov, centre = center, 
                                     level = level.conf, npoints = npoint)
        res <- rbind(res, elli.tmp)
      }
      label <- factor(rep(lev, each = npoint), levels = lev)
      result <- data.frame(facteur = label, res)
      colnames(result)[1] = "facteur"
      colnames(result) <- colnames(coord.simul)[c(1, axes + 1)]
      return(list(res = result, call = npoint))
    }
    

    if (!is.null(selWord)) {
        colCA <- FactoMineR::CA(concCol, col.sup = (ncol(X) + 1):ncol(concCol), graph = FALSE, ncp=ncp)
        aux3 <- colCA$col.sup$coord[, axes]
        rownames(aux3) <- paste("r", 1:nrow(aux3), sep = "")
        aux1 <- cbind.data.frame(label=paste("word", 1:ncol(X), sep = ""), aux3)
        aux1$label <- as.factor(aux1$label)
        ellCol <- Xcoord.ellipse(aux1, level.conf = level.conf)$res
    }
    if (!is.null(selDoc)) {
        rowCA = FactoMineR::CA(concRow, row.sup = (nrow(X) + 1):nrow(concRow), graph = FALSE, ncp=ncp)
        aux2 = cbind.data.frame(label=paste("doc", 1:nrow(X), sep = ""), 
            rowCA$row.sup$coord[, axes])
        aux2$label <- as.factor(aux2$label)
        ellRow = Xcoord.ellipse(aux2, level.conf = level.conf)$res
    }
    if (is.null(xlim)) {
        if (  (!is.null(selWord)) & (!is.null(selDoc))  ) 
            xlim <- c(min(ellCol[, 2], ellRow[, 2]), max(ellCol[, 2], ellRow[, 2]))
        else {
            if (!is.null(selWord))
                xlim <- c(min(ellCol[, 2]), max(ellCol[, 2]))
            if (!is.null(selDoc)) 
                xlim <- c(min(ellRow[, 2]), max(ellRow[, 2]))
        }
    }
    if (is.null(ylim)) {
        if (  (!is.null(selWord)) & (!is.null(selDoc))  ) 
            ylim <- c(min(ellCol[, 3], ellRow[, 3]), max(ellCol[, 3], ellRow[, 3]))
        else {
            if (!is.null(selWord)) 
                ylim <- c(min(ellCol[, 3]), max(ellCol[, 3]))
            if (!is.null(selDoc))
                ylim <- c(min(ellRow[, 3]), max(ellRow[, 3]))
        }
    }
    if (  (!is.null(selWord)) & (!is.null(selDoc))  )
 plot.LexCA(object, axes = axes, selWord=selWord,selDoc=selDoc,xlim = xlim, ylim = ylim, col.doc = col.doc, col.word = col.word,
            cex=cex,title=title, graph.type =graph.type, plot.new=TRUE)
    else {
          if (!is.null(selWord))  
         plot.LexCA(object, axes = axes, selWord=selWord,selDoc=NULL,xlim = xlim, ylim = ylim, col.word = col.word,cex=cex,title=title,
                    graph.type =graph.type, plot.new=TRUE) 
          if (!is.null(selDoc)) 
         plot.LexCA(object, axes = axes, selWord=NULL,selDoc=selDoc,xlim = xlim, ylim = ylim,  col.doc = col.doc,,cex=cex,title=title,
                    graph.type =graph.type, plot.new=TRUE)
         }

    if (!is.null(selDoc)) {
        lev <- paste("doc", 1:nlevels(ellRow[, 1]), sep = "")
        for (e in 1:nlevels(ellRow[, 1])) {
            data.elli <- ellRow[ellRow[, 1] == lev[e], -1]
            lines(x = data.elli[, 1], y = data.elli[, 2], col = vdvdoc[e])
        }
    }
    if (!is.null(selWord)) {
        lev <- paste("word", 1:nlevels(ellCol[, 1]), sep = "")
        for (e in 1:nlevels(ellCol[, 1])) {
            data.elli <- ellCol[ellCol[, 1] == lev[e], -1]
            lines(x = data.elli[, 1], y = data.elli[, 2], col = vdvword[e])
        }
    }
}

#' @importFrom MASS ginv
#' @export
 LexGalt <- function (object, context="ALL", conf.ellip=FALSE, 
                     nb.ellip = 100, graph= TRUE, axes = c(1, 2), label.group=NULL) 

{

  # ncp= number of dimensions kept to compute (by default NULL to indicate all dimensions)
  ncp <- NULL  # LexGalt CAN'T work with Aggregated Lexical Tables (ALT)
  # level.ventil, a proportion corresponding to the level under which the category is ventilated; by default, 0 and no ventilation is done
  # If level.ventil, seed is a random value by defect and results may differ when running the same script twice.
  # Use the same values for the set.seed() function.
  
  # scale. Variables are are scaled to unit variance (standardized) (by default TRUE)
  scale <- TRUE
  level.ventil = 0; set.seed(1234)

 #  return(object$var.agg)  
  if(!is.null(object$var.agg)) stop("LexGalt needs non aggregate TextData objects")
  options(stringsAsFactors = FALSE)
  
  

##################################################
### PHASE 0. Functions
#################################################  
    Xtab.disjonctif <- function(tab)
  {
    tab <- as.data.frame(tab,  stringsAsFactors = TRUE)
    modalite.disjonctif <- function(i) {
      moda <- as.factor(tab[, i])
      n <- length(moda)
      x <- matrix(0L, n, nlevels(moda))
      x[(1:n) + n * (unclass(moda) - 1L)] <- 1L
      return(x)
    }
    if (ncol(tab) == 1) {
      res <- modalite.disjonctif(1)
      dimnames(res) <- list(attributes(tab)$row.names, levels(tab[,1]))
    }
    else {
      variable <- rep(attributes(tab)$names, lapply(tab, nlevels))
      listModa <- unlist(lapply(tab, levels))
      wlistModa <- which((listModa) %in% c("y", "n","Y", "N"))
      if (!is.null(wlistModa)) 
        listModa[wlistModa] <- paste(variable[wlistModa], 
                                     listModa[wlistModa], sep = ".")
      numlistModa <- which(unlist(lapply(listModa, is.numeric)))
      if (!is.null(numlistModa)) 
        listModa[numlistModa] <- paste(variable[numlistModa], 
                                       listModa[numlistModa], sep = ".")
      res <- lapply(1:ncol(tab), modalite.disjonctif)
      res <- as.matrix(data.frame(res, check.names = FALSE))
      dimnames(res) <- list(attributes(tab)$row.names, listModa)
    }
    return(res)
  }

  mean.p<-function(V, poids) res <- sum(V * poids, na.rm = TRUE)/sum(poids[!is.na(V)])
  sd.p<-function(V, poids) res <- sqrt(sum(V^2 * poids, na.rm = TRUE)/sum(poids[!is.na(V)]))
  #######################################################################
  
  
  #######################################################################
  # PHASE 1. - Checking if it is a list (multiple) or only one object (simple)
  #######################################################################
  num.group <- ifelse(is.vector(object), length(object),1)
  if(num.group==1) {
    # Simple, only one TextData object
    # If it is not a TextData object: stop
    if (!inherits(object,"TextData")) stop("Object should be TextData class")
    if(is.null(context)) stop("No contextual variables are selected")
  } else {
    # Multiple. Check all objects in list are TextData objects
    for(i in 1:num.group) 
      if (!inherits(object[[i]],"TextData")) stop("Object should be TextData class")
  }
  #######################################################################
  

  
  
  #######################################################################
  ## PHASE 2. Contextual variables
  context.quanti <- context.quali <- NULL
  #############  Start selecting contextual variables	
  if(length(context)==1)	# Only one contextual variable
    if(context=="ALL") {	# Contextual variable can be "ALL" (defect) (quanti+quali)
      context<- NULL	# New object context to separe quanti and quali variables
      if(num.group==1) {      # It is Simple  LexGalt
        if(!is.null(object$context$quali))     # Check if there are quali variables in TextData object
          context <- as.character(colnames(object$context$quali))
        if(!is.null(object$context$quanti)) # Check if there are quanti variables in TextData object
          context <- c(context,colnames(object$context$quanti))	# context has quanti and quali variables
      } else {    # It is Multiple LexGalt; Only read contextual variables from first TextData object
        if(!is.null(object[[1]]$context$quali)) 	
          context <- as.character(colnames(object[[1]]$context$quali))	
        if(!is.null(object[[1]]$context$quanti)) 
          context <- c(context,colnames(object[[1]]$context$quanti))	
      } # Final num.group
    }  # Final context=1	
  ##########################################################################3

  if(num.group ==1) {  # If it is simple
    if(!is.null(colnames(object$context$quanti))) # Selection quanti variables 
      context.quanti <- colnames(object$context$quanti)[which(colnames(object$context$quanti) %in% context)]
    if(!is.null(colnames(object$context$quali)))  # Selection quali variables
      context.quali <- colnames(object$context$quali)[which(colnames(object$context$quali) %in% context)]
  } else { # If is multiple
    context.quanti <- context.quali <- context
    # Only selection contextual variables if they are in all the groups
    for(i in 1:num.group) {
      if(!is.null(colnames(object[[i]]$context$quanti)))
        context.quanti <- colnames(object[[i]]$context$quanti)[which(colnames(object[[i]]$context$quanti) %in% context.quanti)]
      else   context.quanti <- NULL
      if(!is.null(colnames(object[[i]]$context$quali)))
        context.quali <- colnames(object[[i]]$context$quali)[which(colnames(object[[i]]$context$quali) %in% context.quali)]
      else   context.quali <- NULL
    } # End For
  } # End  if(num.group ==1) 
  ##### End of contextual variables
  # nQN is the number of selected quanti contextual variables
  # nQL is the number of selected quali contextual variables
  nQN <- ifelse(length(context.quanti)==0, 0, length(context.quanti))
  nQL <- ifelse(length(context.quali)==0, 0, length(context.quali))
  ################################################################################

  
  

  
  
  
  #######################################################################
  ##  PHASE 3. Simple LexGalt 
  ####### If LexGalt is simple, not multiple
  if(num.group==1) {  # It is simple, not multiple
    #### There are Quantitative variables
    if(nQN!=0) {
      # df.quanti is a dataframe with quantitative variables in columns
      df.quanti <- data.frame(object$context$quanti[,context.quanti,drop=FALSE])
      # Check if one variable has all cases the same value (variance=0)
      # The following line returns a value for each variable with the range of each variable
      # If the range is 0 then all the vaues of the variable are the same
      fval <- apply(df.quanti, 2, function(x) max(x)-min(x))
      fval <- colnames(df.quanti)[which(fval==0)]
      if(length(fval)>0){
        warning("Quantitative variable with the same value are removed: ",fval)
        context.quanti <- context.quanti[!context.quanti %in% fval]
        nQN <- length(context.quanti) # Compute the new number of quanti variables
      }
      if(nQN<2) stop("The number of quantitative variables must be > 1")
      XQN <- data.frame(object$context$quanti[,context.quanti,drop=FALSE])	
      # minN is the maximum of factors that can be extracted
      minN <- min(nrow(XQN)- 1, ncol(XQN)) 
      # Si se especificó en la llamada el número de factores ncp
      ncpQN <- ifelse(is.null(ncp), minN, min(ncp, minN))
      # Save in two dataframes (XQN and XQN.initial) quanti variables
      XQN.initial<- XQN
    }
    #################### Final quantitative variables
    
        
    ######### Categorical variables
    if(nQL!=0){
      XQL <- data.frame(object$context$quali[,context.quali,drop=FALSE]) 
      minL <- min(nrow(XQL) - 1, ncol(Xtab.disjonctif(XQL)) - ncol(XQL))
      ncpQL <- ifelse(is.null(ncp), minL, min(ncp, minL))
      if(nQL<2) stop("The number of factors for qualitative variables ncp must be > 1")
      XQL.initial <- XQL
    }
    
    if(nQN+nQL==0) stop("There are not contextual variables")
    
 
  
    
    ##############################################
    # PHASE 4.  Starting. LEXICAL MATRIX AND MARGINALS
    Y <- as.matrix(object$DocTerm) # (Documents x words) (Respondents x words)
    P <- as.matrix(Y/sum(Y))       # Proportion matrix 
    PI. <- apply(P, 1, sum)        # Margin vector % words for each document (respondent)
    P.J <- apply(P, 2, sum)        # Margin vector % words for each word
    # I respondents (docs), J words, K quantitative variables (nQN)
    ##############################################

    
    ####################################
    ### PHASE 5 #########################
    ########### Start funct1 ############					
    funct1 <- function(phi.stand,ncpf,XP,su) {					# Quanti or qualitative variables
      # phi.stand have the eigenvectors (docs x dimensiones)
      # su: scale. TRUE if quantitative scaled 
      # su: if FALSE. If qualitative or quantitative variables they are not scaled (yes centered)
      # P is not passed because is always the same 
      # P is % words in matrix documents x words
      # ncpf is the number of factors
      # XP is the quantitative data matrix centered (and scaled if su=TRUE)
      # L Matrix (words x dimensions)
      # L <- t(P) *phi.stand/P.J  
      L <- sweep(crossprod(P, phi.stand), 1, P.J, "/")
      # mT (Words x categories):  (J words x I docs) * (I docs x variables o categorías) 
      mT <- crossprod(P, XP)	
      C <- crossprod(sweep(XP, 1, PI.^(1/2), "*"), sweep(XP, 1, PI.^(1/2),"*"))	
      # J words x L variables or categories
      W <- sweep(crossprod(t(mT), MASS::ginv(C)), 1, P.J, "/")	
            colnames(W) <- colnames(mT)
      # PCA using L as actif and W as supplementary
      diag.L <- PCA(cbind(L, W), quanti.sup = (ncpf + 1):(ncpf + ncol(mT)),
                    scale.unit = FALSE, ncp = ncpf, row.w = P.J, graph = FALSE)
      # Document coordinates		
      coord.ind <- sweep(crossprod(t(P), diag.L$svd$U), 1, PI., "/")
      # Cos2 of documents
      cos2.ind <- sweep(coord.ind^2, 1, apply(coord.ind^2, 1, sum),"/")
      # Building an object res.doc with coordinates and cos2 for the documents
      res.doc <- list(coord = coord.ind, cos2 = cos2.ind)
      # Building an object res.word with coordinates and cos2 for the docs
      res.word <- list(coord = diag.L$ind$coord, cos2 = diag.L$ind$cos2,  
                       contrib = diag.L$ind$contrib,
                       C=C , Cmenos = MASS::ginv(C))
      res <- list(eig = diag.L$eig, doc = res.doc, word = res.word, diag.L=diag.L, L=L, W=W)
    }
    ########### Final of function funct1 ############					

    
    
    
    
    
    ############################################################################
    # =======  PHASE 6. Call when LexGalt has quantitative variables			
    if(nQN!=0) {		
      diag.XQN <- PCA(XQN, scale.unit = scale, ncp = ncpQN, row.w = PI., graph = FALSE)
      # phi.stand.QN has the eigenvectors (Docs x dimensions)
      phi.stand.QN <- diag.XQN$svd$U
      # XQN matriz docs x quanti variables (centered or scale)  
      # XQN always is centered:
      XQN <- as.matrix(sweep(XQN, 2, apply(XQN, 2, mean.p, PI.), "-"))		
      # If scale=TRUE divided for standard deviation
      if (scale == "TRUE") XQN <- sweep(XQN, 2, apply(XQN, 2, sd.p, PI.), "/")	
      resQN <- funct1(phi.stand.QN, ncpQN, XQN, scale)
      resQN$quanti.var <- resQN$diag.L$quanti.sup	
      #  }
      class(resQN) <- c("LexGalt", "list")
    }			
    # =======  Final Call when LexGalt with quantitative variables	
    

    # =======  Call when LexGalt with qualitative variables			
    if(nQL!=0) {			  # Number of qualitative variables
      diag.XQL <- MCA(XQL, row.w = PI., ncp = ncpQL, level.ventil = level.ventil, graph = FALSE)
     # Building disjonctif matrix
      # Revisar si siempre se centra ******   
      if (ncol(XQL) > 1) # Centering qualitative variables
        XQL <- sweep(Xtab.disjonctif(XQL), 2, apply(Xtab.disjonctif(XQL), 2, mean.p, PI.), "-")
       else XQL <- Xtab.disjonctif(XQL)
      phi.stand.QL <- diag.XQL$svd$U
      # El resultado de funct1 var.contexto cualitativo se almacena en resQL
      resQL <- funct1(phi.stand.QL, ncpQL, XQL, FALSE)			
      resQL$quali.var <- list(coord = resQL$diag.L$quanti.sup$coord, cos2 = resQL$diag.L$quanti.sup$cos2)
      class(resQL) <- c("LexGalt", "list")
    }
    # ======= FInal  Call when LexGalt with qualitative variables			
    


    #______________________________ Ellipses ---------------------
    if(conf.ellip==TRUE)
      if(nb.ellip>0) {
        PJ <- vector(mode = "list", length = nb.ellip)
        #     stop("Hace elipses")
        
        for(n in 1:nb.ellip) {
          samp <- sample(1:nrow(Y), replace = TRUE)
          while (sum(apply(Y[samp, ], 2, sum) > 0) != ncol(Y)) samp <- sample(1:nrow(Y), replace = TRUE)
          Y.samp <- Y[samp, ]
          P.samp <- as.matrix(Y.samp/sum(Y.samp))
          PI.samp <- apply(P.samp, 1, sum)
          P.J.samp <- apply(P.samp, 2, sum)
          PJ[[n]] <- P.J.samp
          
          if(nQL>0) {  X.samp <- sweep(Xtab.disjonctif(XQL.initial[samp, ]),2, 
                                       apply(Xtab.disjonctif(XQL.initial[samp, ]), 2, mean.p, PI.samp), "-")
          phi.stand.samp <- phi.stand.QL[samp, ]
          T.samp <- crossprod(P.samp, X.samp)
          C.samp <- crossprod(sweep(X.samp, 1, PI.samp^(1/2), "*"), sweep(X.samp, 1, PI.samp^(1/2), "*"))
          
          if (n == 1) {  
            LQ.samp <- sweep(crossprod(P.samp, phi.stand.samp), 1, P.J.samp, "/")
            WQ.samp <- sweep(crossprod(t(T.samp), MASS::ginv(C.samp)), 1, P.J.samp, "/")
            WQ.stand.samp <- sweep(sweep(crossprod(t(T.samp), 
                                                   MASS::ginv(C.samp)), 1, P.J.samp, "/"), 2, apply(sweep(crossprod(t(T.samp), 
                                                   MASS::ginv(C.samp)), 1, P.J.samp, "/"), 2, sd.p, P.J.samp), "/")
          }
          else {       
            LQ.samp <- rbind(LQ.samp, sweep(crossprod(P.samp, phi.stand.samp), 1, P.J.samp, "/"))
            WQ.samp <- cbind(WQ.samp, sweep(crossprod(t(T.samp),MASS::ginv(C.samp)), 1, P.J.samp, "/"))
            WQ.stand.samp <- rbind(WQ.stand.samp, sweep(sweep(crossprod(t(T.samp), 
                          MASS::ginv(C.samp)), 1, P.J.samp, "/"), 2, apply(sweep(crossprod(t(T.samp),
                          MASS::ginv(C.samp)), 1, P.J.samp, "/"), 2, sd.p, P.J.samp), "/"))
          }
          }  # End nQL>0
          
          if(nQN>0) {
            X.samp <- as.matrix(sweep(XQN.initial[samp, ], 2, apply(XQN.initial[samp,], 2, 
                                                                    mean.p, PI.samp), "-"))
            if(scale==TRUE) 
              X.samp <- sweep(X.samp, 2, apply(X.samp, 2, sd.p, PI.samp), "/")
            phi.stand.samp <- phi.stand.QN[samp, ]
            T.samp <- crossprod(P.samp, X.samp)
            C.samp <- crossprod(sweep(X.samp, 1, PI.samp^(1/2), "*"), sweep(X.samp, 1, PI.samp^(1/2), "*"))
            if (n == 1) {
              LN.samp <- sweep(crossprod(P.samp, phi.stand.samp), 1, P.J.samp, "/")
              WN.samp <- sweep(crossprod(t(T.samp), MASS::ginv(C.samp)), 1, P.J.samp, "/")
              WN.stand.samp <- sweep(sweep(crossprod(t(T.samp), 
                              MASS::ginv(C.samp)), 1, P.J.samp, "/"), 2, apply(sweep(crossprod(t(T.samp), 
                              MASS::ginv(C.samp)), 1, P.J.samp, "/"), 2, sd.p, P.J.samp), "/")
            }
            else {
              LN.samp <- rbind(LN.samp, sweep(crossprod(P.samp, phi.stand.samp), 1, P.J.samp, "/"))
              WN.samp <- cbind(WN.samp, sweep(crossprod(t(T.samp),MASS::ginv(C.samp)), 1, P.J.samp, "/"))
              WN.stand.samp <- rbind(WN.stand.samp, sweep(sweep(crossprod(t(T.samp), 
                              MASS::ginv(C.samp)), 1, P.J.samp, "/"), 2, apply(sweep(crossprod(t(T.samp),
                                                                                                                                           MASS::ginv(C.samp)), 1, P.J.samp, "/"), 2, sd.p, P.J.samp), "/"))
            }
          } # End if(nQN>0) 
        } ### End for nb.ellip  
          
          
        
        if(nQL>0)  {
          rownames(LQ.samp) <- paste(rep(rownames(resQL$L), nb.ellip), rep(1:nb.ellip, each = nrow(resQL$L)), sep = "")
          freq.ellip.coord <- as.data.frame(PCA(rbind(resQL$L, LQ.samp), ncp = ncpQL, 
                                                ind.sup = (ncol(Y) + 1):((nb.ellip + 1) * ncol(Y)), row.w = P.J, 
                                                scale.unit = FALSE, graph = FALSE)$ind.sup$coord)
          colnames(WQ.samp) <- paste(rep(colnames(resQL$W), nb.ellip), rep(1:nb.ellip, each = ncol(resQL$W)), sep = "")
          
          var.ellip.coord <- as.data.frame(PCA(cbind(resQL$L, WQ.samp), 
                                               quanti.sup = (ncpQL + 1):(ncpQL + ncol(WQ.samp)), scale.unit = FALSE, 
                                               ncp = ncpQL, row.w = P.J, graph = F)$quanti.sup$coord)
          freq.ellip.coord$FREQ <- rep(rownames(resQL$L), nb.ellip)
          var.ellip.coord$VAR <- rep(colnames(XQL), nb.ellip)
          resQL$ellip <- list(word = freq.ellip.coord, var = var.ellip.coord)
          class(resQL) <- c("LexGalt", "list")
        } # End if(nQL>0) 
        
        if(nQN>0)  {
          rownames(LN.samp) <- paste(rep(rownames(resQN$L), nb.ellip), rep(1:nb.ellip, each = nrow(resQN$L)), sep = "")
          freq.ellip.coord <- as.data.frame(PCA(rbind(resQN$L, LN.samp), ncp = ncpQN, 
                                                ind.sup = (ncol(Y) + 1):((nb.ellip + 1) * ncol(Y)), row.w = P.J, 
                                               scale.unit = FALSE, graph = FALSE)$ind.sup$coord)
            for (n in 1:nb.ellip) {
            aux <- as.matrix(freq.ellip.coord[(((n - 1) * ncol(Y)) + 1):(n * ncol(Y)), 1:ncpQN])
            aux.cent <- sweep(aux, 2, apply(aux, 2, mean.p, PJ[[n]]), "-")
            aux.stand <- as.matrix(sweep(aux.cent, 2, apply(aux.cent, 2, sd.p, PJ[[n]]), "/"))
            if (n == 1) 
              var.ellip.coord <- as.data.frame(crossprod(WN.stand.samp[(((n -1) * ncol(Y)) + 1):(n * ncol(Y)), ],
                                                         sweep(aux.stand, 1, PJ[[n]], "*")))
            else var.ellip.coord <- rbind(var.ellip.coord, as.data.frame(crossprod(WN.stand.samp[(((n -1)
                                 * ncol(Y)) + 1):(n * ncol(Y)), ], sweep(aux.stand, 1, PJ[[n]], "*"))))
                         }  # End For
          
          freq.ellip.coord$FREQ <- rep(rownames(resQN$L), nb.ellip)
          var.ellip.coord$VAR <- rep(colnames(XQN), nb.ellip)
          resQN$ellip <- list(word = freq.ellip.coord, var = var.ellip.coord)
        } # End if(nQN>0)
        
        #______________________________ Final Ellipses ---------------------   
      }

    if(nQL==0) resQL <- ""
    if(nQN==0) resQN <- ""
    
    resNew <- list(SQL = resQL, SQN=resQN)
    class(resNew) <- c("LexGalt","list") 
    #______________________________ Plots simple ---------------------
    if (graph) {
      if(nQN!=0) { 
        plot.LexGalt(resNew, type="QN", eigen=TRUE, new.plot = TRUE, title="Eigenvalues from Quantitative variables")
        plot.LexGalt(resNew, type="QN", selDoc="ALL",axes = axes, new.plot = TRUE, title="Documents from Quantitative variables")
        plot.LexGalt(resNew, type="QN", selQuantiVar="ALL", axes = axes, conf.ellip = conf.ellip, new.plot = TRUE)
        
        if (nrow(resNew$SQN$word$coord) < 50) {
          plot.LexGalt(resNew, type="QN", selWord="ALL", autoLab="yes", axes = axes, conf.ellip = conf.ellip, new.plot = TRUE,
                       title="Words from Quantitative variables")
        }
        else {
          plot.LexGalt(resNew, type="QN", selWord="coord 49", autoLab="yes", axes = axes, conf.ellip = conf.ellip, new.plot = TRUE,
                       title="Words from Quantitative variables")
          warning("The first 50 words that have the highest coordinates on the 2 dimensions of your plot are drawn.")
        }
      } # End nQN
      if(nQL>0) { 
        plot.LexGalt(resNew, type="QL", selDoc="ALL", axes = axes, new.plot = TRUE, title="Documents from Qualitative variables")
        plot.LexGalt(resNew, type="QL", selQualiVar="ALL", axes = axes, new.plot = TRUE, title="Qualitative variables")
        
        if (nrow(resNew$SQL$word$coord) < 50) {
          plot.LexGalt(resNew, type="QL", selWord="ALL", conf.ellip = conf.ellip, new.plot = TRUE,
                       title="Words from Qualitative variables")
        }
        else {
          plot.LexGalt(resNew, type="QL", selWord="coord 49",autoLab="yes", conf.ellip = conf.ellip, new.plot = TRUE,
                       title="Words from Qualitative variables")
          warning("The first 50 words that have the highest coordinates on the 2 dimensions of your plot are drawn.")     
        }
      } # End nQL
    } # End graph
    
    #___________________________ Final plots
    
    return(resNew)
    
  } # Final of if(num.group==1), simple
  
   else {
          ############################  MULTIPLE

     Y<-vector(mode='list',length=num.group)
     for(i in 1:num.group) Y[[i]] <- as.matrix(object[[i]]$DocTerm)
     
     if(is.null(label.group)) label.group <- paste("GROUP",1:num.group,sep = ".")	
     if(length(label.group) != num.group) stop("The name of groups is != number of groups")
     name.group <- label.group
     
     N<-sum(unlist(Y))
     Nl<-sapply(Y,sum)
     num.ind<-sapply(Y,nrow)
     num.freq<-sapply(Y,ncol)
     names(num.freq)<-names(num.ind)<-names(Nl)<-label.group
     Pl<-mapply(function(y,nl) y/nl,Y,Nl,SIMPLIFY = FALSE)
     pi.l<-lapply(Pl,function(p) apply(p,1,sum))
     p.jl<-lapply(Pl,function(p) apply(p,2,sum))
     wl<-Nl/N
     
     DIl<-mapply(function(pi,wl) pi*wl,pi.l,wl,SIMPLIFY = FALSE)
     DJl<-mapply(function(pj,wl) pj*wl,p.jl,wl,SIMPLIFY = FALSE)
     DI<-unlist(DIl)
     DJ<-unlist(DJl)
     
     nXQL <- nXQN <- 0 

          
     if(nQL!=0) {
       XQL<-vector(mode='list',length=num.group)
       nXQL <- length(context.quali) 
       if(nXQL<2) stop("The number of qualitative variables must be > 1")
       for(i in 1:num.group) XQL[[i]] <- as.matrix(object[[i]]$context$quali[context.quali])
     }
     Xl<-vector(mode='list',length=num.group)
     XG<-data.frame()
     resQL <- resQN <- NULL 
     
     if(nXQL!=0) {
       for(i in 1:num.group) Xl[[i]]<-as.matrix(Xtab.disjonctif(XQL[[i]]))
       if(unique(sapply(XQL,ncol))!=1) Xl<-mapply(function(x,pi) sweep(x,2,apply(x,2,mean.p,pi),"-"),Xl,pi.l,SIMPLIFY = FALSE)
       
       for (i in 1:num.group) XG<-as.matrix(rbind(XG,Xl[[i]]))
       Yal<-mapply(function(y,x) crossprod(y,x),Y,Xl,SIMPLIFY = FALSE)
       Pal<-mapply(function(ya) ya/N,Yal,SIMPLIFY = FALSE)
       C<-crossprod(sweep(XG,1,DI^(1/2),"*"),sweep(XG,1,DI^(1/2),"*"))
       Ml<-mapply(function(di,dj,w,x) diag(dj)%*%matrix(nrow=length(dj),ncol=length(di),1/w)%*%diag(di)%*%x,DIl,DJl,wl,Xl,SIMPLIFY = FALSE)				
       Zl<-mapply(function(pa,m,dj) sweep(pa-m,1,dj,"/")%*%MASS::ginv(C),Pal,Ml,DJl,SIMPLIFY=FALSE)
       ponde<-mapply(function(z,dj) Re(eigen(t(z)%*%diag(dj)%*%z%*%C)$values[1]),Zl,DJl,SIMPLIFY = FALSE)				
       row.w<-unlist(mapply(function(dj,pond) dj/pond,DJl,ponde,SIMPLIFY = FALSE))				
       Z<-data.frame()				
       
       for (i in 1:num.group) Z<-as.matrix(rbind(Z,Zl[[i]]))				
       diag.MFAGALT<-eigen(t(Z)%*%diag(row.w)%*%Z%*%C)				
       eig<-Re(diag.MFAGALT$values)				
       eig<-eig[1:(length(eig)-unique(sapply(XQL,ncol)))]				
       vp<-as.data.frame(matrix(NA,length(eig),3))				
       rownames(vp)<-paste("comp",1:length(eig))				
       colnames(vp)<- c("eigenvalue","percentage of variance","cumulative percentage of variance")				
       vp[,"eigenvalue"]<-eig				
       vp[,"percentage of variance"]<-(eig/sum(eig))*100				
       vp[,"cumulative percentage of variance"]<-cumsum(vp[,"percentage of variance"])	
       U<-sweep(Re(diag.MFAGALT$vectors[,1:length(eig)]),2,sqrt(diag(t(Re(diag.MFAGALT$vectors[,1:length(eig)]))%*%C%*%Re(diag.MFAGALT$vectors[,1:length(eig)]))),"/")				
       coord.freq<-Z%*%C%*%U				
       contrib.freq<-sweep(sweep(coord.freq^2,1,row.w,FUN="*")*100,2,eig,FUN="/")				
       dist2.freq<-diag(Z%*%C%*%t(Z))				
       cos2.freq<-sweep(as.matrix(coord.freq^2),1,dist2.freq,FUN="/")				
       coord.var<-sweep(U,2,sqrt(eig),"*")				
       dist2.var<-apply(sweep(Z,1,sqrt(row.w),FUN="*")^2,2,sum)				
       cos2.var<-sweep(as.matrix(coord.var^2),1,dist2.var,FUN="/")	
       coord.freq.partiel<-mapply(function(z) z%*%C%*%U,Zl,SIMPLIFY=FALSE)				
       coord.var.partiel<-mapply(function(z,f,dj,pond) sweep(t(z)%*%sweep(f,1,dj/pond,"*")*num.group,2,sqrt(eig),"/"),Zl,coord.freq.partiel,DJl,ponde,SIMPLIFY=FALSE)				
       tab<-(Z%*%C%*%t(Z))^2				
       tab<-sweep(sweep(tab,2,row.w,"*"),1,row.w,"*")				
       Lg<-matrix(0,num.group+1,num.group+1)				
       ind.gl<-0				

     
       for (gl in 1:num.group) {				
         ind.gc<-0			
         for (gc in 1:num.group) {			
           Lg[gl,gc]<-Lg[gc,gl]<-sum(tab[(ind.gl+1):(ind.gl+num.freq[gl]),(ind.gc+1):(ind.gc+num.freq[gc])])		
           ind.gc<-ind.gc+num.freq[gc]			
         }			
         Lg[num.group+1,gl]<-Lg[gl,num.group+1]<-sum(tab[(ind.gl+1):(ind.gl+num.freq[gl]),1:ncol(tab)])/eig[1]			
         ind.gl<-ind.gl+num.freq[gl]			
       }	
       Lg[num.group+1,num.group+1]<-sum(tab[1:ncol(tab),1:ncol(tab)])/(eig[1]^2)				
       RV<-sweep(Lg,2,sqrt(diag(Lg)),"/")				
       RV<-sweep(RV,1,sqrt(diag(Lg)),"/")				
       contrib.group<-matrix(NA,num.group,ncol(U))				
       dist2.group<-vector(length=num.group)				
       freq.gr<-0
       
       for (g in 1:num.group) {				
         if (g==1) contrib.group[g,]<-apply(contrib.freq[1:num.freq[g],],2,sum)			
         else contrib.group[g,]<-apply(contrib.freq[(freq.gr+1):(freq.gr+num.freq[g]),],2,sum)			
         ponde.tot<-eigen(t(Zl[[g]])%*%diag(DJl[[g]])%*%Zl[[g]]%*%C)$values	
         dist2.group[g]<-sum((ponde.tot/ponde[[g]])^2)			
         freq.gr<-freq.gr+num.freq[g]			
       }
       coord.group<-sweep(contrib.group/100,2,eig,"*")				
       cos2.group<-sweep(coord.group^2,1,dist2.group,"/")				
       rownames(coord.var)<-rownames(cos2.var)<-colnames(XG)					
       colnames(coord.freq)<-colnames(contrib.freq)<-colnames(cos2.freq)<-colnames(coord.var)<-colnames(cos2.var)<-paste("Dim",c(1:ncol(U)),sep = ".")
       
       names(coord.var.partiel)<-label.group	
       
       for(i in 1:num.group){				
         rownames(coord.var.partiel[[i]])<-colnames(XG)			
         colnames(coord.var.partiel[[i]])<-paste("Dim",c(1:ncol(U)),sep = ".")			
       }
       rownames(coord.group)<-rownames(contrib.group)<-rownames(cos2.group)<-label.group				
       colnames(coord.group)<-colnames(contrib.group)<-colnames(cos2.group)<-paste("Dim",c(1:ncol(U)),sep = ".")				
       rownames(Lg)<-colnames(Lg)<-rownames(RV)<-colnames(RV)<-c(label.group,"MFA-GALT")				
       res<-list()				
       res$eig<-vp				
       res$word<-list(coord=coord.freq,cos2=cos2.freq,contrib=contrib.freq)

       type <- "n"
       res$quali.var<-list(coord=coord.var,cos2=cos2.var,coord.partial=coord.var.partiel)	
       res$group<-list(Lg=Lg,RV=RV,coord=coord.group,contrib=contrib.group,cos2=cos2.group)				
       res$call<-list(num.groups=num.group,name.groups=label.group,num.freq=num.freq,type=type)				
       
       class(res) <- c("LexGalt", "list")	# Remove MFAGALT to the end	
       resQL <- res	
       class(resQL) <- c("LexGalt", "list")		
     } # FInal     if(nXQL!=0) 
     
     if(nQN!=0) {
       XQN<-vector(mode='list',length=num.group)
       nXQN <- length(context.quanti)
       if(nXQN<2) stop("The number of quantitative variables must be > 1")
       for(i in 1:num.group) {
         XQN[[i]] <- as.matrix(object[[i]]$context$quanti[context.quanti])
         Xl[[i]]<-as.matrix(XQN[[i]])}
       Xl<-mapply(function(x,pi) sweep(x,2,apply(x,2,mean.p,pi),"-"),Xl,pi.l,SIMPLIFY = FALSE)
       XG<-data.frame()
       
       for (i in 1:num.group) XG<-as.matrix(rbind(XG,Xl[[i]]))
       Yal<-mapply(function(y,x) crossprod(y,x),Y,Xl,SIMPLIFY = FALSE)
       Pal<-mapply(function(ya) ya/N,Yal,SIMPLIFY = FALSE)
       C<-crossprod(sweep(XG,1,DI^(1/2),"*"),sweep(XG,1,DI^(1/2),"*"))
       Ml<-mapply(function(di,dj,w,x) diag(dj)%*%matrix(nrow=length(dj),ncol=length(di),1/w)%*%diag(di)%*%x,DIl,DJl,wl,Xl,SIMPLIFY = FALSE)				
       Zl<-mapply(function(pa,m,dj) sweep(pa-m,1,dj,"/")%*%MASS::ginv(C),Pal,Ml,DJl,SIMPLIFY=FALSE)
       ponde<-mapply(function(z,dj) Re(eigen(t(z)%*%diag(dj)%*%z%*%C)$values[1]),Zl,DJl,SIMPLIFY = FALSE)				
       row.w<-unlist(mapply(function(dj,pond) dj/pond,DJl,ponde,SIMPLIFY = FALSE))				
       Z<-data.frame()				
       for (i in 1:num.group) Z<-as.matrix(rbind(Z,Zl[[i]]))				
       diag.MFAGALT<-eigen(t(Z)%*%diag(row.w)%*%Z%*%C)				
       eig<-Re(diag.MFAGALT$values)				
       vp<-as.data.frame(matrix(NA,length(eig),3))				
       rownames(vp)<-paste("comp",1:length(eig))				
       colnames(vp)<- c("eigenvalue","percentage of variance","cumulative percentage of variance")				
       vp[,"eigenvalue"]<-eig				
       vp[,"percentage of variance"]<-(eig/sum(eig))*100				
       vp[,"cumulative percentage of variance"]<-cumsum(vp[,"percentage of variance"])
       
       U<-sweep(Re(diag.MFAGALT$vectors[,1:length(eig)]),2,sqrt(diag(t(Re(diag.MFAGALT$vectors[,1:length(eig)]))%*%C%*%Re(diag.MFAGALT$vectors[,1:length(eig)]))),"/")				
       coord.freq<-Z%*%C%*%U				
       contrib.freq<-sweep(sweep(coord.freq^2,1,row.w,FUN="*")*100,2,eig,FUN="/")				
       dist2.freq<-diag(Z%*%C%*%t(Z))				
       cos2.freq<-sweep(as.matrix(coord.freq^2),1,dist2.freq,FUN="/")				
       coord.var<-sweep(U,2,sqrt(eig),"*")				
       dist2.var<-apply(sweep(Z,1,sqrt(row.w),FUN="*")^2,2,sum)				
       cos2.var<-sweep(as.matrix(coord.var^2),1,dist2.var,FUN="/")	
       coord.freq.partiel<-mapply(function(z) z%*%C%*%U,Zl,SIMPLIFY=FALSE)				
       coord.var.partiel<-mapply(function(z,f,dj,pond) sweep(t(z)%*%sweep(f,1,dj/pond,"*")*num.group,2,sqrt(eig),"/"),Zl,coord.freq.partiel,DJl,ponde,SIMPLIFY=FALSE)				
       # Si es numerica
       cor.var<-sweep(sweep(t(Z)%*%sweep(coord.freq,1,row.w,"*"),1,apply(Z,2,function(V, poids) res <- sqrt(sum(V^2 * poids, na.rm = TRUE)),row.w),"/"),2,sqrt(eig),"/")
       tab<-(Z%*%C%*%t(Z))^2				
       tab<-sweep(sweep(tab,2,row.w,"*"),1,row.w,"*")				
       Lg<-matrix(0,num.group+1,num.group+1)				
       ind.gl<-0				
       for (gl in 1:num.group) {				
         ind.gc<-0			
         for (gc in 1:num.group) {			
           Lg[gl,gc]<-Lg[gc,gl]<-sum(tab[(ind.gl+1):(ind.gl+num.freq[gl]),(ind.gc+1):(ind.gc+num.freq[gc])])		
           ind.gc<-ind.gc+num.freq[gc]			
         }			
         Lg[num.group+1,gl]<-Lg[gl,num.group+1]<-sum(tab[(ind.gl+1):(ind.gl+num.freq[gl]),1:ncol(tab)])/eig[1]			
         ind.gl<-ind.gl+num.freq[gl]			
       }				
       
       Lg[num.group+1,num.group+1]<-sum(tab[1:ncol(tab),1:ncol(tab)])/(eig[1]^2)				
       RV<-sweep(Lg,2,sqrt(diag(Lg)),"/")				
       RV<-sweep(RV,1,sqrt(diag(Lg)),"/")				
       contrib.group<-matrix(NA,num.group,ncol(U))				
       dist2.group<-vector(length=num.group)				
       freq.gr<-0				

       
       for (g in 1:num.group) {				
         if (g==1) contrib.group[g,]<-apply(contrib.freq[1:num.freq[g],],2,sum)			
         else contrib.group[g,]<-apply(contrib.freq[(freq.gr+1):(freq.gr+num.freq[g]),],2,sum)			
         ponde.tot<-eigen(t(Zl[[g]])%*%diag(DJl[[g]])%*%Zl[[g]]%*%C)$values			
         dist2.group[g]<-sum((ponde.tot/ponde[[g]])^2)			
         freq.gr<-freq.gr+num.freq[g]			
       }
       
       
       coord.group<-sweep(contrib.group/100,2,eig,"*")				
       cos2.group<-sweep(coord.group^2,1,dist2.group,"/")				
       rownames(coord.var)<-rownames(cos2.var)<-colnames(XG)					
       colnames(coord.freq)<-colnames(contrib.freq)<-colnames(cos2.freq)<-colnames(coord.var)<-colnames(cos2.var)<-paste("Dim",c(1:ncol(U)),sep = ".")
       rownames(cor.var)<-colnames(XG)
       colnames(cor.var)<-paste("Dim",c(1:ncol(U)),sep = ".")
       
       
       names(coord.var.partiel)<-label.group				
       for(i in 1:num.group){				
         rownames(coord.var.partiel[[i]])<-colnames(XG)			
         colnames(coord.var.partiel[[i]])<-paste("Dim",c(1:ncol(U)),sep = ".")			
       }				

       rownames(coord.group)<-rownames(contrib.group)<-rownames(cos2.group)<-label.group				
       colnames(coord.group)<-colnames(contrib.group)<-colnames(cos2.group)<-paste("Dim",c(1:ncol(U)),sep = ".")				
       rownames(Lg)<-colnames(Lg)<-rownames(RV)<-colnames(RV)<-c(label.group,"MFA-GALT")				
       res<-list()				
       res$eig<-vp				
       res$word<-list(coord=coord.freq,cos2=cos2.freq,contrib=contrib.freq)	    		
       res$quanti.var<-list(coord=coord.var,cor=cor.var,cos2=cos2.var,coord.partial=coord.var.partiel)
       res$group<-list(Lg=Lg,RV=RV,coord=coord.group,contrib=contrib.group,cos2=cos2.group)
       #   \item{type}{the type of variables: "c" or "s" for quantitative variables and "n" for categorical variables. The difference is that for "s" variables are scaled to unit variance (by default, variables are scaled to unit variance)}
       
       type <- ifelse(scale==TRUE,"c","s")
       res$call<-list(num.groups=num.group,name.groups=label.group,num.freq=num.freq,type=type)
       
       class(res) <- c("LexGalt", "list")	# Remove MFAGALT	
       resQN <- res	
       class(resQN) <- c("LexGalt", "list")
     }
     
     resNew <- list(MQL=resQL,MQN=resQN)
     class(resNew) <- c("LexGalt", "list") 
 
     
     if(graph) {
       if(nQL!=0){
         plot.LexGalt(resNew, type="QL", eigen=TRUE, new.plot = TRUE, title="Eigenvalues from Quantitative variables")
         plot.LexGalt(resNew, type="QL",plot.group="TRUE",new.plot = TRUE, title="Groups representation (LexGalt) from qualitative variables")
  #       plot.LexGalt(resNew, type="QL", selDoc="ALL",axes = axes, new.plot = TRUE, title="Documents from Qualitative variables")
         plot.LexGalt(resNew, type="QL", selQualiVar="ALL", axes = axes, new.plot = TRUE, title="Qualitative variables")
         plot.LexGalt(resNew, type="QL", selWord="ALL", new.plot = TRUE,
                      title="Words from Qualitative variables")
       }
       
       if(nQN!=0){
         plot.LexGalt(resNew, type="QN", eigen=TRUE, new.plot = TRUE, title="Eigenvalues from Quantitative variables")
                  plot.LexGalt(resNew, type="QN",plot.group="TRUE", new.plot = TRUE,
                      title="Groups representation (LexGalt) from quantitative variables")
         plot.LexGalt(resNew, type="QN", selDoc="ALL",axes = axes, new.plot = TRUE, title="Documents from Quantitative variables")
         plot.LexGalt(resNew , type="QN", selQuantiVar="ALL", axes = axes, new.plot = TRUE, partial=TRUE, title="Quantitative variables")
         plot(resNew, type="QN", selWord="ALL", new.plot=TRUE, title="Words from Quantitative variables")
       } # FINAL nQN
     } # Final graph
     
     return(resNew)
   } # Final multiple or simple, if(num.group==1)
  
} # Final LexGalt

#' @importFrom grDevices adjustcolor col2rgb rgb
#' @importFrom graphics arrows points
#' @export
plot.LexGalt <- function(x, type="QL", selDoc=NULL, selWord=NULL, selQualiVar=NULL, 
  selQuantiVar=NULL, conf.ellip=FALSE, selWordEllip=NULL, selQualiVarEllip=NULL,
  selQuantiVarEllip=NULL, level.conf=0.95, eigen=FALSE, title = NULL, axes = c(1, 2), 
  xlim = NULL, ylim = NULL, col.eig="grey", col.doc = "black", col.word = NULL, 
  col.quali = "blue", 
  col.quanti = "blue",col="grey", pch = 20, label = TRUE, 
  autoLab = c("auto", "yes", "no"),  palette = NULL, unselect = 1,  
  selCov=FALSE, selGroup="ALL", partial=FALSE, plot.group=FALSE,
  col.group=NULL, label.group=NULL, legend=TRUE, pos.legend="topleft", new.plot = TRUE,cex=1, ...)
{
  

    if(!inherits(x,"LexGalt")) stop("non convenient data")
    if(is.null(selDoc) & is.null(selWord) & is.null(selQualiVar)&
     is.null(selQuantiVar) & conf.ellip==FALSE & is.null(selWordEllip) & is.null(selQualiVarEllip) &
     is.null(selQuantiVarEllip) & eigen==FALSE) stop("There is nothing to plot\nSelect some argument")
  
     if(unselect==TRUE) unselect <- 1
#######################  Previous control
    if (is.numeric(unselect)) 
    if ((unselect > 1) | (unselect < 0)) 
      stop("unselect should be between 0 and 1")
  
  somethingsel <- all(sapply(list(selDoc, selWord, selQualiVar, selQuantiVar, selWordEllip, selQualiVarEllip, selQuantiVarEllip,
                                    selGroup, eigen), is.null))
  if(somethingsel) stop("There is nothing selected to plot")

  
####################### Functions
  selX <- function(sel1)
  {
    xx <- gregexpr(pattern =' ',sel1)[[1]][1]-1
    xx <- substr(sel1, 1, xx)
  }
  autoLab <- match.arg(autoLab, c("auto", "yes", "no"))
  if (autoLab == "yes") 
    autoLab = TRUE
  if (autoLab == "no") 
    autoLab = FALSE
  auto.Lab <- autoLab
  

    PALETTE <- palette(c("black", "red", "green3", "blue", "cyan", 
                         "magenta", "darkgray", "darkgoldenrod", "darkgreen", 
                         "violet", "turquoise", "orange", "lightpink", 
                         "lavender", "yellow", "lightgreen", "lightgrey", 
                         "lightblue", "darkkhaki", "darkmagenta", "darkolivegreen", 
                         "lightcyan", "darkorange", "darkorchid", "darkred", 
                         "darksalmon", "darkseagreen", "darkslateblue", 
                         "darkslategray", "darkslategrey", "darkturquoise", 
                         "darkviolet", "lightgray", "lightsalmon", "lightyellow", 
                         "maroon")) 
    if (!is.null(palette)) PALETTE[1:length(palette)] <- palette
                   
  
  selectionX <- function(sel1, xobj, bType, axx, axy)
  {
    if(is.null(sel1)) return(sel1)
    xx <- ""
    if(length(sel1)==1){
      if(sel1=="ALL") sel1 <- c(1:dim(xobj$coord)[1])
    }
    if(length(sel1)==1) {
      if(sel1=="meta") sel1 <- "meta 3"
      # if(sel1=="char") sel1 <- "char 0.05"
      xx <- gregexpr(pattern =' ',sel1)[[1]][1]-1
      xx <- substr(sel1, 1, xx)
    }  }
########## End Functions

  xlimT <- xlim ; ylimT <- ylim


  # Check if it is a Multiple analysis
  typeSM <- ifelse((length(x$SQL)+ length(x$SQN)) > 0, "typeS", "typeM")
  
  if(typeSM=="typeS") {
    if(type=="QL") {        # CATEGORICAL VARIABLES
      if(length(x$SQL)==1) stop("There are not categorical variables selected in LexGalt object")
      res.cagalt <- x$SQL }
    if(type=="QN") {
      if(length(x$SQN)==1)stop("There are not quantitative variables selected in LexGalt object")
      res.cagalt <- x$SQN }
    lab.x <- paste("Dim ", axes[1], " (", format(res.cagalt$eig[axes[1],2],
                                                 nsmall = 2, digits = 2), "%)", sep = "")
    lab.y <- paste("Dim ", axes[2], " (", format(res.cagalt$eig[axes[2],2],
                                                 nsmall = 2, digits = 2), "%)", sep = "")
    

  
###################################### Plot eigenvalues   
  if(eigen) {
    #  nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")) # TRUE  when inside RStudio ,  # FALSE when outside RStudio
    # Cambiado
    if(new.plot) if(!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) {
      dev.new(width = min(14, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1])), height = 8) # Outside RStudio 
    } else dev.new()
    
    if(is.null(title)) titleE <- "Eigenvalues" else titleE <- title
    barplot(res.cagalt$eig[, 1], main = titleE, col=col.eig, cex.axis=cex, cex.names=cex,
            names.arg = paste("dim",1:nrow(res.cagalt$eig)))}
###################################### End Plot eigenvalues 


    
###################################### Document selection selDoc. Only for separate analysis. Answers if open question
  if (!is.null(selDoc)) {
    if (is.null(title)) 
      titre <- "Document factor map (LexGalt)"
    else titre <- title
    coord.doc <- res.cagalt$doc$coord[, axes, drop = FALSE]
    num.doc <- nrow(coord.doc)
    xlim <- xlimT; ylim <- ylimT
    if (is.null(xlim)) xlim <- c(min(coord.doc[, 1]), max(coord.doc[, 1])) * 1.2
    if (is.null(ylim)) ylim <- c(min(coord.doc[, 2]), max(coord.doc[, 2])) * 1.2
    selection <- NULL
    rdo<-""
    
    if (mode(selDoc) == "numeric") selection <- selDoc
    else if(length(selDoc)==1) {
      if (selDoc=="ALL")  selection <- c(1:num.doc)
      else rdo <- selX(selDoc)
    } else { selection <-  which(rownames(coord.doc) %in% selDoc)}
  
    
    
    if (rdo=="coord") 
      selection <- (rev(order(apply(coord.doc^2, 1, max))))[1:min(num.doc,
                 sum(as.integer(unlist(strsplit(selDoc, "coord"))), na.rm = T))]
    if (rdo=="cos2") {
      selcos2 <- as.numeric(substr(selDoc, 5, nchar(selDoc)))
      selection <- which(apply(res.cagalt$doc$cos2[, axes], 1, sum)> selcos2)
    }
    if (rdo=="contrib") stop("There is not possible to use contrib selection for selDoc")
    
    if(is.null(selection)) if(rdo=="") if(length(selDoc)==1) selection <- which(rownames(coord.doc) %in% selDoc)
    if(length(selection)==0) stop("There are not selected elements to plot")
    
    if(new.plot) if(!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) {  
      dev.new(width = min(14, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1])), height = 8) # Outside RStudio 
   } else dev.new() 

    plot(0, 0, main = titre, xlab = lab.x, ylab = lab.y, xlim = xlim, ylim = ylim, col = "white", asp = 1, ...)
    abline(v = 0, lty = 2, ...)
    abline(h = 0, lty = 2, ...)
 ipch <- 20
    if(!is.null(pch)) {                                     # symbols in graphs
      ipch <- rep(pch[1], num.doc)   # By defect symbol 20
    } 
    coo <- labe <- coll <- fonte <- NULL
    
  #  coo <- labe <- coll <- ipch <- fonte <- NULL
    coo <- coord.doc
    if (label) 
      labe <- rownames(coord.doc)
    else labe <- rep("", num.doc)

        
    if(length(col.doc)==1)  coll <- rep(col.doc, num.doc) else{
     if(length(col.doc)< num.doc) col.doc <- c(col.doc, rep("grey", num.doc-length(col.doc)))
      if(length(col.doc)> num.doc) col.doc <- col.doc[1:num.doc] 
      coll <- col.doc
    }

    fonte <- rep(1, num.doc)

    if (!is.null(selection)) 
        coll[!((1:length(coll)) %in% selection)] = rgb(t(col2rgb(coll[!((1:length(coll)) %in% selection)])), 
                                                       alpha = 255 * (1 - unselect), maxColorValue = 255)

    pos.text <- NULL
    if (any(labe != "")) {
      autoLab <- FALSE
      if (auto.Lab == "auto") autoLab = (length(selection) < 50)
      if (auto.Lab == TRUE) autoLab <- TRUE
      if (autoLab == TRUE) {
        if(is.null(pch))    warning("autoLab must draw points")
        autoLab(coo[selection, 1], y = coo[selection,2],
                labels = labe[selection], col = coll[selection], 
                font = fonte[selection], cex=cex,...)
        points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
      }  # End of autolab ==TRUE
      else {  
        # It is not autolab
        if(!is.null(pch)) { 
          points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
          pos.text <- 3
        }
        text(coo[labe != "", 1], y = coo[labe != "",2],
             labels = labe[labe != ""], col = coll[labe != ""], 
             font = fonte[labe != ""], pos=pos.text , cex=cex,...)
      }
    } else {
      # Without labels
      if(is.null(pch)) stop("You must plot points and/or text")
      points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
    }
   }  # End plot documents
###################################### Document selection selDoc

    
    
##############################  Selection of words selWord
  if (!is.null(selWord)) {
    if (is.null(title)) 
      titre <- "Word factor map (LexGalt)"
    else titre <- title
    coord.word <- res.cagalt$word$coord[, axes, drop = FALSE]	
    num.word <- nrow(coord.word)
    
    xlim <- xlimT; ylim <- ylimT
    if (is.null(xlim)) 
      xlim <- c(min(coord.word[, 1]), max(coord.word[, 1])) * 1.2
    if (is.null(ylim)) 
      ylim <- c(min(coord.word[, 2]), max(coord.word[, 2])) * 1.2
    selection <- NULL
    rdo<-""
    
    # Selection of words
    if (mode(selWord) == "numeric") 
      selection <- selWord
    else if(length(selWord)==1) {
      if (selWord=="ALL")  selection <- c(1:num.word)
      else rdo <- selX(selWord)
    } else { selection <-  which(rownames(coord.word) %in% selWord)}
  
    
    if (rdo=="coord") 	
      selection <- (rev(order(apply(coord.word, 1, max))))[1:min(nrow(coord.word), 	
            sum(as.integer(unlist(strsplit(selWord, "coord"))), na.rm = TRUE))]	
    if (rdo=="cos2") {	
      selcos2 <- as.numeric(substr(selWord, 5, nchar(selWord)))	
      selection <- which(apply(res.cagalt$word$cos2[, axes], 1, sum)> selcos2) }	
    if (rdo=="contrib") {	
      selcontrib <- as.numeric(substr(selWord, 8, nchar(selWord)))	
      dft <- data.frame(res.cagalt$word$contrib[,c(axes[1],axes[2]),drop=FALSE])
      fval <- apply(dft, 1, function(z) max(abs(z)))
      selection <- which(fval>= selcontrib) }	
    if (rdo=="meta") {	
      selmeta <- as.numeric(substr(selWord, 5, nchar(selWord)))	
      sMeta <- which(res.cagalt$word$contrib[,1] > selmeta*mean(res.cagalt$word$contrib[,1]))
      sMeta <- c(sMeta, which(res.cagalt$word$contrib[,2] > selmeta*mean(res.cagalt$word$contrib[,2])))
      selection <- unique(sMeta) }	
    
    if(is.null(selection)) if(rdo=="") if(length(selWord)==1) selection <- which(rownames(coord.word) %in% selWord)
    if(length(selection)==0) stop("There are not selected words to plot")	
    
    if(new.plot) if(!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")))   
      dev.new(width = min(14, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1])), height = 8) # Outside RStudio 
    else dev.new()
    plot(0, 0, main = titre, xlab = lab.x, ylab = lab.y, 	
         xlim = xlim, ylim = ylim, col = "white", asp = 1, ...)	
    abline(v = 0, lty = 2, ...)	
    abline(h = 0, lty = 2, ...)	
    
    ipch <- 20
    if(!is.null(pch)) {                                     # symbols in graphs
      ipch <- rep(pch[1], nrow(coord.word))   # By defect symbol 20
    } 
    
    coo <- labe <- coll <- fonte <- NULL	
    coo <- coord.word	

    if (label) 	
      labe <- rownames(coord.word)	
    else labe <- rep("", nrow(coord.word))	


    if(length(col.word)==1)  coll <- rep(col.word, nrow(coord.word)) else{
      if(length(col.word)< nrow(coord.word)) col.word <- c(col.word, rep("darkred", nrow(coord.word)-length(col.word)))
      if(length(col.word)> nrow(coord.word)) col.word <- col.word[1:nrow(coord.word)] 
      coll <- col.word
    }

  #  coll <- rep(col.word, nrow(coord.word))	
  #  ipch <- rep(15, nrow(coord.word))	
    fonte <- rep(1, nrow(coord.word))	
    
    
    if (!is.null(selection)) 
      coll[!((1:length(coll)) %in% selection)] = rgb(t(col2rgb(coll[!((1:length(coll)) %in% selection)])), 
                                                     alpha = 255 * (1 - unselect), maxColorValue = 255)
    pos.text <- NULL
    
    if (any(labe != "")) {
      autoLab <- FALSE
      if (auto.Lab == "auto") autoLab = (length(selection) < 50)
      if (auto.Lab == TRUE) autoLab <- TRUE
      if (autoLab == TRUE) {
        if(is.null(pch))    warning("autoLab must draw points")
        autoLab(coo[selection, 1], y = coo[selection,2],
                labels = labe[selection], col = coll[selection], 
                font = fonte[selection], cex=cex,...)
        points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
      }  # Final de autolab ==TRUE
      else {  
        # NO es autolab
        if(!is.null(pch)) { 
          points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
          pos.text <- 3
        }
        text(coo[labe != "", 1], y = coo[labe != "",2],
             labels = labe[labe != ""], col = coll[labe != ""], 
             font = fonte[labe != ""], pos=pos.text , cex=cex,...)
      }
    } else {
      # Sin etiquetas
      if(is.null(pch)) stop("You must plot points and/or text")
      points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
    }
  }  # Final plot words

       
# Final selection of words    
#################################################3

    
    
    
  # Word ellipses selected
  if(!is.null(selWordEllip)){
    selectionEll <- NULL
    rdoEllip<-""
    if (mode(selWordEllip) == "numeric") 
      selWordEllip <- rownames(coord.word)[selWordEllip]
    else if(length(selWordEllip)==1) {
      if (selWordEllip=="ALL") selWordEllip <- rownames(coord.word)
      else rdoEllip <- selX(selWordEllip)
    }
    if (rdoEllip=="coord") 
      selectionEll  <- (rev(order(apply(coord.word[,axes]^2, 1, max))))[1:min(num.word, 
                  sum(as.integer(unlist(strsplit(selWordEllip, "coord"))), na.rm = TRUE))]
    if (rdoEllip=="cos2") {
      selcos2 <- as.numeric(substr(selWordEllip, 5, nchar(selWordEllip)))
      selectionEll  <- which(apply(res.cagalt$word$cos2[, axes], 1, sum)> selcos2) }
    if (rdoEllip=="contrib") {
      selcontrib <- as.numeric(substr(selWordEllip, 8, nchar(selWordEllip)))
      dft <- data.frame(res.cagalt$word$contrib[,c(axes[1],axes[2]),drop=FALSE])
      fval <- apply(dft, 1, function(z) max(abs(z)))
      selectionEll  <- which(fval>= selcontrib) }
    if (rdoEllip=="meta") {
      selmeta <- as.numeric(substr(selWordEllip, 5, nchar(selWordEllip)))
      sMeta <- which(apply(res.cagalt$word$contrib[, axes], 1, function(a) a[axes]> selmeta*mean(a[axes])))
      selectionEll  <- unique(sMeta) }
    if(!is.null(selectionEll)) selWordEllip <- rownames(coord.word)[selectionEll]
    selWordEllip <-selWordEllip[which(selWordEllip %in% rownames(coord.word)[selection])]
    if(length(selWordEllip)==0) stop("There are not selected words to plot ellipses")

            
    dfEll <- res.cagalt$ellip$word[,c(ncol(res.cagalt$ellip$word), axes),drop=FALSE]
    dfEll$FREQ <- as.factor(dfEll$FREQ)
    coord.ellip <- FactoMineR::coord.ellipse(dfEll,level.conf=level.conf,bary = FALSE)
    

    col.word <- coll[which(rownames(coord.word) %in% selWordEllip)]      
    for(i in 1:length(selWordEllip)){
      lines(coord.ellip$res[coord.ellip$res$FREQ==selWordEllip[i],2], 
            coord.ellip$res[coord.ellip$res$FREQ==selWordEllip[i],3], lty = 2, lwd = 2, col = col.word[i])
    }
}# Final Words and ellipses of words

    
    
  
# Selection of quantitative variables  
  if (!is.null(selQuantiVar)) {
    if (is.null(res.cagalt$quanti.var)) 
      stop("Variables are not quantitative")
    
}
# Final quantitative variables  
  

##############################################  Qualitative variables
  if (!is.null(selQualiVar)) { 
    if (is.null(res.cagalt$quali.var)) 
      stop("Variables are not categorical")
    if (is.null(title)) 
      titre <- "Categories factor map (CaGalt)"
    else titre <- title
    coord.var <- res.cagalt$quali.var$coord[, axes, drop = FALSE]
    num.var <-  nrow(coord.var)
    
    xlim <- xlimT; ylim <- ylimT
    
    if (is.null(xlim)) 
      xlim <- c(min(coord.var[, 1]), max(coord.var[, 1])) * 1.2
    if (is.null(ylim)) 
      ylim <- c(min(coord.var[, 2]), max(coord.var[, 2])) * 1.2
    
    selection <- NULL
    rdo<-""
    
    if (mode(selQualiVar) == "numeric") selection <- selQualiVar
    else if(length(selQualiVar)==1) {
      if (selQualiVar=="ALL")  selection <- c(1:num.var)
      else rdo <- selX(selQualiVar)
    } else { selection <-  which(rownames(coord.var) %in% selQualiVar)}
    
    if (rdo=="coord") 	
      selection <- (rev(order(apply(coord.var^2, 1, max))))[1:min(num.var, 	
           sum(as.integer(unlist(strsplit(selQualiVar, "coord"))), na.rm = TRUE))]	
    if (rdo=="cos2") {	
      selcos2 <- as.numeric(substr(selQualiVar, 5, nchar(selQualiVar)))	
      selection <- which(apply(res.cagalt$quali.var$cos2[, axes], 1, sum)> selcos2)	
    }	
    
    if (rdo=="contrib") stop("There is not possible to use contrib selection for selQualiVar")
    if (rdo=="meta") stop("There is not possible to use meta selection for selQualiVar")
    
    
    
    if(is.null(selection)) if(rdo=="") if(length(selQualiVar)==1) selection <- which(rownames(coord.var) %in% selQualiVar)
    if(length(selection)==0) stop("There are not selected elements to plot")
    
    if(new.plot) if(!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")))   
      dev.new(width = min(14, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1])), height = 8) # Outside RStudio 
    else dev.new()
    plot(0, 0, main = titre, xlab = lab.x, ylab = lab.y, 
         xlim = xlim, ylim = ylim, col = "white", asp = 1, ...)
    abline(v = 0, lty = 2, ...)
    abline(h = 0, lty = 2, ...)

    ipch <- 20      
    if(!is.null(pch)) {                                     # symbols in graphs
      ipch <- rep(pch[1], nrow(coord.var))   # By defect symbol 20
    } 
    
    
    coo <- labe <- coll <-  fonte <- NULL
    coo <- coord.var
    if (label) 
      labe <- rownames(coord.var)
    else labe <- rep("", nrow(coord.var))
    coll <- rep(col.quali, nrow(coord.var))

    if(length(col.quali)==1)  coll <- rep(col.quali, nrow(coord.var)) else{
      if(length(col.quali)< nrow(coord.var)) col.quali <- c(col.quali, rep(col, nrow(coord.var)-length(col.quali)))
      if(length(col.quali)> nrow(coord.var)) col.quali <- col.quali[1:nrow(coord.var)] 
      coll <- col.quali
    }
    
    fonte <- rep(1, nrow(coord.var))
    
    if (!is.null(selQualiVar)) {
        coll[!((1:length(coll)) %in% selection)] = rgb(t(col2rgb(coll[!((1:length(coll)) %in% 
           selection)])), alpha = 255 * (1 - unselect), maxColorValue = 255)
    }
   
    pos.text <- NULL
      if (any(labe != "")) {
      
      autoLab <- FALSE
      if (auto.Lab == "auto") autoLab = (length(selection) < 50)
      if (auto.Lab == TRUE) autoLab <- TRUE
      
      if (autoLab == TRUE) {
        if(is.null(pch))    warning("autoLab must draw points")
        autoLab(coo[selection, 1], y = coo[selection,2],
                labels = labe[selection], col = coll[selection], 
                font = fonte[selection], cex=cex,...)
        points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
      }  # Final de autolab ==TRUE
      else {  
        # NO es autolab
        if(!is.null(pch)) { 
          points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
          pos.text <- 3
        }
        text(coo[labe != "", 1], y = coo[labe != "",2],
             labels = labe[labe != ""], col = coll[labe != ""], 
             font = fonte[labe != ""], pos=pos.text , cex=cex,...)
      }
     
      } else {
        # Sin etiquetas
        if(is.null(pch)) stop("You must plot points and/or text")
        points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
      }

   
    
    
#########################################  Final qualitative variables


  # Qualitative categories ellipses
    if(!is.null(selQualiVarEllip)){
      selectionEll <- NULL
      rdoEllip<-""
      if(selQualiVarEllip[1]=="ALL")  selQualiVarEllip <- "ALL"

      if (mode(selQualiVarEllip) == "numeric") 
        selQualiVarEllip <- rownames(coord.var)[selQualiVarEllip]
      else if(length(selQualiVarEllip)==1) {
        if (selQualiVarEllip=="ALL") selQualiVarEllip <- rownames(coord.var)
        else rdoEllip <- selX(selQualiVarEllip)
      }# 
      
      if (rdoEllip=="coord") 	
        selectionEll  <- (rev(order(apply(coord.var[,axes]^2, 1, max))))[1:min(num.var, 	
                        sum(as.integer(unlist(strsplit(selQualiVarEllip, "coord"))), na.rm = TRUE))]	
      if (rdoEllip=="cos2") {	
        selcos2 <- as.numeric(substr(selQualiVarEllip, 5, nchar(selQualiVarEllip)))	
        selectionEll  <- which(apply(res.cagalt$quali.var$cos2[, axes], 1, sum)> selcos2) }	
      if (rdoEllip=="contrib") {	
        selcontrib <- as.numeric(substr(selQualiVarEllip, 8, nchar(selQualiVarEllip)))	
        dft <- data.frame(res.cagalt$quali.var$contrib[,c(axes[1],axes[2]),drop=FALSE])	
        fval <- apply(dft, 1, function(z) max(abs(z)))	
        selectionEll  <- which(fval>= selcontrib) }	
      if (rdoEllip=="meta") {
        selmeta <- as.numeric(substr(selQualiVarEllip, 5, nchar(selQualiVarEllip)))
        sMeta <- which(apply(res.cagalt$quali.var$contrib[, axes], 1, function(a) a[axes]> selmeta*mean(a[axes])))
        selectionEll  <- unique(sMeta) }
      
      if(!is.null(selectionEll)) selQualiVarEllip <- rownames(res.cagalt$quali.var$coord)[selectionEll]
      selQualiVarEllip <- selQualiVarEllip[which(selQualiVarEllip %in% rownames(res.cagalt$quali.var$coord)[selection])]
      if(length(selQualiVarEllip)==0) stop("There are not selected elements to plot ellipses")

      # Coordinates of all simulations. Last column has the word
      dfEll <- res.cagalt$ellip$var[,c(ncol(res.cagalt$ellip$var), axes),drop=FALSE]
      dfEll$VAR <- as.factor(dfEll$VAR)
      coord.ellip <- FactoMineR::coord.ellipse(dfEll,level.conf=level.conf,bary = FALSE)


col.quali <- coll[which(rownames(res.cagalt$quali.var$coord) %in% selQualiVarEllip)]      
      for(i in 1:length(selQualiVarEllip )){
        lines(coord.ellip$res[coord.ellip$res$VAR==selQualiVarEllip[i],2], 
              coord.ellip$res[coord.ellip$res$VAR==selQualiVarEllip[i],3], lty = 2, lwd = 2, col = col.quali[i])
      }
    } # Final ellipses for qualitative variables
  } # Final qualitative variables    


    
    
    
          
########################  Quantitative variables
  if (!is.null(selQuantiVar)) {
    if (is.null(res.cagalt$quanti.var)) 
      stop("Variables are not quantitative")
    if (is.null(title)) 
      titre <- "Variables factor map (CaGalt)"
    else titre <- title
 
    # Review if selCov is necessary   
    if(selCov==TRUE) {
      coord.Q <- res.cagalt$quanti.var$coord[, axes, drop = FALSE]
      xlim <- xlimT; ylim <- ylimT
      if (is.null(xlim)) 
        xlim <- c(min(coord.Q[, 1]), max(coord.Q[, 1])) * 1.2
      if (is.null(ylim)) 
        ylim <- c(min(coord.Q[, 2]), max(coord.Q[, 2])) * 1.2
    } else {
      coord.Q <- res.cagalt$quanti.var$cor[, axes, drop = FALSE]
      xlim <- ylim <- c(-1, 1)
    }
    selection <- NULL
    rdo<-""
    
    if (mode(selQuantiVar) == "numeric") 
      selection <- selQuantiVar
    else if(length(selQuantiVar)==1) {
      if (selQuantiVar=="ALL")  selection <- c(1:nrow(coord.Q))
      else rdo <- selX(selQuantiVar)
    } else { selection <-  which(rownames(coord.Q) %in% selQuantiVar)}
    
    if (rdo=="coord") 
      selection <- (rev(order(apply(coord.Q^2, 1, max))))[1:min(nrow(coord.Q), 
                                                                       sum(as.integer(unlist(strsplit(selQuantiVar, "coord"))), na.rm = T))]
    if (rdo=="cos2") {
      selcos2 <- as.numeric(substr(selQuantiVar, 5, nchar(selQuantiVar)))
      selection <- which(apply(res.cagalt$quanti.var$cos2[, axes], 1, sum)> selcos2)
    }
    if (rdo=="contrib") stop("There is not possible to use contrib selection for selQuantiVar")
    
    
    if(is.null(selection)) if(rdo=="") if(length(selQuantiVar)==1) selection <- which(rownames(coord.Q) %in% selQuantiVar)
    if(length(selection)==0) stop("There are not selected elements to plot")
 
#    if(new.plot) if(!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")))   
#      dev.new(width = min(14, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1])), height = 8) # Outside RStudio 
#    else if(new.plot==TRUE) dev.new()

    if ((new.plot) & !nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) 
      dev.new()

   
        plot(0, 0, xlab = lab.x, ylab = lab.y, xlim = xlim, ylim = ylim, 
         col = "white", asp = 1, main = titre, ...)
    if(selCov==FALSE){
      x.cercle <- seq(-1, 1, by = 0.01)
      y.cercle <- sqrt(1 - x.cercle^2)
      lines(x.cercle, y = y.cercle, ...)
      lines(x.cercle, y = -y.cercle, ...)}
    
    abline(v = 0, lty = 2, ...)
    abline(h = 0, lty = 2, ...)
    coll <- coo <- labe <- posi <- NULL
    
    coll <- rep(col.quanti, nrow(coord.Q))
    coo <- coord.Q                                 # Coordinates quantitative variables without simulation

    
    if (label) 	
      labe <- rownames(coord.Q)	
    else labe <- rep("", nrow(coord.Q))	
    
    if (!is.null(selection)) {	
      if (is.numeric(unselect)) 	
        coll[!((1:length(coll)) %in% selection)] = rgb(t(col2rgb(coll[!((1:length(coll)) %in% 	
                                                                          selection)])), alpha = 255 * (1 - unselect),maxColorValue = 255)	
      else coll[!((1:length(coll)) %in% selection)] = unselect	
      labe[!((1:length(coll)) %in% selection)] <- ""	
    }	
    
    for (v in 1:nrow(coord.Q)) {	
      arrows(0, 0, coord.Q[v, 1], coord.Q[v, 2], 	
             length = 0.1, angle = 15, code = 2, col = coll[v])	
      if (label) {	
        if (abs(coord.Q[v, 1]) > abs(coord.Q[v,2])) {	
          if (coord.Q[v, 1] >= 0) 	
            posi <- c(posi, 4)	
          else posi <- c(posi, 2)	
        }	
        else {	
          if (coord.Q[v, 2] >= 0) 	
            posi <- c(posi, 3)	
          else posi <- c(posi, 1)	
        }	
      }	
    }
    

    if (any(labe != "")) {	
      autoLab <- FALSE	
      if (auto.Lab == "auto")  autoLab = (length(which(labe != "")) < 50)	
      if (auto.Lab == TRUE) autoLab <- TRUE
      if (autoLab == TRUE){ 	
        autoLab(coo[labe != "", 1], y = coo[labe != "", 2], labels = labe[labe != ""],	
      #          col = coll[labe != ""], shadotext = shadowtext,cex=cex, ...)	}
      col = coll[labe != ""], cex=cex, ...)	}
      else	
        text(coo[labe != "", 1], y = coo[labe != "", 2], labels = labe[labe != ""], 	
             pos = posi[labe != ""], col = coll[labe != ""], cex=cex,...)	
    }	
    
    # Ellipses selection for Quantitative analysis
    if(!is.null(selQuantiVarEllip)){	
      selectionEll <- NULL	
      rdoEllip<-""	
      if (mode(selQuantiVarEllip) == "numeric") 	
        selQuantiVarEllip <- rownames(res.cagalt$quanti.var$coord)[selQuantiVarEllip]	
      else if(length(selQuantiVarEllip)==1) {	
        if (selQuantiVarEllip=="ALL") selQuantiVarEllip <- rownames(res.cagalt$quanti.var$coord)	
      }# Final no 1	
      

      selQuantiVarEllip <- selQuantiVarEllip[which(selQuantiVarEllip %in% rownames(res.cagalt$quanti.var$coord)[selection])]
      if(length(selQuantiVarEllip)==0) stop("There are not selected elements to plot ellipses")
      

   
      # Contiene las coordenadas de todas las simulaciones para los ejes seleccionados,
      # La primera columna contiene la palabra
      # Contiene las correlaciones en todos los casos
      dfEll <- res.cagalt$ellip$var[,c(ncol(res.cagalt$ellip$var), axes),drop=FALSE]
      dfEll$VAR <- as.factor(dfEll$VAR)

      if(selCov==TRUE)
      {
        factorBoot  <- data.frame(res.cagalt$quanti.var$coord[,1]/res.cagalt$quanti.var$cor[,1],
                                  rownames(res.cagalt$quanti.var$cor))
        colnames(factorBoot) <- c("Value","VAR")
        tmp <- merge(x =dfEll, y = factorBoot, by = "VAR", all.x = TRUE)
        tmp[,2:3] <- tmp[,2:3]*tmp[,"Value"]
        dfEll <- tmp[,1:3] 
      }
      
   
      coord.ellip <- coord.ellipse(dfEll,level.conf=level.conf,bary = FALSE)

      if(selCov==FALSE) {
        tmp <- sqrt(coord.ellip$res[,2]^2+coord.ellip$res[,3]^2)
        coord.ellip$res[which(tmp>1),2] <- coord.ellip$res[which(tmp>1),2]/tmp[which(tmp>1)]
        coord.ellip$res[which(tmp>1),3] <- coord.ellip$res[which(tmp>1),3]/tmp[which(tmp>1)]
      }
      
      


      for(i in 1:length(selQuantiVarEllip )){
        lines(coord.ellip$res[coord.ellip$res$VAR==selQuantiVarEllip[i],2], 
              coord.ellip$res[coord.ellip$res$VAR==selQuantiVarEllip[i],3], 
              lty = 2, lwd = 2, col = col.quanti)
      }
    } # Final selQuantiVarEllip
    
  } # Final Quantitative variables    

  } else { 
# This is the multiple case
    
    
    
    
    if(type=="QL") {		
      if(is.null(x$MQL) | length(x$MQL)==1) stop("There are not categorical variables selected or in LexGalt object")		
      res.mfagalt <- x$MQL }		
    if(type=="QN") {			
      if(is.null(x$MQN) | length(x$MQN)==1)stop("There are not quantitative variables selected or in LexGalt object")		
      res.mfagalt <- x$MQN }

    # Selection of groups
    ## Problema. Tenemos en res.mfagalt$call el nombre de los grupos name.groups y el número num.groups
    ## Podemos tener seleccionados grupos en selGroup
    ## Podemos tener nuevas etiquetas en label.group
    name.groups <- res.mfagalt$call$name.groups
    num.groups <-res.mfagalt$call$num.groups
    if(is.null(selGroup)) selGroup <- c(1:num.groups)	
    if(length(selGroup)==1) if(selGroup=="ALL") selGroup <- c(1:num.groups)
    if (is.numeric(selGroup)) selGroup <- name.groups[selGroup]
    selGroup <- selGroup[which(selGroup %in%  name.groups)]
    if(length(selGroup)==0) stop("There is not selected groups")
    noselGroup <- name.groups[-which(name.groups %in% selGroup)]		
  
    # Labels of groups
    # Tienen que ser el mismo número que name.groups, estén seleccionadas o no

    
   old.name.groups <- name.groups
        if(is.null(label.group)) label.group <- name.groups
    if(length(label.group)==num.groups) name.groups <- label.group 
    else if(length(label.group)==length(selGroup)) name.groups[which(name.groups %in% selGroup)] <- label.group
         else      warning("Number of label.group is not the same as number of groups or selected selGroup) \n     label.group will be ignorated")
     label.group <- name.groups 
  
    
      
    # Color selection		
    if(is.null(col.group)) col.group <- PALETTE[1:res.mfagalt$call$num.groups]	
    if (is.numeric(unselect)) if ((unselect > 1) | (unselect < 0)) stop("unselect should be between 0 and 1")		
    # Labels 
    lab.x <- paste("Dim ", axes[1], " (", format(res.mfagalt$eig[axes[1], 2], nsmall = 2, digits = 2), "%)", sep = "")		
    lab.y <- paste("Dim ", axes[2], " (", format(res.mfagalt$eig[axes[2], 2], nsmall = 2, digits = 2), "%)", sep = "")		

    ###################################### Plot eigenvalues   ########   Same as simple
    if(eigen) {
      #  nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")) # TRUE  when inside RStudio ,  # FALSE when outside RStudio
      
      if(new.plot) if(!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")))   
        dev.new(width = min(14, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1])), height = 8) # Outside RStudio 
      else if(new.plot==TRUE) dev.new()
      
      if(is.null(title)) titleE <- "Eigenvalues" else titleE <- title
      barplot(res.mfagalt$eig[, 1], main = titleE, col=col.eig, cex.axis=cex, cex.names=cex,
              names.arg = paste("dim",1:nrow(res.mfagalt$eig)))}
    ###################################### End Plot eigenvalues 


    
    # selection <- NULL  # Number of non selected groups
    if(length(noselGroup)>0)
      selection <- which(!rownames(res.mfagalt$group$coord[, axes, drop = FALSE]) %in% noselGroup)
        else selection <- c(1:length(selGroup))

    
    
#######################################################################
#### Gráfico en donde aparecen como modalidades los grupos
# If plot groups					

    
if(plot.group==TRUE){	
  coord.actif <- res.mfagalt$group$coord[, axes, drop = FALSE]	
  xlim <- xlimT; ylim <- ylimT	
  if(new.plot) if(!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")))   
    dev.new(width = min(14, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1])), height = 8) # Outside RStudio 
  else dev.new()
  
  coo <- labe <- coll <- ipch <- fonte <- NULL			
  if (is.null(xlim)) xlim <- c(0,1)			
  if (is.null(ylim)) ylim <- c(0,1)			
  if (is.null(title)) title <- "Groups representation (MfaGalt)"			
  plot(0, 0, main = title, xlab = lab.x, ylab = lab.y, xlim = xlim, ylim = ylim, asp = 1, col = "white", ...)			
  abline(v = 0, lty = 2, ...)			
  abline(h = 0, lty = 2, ...)			
  # coo <- rbind(coo, coord.actif)	
  coo <- coord.actif
  if(length(col.group)==1) col.group[1:num.groups] <- col.group
  coll <- col.group

 # label.group
    if(length(noselGroup)>0) 
       coll[!((1:length(coll)) %in% selection)] <- rgb(t(col2rgb(coll[!((1:length(coll)) %in% selection)])), 
                                               alpha = 255 * (1 - unselect), maxColorValue = 255)


  # Posibilidad de que el número de grupos no sea igual que el de símbolos
  
  if(is.null(pch)) ipch <- c(15:(15+num.groups-1))
  if(length(pch)==1)  ipch <- c(pch:(pch+num.groups-1))	else ipch <- pch	
  if(length(pch)>length(ipch)) maxgr <- pch else maxgr <- ipch
     pch <- ipch <- maxgr

  fonte <- rep(1, nrow(coord.actif))

  

  
    if(label) labe <- label.group
  pos.text <- NULL
    if (any(labe != "")) {
    autoLab <- FALSE
    if (auto.Lab == "auto") autoLab = (length(selection) < 50)
    if (auto.Lab == TRUE) autoLab <- TRUE
    if (autoLab == TRUE) {
      if(is.null(pch))    warning("autoLab must draw points")
      autoLab(coo[, 1], y = coo[,2],
              labels = labe,  col=coll, 
              font = fonte, cex=cex,...)
      points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
    }  # Final de autolab ==TRUE
    else {  
      # NO es autolab
      if(!is.null(pch)) { 
        points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
        pos.text <- 3
      }
      # text(coo[labe != "", 1], y = coo[labe != "",2],
      #     labels = labe[labe != ""], col = coll[labe != ""], 
      #     font = fonte[labe != ""], pos=pos.text , cex=cex,...)
      text(coo[, 1], y = coo[,2], labels = labe, col = coll, 
               font = fonte, pos=pos.text , cex=cex,...)
           
    }
  } else {
    # Sin etiquetas
    if(is.null(pch)) stop("You must plot points and/or text")
    points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
  }
} # Final if plot.group
#### Finalizado el plot.group


    
    
    
    
    
    
if(!is.null(selWord)){		
  #############  Plot selected words		
  titre <- ifelse(is.null(title), "Word factor map (M LexGalt)", title)		
  coord.word <- res.mfagalt$word$coord[, axes, drop = FALSE]	  # Cambiar después para simplificar
  num.word <- nrow(coord.word)
  selection.word <- NULL                                                    #  <-------------- PDte 

    rdo <-""                                                              # Para la selección cos2...
  sel <- nosel <- NULL
  xlim <- xlimT; ylim <- ylimT		
  if (is.null(xlim)) xlim <- c(min(coord.word[,1]),max(coord.word[,1])) * 1.2		
  if (is.null(ylim)) ylim <- c(min(coord.word[,2]),max(coord.word[,2])) * 1.2		

#  if(length(col.word)==1)  coll <- rep(col.word, nrow(coord.word)) else{
#    if(length(col.word)< nrow(coord.word)) coll <- c(col.word, rep(col.word, nrow(coord.word)-length(col.word)))
#    if(length(col.word)> nrow(coord.word)) coll <- col.word[1:nrow(coord.word)] 
#    #  coll <- col.word
#  }
  
    # First of all we must to remove not selected words by groups
  # Selecting rows (words)	from groups	
  pos <- matrix(, nrow = num.groups, ncol = 3)		
  colnames(pos) <- c("group","start","end")		
  pos[,1] <- name.groups	
  ncont <- 0			
  for(i in 1:num.groups) {			
    pos[i,2] <- ncont+1			
    ncont <- ncont+res.mfagalt$call$num.freq[i]			
    pos[i,3] <- ncont    }			
  # pos has the position of each group
  # group     start end  
  # [1,] "GROUP.1" "1"   "169"
  # [2,] "GROUP.2" "170" "316"
  numnoselgroup <- length(noselGroup)	
    if(numnoselgroup>0) {
     for(i in 1:num.groups) {						
      tmp <- c(pos[i,2]:pos[i,3])						
      tmp2 <- which(name.groups[i] %in% noselGroup)
      if(length(tmp2)==0) sel <- c(sel,tmp) # else nosel <- c(nosel,tmp)	
    }						
  } else { sel <- c(1:num.word)}	 # Final numnoselgroup>0
  # sel contiene las posiciones de los individuos seleccionados por grupos, nosel los no seleccionados
nosel <- which(!c(1:num.word) %in% sel)
noselGROUP <- nosel

if(is.null(pch)) ipch <- c(15:(15+num.groups-1))
if(length(pch)==1)  ipch <- c(pch:(pch+num.groups-1))	else ipch <- pch	


   ########################### Selection of words
 if(!is.list(selWord)) {
     rdo <- selX(selWord)
    if(length(rdo)==1){
    if (rdo=="cos2") {						
      selcos2 <- as.numeric(substr(selWord, 5, nchar(selWord)))	
      # Calcula las palabras que no tienen una suma de cos2 entre los dos factores mayor que la seleccionada
      noselcos2 <- which(apply(res.mfagalt$word$cos2[, axes], 1, sum) < selcos2)
      nosel <- unique(c(nosel,noselcos2)) }		
    
     if (rdo=="coord") {						
       selcoord <- as.numeric(substr(selWord, 6, nchar(selWord)))	
       C <- res.mfagalt$word$coord						
       rownames(C) <- c(1:nrow(C))		
       C <- data.frame(C[,axes], c(1:nrow(C)), rownames(coord.word))	# Añade a la última columna la palabra
       C2 <- data.frame(apply(C^2, 1, max), c(1:nrow(C)), rownames(coord.word))[sel,]						
       selcoord <- rownames(C2[ order(-C2[,1]), ]  [c(1:selcoord),])		
       temp <- 1:nrow(C)						
       noselcoord <- temp[!temp %in% selcoord]						
       nosel <- unique(c(nosel,noselcoord)) }
     
     if (rdo=="contrib") {						
       selcontrib <- as.numeric(substr(selWord, 8, nchar(selWord)))						
       C <- res.mfagalt$word$contrib						
       rownames(C) <- c(1:nrow(C))		
       C <- data.frame(C[,axes], c(1:nrow(C)), rownames(res.mfagalt$word$contrib))	
       
       C2 <- data.frame(apply(C[,c(1,2)], 1, max), c(1:nrow(C)), rownames(res.mfagalt$word$coord))[sel,]
       noselcontrib <- which(C2[,1]< selcontrib)						
       nosel <- unique(c(nosel,noselcontrib))
       }   
     
     if (rdo=="meta") {						
       selmeta <- as.numeric(substr(selWord, 5, nchar(selWord)))						
       C <- res.mfagalt$word$contrib						
       rownames(C) <- c(1:nrow(C))						
       C <- data.frame(C[,axes], c(1:nrow(C)), rownames(res.mfagalt$word$contrib))						
       sMeta <- which(C[,1] > selmeta*mean(C[,1]))						
       sMeta <- c(sMeta, which(C[,2] > selmeta*mean(C[,2])))						
       #       sMeta <- which(apply(C[, axes], 1, function(a) a[axes]> selmeta*mean(a[axes])))						
       sMeta <- unique(sMeta)   						
       temp <- 1:nrow(C)						
       nosel <- unique(c(nosel,temp[!temp %in% sMeta]))}   

      sel <- which(!c(1:nrow(res.mfagalt$word$coord)) %in% nosel)
     } # Final de si es solo un valor

     # nosel contiene los no seleccionados por palabras


     if(selWord[1]!="ALL"){
      if(rdo[1]=="") {

       if(is.numeric(selWord)) selWord <- rownames(coord.word)[selWord] 
       nosel2 <-  which(!rownames(coord.word) %in% selWord)
       nosel <- unique(c(nosel2,nosel))
       sel <- which(!c(1:num.word) %in% nosel)
     }} # Final rdo==""
    #  stop("NO Es una lista")
  #stop("Final de no es una lista")
  } else {
    # is.list(selWord)
    
    numselgroup <- num.groups-numnoselgroup
    
    if(numselgroup != length(selWord)) 
      stop(paste0("Error, number of selected groups is " , numselgroup, 
                  "\n  but the number of groups in selWord is ", length(selWord) ))
    
      sel <- NULL
    cont <- 1
    
    
    for(i in 1:num.groups) {						
      if(name.groups[i] %in% selGroup) {
        strtmp <- rownames(coord.word)[pos[i,2]:pos[i,3]]  # Palabras totales del grupo i
        sel <-  c(sel, (which(strtmp %in%  selWord[[cont]])-1+as.numeric(pos[i,2])))  # MOdificar quitar primer sel
        cont <- cont+1
      }
      #            
    } 
      #         stop("Es una lista")
    nosel <- which(!c(1:num.word) %in% sel)
  }
  
  ############## Graficos
# 1.- Poner el color de la palabra dependiendo del grupo al que pertenece
  if(length(sel)==0) stop("There are no words to plot")
  coo <- labe <- coll <- fonte <- NULL			
  # coord.word is a matrix, with words in rows and dimensions in columns
  coo <- coord.word
  

  # label.group
  if(length(col.group)!= num.groups) {
    warning("col.group is bad defined, it will be ignorated")
    col.group <- PALETTE[1:res.mfagalt$call$num.groups]
  }

   ipch <- NULL
  # Posibilidad de que el número de grupos no sea igual que el de símbolos
  if(is.null(pch)) 
    ipch <- rep(26,num.word) else {
    pch <- rep(pch,num.groups)
  if(length(pch)==1)  {  ipch <- rep(as.numeric(pch),num.word) 
  }else {
  if(length(pch)>num.groups) pch <- pch[1:num.groups] 
  if(length(pch)<num.groups) stop("length pch is < than num.groups")
  if(is.null(ipch))
  for(i in 1:num.groups) {						
      itmp <- as.numeric(pos[i,3]) - as.numeric(pos[i,2]) +1 
      ipch <- c(ipch, rep(pch[i], itmp))
  } 
  }}

   
      coll <- rep(col.group,res.mfagalt$call$num.freq)	 
  if(!is.null(col.word)) {
    if(length(col.word)!=length(coll)) stop(paste0("length.col must have ",length(coll), " values, not ", length(col.word) ))
   coll <- col.word 
  }
 coll[c(1:num.word) %in% nosel] <- rgb(t(col2rgb(coll[c(1:num.word) %in% nosel])), 
              alpha = 255 * (1 - unselect), maxColorValue = 255)	
 coll[((1:num.word) %in% noselGROUP)] <- rgb(t(col2rgb(coll[((1:num.word) %in% noselGROUP)])), 
                                         alpha = 0, maxColorValue = 255)

 if(new.plot) if(!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")))   
   dev.new(width = min(14, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1])), height = 8) # Outside RStudio 
 else dev.new()
 
  plot(0, 0, main = titre, xlab = lab.x, ylab = lab.y, xlim = xlim, ylim = ylim, col = "white", asp = 1, ...)		
  abline(v = 0, lty = 2, ...)		
  abline(h = 0, lty = 2, ...)		


  if(label)
    labe <-  rownames(coord.word)
   else labe <- rep("",nrow(coord.word))

  fonte <- rep(1, nrow(coord.word))	
#  ipch <- 20
#  if(!is.null(pch)) {                                     # symbols in graphs
#    ipch <- rep(pch[1], nrow(coord.word))   # By defect symbol 20
#  } 
  #if(length(nosel)!=0)
  #  text(coo[nosel, 1], y = coo[nosel, 2], labels = rownames(coord.word)[nosel], cex=cex, col=coll[nosel], ...)	 
  

  if (any(labe != "")) {
    autoLab <- FALSE
    if (auto.Lab == "auto") autoLab = (length(sel) < 50)
    if (auto.Lab == TRUE) autoLab <- TRUE
    if (autoLab == TRUE) {
      if(is.null(pch))    warning("autoLab must draw points")
      autoLab(coo[sel, 1], y = coo[sel,2],
              labels = labe[sel], col = coll[sel], 
              font = fonte[sel], cex=cex,...)
      points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
    }  # Final de autolab ==TRUE
    else {  
      # NO es autolab
      if(!is.null(pch)) { 
        points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
        pos.text <- 3
      } else pos.text <- NULL
            text(coo[labe != "", 1], y = coo[labe != "",2],
           labels = labe[labe != ""], col = coll[labe != ""], 
           font = fonte[labe != ""], pos=pos.text , cex=cex,...)
    }
  } else {
    # Sin etiquetas
    if(is.null(pch)) stop("You must plot points and/or text")
    points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
  }
  
#  text(coo[sel, 1], y = coo[sel, 2], labels = labe, cex=cex, col=coll[sel], ...)	
  
# Controlar antes los pch si solo es uno o hay varios

  

pch2 <- pch
if(is.null(pch2)) pch2 <- rep(NA, num.groups)

   if(legend) if(length(label.group)==num.groups){						
#    legend(pos.legend,label.group,pch=15:(15+res.mfagalt$call$num.group-1),cex=cex*1.5,text.col=col.word,col=col.word)				
  #  legend(pos.legend,label.group,pch=pch2,cex=cex*1.1,text.col=col.group,col=col.group)	
   
     legend(pos.legend,label.group[which(old.name.groups %in% selGroup)],pch=pch2,cex=cex*1.1,text.col=col.group,col=col.group)			

    } else { warning("The number of groups of name.groups is ",  res.mfagalt$call$num.groups, " not " , length(label.group))}						
  } # Final if(!is.null(selWord)){		

    
###########################################################
    if(!is.null(selQualiVar)){				
      if (is.null(title)) titre <- "Categories factor map (MfaGalt)"					
        else titre <- title					
      coord.var <- res.mfagalt$quali.var$coord[, axes]					
      xlim <- xlimT; ylim <- ylimT						
      # Si tiene parcial o no, calculo de los limites		
      # col.var.partial <- col.group	
      ncateg <- nrow(coord.var)
      col.var.partial <- rep(col.quali[1], ncateg)
      
 
     # ipch <- 20
     if(!is.null(pch)) {                                     # symbols in graphs
         ipch <- rep(pch[1],ncateg)   # By defect symbol 20
      } else ipch <- rep(26,ncateg) 


      ############ Seleccion de variables y/o modalidades					
      selection <- NULL						
      if (is.numeric(selQualiVar)) selQualiVar <- rownames(coord.var)[selQualiVar]						
      if(selQualiVar[1]=="ALL") selQualiVar <- rownames(coord.var)						
      selection  <- which(rownames(coord.var) %in% selQualiVar)					
      
      if(length(selQualiVar)==1){						
        rdo <- selX(selQualiVar)						
        if (rdo=="cos2") {						
          selcos2 <- as.numeric(substr(selQualiVar, 5, nchar(selQualiVar)))						
          selection <- which(apply(res.mfagalt$quali.var$cos2[, axes], 1, sum) >= selcos2) }		
        if (rdo=="coord") {						
          selcoord <- as.numeric(substr(selQualiVar, 6, nchar(selQualiVar)))						
          selection <- rev(order(apply(res.mfagalt$quali.var$coord[,axes]^2, 1, max)))[1:selcoord] }	
      }  # Final selQualiVar
      
  #    return(selection)
      
      if(partial==TRUE | partial=="ALL"){				
        strcateg <- rownames(coord.var)
        for(i in 1:res.mfagalt$call$num.groups){	
          new.coord <- res.mfagalt$quali.var$coord.partial[[i]][, axes, drop = FALSE]
          rownames(new.coord) <- paste0(name.groups[i],".", strcateg)
          coord.var <- rbind(coord.var,new.coord)
          if(is.null(col.group[i])) tmp <- rep(col.quali[1], ncateg) else tmp<- rep(col.group[i], ncateg)
          col.var.partial <- c( col.var.partial, tmp)
          if(length(pch) ==i) 
              {pch <- c(pch,pch[i])
            tmp <- rep(pch[1], ncateg)} else tmp<- rep(pch[i+1], ncateg)
                    ipch <- c(ipch, tmp)
                            }
       } # Final partial TRUE
      if (is.null(xlim)) xlim <- c(min(coord.var[,1]),max(coord.var[,1])) * 1.2						
      if (is.null(ylim)) ylim <- c(min(coord.var[,2]),max(coord.var[,2])) * 1.2

      coll <- col.var.partial
      

      #       if (is.null(xlim)) xlim <- c(min(min(coord.var[,1]),xPmin),max(max(coord.var[,1]),xPmax)) * 1.2						
      #      if (is.null(ylim)) ylim <- c(min(min(coord.var[,2]),yPmin),max(max(coord.var[,2]),yPmax)) * 1.2						
      #      } else {						
      #        if (is.null(xlim)) xlim <- c(min(coord.var[,1]),max(coord.var[,1])) * 1.2						
      #        if (is.null(ylim)) ylim <- c(min(coord.var[,2]),max(coord.var[,2])) * 1.2			
      
      if ((new.plot) & !nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) 
        dev.new(width = min(14, max(8, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1]))), height = 8)	
      plot(0, 0, main = title, xlab = lab.x, ylab = lab.y, xlim = xlim, ylim = ylim, col = "white", asp = 1, ...)						
      abline(v = 0, lty = 2, ...)					
      abline(h = 0, lty = 2, ...)					
    
      coo <- labe <- fonte <- NULL	
      labe <- trimws(rownames(coord.var))
      coo <-  coord.var
   
      # ipch <- rep(20,nrow(coord.var))
 
  #    if(!is.null(col.word)) {
  #      if(length(col.word)!=length(coll)) stop(paste0("length.col must have ",length(coll), " values, not ", length(col.word) ))
  #      coll <- col.word 
  #    }
      # selection
      
      nosel <- which(!c(1:ncateg) %in% selection)
      coll[c(1:ncateg) %in% nosel] <- rgb(t(col2rgb(coll[c(1:ncateg) %in% nosel])), 
                                            alpha = 255 * (1 - unselect), maxColorValue = 255)	
      
      if(partial==TRUE | partial=="ALL"){		
        for(i in 1:res.mfagalt$call$num.groups){	
          if(name.groups[i] %in% noselGroup) 
            coll[((i*ncateg) + c(1:ncateg))] <- rgb(t(col2rgb(coll[((i*ncateg) + c(1:ncateg))])), 
                                                alpha = 255 * (1 - unselect), maxColorValue = 255)	
        }
      }
    
      sel <- selection
      if (any(labe != "")) {
        autoLab <- FALSE
        if (auto.Lab == "auto") autoLab = (length(sel) < 100)
        if (auto.Lab == TRUE) autoLab <- TRUE
        if (autoLab == TRUE) {
          if(is.null(pch))    warning("autoLab must draw points")
          autoLab(coo[, 1], y = coo[,2],
                  labels = labe, col = coll, 
                  font = fonte, cex=cex,...)
          points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
        }  # Final de autolab ==TRUE
        else {  
          # NO es autolab
          if(!is.null(pch)) { 
            points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
            pos.text <- 3
          } else pos.text <- NULL
          text(coo[labe != "", 1], y = coo[labe != "",2],
               labels = labe[labe != ""], col = coll[labe != ""], 
               font = fonte[labe != ""], pos=pos.text , cex=cex,...)
        }
      } else {
        # Sin etiquetas
        if(is.null(pch)) stop("You must plot points and/or text")
        points(coo[, 1], y = coo[, 2], pch = ipch, col = coll, cex=cex,...)
      }

      
      pch2 <- pch
      if(is.null(pch2)) pch2 <- rep(NA, num.groups)
      if(length(label.group)==num.groups){						
        #    legend(pos.legend,label.group,pch=15:(15+res.mfagalt$call$num.group-1),cex=cex*1.5,text.col=col.word,col=col.word)				
        #  legend(pos.legend,label.group,pch=pch2,cex=cex*1.1,text.col=col.group,col=col.group)				
        legend(pos.legend,label.group,pch=pch2,cex=cex*1.1,text.col=col.group,col=col.group)				
      } else { warning("The number of groups of name.groups is ",  res.mfagalt$call$num.groups, " not " , length(label.group))}						
    } # Final is.null(selQualiVar)						
  
 
    
    
 
    
    

    #####################################################################3			
    if(!is.null(selQuantiVar)){			
      if (is.null(res.mfagalt$quanti.var)) 			
        stop("Variables are not quantitative")			
      if (is.null(title)) 			
        titre <- "Variables factor map (LexGalt)"			
      else titre <- title			
      selection <- NULL			
      rdo<-""			
      
      

      xlim <- xlimT; ylim <- ylimT		
      if(selCov==TRUE) {		
        coord.Q <- res.mfagalt$quanti.var$coord[, axes, drop = FALSE]		
        if (is.null(xlim)) 		
          xlim <- c(min(coord.Q[, 1]), max(coord.Q[, 1])) * 1.2		
        if (is.null(ylim)) 		
          ylim <- c(min(coord.Q[, 2]), max(coord.Q[, 2])) * 1.2		
      } else {		
        coord.Q <- res.mfagalt$quanti.var$cor[, axes, drop = FALSE]		
        xlim <- ylim <- c(-1, 1)		
      } # Final selCov		
      

      
      if (mode(selQuantiVar) == "numeric") 			
        selection <- selQuantiVar			
      else if(length(selQuantiVar)==1) {			
        if (selQuantiVar=="ALL")  selection <- c(1:nrow(coord.Q))			
        else rdo <- selX(selQuantiVar)			
      } else { selection <-  which(rownames(coord.Q) %in% selQuantiVar)}			


      if (rdo=="coord") 			
        selection <- (rev(order(apply(coord.Q^2, 1, max))))[1:min(nrow(coord.Q), 			
                                                                         sum(as.integer(unlist(strsplit(selQuantiVar, "coord"))), na.rm = T))]			
      if (rdo=="cos2") {			
        selcos2 <- as.numeric(substr(selQuantiVar, 5, nchar(selQuantiVar)))			
        selection <- which(apply(res.mfagalt$quanti.var$cos2[, axes], 1, sum)> selcos2)			
      }			
      if (rdo=="contrib") stop("There is not possible to use contrib selection for selQuantiVar")			
      
      if(is.null(selection)) if(rdo=="") if(length(selQuantiVar)==1) selection <- which(rownames(coord.Q) %in% selQuantiVar)		
      if(length(selection)==0) stop("There are not selected elements to plot")		
  

      
      
      if(new.plot) if(!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY")))   
        dev.new(width = min(14, 8 * (xlim[2] - xlim[1])/(ylim[2] - ylim[1])), height = 8) # Outside RStudio 
      else if(new.plot==TRUE) dev.new()
      
      
      coo <- labe <- coll <- ipch <- fonte <- NULL	
    
      
      
      
      
      if (label) labe <- rownames(coord.Q)		
      else labe <- rep("", nrow(coord.Q))	
      #		if(is.null(col.quanti)) col.quanti="blue"	
      coll <- rep(col.quanti, nrow(coord.Q))			
      #      	coo <- coord.Q		
      plot(0, 0, xlab = lab.x, ylab = lab.y, xlim = xlim, ylim = ylim, col = "white", asp = 1, main = title, ...)
      
      if(selCov==FALSE) {				
        x.cercle <- seq(-1, 1, by = 0.01)	
        y.cercle <- sqrt(1 - x.cercle^2)	
        lines(x.cercle, y = y.cercle, ...)		
        lines(x.cercle, y = -y.cercle, ...)		
      } # Final selCov==FALSE
      abline(v = 0, lty = 2, ...)		
      abline(h = 0, lty = 2, ...)		
      
      

      col.var.partial <- col.group					

    
   
      if(partial==FALSE | partial=="ALL"){				
        posi <- NULL
        
        if(length(col.quanti)== 1)
        coll <- rep(col.quanti, nrow(coord.Q))		
        else {
          coll <- rep("blue", nrow(coord.Q))
          coll[1:length(col.quanti)] <- col.quanti
        }
        
        coo <- coord.Q					
        
        if (label) 					
          labe <- rownames(coord.Q)					
        else labe <- rep("", nrow(coord.Q))					
        

        if (!is.null(selection)) {					
          if (is.numeric(unselect)) 					
            coll[!((1:length(coll)) %in% selection)] = rgb(t(col2rgb(coll[!((1:length(coll)) %in% 					
                                                                              selection)])), alpha = 255 * (1 - unselect),maxColorValue = 255)					
          else coll[!((1:length(coll)) %in% selection)] = unselect					
          labe[!((1:length(coll)) %in% selection)] <- ""					
        }				
        

        
        for (v in 1:nrow(coord.Q)) {					
          arrows(0, 0, coord.Q[v, 1], coord.Q[v, 2], 					
                 length = 0.1, angle = 15, code = 2, col = coll[v])					
          if (label) {					
            if (abs(coord.Q[v, 1]) > abs(coord.Q[v,2])) {					
              if (coord.Q[v, 1] >= 0) 					
                posi <- c(posi, 4)					
              else posi <- c(posi, 2)					
            }					
            else {					
              if (coord.Q[v, 2] >= 0) 					
                posi <- c(posi, 3)					
              else posi <- c(posi, 1)					
            }					
          }		
        }


          if (any(labe != "")) {					
            autoLab <- FALSE					
            if (auto.Lab == "auto")  autoLab = (length(which(labe != "")) < 50)					
            if (auto.Lab == TRUE) autoLab <- TRUE					
            if (autoLab == TRUE){ 					
              autoLab(coo[labe != "", 1], y = coo[labe != "", 2], labels = labe[labe != ""],					
                      col = coll[labe != ""], cex=cex, ...)	}				
            else					
              text(coo[labe != "", 1], y = coo[labe != "", 2], labels = labe[labe != ""], 					
                   pos = posi[labe != ""], col = coll[labe != ""], cex=cex,...)					
         #  }		
      }	# Final partial				
  
} # End Temporal      

      
      



      #==============================================================	
      if(partial==TRUE | partial=="ALL"){	
        n.group <- res.mfagalt$call$num.groups	
        n.var <- nrow(res.mfagalt$quanti.var$coord)	
        nam.group <- res.mfagalt$call$name.groups	
        coord.Q <- NULL			
        
        for(i in 1:n.group) {
          coord.Q <- rbind(coord.Q, res.mfagalt$quanti.var$coord.partial[[i]])
        }
        if(selCov==FALSE) {			
          coord.Q  <- coord.Q/sqrt(apply(coord.Q^2,1,sum))
          xlim <- ylim <- c(-1, 1)	
        } # End selCov			
        
        coord.Q <- cbind(coord.Q,rep(nam.group, each=n.var))		
        coord.Q <-  coord.Q[,c(axes,ncol(coord.Q))]	

        posi <- NULL			
        coll <- rep(col.var.partial, each=n.var)			# Basic colors of variables by groups
        coo <- coord.Q	
        if (label) 			
          labe <- rownames(coord.Q)			
        else labe <- rep("", nrow(coord.Q))			
       
       selection <-  which(rownames(coord.Q) %in% rownames(coord.Q)[selection] & coord.Q[,3] %in% selGroup)

        
        
        if (!is.null(selection)) {			
          if (is.numeric(unselect)) 			
            coll[!((1:length(coll)) %in% selection)] = rgb(t(col2rgb(coll[!((1:length(coll)) %in% selection)])),
                                                           alpha = 255 * (1 - unselect),maxColorValue = 255)			
          else coll[!((1:length(coll)) %in% selection)] = unselect			
          labe[!((1:length(coll)) %in% selection)] <- ""			
        }			
        ipch <- rep(25, nrow(coord.Q))				
        fonte <- rep(1, nrow(coord.Q))				
        
        for (v in 1:nrow(coord.Q)) {		
          arrows(0, 0, as.numeric(coord.Q[v, 1]), as.numeric(coord.Q[v, 2]), 		
                 length = 0.1, angle = 15, code = 2, col = coll[v])		
          #   if (shadowtext) points(coo[, 1], y = coo[, 2], pch = ipch, col = coll,cex=cex, ...)				
          
        } # End for

        posi <- NULL			
        coll <- rep(col.var.partial, each=n.var)			
        coo <- coord.Q		
        if (label) 			
          labe <- rownames(coord.Q)			
        else labe <- rep("", nrow(coord.Q))			
        
        if (!is.null(selection)) {			
          if (is.numeric(unselect)) 			
            coll[!((1:length(coll)) %in% selection)] = rgb(t(col2rgb(coll[!((1:length(coll)) %in% 			
                                                                              selection)])), alpha = 255 * (1 - unselect),maxColorValue = 255)			
          else coll[!((1:length(coll)) %in% selection)] = unselect		
          
          # labe[!((1:length(coll)) %in% selection)] <- ""
          # labe[((1:length(coll)) %in% selection)] <- rownames(coord.Q)[selection]
        }			
        ipch <- rep(25, nrow(coord.Q))				
        fonte <- rep(1, nrow(coord.Q))				
        
        for (v in 1:nrow(coord.Q)) {		
          arrows(0, 0, as.numeric(coord.Q[v, 1]), as.numeric(coord.Q[v, 2]), 		
                 length = 0.1, angle = 15, code = 2, col = coll[v])		
       #   if (shadowtext) points(coo[, 1], y = coo[, 2], pch = ipch, col = coll,cex=cex, ...)				
          
        } # End for
        if (any(labe != "")) {				
          labe <- rownames(coord.Q)
          autoLab(as.numeric(coord.Q[labe != "", 1]), y = as.numeric(coord.Q[labe != "", 2]), 
                  labels = labe[labe != ""], col = coll[labe != ""], font = fonte[labe != ""], cex=cex, ...)					
        }


             

        nselGr <- which(old.name.groups %in% selGroup)
       
        
        if(legend) 
         legend(pos.legend,label.group[nselGr],pch=NULL,cex=cex*1.5,text.col=col.var.partial[ nselGr],
                          col=col.var.partial[ nselGr])	  
                 
      } # End partial
      #==============================================================	

      
      
    } ########################## Final selQuantiVar
    
    

  } # Final multiple case
  
}

#' @rawNamespace import(ggpubr, except = c(rotate))
#' @import gridExtra
#' @import stringr 
#' @import patchwork
#' @importFrom utils modifyList
#' @export
plot.LexChar <- function (x, char.negat=TRUE, col.char.posit="blue", col.char.negat="red",
  col.lines="black", theme=theme_bw(), text.size=12,numr=1,numc=2, top=NULL, 
  max.posit=15, max.negat=15, type=c("CharWord","quanti","quali"), sel.var.cat="ALL", 
  txt.var.cat=NULL, sel.words="ALL",  ...)   
{
 ## Pendiente ayudas:
  # quanti, variables en orden alfabético
  # xlab

  dots <- list(...)
## Pendiente hjust
 # if("hjust" %in% names(dots)) hjust<- dots$hjust else hjust <- 0  
  ### <------------------------ Pendiente
 if("ylab" %in% names(dots)) ylab <- dots$ylab else ylab <- "vtest"
  # text.size

  
  if(is.null(type)) type <- "CharWord"
  
  options(stringsAsFactors = FALSE)
  if (!inherits(x, "LexChar"))  stop("x object should be LexChar class")
  words <-vtest <- NULL
  
  
  
  type <- match.arg(type[1], c("CharWord", "quanti", "quali"))
  if(max.negat<1) char.negat<-FALSE 
  if(char.negat==FALSE) max.negat<-0
  
    if(is.null(top)) top <- paste0("Characteristic words. Proba= ", x$Proba)

  theme$text$size <- text.size
  icont<-0
  
  marrangeGrob2<- function(grobs, ncol, nrow, ..., top ) 
  {
  # grobs <- lapply(grobs, ggplotGrob)
  # gridExtra::grid.arrange(grobs = grobs)
    #######################################
    
    n <- length(grobs)
    nlay <- nrow * ncol
    pages <- n%/%nlay + as.logical(n%%nlay)
    
    groups <- split(seq_along(grobs), gl(pages, nlay, n))

    figure <- ggpubr::ggarrange(plotlist=grobs, 
                       # labels = c("A", "B", "C"),
                       ncol = ncol, nrow = nrow, #,
                       # common.legend = TRUE, legend = "bottom",
                       legend = "top",
                       common.legend = TRUE
                       )

    if(pages==1)  figure <- figure +
      patchwork::plot_annotation(title=eval(top),theme = theme(plot.title = element_text(hjust = 0.5)))
    else {
      for(i in 1:pages) {
        figure[[i]] <- figure[[i]] +
          patchwork::plot_annotation(title=eval(top),theme = theme(plot.title = element_text(hjust = 0.5)))
      }
    }
    
           return(figure)
  }
  

  
  
  # -------------------------------------------------------------------------------------
  
  fCharWord <- function(x, top)  {
    svc <- NULL
    icont<-0
    ldoc<-names(x)
    ntdoc<-length(ldoc)
    pword <-list()

    theme$text$size <- text.size
    
    if(length(sel.var.cat)==1) if(sel.var.cat=="ALL") sel.var.cat <- ldoc
    
    if(is.numeric(sel.var.cat)) 
      svc <- ldoc[sel.var.cat] else svc <- sel.var.cat[sel.var.cat %in% ldoc]
    # if(is.null(svc)) svc <- ldoc
    n.svc <- length(svc)
    

    for(i.svc in 1:n.svc)
      #   for (idoc in 1:ntdoc)
    {
      name.svc <- svc[i.svc]


      if(!is.null(x[[name.svc]])) {
        df <- as.data.frame(x[name.svc])
        df[,1] <- rownames(df)
        df[,2] <- df[,6] 
        df[,3:6] <- NULL

        rownames(df) <- NULL
        colnames(df) <- c("words", "vtest")
        df$words <- reorder(df$words,df$vtest)
      
        sel.words.S <- NULL
        if(length(sel.words)==1) {if(sel.words=="ALL") 
          sel.words.S <- df$words
          else sel.words.S <- sel.words
        }
        if(is.null(sel.words)) sel.words.S <- df$words
        if(length(sel.words)>1) sel.words.S <- sel.words
        
       
        
        
        
        
          
        df <- df[df$words %in% sel.words.S,,drop=FALSE]
        
          if(!char.negat)      df <- df[-df$vtest<0,,drop=FALSE]
        
        numposit<-nrow(df[df$vtest>0,])
        numnegat <- nrow(df[df$vtest<0,])
        
        if(numposit > max.posit) {
          df <- df[-((max.posit+1):numposit),,drop=FALSE]
          numposit <- max.posit
        }
        
        if(numnegat > max.negat) {
          df <- df[-((numposit+1):(nrow(df)-max.negat)),,drop=FALSE]
          numnegat <- max.negat
        }
        
       colorXX <- c(rep(col.char.posit,numposit),rep(col.char.negat,nrow(df)- numposit)) 
        
        icont <- icont+1
        

        if(!is.null(txt.var.cat))  subtitle <- txt.var.cat[i.svc] else
        subtitle <- names(x[name.svc])
        
  
       
        pword[[i.svc]] <- ggplot(df) + geom_bar(aes(x=words,y=vtest),stat = "identity", color = col.lines,
                                                   fill = colorXX)+ coord_flip() +
          labs(title = subtitle)+ ylab(ylab) + xlab("") + 
          # ggtitle(title) + 
          theme(axis.text = element_text(size = text.size))+theme
      }  else {  # NO words to select
        
        if(!is.null(txt.var.cat))  subtitle <- txt.var.cat[i.svc] else
          subtitle <- names(x[name.svc])
        
        pword[[i.svc]] <- ggplot() + geom_bar(aes(x="",y=0),stat = "identity")+
                                              coord_flip()+
          labs(title = subtitle)+ ylab(ylab) + xlab("") + theme
          # ggtitle(title) + 
          theme(axis.text = element_text(size = text.size))
      }
    }
    return(pword)
  }

  
  if(type=="CharWord")  {
    if(is.null(top)) top <- paste0("Characteristic words. Proba= ", x$Proba)
    pword <- fCharWord(x$CharWord,  top)
  } # End CharWord
  
  if(type=="quali")  {
    if(is.null(x$quali$CharWord)) stop("There are not qualitative variable to plot in LexChar object")
    if(is.null(top)) top <- paste0("Characteristic words. Proba= ", x$Proba)
    pword <- fCharWord(x$quali$CharWord,  top)
  } # End CharWord
  

  fCharWord.Quanti <- function(x, top, p.Proba)  {
    tac <- NULL
    pword <-list()
    str.colnames<- c("Word", "GlobalAverage", "AverageWord","Differ.", "vtest", "pvalue")
    if(is.null(top)) top <- paste0("Characteristic words. Proba= ", p.Proba)
    theme$text$size <- text.size
    
    # Variable selection
    if(length(sel.var.cat)==1) if(sel.var.cat=="ALL") sel.var.cat <- names.quanti
        if(is.numeric(sel.var.cat)) 
      svc <- names.quanti[sel.var.cat] else svc <- sel.var.cat[sel.var.cat %in% names.quanti]
    if(length(svc)==0) stop("There are no quantitative variables to select")
    n.svc <- length(svc)

    strcolnames<- c("GlobalAverage", "AverageWord","Difer.", "vtest", "pvalue", "Word", "Variable")

  

    for(i in 1:length(x)) {
      t1<- as.data.frame(x[i,drop=FALSE])
      t2 <- data.frame(t1,rep(names(x)[i],length(x[i])), rownames(t1))
      colnames(t2) <- strcolnames
      if(is.null(tac)) tac <- t2 else  tac <- rbind(tac,t2)
    }

    total.words <- unique(tac$Word)
    
    if(length(sel.words)==1) if(sel.words=="ALL") sel.words <- total.words
    sel.words <- sel.words[sel.words %in% total.words]
    tac <- tac[tac$Word %in% sel.words, , drop=FALSE]

    
    for(i.svc in 1:n.svc) {
      SP <- tac[tac$Variable==svc[i.svc],,drop=FALSE]
      t1 <- as.data.frame(tac[tac$Variable==svc[i.svc],,drop=FALSE])
     # t1 <- t1[t1$Word %in% sel.words,,drop=FALSE]
      t2.pos <- t1[t1[,3]>0, ,drop=FALSE]
      t2.neg <- t1[t1[,3]<0, ,drop=FALSE] 
      
      if(nrow(t2.pos)>0) {
        t2.pos <- t2.pos[order(-t2.pos[,4]),,drop=FALSE]
        rownames(t2.pos) <- paste0("P", c(1:nrow(t2.pos)))  
        t2.pos <- t2.pos[,c(6,4,5)]
        t2.pos <- t2.pos[1:min(max.posit,nrow(t2.pos)) ,,drop=FALSE]
      }
      if(nrow(t2.neg)>0) {
        t2.neg <- t2.neg[order(t2.neg[,4]),,drop=FALSE]
        rownames(t2.neg) <- paste0("N", c(1:nrow(t2.neg)))  
        t2.neg <- t2.neg[,c(6,4,5)]
        t2.neg <- t2.neg[1:min(max.negat,nrow(t2.neg)) ,,drop=FALSE]
        t2.neg <- t2.neg[order(-t2.neg[,2]),,drop=FALSE]
      } 
      
      numposit <- nrow(t2.pos)
      numnegat <- nrow(t2.neg)
      colorXX <- c(rep(col.char.posit,numposit),rep(col.char.negat,numnegat)) 

      df <- rbind(t2.pos, t2.neg)

      if(!is.null(txt.var.cat))  subtitle <- txt.var.cat[i.svc] else
        subtitle <- svc[i.svc]
      df$word <- factor(df$Word,                                    # Factor levels in decreasing order
                       levels = df$Word[order(df$vtest, decreasing = FALSE)])
      pword[[i.svc]] <- ggplot(df) + geom_bar(aes(x=word,y=vtest),stat = "identity", color = col.lines,
                                              fill = colorXX)+ coord_flip() + ylab(ylab) + xlab("") +
        labs(title = subtitle)+ 
        # ggtitle(title) + 
        theme(axis.text = element_text(size = text.size))+ theme 
    } # End for
    return(pword)

  } # End fCharWord.Quanti
  
  

  
  
  if(type=="quanti")  {
    if(is.null(x$quanti$CharWord)) stop("There are not quantitative variables to plot in LexChar object")
    if(is.null(top)) top <- paste0("Characteristic words. Proba= ", x$Proba)
    names.quanti <- rownames(x$quanti$stats)
    pword <- fCharWord.Quanti(x$quanti$CharWord, top, x$Proba)
    
  } # End CharWord
  
  
if(length(pword)==1) numr <- numc <- 1
  suppressWarnings(marrangeGrob2(grobs = pword, nrow = numr, ncol = numc, top=top))
}

#' @import stringr
#' @export
LexChar <- function(object,  proba = 0.05, maxCharDoc = 10, maxPrnDoc=100, marg.doc="before",
                    context=NULL, correct=TRUE,  nbsample=500, seed=12345, ...)
 { 
  set.seed(seed)
  options(stringsAsFactors = FALSE)
  # maxCharDoc: Maximum number of characteristic documents to show
  # correct = TRUE and Correction for pvalue of Characteristic words (FactoMineR, Agresti...)
  # marg.doc; after/before/before.RW

  # To compute vtest from pvalue when pvalue=0.000
  dots <- list(...)
  if("eps" %in% names(dots)) eps <- dots$eps else eps <- 0.0001

  # Version 1.4.1 used context.sup, version 1.4.2. use context argument
  varnames<- lapply(substitute(list(...))[-1], deparse)
  if(!is.null(varnames$context.sup)) {
    context <- gsub("[[:punct:]]", "", varnames$context.sup)   # Only for Compatibility version 1.4.1
    warning("Xplortext Versions > 1.4.1 use context, no context.sup argument") 
  }
  ###################################################################

  if(marg.doc!="before"& marg.doc!="before.RW" & marg.doc!="after") stop("Error in marg.doc argument")
  
  # Checking object type, proba limits and maxCharDoc values
  if (!inherits(object,"TextData") & !inherits(object,"DocumentTermMatrix") & !inherits(object,"matrix")
      & !inherits(object,"data.frame"))
    stop("Object should be TextData, DocumentTermMatrix, matrix or data frame")
  if(proba<0|proba>1) stop("proba should be between 0 and 1")
  
  if(is.null(maxCharDoc) |  maxCharDoc <0) maxCharDoc <- 0 


  ###################################################################
  ## Functions
  ############### descfreq_New function #############################################
  descfreq_NewChar <- function (data, proba = 0.05, marge.li, marge.col) 				
  {
    lab.sauv <- lab <- colnames(data)				
    for (i in 1:length(lab)) lab[i] = gsub(" ", ".", lab[i])				
    colnames(data) <- lab
    
    old.warn = options("warn")			
    options(warn = -1)			# suppress warnings globally
    
    nom <- tri <- structure(vector(mode = "list", length = nrow(data)), names = rownames(data))		
    if(is.data.frame(marge.li)) marge.li <- as.vector(marge.li[,1]) else marge.li <- as.vector(marge.li)
    if(is.data.frame(marge.col)) marge.col <- as.vector(marge.col[,1]) else marge.col <- as.vector(marge.col)
    
    
    SumTot<-sum(marge.li)				
    nr <- nrow(data)  				
    nc <- ncol(data) 				
    
    for (j in 1:nr) {
      aux3 <- marge.li[j]/SumTot				# % Occurrences before or after
      
      for (k in 1:nc) {
        # if(j==4 & k==1) stop(colnames(data)[7])  # balance
        aux2 <- data[j, k]/marge.col[k]
        daux <- dhyper(data[j, k], marge.col[k], SumTot - marge.col[k], marge.li[j])
        

        if(correct==FALSE) daux <- daux*2  
        #  P[X <= OCURR.W-1]
        A <- phyper(data[j, k]-1, marge.col[k], 				
                    SumTot - marge.col[k], marge.li[j],lower.tail = TRUE)
        #  P[X > OCURR.W]
        B <- phyper(data[j, k], marge.col[k], SumTot - marge.col[k], marge.li[j],lower.tail = FALSE)
        aux4 <- min(A,B)*2+daux
       

        if (aux4 > 1) aux4 <- 2 - aux4

        if (aux4 <= proba) {
          aux5 = (1 - 2 * as.integer(aux2 > aux3)) * qnorm(aux4/2)  # vtest			
          aux1 = data[j, k]/marge.li[j]				
          tri[[j]] = rbind(tri[[j]], c(aux1 * 100, sum(marge.col[k])/SumTot * 				
                                         100, data[j, k], marge.col[k], aux4, aux5))	
          nom[[j]] = rbind(nom[[j]], c(colnames(data)[k], colnames(data)))				
        }
      }
    }
    
    
    for (j in 1:nr) {				
      if (!is.null(tri[[j]])) {				
        oo = rev(order(tri[[j]][, 6]))				
        tri[[j]] = tri[[j]][oo, ]				
        nom[[j]] = nom[[j]][oo, ]				
        if (nrow(matrix(tri[[j]], ncol = 6)) > 1) 				
          rownames(tri[[j]]) = nom[[j]][, 1]				
        else {				
          tri[[j]] = matrix(tri[[j]], ncol = 6)				
          rownames(tri[[j]]) = nom[[j]][1]				
        }				
        colnames(tri[[j]]) = c("Intern %", "glob %", "Intern freq", 				
                               "Glob freq ", "p.value", "v.test")				
      }				
    }				
    res = tri				
    options(old.warn)				
    class(res) <- c("descfreq", "list ")				
    return(res)				
  }		  # End   descfreq_New
  
  
  
  
 

   ##############  Function to compute Vocab$quali$stats  
  chi.quali<- function(X, QL)  {
    list.Agg <- lapply(seq_along(QL),FUN=function(i) t(t(as.matrix(X))%*% as.matrix(QL[[i]])))
    
    Nq <- sum(X)
    
    dfq<-NULL
    old.warn = options("warn")				
    options(warn = -1)			# suppress warnings globally
    for(i in 1:length(list.Agg)) {
      ch <- chisq.test(list.Agg[[i]], correct=FALSE)[1:3]
      ph2 <- ch$statistic/Nq
      ph <- sqrt(ph2)
      minrowcol <- min(ncol(list.Agg[[i]]),nrow(list.Agg[[i]]))-1
      VCr <-sqrt(ph2/ minrowcol )
      d <- data.frame(ch,ph,VCr)
      dfq <- rbind(dfq,d)
    }
    colnames(dfq) <- c("Chi.Squared", "df", "p.value", "phi", "Cramer's V")
    rownames(dfq) <-  names(QL)
    options(old.warn)	
    return(dfq)
  }  
  chi.quali.single <- function(X)  {
    old.warn = options("warn")				
    options(warn = -1)			# suppress warnings globally
    ch <- chisq.test(X, correct=FALSE)[1:3]
    ph2 <- ch$statistic/sum(X)
    ph <- sqrt(ph2)
    minrowcol <- min(ncol(X),nrow(X))-1
    VCr <-sqrt(ph2/ minrowcol )
    d <- data.frame(ch,ph,VCr)    
    colnames(d) <- c("Chi.Squared", "df", "p.value", "phi", "Cramer's V")
    rownames(d) <-  "LexicalTable"
    options(old.warn)	
    return(d)
  }

  ##############  Function to compute Vocab$quanti$stats    
  mean.p <- function(V,poids) res<-sum(V*poids,na.rm=TRUE)/sum(poids[!is.na(V)])
  var.p <- function(V,poids) res<-sum(V^2*poids,na.rm=TRUE)/sum(poids[!is.na(V)])

  vocabQuanti <- function(vdt,vX, vrow.INIT ) {
    vrow.INIT <- vrow.INIT[,1]
    vX <- as.data.frame(vX)  # Quantitative values of variables

    for(i in 1:ncol(vX)){
      if(any(is.na(vX[,i]))) warning("\n", names(vX)[i], " variable has missing values. They will be replaced by the mean\n") 
      vX[is.na(vX[,i]), i] <- mean(vX[,i], na.rm = TRUE)
    }
 
    # Weight documents. Different from marg.doc
    Wi <- vrow.INIT/sum(vrow.INIT)
    col.INIT <- colSums(vdt)
  
    y <- sum(col.INIT)
    #  Weighted average of quantitative variables  (1 x 2 variables)
    
    Aver.X <- apply(vX,2,mean.p,Wi)
    Var.X<-apply(sweep(vX,2,Aver.X,"-"),2,var.p,Wi)

    #Average of quantitative variables for each word
    Wij.cj <- as.matrix(sweep(vdt,2,col.INIT ,"/"))
    
    nk <- colSums(vdt)

    ntot <- sum(vrow.INIT)
    MeanWord <- t(vX) %*% Wij.cj 

    # Variance of words
    coef <- as.matrix((y/col.INIT-1)/(y-1))
    Var.Y <- coef %*% Var.X
    sc.inter <-   apply(sweep(t(sweep(t(MeanWord), 2, Aver.X,"-")^2),2,col.INIT,"*"),1,sum)/y
    pct.expl <- sc.inter/Var.X
    Desv.Y <- sqrt(Var.Y)
    dj <- t(sweep(MeanWord,1,Aver.X,"-")) / Desv.Y
    
    # colSums(vdt)[colnames(vdt)[i]] 
    

    ### Permutations
    # Doing list with vocabulary elements
    nWords <- ncol(vdt)                                                       
    nDocs <- nrow(vdt)                                                          
    
    relcq.perm <-vector(mode='list',length=nWords)
    for (i in 1:nWords) relcq.perm[[i]]<-matrix(nrow=nbsample,ncol=ncol(vX),0) 
    relcq.SS.perm <-vector(mode='list',length=ncol(vX))
    for (i in 1:ncol(vX)) relcq.SS.perm[[i]]<- matrix(nrow=nbsample,ncol=1,0) 	 
    
 
    for (i in 1:nbsample){                                                      
      X.perm <- vX[sample(1:nDocs),,drop=FALSE]
      Aver.X.perm<-apply(X.perm,2,mean.p,Wi)
      MeanWord.perm <- t(X.perm) %*% Wij.cj
      Var.X.perm<-apply(sweep(X.perm,2,Aver.X.perm,"-"),2,var.p,Wi)
      Var.Y.perm <- coef %*% Var.X.perm
      # dj words x variables
      dj.perm <- t(sweep(MeanWord.perm,1,Aver.X.perm,"-")) / sqrt(Var.Y.perm)   
      

      for (iw in 1:nWords) 
        relcq.perm[[iw]][i,]<-dj.perm[iw,]    
      sc.inter.perm <-   apply(sweep(t(sweep(t(MeanWord.perm), 2, Aver.X.perm,"-")^2),2,col.INIT,"*"),1,sum)/y
      pct.expl.perm <- sc.inter.perm/Var.X.perm
      
      for (iw in 1:ncol(vX)) relcq.SS.perm[[iw]][i,] <- pct.expl.perm[iw] # /  SC.X.perm[iw]
    }
  
  
    #  names(relcq.perm) <- colnames(vdt)
    a <- matrix(nrow=ncol(vdt),ncol=ncol(vX),0)
    b <- matrix(nrow=1,ncol=ncol(vX),0)


    for (k in 1:ncol(vX)){
      for (i in 1:ncol(vdt)){
        if (dj[i,k]>0) a[i,k] <- sum(relcq.perm[[i]][,k]>dj[i,k])/nbsample
        if (dj[i,k]<0) a[i,k] <- sum(relcq.perm[[i]][,k]<dj[i,k])/nbsample
      }
      b[1,k] <- sum(relcq.SS.perm[[k]]> pct.expl[k]) /nbsample
    }
    
    # res.mat <- matrix(nrow=ncol(vX),ncol=4,0) 
    res.mat <- matrix(nrow=ncol(vX),ncol=5,0) 	
    rownames(res.mat)<-colnames(vX)
   # colnames(res.mat )<- c("GlobalAverage","AverageWord","Differ.","pvalue")
    colnames(res.mat )<- c("GlobalAverage","AverageWord","Differ.","vtest", "pvalue")
    #    names(relcq.perm) <- colnames(vdt)


    
    for (i in 1:nWords) {
      relcq.perm[[i]] <- res.mat
      relcq.perm[[i]][,1] <- t(Aver.X)
      relcq.perm[[i]][,2] <- MeanWord[,i]
      relcq.perm[[i]][,3] <-  relcq.perm[[i]][,2]- relcq.perm[[i]][,1]
      # vtest non weighted
      # relcq.perm[[i]][,4] <- (relcq.perm[[i]][,2]- relcq.perm[[i]][,1] )/ 
      #    sqrt( (ntot- nk[colnames(MeanWord)[i]]) * t(Var.X)/ ((ntot-1)* nk[colnames(MeanWord)[i]]) )
      # 1.- Numerator relcq.perm[[i]][,2]- relcq.perm[[i]][,1]
      # 2.- Variance quantitative variable t(Var.X)
      # 3. n is ntot (total occurrences)
      # 4. nk occurrences word k
      relcq.perm[[i]][,4] <- relcq.perm[[i]][,5] <- t(a[i,])
       relcq.perm[[i]][,4][relcq.perm[[i]][,4]<eps ] <- eps 
     # relcq.perm[[i]][,4] <- sign(relcq.perm[[i]][,3]) * abs(qnorm(relcq.perm[[i]][,4]))
       relcq.perm[[i]][,4] <- sign(relcq.perm[[i]][,3]) * abs(qnorm(relcq.perm[[i]][,4]/2))

      names(relcq.perm)[[i]] <- colnames(vdt)[i]
      
      relcq.perm[[i]] <- subset(relcq.perm[[i]], relcq.perm[[i]][,5] < proba)
    }

    relcq.perm <- relcq.perm[sapply(relcq.perm, function(x) ifelse(nrow(x)==0,F,T))]
    res.mat.SS <- matrix(nrow=ncol(vX),ncol=4,0) 	
    rownames(res.mat.SS)<-colnames(vX)
    colnames(res.mat.SS)<- c("TotSumSquares","BetweenSquares","%Explained","pvalue")
    for (i in 1:ncol(vX)) {
      res.mat.SS[i,1] <- Var.X[i]
      res.mat.SS[i,2] <- sc.inter[i]
      res.mat.SS[i,3] <- pct.expl[i]
      res.mat.SS[i,4] <- b[1,i]
    }
    
    quanti <- list(CharWord=relcq.perm, stats=res.mat.SS)
    return(quanti)
  }
  

  ##### End of functions #############################################

  
  #### step 1. Selecting the type of object ##############
  ### Verifying if it is a TextDataObject and aggregated TextData
  bTextData <- ifelse(inherits(object,"TextData"),TRUE,FALSE)
  ###  To know if it is an aggregate analysis for an TextData object
  # If it is an aggregated table
  bvaragg <- FALSE
  if(inherits(object,"TextData"))
    if(!is.null(object$info)) 
      if(object$info$name.var.agg[[1]]!="") bvaragg <- TRUE 
  


  #=================================================================	
  # Computing strquanti and strquali
  #### step 2. Detecting names contextual variables ##############
  if(bTextData) {
    # Is bTextData
    # df.qual<- strquali <- strquanti <- NULL
    strquali <- strquanti <- NULL

    
    
    if(!is.null(context))
      if(length(context)==1) if(context=="ALL") {
        if(bvaragg) {
          context  <- colnames(object$SourceTerm.qual)
          context  <- c(context,colnames(object$SourceTerm.quant))  
        } else {
          context  <- colnames(object$context$quali)
          context  <- c(context,colnames(object$context$quanti))
        } }

    # context, quanti and quali names of variables
    strquali<- context[which(context %in% colnames(object$SourceTerm.qual))]
    # strquanti<- colnames(object$SourceTerm.quant)[which(colnames(object$SourceTerm.quant) %in% context)]
    strquanti<- context[which(context %in% colnames(object$SourceTerm.quant))]
    if(!is.null(strquali)) if(length(strquali)==0) strquali <- NULL
    if(!is.null(strquanti)) if(length(strquanti)==0) strquanti <- NULL
    
  } else {
    # Is not bTextData
    # df.context.quali <- df.context.quanti <-NULL
    if("DocumentTermMatrix" %in% class(object)) object <- as.matrix(object)
    if(is.matrix(object))      object <- as.data.frame(object)
    if(!is.data.frame(object)) stop("Error: object must be a dataframe")
    
    if(marg.doc!="after") {
      options(warn=0)
      warning("Only marg.doc==after is allowed ; is changed to after")
      marg.doc <- "after"
    }
    # DocTerm <- object
    if(length(context)==1)
      if(context=="ALL") stop("You must define context variables by index or name of object")

    if(is.numeric(context)) context <- colnames(object)[context]
   #  sel.context <- colnames(object)[which(context %in% colnames(object))]
    sel.context <- context[which(context %in% colnames(object))]

    strquali <- strquanti <- NULL
    

    if(length(sel.context)!=0) {
      df.context <- object[,sel.context,drop=FALSE]
      context.ordered <-  names(df.context)[sapply(df.context, is.ordered)]
      context.type <- sapply(df.context, class)
      
      strquali <-  names(context.type[which(context.type %in% c("factor","character", "logical", "Date"))])
      strquali <-  c(strquali,context.ordered)
      strquanti <- sel.context[!sel.context %in% strquali]

      if(length(strquanti)==0) strquanti <- NULL
      if(length(strquali)==0) strquali <- NULL
    }
    
  }  # End step 2
  
 

  # STEP 3. 
  #=================================================================	
  # if after/before, frequencies after/before TextData selection are used as document weighting (by default "before");
  #  if before.RW all words under threshold in TextData function are included as a new word named RemovedWords
  ##### step 3. Building object to descfreq_NewChar function ##############
  if(bTextData) { # There is TextData object
    # 1. Computing DocTerm
    DT2 <- as.matrix(object$DocTerm)
    if(marg.doc=="after") {
      DT2 <- DT2[rowSums(DT2)!=0,]
      row.INIT.2 <- data.frame(Occurrences.after=rowSums(DT2))
    }

    if(marg.doc=="before") {
      DT2 <- DT2[rowSums(DT2)!=0,]
      row.INIT.2 <- as.data.frame(object$rowINIT)        # Frequency Occurrences.before
      row.INIT.2 <- row.INIT.2[rownames(DT2),,drop=FALSE]
    }

     if(marg.doc=="before.RW") {
      row.INIT.2 <- as.data.frame(object$summDoc[, "Occurrences.before",drop=FALSE])  # All documents
      rownames(row.INIT.2) <- object$summDoc$DocName
      row.INIT.2 <- row.INIT.2[row.INIT.2$Occurrences.before!=0,,drop=FALSE] 
      # NoNullBefore rows with no null documents before but after threshold
      NoNullBefore <- row.INIT.2[!rownames(row.INIT.2) %in% rownames(DT2),,drop=FALSE]
  
      if(nrow(NoNullBefore)>0){
        DT2 <- cbind(DT2,as.data.frame(row.INIT.2[rownames(DT2),]))
        nrDT2 <- nrow(DT2)
        strnames <- rownames(NoNullBefore)
        # Add NoNullBefore rows to dataframe and fill them with 0
        DT2[( nrDT2+1):( nrDT2+  nrow(NoNullBefore)),] <- 0
        # Add rownames to new rows from strnames
        rownames(DT2)[(nrDT2+1):(nrDT2+nrow(NoNullBefore))]  <-  strnames 
        DT2[(nrDT2+1):( nrDT2+  nrow(NoNullBefore)),ncol(DT2)] <- NoNullBefore[,1]
        DT2 <- DT2[rownames(row.INIT.2),]
      } 
      else {
        DT2 <- cbind(DT2,as.data.frame(row.INIT.2[rownames(DT2),]))
      }# End if(nrow(NoNullBefore)>0)
      
      # Is before.RW"
      freq.after <- apply(DT2[,c(1:(ncol(DT2)-1))],1,sum)
      freq.before <- DT2[,ncol(DT2)]  
      # Adding a column with the name of RemovedWords
      DT2 <- cbind(DT2,as.data.frame(freq.before-freq.after))
      colnames(DT2)[ncol(DT2)] <- "RemovedWords"
      DT2 <- DT2[,-(ncol(DT2)-1)]
      #  colnames(DT2)[ncol(DT2)] <- "RemovedWords"
    }  # End before.RW
    # DocTerm can have some document/aggregate document with margin zero
    
    row.INIT.2 <- row.INIT.2[rownames(DT2),,drop=FALSE]
    pos.0 <- which(rowSums(DT2)==0)
    if(length(pos.0)!=0) {
      # Remove empty documents or aggregate documents 
      DT2 <- DT2[!rownames(DT2) %in% names(pos.0),,drop=FALSE]
      row.INIT.2 <- row.INIT.2[!rownames(row.INIT.2) %in% names(pos.0),,drop=FALSE]
    }  
    # dbTextData 
  } # End bTextData for step3
  else{   # Is not TextData
    # Is not bTextData
    # We must have DocTerm without quali and quanti variables
    
     strQLQN <- c(strquali, strquanti)
    DT2.init <- data.frame(object)
    DT2 <- object[,!colnames(DT2.init) %in% strQLQN,drop=FALSE]  # without context variables
    
    df.context.quali <- df.context.quanti <- NULL
    #  pos.0 <- which(rowSums(DT2)==0)
    # Remove empty documents of non TextData object 
    #   if(length(pos.0)!=0)   DT2 <- DT2[!rownames(DT2) %in% rownames(DT2)[pos.0],,drop=FALSE]

    row.INIT.2 <- data.frame(rowSums(DT2))

    if(!is.null(strquali)) df.context.quali<- DT2.init[rownames(DT2),strquali,drop=FALSE]
    if(!is.null(strquanti)) df.context.quanti<- DT2.init[rownames(DT2),strquanti,drop=FALSE]
    #    stop("Is not bTextData")
  }
  #--------------------------------------
  # This is valid for any kind of objects
  col.INIT <- as.vector(colSums(DT2))
  # row.INIT.B <- row.INIT
  #  Computing Lexical Table: Chi.Squared  df   p.value       phi Cramer's V. No weigthing is applied
  n.words<- ncol(DT2)
  d.single <- chi.quali.single(DT2[rowSums(DT2)>0,c(1:n.words)]) 
  #--------------------------------------------------------------------------------------------
  
  resCharWord <- descfreq_NewChar(DT2, proba = proba, row.INIT.2, col.INIT) 	
  
  
  # Return words for active categories
  res <- list(CharWord=resCharWord, stats=d.single)
  res$Proba <- proba
  # Chi square not depend on row.INIT
  


  ###########################################################################################################
  if(length(strquali)==0) strquali <- NULL
  if(length(strquanti)==0) strquanti <- NULL
  strQNQL <- c(strquali, strquanti)
  DT.Z <- NULL
  

  
  ##########################  Contextual variables. DT3
  new.QUALI <- strquali 
  if(bTextData & bvaragg)  new.QUALI <- c(colnames(object$var.agg), new.QUALI)
  # new.QUALI <- unique(c(strQNQL, new.QUALI))  # Quanti, Quali and var.agg
  


  ###################  NO QUALI BUT QUANTI CONTEXTUAL VARIABLES ###################
  
  ##### step XX. Quali / quanti contextual variables ##############
  if(is.null(strquali) & !is.null(strquanti)) {
    # Only quanti
    if(bTextData) {
      if(bvaragg) { # TextData, bvaragg, no quali, yes quanti
        df.context.quanti <- as.matrix(object$context$quanti)
        ROW.Q <- row.INIT.2
      } else { # "TextData, no bvaragg, no quali, yes quanti"
        ROW.Q <- row.INIT.2[rownames(DT2),,drop=FALSE]
        df.context.quanti <- object$SourceTerm.quant
        df.context.quanti <-  df.context.quanti[rownames(DT2),,drop=FALSE]
      }
      #  ROW.W <- ROW.W[rownames(DT2),,drop=FALSE ]
      res$quanti <- vocabQuanti(DT2, df.context.quanti, ROW.Q)
    } else { # 
      # No TextData
      res$quanti <- vocabQuanti(DT2, df.context.quanti, row.INIT.2)
    }
  }
  
  if(length(new.QUALI)==0)  new.QUALI <- NULL
  
  

  
  ##################  If there are context qualitative variables
  if(!is.null(new.QUALI)){  # new.QUALI has the names of var.agg and strquali
    DT.Z2 <- NULL
    if(bvaragg){  # Es agregado
      DT.Z <- data.frame(as.matrix(object$SourceTerm))       # 292 x 24
      df.AGGR <- data.frame(object$var.agg)                 # 292
      df.AGGR <- df.AGGR[rownames(DT.Z),,drop=FALSE]
      
      if(!is.null(strquali)) {
        df.context.quali <- object$SourceTerm.qual[,strquali,drop=FALSE]
        df.context.quali <- df.context.quali[rownames(DT.Z),,drop=FALSE]
        df.AGGR <- cbind(df.AGGR,df.context.quali)               # 292
      }
      
      df.context.quali <- df.AGGR
      rm(df.AGGR)
      strquali <- colnames(df.context.quali)
      
      if(marg.doc=="after") {
        ROW.W <- data.frame(rowSums(DT.Z))
        rownames(ROW.W) <- rownames(DT.Z)  # 292
      }
      
      if(marg.doc=="before") {
        ROW.W <- object$SourceTerm.freq # 292
        rownames(ROW.W) <- ROW.W$DocName
        ROW.W <- ROW.W[,-1,drop=FALSE]
      }
      
      if(marg.doc=="before.RW") {
        DT.Temp <- as.matrix(object$SourceTerm.dtm)
        DT.Temp <- DT.Temp[rowSums(DT.Temp)!=0,]   # 298
        
        ROW.W <- data.frame(rowSums(DT.Temp))
        rownames(ROW.W) <- rownames(DT.Temp)       # 298
        rm(DT.Temp)
        NoNullBefore <- ROW.W[!rownames(ROW.W) %in% rownames(DT.Z),,drop=FALSE]
        
        if(nrow(NoNullBefore)>0){
          DT.Z2 <- cbind(DT.Z,as.data.frame(ROW.W[rownames(DT.Z),]))  # 292
          nrDT2 <- nrow(DT.Z2)
          strnames <- rownames(NoNullBefore)
          
          # Add NoNullBefore rows to dataframe and fill them with 0
          DT.Z2[( nrDT2+1):( nrDT2+  nrow(NoNullBefore)),] <- 0     # 298
          rownames(DT.Z2)[(nrDT2+1):(nrDT2+nrow(NoNullBefore))]  <-  strnames 
          DT.Z2[(nrDT2+1):( nrDT2+  nrow(NoNullBefore)),ncol(DT.Z2)] <- NoNullBefore[,1]
          DT.Z <- DT.Z2[rownames(ROW.W),]
          # Add rownames to new rows from strnames
        } 
        else {
          DT.Z2 <- cbind(DT.Z,as.data.frame(ROW.W[rownames(DT.Z),]))
        }# End if(nrow(NoNullBefore)>0)
        
        # Is before.RW"
        freq.after <- apply(DT.Z2[,c(1:(ncol(DT.Z2)-1))],1,sum)
        freq.before <- DT.Z2[,ncol(DT.Z2)]  
        # Adding a column with the name of RemovedWords
        DT.Z2 <- cbind(DT.Z2,as.data.frame(freq.before-freq.after))
        colnames(DT.Z2)[ncol(DT.Z2)] <- "RemovedWords"
        DT.Z <- DT.Z2[,-(ncol(DT.Z2)-1)]
        #  colnames(DT2)[ncol(DT2)] <- "RemovedWords"
        
        df.context.quali <- object$SourceTerm.var.agg[rownames(DT.Z),,drop=FALSE]
        if(!is.null(object$SourceTerm.qual))
          df.context.quali <- cbind(df.context.quali,object$SourceTerm.qual[rownames(DT.Z),,drop=FALSE] )
      }
      if(!is.null(strquanti)) {
        df.context.quanti <- data.frame(object$SourceTerm.quant)  # <-------------------------------300.
        df.context.quanti <- df.context.quanti[rownames(DT.Z),,drop=FALSE]  # 292 o 298
      }
    } # End bvaragg aggregate
    
    
  
  
    
    
    if(!bvaragg & bTextData){  # Is not aggregate
      #  df.context.quali <- object$context$quali   # 292
      df.context.quali <- object$SourceTerm.qual[,strquali,drop=FALSE]  # 300
      DT.Z <- data.frame(as.matrix(object$DocTerm))      # 292 (always) x 24
      
      
      if(marg.doc=="after") {
        ROW.W <- data.frame(rowSums(DT.Z))
        rownames(ROW.W) <- rownames(DT.Z)  # 292  
        DT.Z2 <- DT.Z[rownames(ROW.W),,drop=FALSE]
        if(!is.null(strquanti)) {
          df.context.quanti <- as.matrix(object$context$quanti)  # 292
          df.context.quanti <- df.context.quanti[rownames(DT.Z),,drop=FALSE]
        }
      }
      
      if(marg.doc=="before"){
        ROW.W <- object$rowINIT                 # 300  
        ROW.W <- ROW.W[rownames(DT.Z),,drop=FALSE]  # 292
        DT.Z2 <- DT.Z[rownames(ROW.W),,drop=FALSE]
        if(!is.null(strquanti)) {
          df.context.quanti <- as.matrix(object$context$quanti)  # 292
          df.context.quanti <- df.context.quanti[rownames(DT.Z),,drop=FALSE]
        }
      } 
      
      if(marg.doc=="before.RW") {  
        ROW.W <- object$rowINIT                 # 300  
        ROW.W <- ROW.W[ROW.W$Occurrences.before!=0,,drop=FALSE]  # 298
        
        # NoNullBefore rows with no null documents before but after threshold
        NoNullBefore <- ROW.W[!rownames(ROW.W) %in% rownames(DT.Z),,drop=FALSE]
        
        if(nrow(NoNullBefore)>0){
          DT.Z2 <- cbind(DT.Z,as.data.frame(ROW.W[rownames(DT.Z),]))  # 292
          nrDT2 <- nrow(DT2)
          strnames <- rownames(NoNullBefore)
          # Add NoNullBefore rows to dataframe and fill them with 0
          DT.Z2[( nrDT2+1):( nrDT2+  nrow(NoNullBefore)),] <- 0     # 304
          # Add rownames to new rows from strnames
          rownames(DT.Z2)[(nrDT2+1):(nrDT2+nrow(NoNullBefore))]  <-  strnames 
          DT.Z2[(nrDT2+1):( nrDT2+  nrow(NoNullBefore)),ncol(DT.Z2)] <- NoNullBefore[,1]
          DT.Z2 <- DT.Z2[rownames(ROW.W),]
          DT.Z <- DT.Z2
        } 
        else {
          DT.Z2 <- cbind(DT.Z2,as.data.frame(ROW.W[rownames(DT.Z2),]))
        }# End if(nrow(NoNullBefore)>0)
        # Is before.RW"
        freq.after <- apply(DT.Z2[,c(1:(ncol(DT.Z2)-1))],1,sum)
        freq.before <- DT.Z2[,ncol(DT.Z2)]  
        # Adding a column with the name of RemovedWords
        DT.Z2 <- cbind(DT.Z2,as.data.frame(freq.before-freq.after))
        
        colnames(DT.Z2)[ncol(DT.Z2)] <- "RemovedWords"
        DT.Z2 <- DT.Z2[,-(ncol(DT.Z2)-1)]
        DT.Z <- DT.Z2
      }
      
      df.context.quali <- df.context.quali[rownames(DT.Z),,drop=FALSE]
      
      if(!is.null(strquanti)) {
        df.context.quanti <- data.frame(object$SourceTerm.quant)
        df.context.quanti <- df.context.quanti[rownames(DT.Z),,drop=FALSE]
      }
    } # End !bvaragg
    
    if(!is.null(DT.Z2)) rm(DT.Z2)
    
   } ####### End quali
  
  

  
  ####### 
  if(!is.null(new.QUALI)) {
 
    if(!bTextData)  {
      DT.Z <- DT2
      ROW.W <- data.frame(rowSums(DT.Z))
      rownames(ROW.W) <- rownames(DT.Z)
    }

    
    # df.context.quali si es mayor que uno unir con @
    # if df.context.quanti
    # ROW.W
    # return(DT.Z)
    df.context.quali$new.Cat <- ""
    df.context.quali$new.Cat  <- paste0(df.context.quali$new.Cat, paste0(colnames(df.context.quali)[1],"."), df.context.quali[,1])
    if(ncol(df.context.quali)>2)
      for(i in 2:(ncol(df.context.quali)-1)) {
        df.context.quali$new.Cat  <- paste0(df.context.quali$new.Cat, 
                                            paste0("@",colnames(df.context.quali)[i],"."), df.context.quali[,i])
        df.context.quali <- df.context.quali[rownames(DT.Z),,drop=FALSE]          
      }
   

    if(length(strquanti)==0) strquanti <- NULL
    ROW.W <- ROW.W[rownames(DT.Z),,drop=FALSE]
    df.context.quali <- df.context.quali[rownames(DT.Z),,drop=FALSE]
    ROW.W$new.Cat <- df.context.quali$new.Cat
    DT.Z$new.Cat <-  df.context.quali$new.Cat

    # -------  Do aggregate
    DT.Z.aggr <- aggregate(.~new.Cat, DT.Z, sum) 
    rownames(DT.Z.aggr) <- DT.Z.aggr$new.Cat
    DT.Z.aggr <- DT.Z.aggr[,-1,drop=FALSE]
  

    ROW.W.aggr <- aggregate(.~new.Cat, ROW.W, sum) 
    rownames(ROW.W.aggr) <- ROW.W.aggr$new.Cat
    ROW.W.aggr <- ROW.W.aggr[,-1,drop=FALSE]
    

    if(!is.null(strquanti)) {
      df.context.quanti <- df.context.quanti[rownames(DT.Z),,drop=FALSE]
      df.context.quanti$new.Cat <- df.context.quali$new.Cat
      df.context.quanti.aggr <- aggregate(.~new.Cat, df.context.quanti, mean) 
      rownames(df.context.quanti.aggr) <- df.context.quanti.aggr$new.Cat
      df.context.quanti.aggr <- df.context.quanti.aggr[,-1,drop=FALSE]
      res$quanti <- vocabQuanti(DT.Z.aggr,
                                df.context.quanti.aggr[rownames(DT.Z.aggr),strquanti,drop=FALSE],
                                ROW.W.aggr[rownames(DT.Z.aggr),,drop=FALSE])
    }
    res$quali$CharWord  <- descfreq_NewChar(DT.Z.aggr, proba = proba, ROW.W.aggr[rownames(DT.Z.aggr),,drop=FALSE],
                                            col.INIT) 
    res$quali$stats <- chi.quali.single(DT.Z.aggr)
  }
  
  
 
  
  ############################################3
  fCharDoc  <- function(CharWord.z, df.QUAL.z, DT.z) {
    
    motsval <- sapply(CharWord.z ,function(x) if(!is.null(x))  x[order(rownames(x)),6,drop=FALSE],simplify = FALSE)
    ########################################
    # motsval vtest of words for each group
    # Intern %     glob % Intern freq Glob freq       p.value    v.test
    # For each group, alphabetical order of words and |vtest|>1.96 
    # motsval <- sapply(resCharWord.QL,function(x) if(!is.null(x))  x[order(row.names(x)),6,drop=FALSE],simplify = TRUE)
    # vlev Name of supplementary categories 
    vlev <- names(CharWord.z)
    nlev <- length(vlev)	
    lisresult <- vector(mode="list",length=nlev)
    DT.z <- DT.z[,-ncol(DT.z),drop=FALSE]
    
    
    ######### -----------------------------  ´
    for (ilev in 1:nlev)     {
      # docpos doc position for each group, starting the firs ilev=1)
      docpos <- which(df.QUAL.z$new.Cat == vlev[ilev])  # vlev vector with the docs names of aggregated categories

      
      # ntrep maximum number of documents to print, minimum between maxPrnDoc and the size of the group
      ntrep <- min(maxCharDoc, length(docpos))   # length(docpos) is the number of documents in each group
      
      lisresult[[ilev]] <- data.frame(nlev,3)  # Name of the group

      SourceTermcurs <- DT.z[docpos,,drop=FALSE ] 
      # SourceTermcurs the selected rows for non aggregated table
      #lisresult[[ilev]] <- SourceTermcurs
      ly<-as.matrix(rep(0,ncol(DT.z)),drop=FALSE)
      # Rows are the vocabulary of after words
      rownames(ly)<-colnames(DT.z)

      
      if(is.null(motsval[[ilev]])) {  
        lisresult[ilev] <- list(NULL)
      } else {
        # There are significant words for this category
        motsvalcurs<- as.matrix(motsval[[ilev]])
        motsvalcurs<-	motsvalcurs[which(rownames(motsvalcurs)!="RemovedWords"),,drop=FALSE ]
        # ly has the words in rows + vtest =0 if not significant, else the vtest (+ or -)
        #ly[rownames(motsvalcurs),1] <- motsvalcurs[,1]
        # str.motsvalcurs <- motsvalcurs %in% 
        ly[rownames(motsvalcurs),1] <- motsvalcurs[,1]
        
        if(sum(motsvalcurs[,1])==0) {
          lisresult[ilev] <- list(NULL)
        } else
        {          
          
          # SourceTermcurs has the docs of the category in rows and the vocabulary in columns
          # a use vtest + and 
          a <- crossprod(ly,t(SourceTermcurs))
          b <- rowSums(SourceTermcurs)          # sum of vocabulary after of category, categories in columns
          repvaltest <- a
          repvaltest[b > 0] <- a[b > 0]/b[b > 0]
          # New the following line
          # The next is new. To remove CRITERIO zero
          repvaltest <- repvaltest[,colSums(repvaltest)>0,drop=FALSE]
          # Order of the docs into category
          ordrep <- order(repvaltest, decreasing = "TRUE")

         ntrep2 <- min(ntrep, length(ordrep))

          ## return(length(ordrep))  30
          
          # ntrep is the number of docs to print
          for (i in 1:ntrep2)
          {
            lisresult[[ilev]][i,1] <- rownames(DT.z)[docpos[ordrep[i]]]
            lisresult[[ilev]][i,2] <- repvaltest[ordrep[i]]
            
            if(bTextData) {
              lisresult[[ilev]][i,3] <- substr(corpus[docpos[ordrep[i]],], start = 1, stop = maxPrnDoc)
              #  if(length(ordrep)==0) lisresult[[ilev]][i,3] <- NULL
            } else {lisresult[[ilev]][i,3] <- ""}
          }


          if(bTextData) {
            colnames(lisresult[[ilev]]) <- c("DOCUMENT", "CRITERION", "---------------------TEXT---------------------")	
          } else { colnames(lisresult[[ilev]]) <- c("DOCUMENT", "CRITERION", "")	}
        } # End is.null(motsval[[ilev]]
      } # New is. sum(ly[,])==0)
      if(length(ordrep)==0) lisresult[ilev] <- list(NULL)

    } # End for
    names(lisresult) <- vlev
    return(lisresult)
  }  

  
  if(bTextData & !bvaragg & is.null(strquali)) maxCharDoc <- 0 # maxDocs <- 0
  if(!bTextData & is.null(strquali)) maxCharDoc <- 0# maxDocs <- 0
  



 #  if(maxDocs>0)  {
  if(maxCharDoc>0)  {

    if(bTextData) {
      base.new <- object$info$base[[1]]
      var.text <- object$info$var.text[[1]]   #  Important  Relaunch
     # str.base <- object$info$base[[1]]    # open.question, character
    #  str.envir <- object$info$menvir[[1]]

        base.new[, var.text[1]] <- as.character(base.new[, var.text[1]])
        base.new[is.na(base.new[var.text[1]]), var.text[1]] <- ""
        corpus <-  base.new[, var.text[1]]

      if(length(var.text) > 1) {					
        for (i in 2:length(var.text)){	
          corpus2 <- as.character(base.new[, var.text[i]])
          dos <-which(corpus!="" & corpus2!="")
          corpus[dos] <- paste(corpus[dos], corpus2[dos], sep=". ")
          uno <-which(corpus=="" & corpus2!="")
          corpus[uno] <- corpus2[uno]
        }
        rm(corpus2)
      }					
      corpus <- data.frame(corpus, stringsAsFactors = FALSE)	
      rownames(corpus) <- rownames(base.new)					# 300
      
      corpus <- corpus[rownames(DT.Z),,drop=FALSE]   # 292 o 298 for Fmin
      DT6 <- DT.Z[rowSums(DT.Z[,1:(ncol(DT.Z)-1)])!=0,,drop=FALSE]
      corpus <- corpus[rownames(DT6),,drop=FALSE]
      df.QUAL<- df.context.quali[rownames(DT6),"new.Cat",drop=FALSE]
      res$quali$CharDoc <- fCharDoc(res$quali$CharWord, df.QUAL, DT6)
    }
    
     if(!bTextData & !is.null(strquali)){
       df.QUAL<- df.context.quali[rownames(DT.Z),"new.Cat",drop=FALSE]
       res$quali$CharDoc <- fCharDoc(res$quali$CharWord, df.QUAL, DT.Z)
     } 
    
    
  } # End maxDocs maxCharDoc
  

  
  res$Proba <- proba
  class(res) <- c("LexChar", "list")    
  return(res)
}

#' @export

LabelTree<-function(object,proba=0.05){
  options(stringsAsFactors = FALSE)
  
    if (!inherits(object, "LexHCca") & !inherits(object, "LexCHCca"))
      stop("Object should be LexHCca or LexCHCca class")
mots.h=function(LTab){
	if (length(LTab)==0) stop("No hierarchical words are identified")
	table=NULL
	for (i in 1:length(LTab)){
		tab=cbind(LTab[[i]],rep(i,nrow(LTab[[i]])))
		tab=tab[tab[,6]>0,,drop=F]
		nuevas=!row.names(tab)%in%row.names(table)
		for (pal in row.names(tab)[!nuevas]){
			if (table[pal,6]<tab[pal,6]) table[pal,]=tab[pal,]
		                                     }
		table=rbind(table,tab[nuevas,,drop=F])            
	                        }
	
	nam=factor(table[,7],levels=1:length(LTab),labels=names(LTab))
      b=split(as.data.frame(table[,1:6]),nam,drop=F)
      lisa<-lapply(b, function(el) if(nrow(el)==0) NULL else el[order(el[,6],decreasing=TRUE),])    
      for(i in sort(which(unlist(lapply(lisa,is.null))),decreasing=T)) lisa[[i]]<-NULL
      return(lisa) 
}
       

## 


###############  <----------------- Comprobar la siguiente, no funciona

if (inherits(object, "LexCHCca")) dendro<-object$dendro$clust
else {
  mat.merge <- object$call$t$tree$merge
  dendro<- list()
  for(i in 1:nrow(mat.merge)) {
    if(mat.merge[i,1]<0) V1 <- -mat.merge[i,1] else V1 <- unlist(dendro[mat.merge[i,1]])
    if(mat.merge[i,2]<0) V2 <- -mat.merge[i,2] else V2 <-unlist(dendro[mat.merge[i,2]])
    a1 <- list(V1) ; a2 <- list(V2)
    dendro[[i]] <- c(a1,a2)
  }
}

DocTerm<-object$data.clust[,1:(dim(object$data.clust)[2]-1)]
nodt<-nrow(DocTerm)  
lista_tabmots<-list()
#lista_tabmots<-structure((vector(mode= "list", length=(nodt*2-2)),
#              names =rep(" ",(nodt*2-2) ))

# nodt number of documents or categories

inodA<-0
for (inode in 1:(nodt-2)) 
 {
  nodecours<-(nodt)-(inode+1)   
  don<-unlist(dendro[[nodecours]])
    label.node<-paste(rownames(DocTerm)[don],collapse=" ")
    by.quali<-rep(2,dim(DocTerm)[1])
    by.quali[don]<-1
    res<-descfreq(DocTerm,by.quali=by.quali,proba=proba)
    if (!is.null(res[[1]]))  {
    inodA<-inodA+1
    lista_tabmots[[inodA]]<- as.data.frame(res[[1]])
    names(lista_tabmots)[inodA]<-label.node 
                              } 
   }
res<-descfreq(DocTerm,proba=proba)


for (inode in 1:nodt) {                         

   label.node<-rownames(DocTerm)[inode]
   if( !is.null(res[[inode]]))  {
   inodA<-inodA+1  
   lista_tabmots[[inodA]]<- as.data.frame(res[[inode]])
   names(lista_tabmots)[inodA]<-label.node  
                                  }    
} 



#  only the non-doubled are kept
res<-NULL
   hierWord <- mots.h(lista_tabmots)
   class(hierWord) = c("LabelTree")
      return(hierWord)
}

#' @rawNamespace import(ggdendro, except = c(theme_dendro))
#' @importFrom grDevices palette
#' @importFrom graphics layout legend par plot text
#' @export
 
plot.LexCHCca <- function(x, axes=c(1, 2), type=c("tree","map","bar"), rect=TRUE, 
                          title=NULL, subtitle=NULL, ind.names=TRUE, new.plot=FALSE, max.plot=15, 
                          tree.barplot=TRUE, ...) 
{
  if(is.null(x)) stop("Missing argument for x object")
  if (!inherits(x,"LexCHCca")) stop("x object should be LexCHCca class")   
  dots <- list(...)
  if(!is.null(dots$choice)) stop("Change choice argument by type argument")
  if(length(type)>1) type <- type[1]
  
  # 1. GUARDAMOS LOS PARÁMETROS ORIGINALES Y ASEGURAMOS SU RESTAURACIÓN AL SALIR
  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  
  res <- x
  X <- res$call$X
  max <- res$call$max
  min <- res$call$min
  max.plot <- max(res$call$max,max.plot)
  nb.clust <- length(levels(X$clust))
  levs <- levels(X$clust)
  
  if (type == "tree") {
    if ((new.plot) & !nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) 
      dev.new()
    
    if (is.null(title))
      title = "Constrained hierarchical clustering"
    
    if (tree.barplot) {
      par(mar = c(0.5, 2, 0.75, 0))
      lay = matrix(ncol = 5, nrow = 5, c(2, 4, 4, 4, 4, 
                                         2, 4, 4, 4, 4, 2, 4, 4, 4, 4, 2, 4, 4, 4, 4, 
                                         1, 3, 3, 3, 3))
      layout(lay, respect = TRUE)
      
      n.tot <- min(15, nrow(res$data.clust), max.plot)
      n.black <- min(n.tot, res$call$t$nb.clust)
      n.grey <- max(0, n.tot-n.black)
      
      inv.height = rev(res$call$t$height)
      vec <- inv.height[1:max.plot]
      
      barplot(height = vec, space = 0.7)
      
      plot(x = 1, xlab = "", ylab = "", main = "", col = "white", axes = FALSE)
      text(1, 1, title, cex = 2)
      plot(x = 1, xlab = "", ylab = "", main = "", col = "white", axes = FALSE)
      legend("top", "Agg. criterion", box.lty = NULL, cex = 1)
    }
    
    plot(res$call$t$tree, hang = -1, xlab = "", sub = "", main=subtitle, ...) 
    
    if (rect) {
      y = (res$call$t$tree$height[length(res$call$t$tree$height) - nb.clust + 2] + 
             res$call$t$tree$height[length(res$call$t$tree$height) - nb.clust + 1])/2
      ordColo <- unique(res$call$X$clust[res$call$t$tree$order])
      rect = rect.hclust(res$call$t$tree, h = y, border = ordColo)
    }
  }
  
  if (type == "map") {
    if (!res$call$vec) {
      if (is.null(title)) 
        title = "CA map"
      Y = X[, -ncol(X)]
      leg.map = NULL
      for (p in 1:nrow(X)) leg.map[p] = paste("cluster", X$clust[p], " ", sep = " ")
      Y = cbind.data.frame(Y, as.factor(leg.map))
      res2 = PCA(Y, quali.sup = ncol(Y), scale.unit = FALSE, 
                 row.w = res$call$t$res$call$row.w, ncp = Inf, graph = FALSE)
      res2$eig <- res$call$t$res$eig
      
      if (ind.names) {
        plot.PCA(res2, title = title, habillage = ncol(Y), 
                 cex = 0.8, axes = axes, new.plot = new.plot, 
                 palette = palette(), graph.type = "classic", ...)
      } else { 
        plot.PCA(res2, title = title, habillage = ncol(Y), 
                 cex = 0.8, axes = axes, label = "none", new.plot = new.plot, 
                 palette = palette(), graph.type = "classic", ...)
      }
    }
  }
  
  if (type == "bar") {
    if ((new.plot) & !nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) 
      dev.new()
    inv.height = rev(res$call$t$height)
    vec <- inv.height[1:max.plot]
    names.arg = NULL
    if (is.null(title)) 
      title = "Aggregation criterion"
    for (i in 1:(max.plot)) names.arg = c(names.arg, paste(i, "-", i + 1, sep = ""))
    
    barplot(vec, names.arg = names.arg, col = c(rep(1, nb.clust - 1),
                                                rep(18, length(vec) - nb.clust + 1)), main = title, 
            xlab = "level of cutting", ylab = "Criterion")
  }
  
  invisible()
}

\name{plot.LexGalt}
\encoding{UTF-8}
\alias{plot.LexGalt}

\title{Plot LexGalt objects}

\description{
Plots Generalised Aggregate Lexical Tables (LexGalt) graphs from a LexGalt object
}

\usage{
\method{plot}{LexGalt}(x,type="QL", selDoc=NULL, selWord=NULL, selQualiVar=NULL,
  selQuantiVar=NULL, conf.ellip=FALSE, selWordEllip=NULL, selQualiVarEllip=NULL,
  selQuantiVarEllip=NULL, level.conf=0.95, eigen=FALSE, title = NULL, axes = c(1, 2),
  xlim = NULL, ylim = NULL, col.eig="grey", col.doc = "black", col.word = NULL,
  col.quali = "blue", col.quanti = "blue", col="grey", pch = 20, label = TRUE, 
  autoLab = c("auto", "yes", "no"), palette = NULL, unselect = 1, 
  selCov=FALSE, selGroup="ALL", partial=FALSE, plot.group=FALSE, 
  col.group=NULL, label.group=NULL, legend=TRUE, pos.legend="topleft", 
  new.plot = TRUE, cex=1,...)
}
\arguments{
  \item{x}{object of LexGalt class}
  \item{type}{results from a qualitative analysis (type="QL") or quantitative analysis (type="QN"); see details; by default Q)}
  \item{selDoc}{vector with the documents to plot (indexes, names or rules; see details; by default NULL)}
  \item{selWord}{vector with the words to plot (indexes, names or rules (indexes, names or rules; see details; by default NULL)}
  \item{selQualiVar}{vector with the categories of categorical variables to plot (indexes, names or rules; see details; by default NULL)}
  \item{selQuantiVar}{vector with the numerical variables to plot (indexes, names or rules; see details; by default NULL)}
  \item{conf.ellip}{to drawn confidence ellipses, by default FALSE}
  \item{selWordEllip}{vector with the words that defines which ellipses are drawn (indexes, names or rules; see details; by default NULL)}
  \item{selQualiVarEllip}{vector with the categories of categorical variables which ellipses are drawn (indexes, names or rules; see details; by default NULL)}
  \item{selQuantiVarEllip}{vector with the numerical variables which ellipses are drawn(indexes, names or rules; see details; by default NULL)}
  \item{level.conf}{level of confidence used to construct the ellipses; by default 0.95}
  \item{eigen}{if TRUE, the eigenvalues barplot is drawn (by default FALSE); other elements can be simultaneously selected}
  \item{title}{title of the graph (by default NULL and the title is automatically assigned)}
  \item{axes}{length-2 vector indicating the axes considered in the graph; by default c(1,2)}
  \item{xlim}{range for 'x' values on the graph, defaulting to the finite values of 'x' range (by default NULL)}
  \item{ylim}{range for the 'y' values on the graph, defaulting to the the finite values of 'y' range (by default NULL)}
  \item{col.eig}{value or vector with colors for the bars of eigenvalues (by default "grey")}
  \item{col.doc}{color for the point-documents(by default "black")}
  \item{col.word}{color for the point-words (by default NULL is darkred in simple analysis; see details)}
  \item{col.quali}{color for the categories of categorical variables (by default "blue")}
  \item{col.quanti}{color for the numerical variables (by default "blue")}
  \item{col}{color for the bars in the eigenvalues barplot (by default "grey")}
  \item{pch}{plotting character for coordinates, cf. \code{\link[graphics]{points}} function in the graphics package}
  \item{label}{a list of character for the elements which are labelled (by default TRUE and all the drawn elements are labelled).}
  \item{autoLab}{if autoLab="auto", autoLab turns to be equal to "yes" if there are less than 50 elements and equal to "no" otherwise; if "yes", the labels are moved, as little as possible, to avoid overlapping (time-consuming if many elements); if "no" the labels are placed quickly but may overlap}
  \item{palette}{the color palette used to draw the points. By default colors are chosen. If you want to define the colors : palette=c("black", "red", "blue"); or you can use: palette=rainbow(10), or in black and white for example: palette=gray(seq(0,.9,len=3))}
  \item{unselect}{may be either a value between 0 and 1 that gives the transparency of the unselected objects (if unselect=1 the transparceny is total and the elements are not drawn, if unselect=0 the elements are drawn as usual but without any label) or may be a color (for example unselect="grey60")}
  \item{selCov}{a boolean, if TRUE then data are scaled to unit variance (by default TRUE)}
  \item{selGroup}{vector with the groups to plot if multiple analysis was performed (indexes, names or rules; see details; by default NULL)}
  \item{partial}{if TRUE partial elements (results for the groups) are shown, if ALL results for the conjoint analysis are superimposed; by default FALSE}
  \item{plot.group}{draw a plot comparing the groups in multiple case (by default TRUE)} 
  \item{col.group}{color for the groups if multiple analysis was performed (by default NULL and they are selected from palette)}
  \item{label.group}{a vector containing the new name of the groups. If "BLANK" no labels with the group are added at the end of the drawn elements (by default, NULL and the name of each group is added)}
  \item{legend}{show the legend of labels of groups. See \code{\link[graphics]{legend}} from graphics package (by default TRUE}
  \item{pos.legend}{position of the legend of labels of groups. See \code{\link[graphics]{legend}} from graphics package (by default "topleft")}
  \item{new.plot}{if TRUE, a new graphical device is created (by default TRUE)}
  \item{cex}{text and symbol size is scaled by cex, in relation to size 1 (by default 1)}
  \item{\dots}{further arguments passed from other methods...}
}

\details{
The argument autoLab = "yes" is time-consuming if many overlapping labels. Furthermore, the visualization of the words cloud can result distorted because of the apparent greater dispersion of the words labels. An alternative would be reducing the character size of the words labels to reduce overlapping (e.g. cex=0.7).

selDoc, selWord, selQualiVar, selQuantiVar, selWordEllip, selQualiVarEllip,
    selQuantiVarEllip allow for selecting all or part of the elements of the corresponding type, using either labels, indexes or rules.\cr

The syntax is the same for all types.

1. Using labels:
\preformatted{
selDoc = c("doc1","doc5"): only the documents with labels doc1 and doc5 are plotted.
selQualiVar=c("category1","category2"): only the categories labeled category1 and
 category2 are plotted.}

2.- Using indexes:
\preformatted{
selDoc = c(1:5): documents 1 to 5 are plotted.
quali.sup=c(1:5,7): categories 1 to 5 and 7 are plotted. The numbering of the
   categories have to be consulted in the LexGalt numerical results.}

3.- Using rules:    
 Rules are based on the coordinates (coord), the contribution (contrib or meta) or the square cosine (cos2).\cr
 Somes examples are given hereafter:

\preformatted{
selDoc="coord 10": only the 10 documents with the highest coordinates, as globally
   computed on the 2 axes, are plotted.
selWord="contrib 10": the words with a contribution to the inertia, of any of 
   the 2 axes.
selWord="meta 3": the words with a contribution over 3 times the average word 
   contribution on any of the two axes are plotted.
selWord="cos2 .85": the words with a cos2 over 0.85, as summed
   on the 2 axes, are plotted.
 }
\preformatted{
col.word by default NULL is "darkred" for simple analysis, if it is null takes
the colors from col.group 
i.e. col.group=c("red","blue"). To select the colors for some words in object res, 
we can use:
str.col.words <- rep("darkred",nrow(res$MQL$word$coord))
str.col.words[which(rownames(res$MQL$word$coord) == "kids")] <- "red"
str.col.words[which(rownames(res$MQL$word$coord) == "friends")] <- "green"
str.col.words[which(rownames(res$MQL$word$coord) == "job")] <- "pink"
plot(res, selGroup=1, selWord=c("friends", "job", "kids", "at"),new.plot=FALSE, 
col.group=c("darkred","blue"), autoLab = "yes", col.word=str.col.words)
} 
 
}


\references{
 
Bécue-Bertaut M. and Pagès J. (2015). Correspondence analysis of textual data involving contextual information: CA-GALT on principal components. Advances in Data Analysis and Classification, vol.(9) 2: 125-142.

Bécue-Bertaut M., Pagès J. and Kostov B. (2014). Untangling the influence of several contextual variables on the respondents' lexical choices. A statistical approach. SORT - Statistics and Operations Research Transactions, vol.(38) 2: 285-302.

Kostov B. A. (2015). A principal component method to analyse disconnected frequency tables by means of contextual information. (Doctoral dissertation). Retrieved from \url{http://upcommons.upc.edu/handle/2117/95759}.

}

\author{
Belchin Kostov, Monica Bécue-Bertaut, Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, 
Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{LexGalt}}}

\examples{
\dontrun{
data(open.question)

res.TD<-TextData(open.question,var.text=c(9,10),  Fmin=10, Dmin=10,
 context.quali=c("Gender", "Age_Group", "Education"),
 remov.number=TRUE, stop.word.tm=TRUE)

res.LexGalt <- LexGalt(res.TD, graph=FALSE, nb.ellip =0)
plot(res.LexGalt, selQualiVar="ALL")
}
}

\keyword{plot}

\name{LabelTree}
\encoding{UTF-8} 
\alias{LabelTree}

\title{Hierarchical words (LabelTree)}

\description{Extracts the hierarchical characteristic words associated to the nodes of a hierarchical tree; the characteristic words of each node are extracted, then each word is associated to the node that it best characterizes.}

\usage{LabelTree(object, proba=0.05)}

\arguments{
 \item{object}{object of LexHCca or LexCHCca class}
 \item{proba}{threshold on the p-value when the characteristic words are computed (by default 0.05)}
}

\value{
Returns a list including:
 
\item{hierWord}{list of the characteristic words associated to the nodes of a hierarchical tree; 
only the non-empty nodes are included}

}

\references{
Bécue-Bertaut, M., Kostov, B., Morin, A., & Naro, G. (2014). Rhetorical Strategy in Forensic Speeches: 
Multidimensional Statistics-Based Methodology. Journal of Classification,31,85-106. \doi{10.1007/s00357-014-9148-9}.

Lebart, L., Salem, A., & Berry, L. (1998). Exploring textual data. (D. Kluwer, Ed.). \doi{10.1007/978-94-017-1525-6}.
}

\author{Monica Bécue-Bertaut, Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Josep-Anton 
Sánchez-Espigares, Belchin Kostov}

\seealso{ \code{\link{LexCA}}, \code{\link{LexCHCca}}}


\examples{
data(open.question)
res.TD<-TextData(open.question,var.text=c(9,10), var.agg="Age_Group", Fmin=10, Dmin=10,
        stop.word.tm=TRUE)
 res.LexCA<-LexCA(res.TD, graph=FALSE)
 res.LexCHCca<-LexCHCca(res.LexCA, nb.clust=4, min=3)
 res.LabelTree<-LabelTree(res.LexCHCca)
}

\keyword{multivariate}

\name{summary.LexCA}
\encoding{UTF-8}
\alias{summary.LexCA}

\title{Summary LexCA object}

\description{
Summarizes LexCA objects
}

\usage{
\method{summary}{LexCA}(object, ncp=5, nb.dec = 3, ndoc=10, nword=10, nseg=10, 
 nsup=10, metaDocs=FALSE, metaWords=FALSE, file = NULL, ...)

}

\arguments{
  \item{object}{object of LexCA class}
  \item{ncp}{number of dimensions to be printed (by default 5)}
  \item{nb.dec}{number of decimal digits to be printed (by default 3)}
   \item{ndoc}{number of documents whose coordinates are listed (by default 10).
   Use ndoc="ALL" to have the results for all the documents. Use 
   ndoc=0 or ndoc=NULL if the results for documents are not wanted.}
  \item{nword}{number of words whose coordinates are listed (by default 10). 
   Use nword="ALL" to have the results for all the words. Use
   nword=0 or nword=NULL if the results for words are not wanted} 
  \item{nseg}{number of repeated segments whose coordinates are listed (by default 10). 
   Use nseg="ALL" to have the results for all the segments. Use 
   nseg=0 or nseg=NULL if the results for segments are not wanted} 
 \item{nsup}{number of supplementary elements whose coordinates are listed (by default 10). 
   Use nsup="ALL" to have the results for all the elements. Use 
   nsup=0 or nsup=NULL if the results for the supplementary elements are not wanted}
  \item{metaDocs}{axis by axis, the highest contributive documents are listed, separately for negative-part and positive-part documents; these documents have been identified in LexCA, taking into account lmd value (by default FALSE)}
  \item{metaWords}{axis by axis, the highest contributive words are listed, separately for negative-part and positive-part words; these words have been identified in LexCA, taking into account lmw value (by default FALSE)}
\item{file}{a connection, or a character string naming the file to print to (csv format). If NULL (the default), the results are not printed in a file}
 \item{\dots}{further arguments passed from other methods}
}

\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Monica Bécue-Bertaut, Josep-Antón Sánchez-Espigares}
\seealso{ \code{\link{LexCA}}, \code{\link{print.LexCA}},  \code{\link{plot.LexCA}}}

\examples{
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), Fmin=10, Dmin=10, stop.word.tm=TRUE)
res.LexCA<-LexCA(res.TD, lmd=1, lmw=1)
summary(res.LexCA)
}

\keyword{summary}

\name{summary.TextData}
\encoding{UTF-8}
\alias{summary.TextData}

\title{Summary of TextData objects}

\description{
Summarizes TextData objects.
}

\usage{
\method{summary}{TextData}(object, ndoc=10, nword=50, nseg=50, ordFreq = TRUE, file = NULL, sep=";",
   dec=".", info=TRUE,...) 
}
\arguments{
  \item{object}{object of TextData class}
  \item{ndoc}{statistical report on the first ndoc documents (by default 10). 
   Use ndoc="ALL" to have the results for all the documents. Use
   ndoc=0 or ndoc=NULL if the results on the documents are not wanted } 
  \item{nword}{index of the nword first words (by default 50). 
   Use nword="ALL" to have the complete index. Use
   nword=0 or nword=NULL if the results on the words are not wanted } 
  \item{nseg}{index of the nfirst nseg repeated segments (by default 50). 
   Use nseg="ALL" to have the complete list of segments. Use 
   nseg=0 or nseg=NULL if the results on the segments are not wanted } 
  \item{ordFreq}{if ordFreq=TRUE, glossaries of words and repeated segments, are listed in frequency order; 
if ordFreq=FALSE, glossaries are listed  in alphabetic order (by default TRUE)}
  \item{file}{a connection, or a character string naming the file to print to in csv format. If NULL (the default), the results are not printed in a file}
   \item{sep}{character inserted between the objects to print (if file argument is non-NULL) (by default ";"\\t" for tab)}
   \item{dec}{the string to use for decimal points in numeric or complex columns: must be a single character (by default ".")}
  \item{info}{if TRUE the selection criteria of the words are shown(by default TRUE)}
  \item{\dots}{further arguments passed to or from other methods,...}
}

\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Monica Bécue-Bertaut, Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{TextData}}, \code{\link{print.TextData}},  \code{\link{plot.TextData}}}


\examples{
# Non aggregate analysis
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), remov.number=TRUE, Fmin=10, Dmin=10,  
 stop.word.tm=TRUE, context.quali=c("Gender","Age_Group","Education"), context.quanti=c("Age"))
summary(res.TD)

# Aggregate analysis and repeated segments
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), var.agg="Gen_Age", remov.number=TRUE, 
 Fmin=10, Dmin=10, stop.word.tm=TRUE, context.quali=c("Gender","Age_Group","Education"), 
 context.quanti=c("Age"), segment=TRUE)
summary(res.TD)
}

\keyword{summary}

\name{plot.LexCA}
\encoding{UTF-8}
\alias{plot.LexCA}

\title{Plot of LexCA objects}

\description{
Plots textual correspondence analysis (CA) graphs from a LexCA object. 
}

\usage{
\method{plot}{LexCA}(x, selDoc="ALL", selWord="ALL", selSeg=NULL, selDocSup=NULL,
  selWordSup=NULL, quanti.sup=NULL, quali.sup=NULL, maxDocs=20, eigen=FALSE, 
  title=NULL, axes=c(1,2), col.doc="blue", col.word="red", col.doc.sup="darkblue", 
  col.word.sup="darkred", col.quanti.sup = "blue", col.quali.sup="darkgreen", 
  col.seg="cyan4", col="grey", cex=1, xlim=NULL, ylim=NULL, shadowtext=FALSE,
  habillage="none", unselect=1, label="ALL", autoLab=c("auto", "yes", "no"), 
  new.plot=TRUE, graph.type = c("classic", "ggplot"),...)
}
\arguments{
  \item{x}{object of LexCA class}
  \item{selDoc}{vector with the active documents to plot (indexes, names or rules; see details; by default "ALL")}
  \item{selWord}{vector with the active words to plot (indexes, names or rules; see details; by default "ALL")}
  \item{selSeg}{vector with the supplementary repeated segments to plot (indexes, names or rules; see details; by default NULL)}
  \item{selDocSup}{vector with the supplementary documents to plot (indexes, names or rules; see details; by default NULL)}
  \item{selWordSup}{vector of the supplementary words to plot (indexes, names or rules; see details; by default NULL)} 
  \item{quanti.sup}{vector of the supplementary quantitative variables to plot (indexes, names or rules; see details; by default NULL)}
  \item{quali.sup}{vector with the supplementary categorical variables/categories to plot (indexes, names or rules; see details; by default NULL). The selected categories (through the variables or directly) are plotted}
  \item{maxDocs}{limit to the number of active documents in the lexical table when selecting the words to be plotted for being characteristic of the selected documents (by default 20)}
  \item{eigen}{if TRUE, the eigenvalues barplot is drawn (by default FALSE); no other elements can be simultaneously selected}
  \item{title}{title of the graph (by default NULL and the title is automatically assigned)}
  \item{axes}{length-2 vector indicating the axes considered in the graph (by default c(1,2))}
  \item{col.doc}{color for the point-documents (by default "blue")}
  \item{col.word}{color for the point-words (by default "red")}
  \item{col.doc.sup}{color for the supplementary point-documents (by default "darkblue")}
  \item{col.word.sup}{color for the supplementary point-words (by default "darkred")}
  \item{col.quanti.sup}{color for the quanti.sup variables (by default "blue")}
  \item{col.quali.sup}{color for the categorical supplementary point-categories, (by default "darkgreen")}
  \item{col.seg}{color for the supplementary point-repeated segments, (by default "cyan4")}
  \item{col}{color for the bars in the eigenvalues barplot (by default "grey")}
  \item{cex}{text and symbol size is scaled by cex, in relation to size 1 (by default 1)}
  \item{xlim}{range for 'x' values on the graph, defaulting to the finite values of 'x' range (by default NULL)}
  \item{ylim}{range for the 'y' values on the graph, defaulting to the the finite values of 'y' range (by default NULL)}
  \item{shadowtext}{if TRUE, shadow on the labels (rectangles are written under the labels which may lead to difficulties to modify the graph with another program) (by default FALSE)}
  \item{habillage}{index or name of the categorical variable used to differentiate the documents by colors given according to the category; by default "none")}
  \item{unselect}{either a value between 0 and 1 or a color. In the first case, transparency level of the unselected objects (if unselect=1 the transparency is total and the elements are not represented; if unselect=0 the elements are represented as usual but without any label); in the case of a color (e.g. unselect="grey60"), the non-selected points are given this color (by default 1)}
  \item{label}{a list of character for the variables which are labelled (by default ALL and all the drawn variables are labelled). You can label all the active variables by putting "var" and/or all the supplementary variables by putting "quanti.sup" and/or a list with the names of the variables which should be labelled. Value should be one of "all", "none", "row", "row.sup", "col", "col.sup", "quali.sup" or NULL.}
  \item{autoLab}{if autoLab="auto", autoLab turns to be equal to "yes" if there are less than 50 elements and equal to "no" otherwise; if "yes", the labels are moved, as little as possible, to avoid overlapping (time-consuming if many elements); if "no" the labels are placed quickly but may overlap}
  \item{new.plot}{if TRUE, a new graphical device is created (by default FALSE)}
  \item{graph.type}{a string that gives the type of graph used: "ggplot" or "classic" (by default classic)}
  \item{\dots}{further arguments passed from other methods...}
}


\details{
The argument autoLab = "yes" is time-consuming if many overlapping labels. Furthermore, the visualization of the words cloud can result distorted because of the apparent greater dispersion of the words labels. An alternative would be reducing the character size of the words labels to reduce overlapping (e.g. cex=0.7).

selDoc, selWord, selSeg, selDocSup, selWordSup, quanti.sup and quali.sup allow for selecting all or part of the elements of the corresponding type, using either labels, indexes or rules.\cr

The syntax is the same for all types.

1. Using labels:
\preformatted{
selDoc = c("doc1","doc5"): only the documents with labels doc1 and doc5 are plotted.
quali.sup=c("varcateg1","category12"): only the categories (all of them) of 
   categorical variable labeled "varcateg1" and the category labeled "category12"
   are plotted.}

2.- Using indexes:
\preformatted{
selDoc = c(1:5): documents 1 to 5 are plotted.
quali.sup=c(1:5,7): categories 1 to 5 and 7 are plotted. The numbering of the
   categories have to be consulted in the LexCA numerical results.}

3.- Using rules:    
 Rules are based on the coordinates (coord), the contribution (contrib or meta; concerning only
 active elements) or the square cosine (cos2).\cr
 Somes examples are given hereafter:

\preformatted{
selDoc="coord 10": only the 10 documents with the highest coordinates, as globally
   computed on the 2 axes, are plotted.
selWord="contrib 10": the words with a contribution to the inertia, of any of 
   the 2 axes.
selWord="meta 3": the words with a contribution over 3 times the average word 
   contribution on any of the two axes are plotted. Only active words or documents 
   can be selected.
selDocSup="cos2 .85": the supplementary documents with a cos2 over 0.85, as summed
   on the 2 axes, are plotted.
selWord="char 0.05": only the characteristic words of the documents selected in 
   SelDoc are plotted. The selection of the words follow the rationale used in 
   function LexChar using as limit for the p-value the value given, here.0.05.}

}



\references{
Husson F., Lê S., Pagés J. (2011). Exploratory Multivariate Analysis by Example Using R. Chapman & Hall/CRC. \doi{10.1201/b10345}.

}

\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Mónica Bécue-Bertaut, Josep-Antón Sánchez-Espigares}

\seealso{\code{\link{LexCA}}, \code{\link{print.LexCA}}, \code{\link{summary.LexCA}}}

\examples{
data(open.question)
res.TD<-TextData(open.question,var.text=c(9,10), var.agg="Age_Group", Fmin=10, Dmin=10,
        remov.number=TRUE, stop.word.tm=TRUE)
res.CA <- LexCA(res.TD, graph=FALSE)
plot(res.CA, selDoc="contrib 30", selWord="coord 20")
}


\keyword{plot}

\name{plot.LexCHCca}
\encoding{UTF-8}
\alias{plot.LexCHCca}
\title{Plots for Chronological Constrained Hierarchical Clustering from LexCHCca Objects}

\description{Plots graphs from LexCHCca results: tree, barplot of the aggregation criterion values and first CA map with the documents colored in accordance with the cluster.

}

\usage{
\method{plot}{LexCHCca}(x, axes=c(1, 2), type=c("tree","map","bar"), rect=TRUE, title=NULL, 
 subtitle=NULL, ind.names=TRUE, new.plot=FALSE, max.plot=15, tree.barplot=TRUE,...)  
}
\arguments{
  \item{x}{object of LexCHCca class}
  \item{axes}{length-2 vector defining the axes of the CA map to plot (by default (1,2))}
  \item{type}{type of graph. "tree" plots the tree; "bar" plots the barplot of the successive 
    values of the aggregation criterion (downward reading of the tree); "map" plots the CA map where the individuals 
    are colored in accordances with the cluster of belonging (by default "tree")}
  \item{rect}{if TRUE, when choice="tree" rectangles are drawn around the clusters (by default TRUE)}
  \item{title}{title of the graph. If NULL, a title is automatically defined (by default NULL)}
  \item{subtitle}{subtitle of the graph. Not available for all the plots. If NULL, a subtitle is automatically defined (by default NULL)}
  \item{ind.names}{if TRUE, the document labels are written on the CA map (by default TRUE)}
  \item{new.plot}{if TRUE, a new window is opened (by default FALSE)}
  \item{max.plot}{maximum of bars in the bar plot of the aggregation criterion (by default 15)}
  \item{tree.barplot}{if TRUE, the barplot of intra inertia losses is added on the tree graph (by default TRUE)}
  \item{\dots}{further arguments passed from other methods...}
}


\value{
Returns the chosen plot
}

\author{Mónica Bécue-Bertaut, Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Josep-Anton Sánchez-Espigares}
\seealso{\code{\link{LexCHCca}} }


\examples{
\dontrun{
data(open.question)
res.TD<-TextData(open.question,var.text=c(9,10), var.agg="Age_Group", Fmin=10, Dmin=10,
        stop.word.tm=TRUE)
res.LexCA<-LexCA(res.TD, graph=FALSE)
res.chcca<-LexCHCca(res.LexCA, nb.clust=4, min=3, graph=FALSE)
plot(res.chcca, type="tree")
plot(res.chcca, type="map")
plot(res.chcca, type="bar", max.plot=5)
}
}

\keyword{plot}

\name{LexChar}
\encoding{UTF-8}
\alias{LexChar}

\title{Characteristic words and documents (LexChar)}

\description{Measure of the association between vocabulary or words and quantitative or qualitative contextual variables.}

\usage{LexChar(object, proba=0.05, maxCharDoc=10, maxPrnDoc=100, 
              marg.doc="before",  context=NULL, correct=TRUE, nbsample=500,
              seed=12345,...)}

\arguments{
 \item{object}{TextData, DocumentTermMatrix, dataframe or matrix object}
 \item{proba}{threshold on the p-value used when selecting the characteristic words (by default 0.05)}
 \item{maxCharDoc}{maximum number of characteristic source-documents to extract (by default 10). See details}
 \item{maxPrnDoc}{maximum length to be printed for a characteristic document (by default 100 characters)}
 \item{marg.doc}{if after/before, frequencies after/before TextData selection are used as document weighting (by default "before"); 
  if before.RW all words under threshold in TextData function are included as a new word named RemovedWords}
 \item{context}{name of quantitative or qualitative variables}
 \item{correct}{if TRUE, pvalue correction test is applied for quantitative contextual variables (by default TRUE)}
 \item{nbsample}{number of samples drawn to evaluate the pvalues in quantitative contextual variables}
 \item{seed}{Seed to obtain the same results using permutation tests (by default 12345)}
 \item{\dots}{further arguments passed to or from other methods}
}

\value{
Returns a list including:
 
\item{CharWord}{characteristic words of all the documents}
\item{stats}{association statistics of the lexical table}
\item{CharDoc}{characteristic source-documents of all the aggregate-documents including qualitative contextual variables}
\item{Vocab}{characteristic quantitative and qualitative variables of the words. CharWord and stats are provided.}
}

\details{
The lexical table provided by TextData can consider either source-documents or aggregate-documents, in accordance with the value of argument "var.agg" in TextData.
Context cualitative variables allow to aggregate documents by combining the categories of the qualitative variables and the aggregation variable if any.

Extracting the characteristic words (CharWord) for a too high number of documents is of no interest and time-consuming. 

In any case, only the first maxPrnDoc characters of each characteristic document are printed (by default 100).

In the case of the association between words and qualitative variables, the usual characteristic words are provided.

quali$CharWord provides the qualitative variables (including the aggregation variable) and their categories.
quali$stats provides association statistics for vocabulary and qualitative variables (including the aggregation variable).
quali$CharDoc provides characteristic source-documents for the categories.
quanti$CharWord provides characteristic quantitative variables for each word. If there are aggregation variable and/or qualitative contextual variable, from aggregated lexical table.
quanti$stats provides statistics for vocabulary and quantitative variables. If there are aggregation variable and/or qualitative contextual variable, from aggregated lexical table.

If the lexical table (object) is not a TextData object, context argument can be columns of the same dataframe.
The aggregate lexical table is constructed from the combinations of the categories of the qualitative variables (including the aggregation variable).
}

\references{
 
Lebart, L., Salem, A., & Berry, L. (1998). Exploring textual data. (D. Kluwer, Ed.). \doi{10.1007/978-94-017-1525-6}.
}

\author{Monica Bécue-Bertaut, Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Josep-Antón
Sánchez-Espigares, Belchin Kostov}

\seealso{ \code{\link{TextData}}, \code{\link{print.LexChar}}, \code{\link{plot.LexChar}}, \code{\link{summary.LexChar}}}

\examples{
data(open.question)
 res.TD<-TextData(open.question, var.text=c(9,10), var.agg="Gen_Edu", Fmin=10, Dmin=10,
                   remov.number=TRUE, stop.word.tm=TRUE)
 res.LexChar <-LexChar(res.TD)
 summary(res.LexChar)
}

\keyword{multivariate}

\name{open.question}
\encoding{UTF-8}
\alias{open.question}
\title{Open.question (data)}
\docType{data}

\description{

Extract of the answers provided in a survey designed to better know opinions about what is most important in life.

Two open-ended questions are included in the questionnaire "What is most important to you in life?" and
"What are other very important things to you? (relaunch of the first question).
}

\usage{data(open.question)}


\format{
Data frame with 300 rows and 10 columns. The rows correspond to the respondents. The first 8 columns correspond to socio-demographic variables collected through closed questions: Gender, Age_Group, Age, Education level, Genre crossed with Age, Genre crossed with Education level, Age crossed with Education level and, finally Genre crossed with Education level and Age. Age is a quantitative variable while the other variables are qualitative. The last two columns contain the answers to the open-ended questions. 
}


\keyword{datasets}


	

\name{ellipseLexCA}
\encoding{UTF-8} 
\alias{ellipseLexCA}

\title{Confidence ellipses on textual correspondence analysis graphs}

\description{
Draws confidence ellipses around documents and/or words on a textual CA graph.
}


\usage{
ellipseLexCA(object, selWord="ALL", selDoc="ALL", nbsample=100, level.conf=0.95,
    axes=c(1, 2), ncp=NULL, xlim=NULL, ylim=NULL, title=NULL, col.doc="blue",
    col.word="red", col.doc.ell=col.doc, col.word.ell=col.word, cex=1) 
}

\arguments{
 \item{object}{object of LexCA class}
 \item{selWord}{selected words (indexes or names; by default "ALL"); see the details section}
 \item{selDoc}{selected docs (indexes or names; by default "ALL"); see the details section}
 \item{nbsample}{number of samples drawn to evaluate the stability of the points}
 \item{level.conf}{confidence level used to construct the ellipses (by default 0.95)}
  \item{axes}{length 2 vector specifying the dimensions to plot}
 \item{ncp}{maximum number of dimension to draw (by default NULL and ncp is the number of dimensions from LexCA object)}
 \item{xlim}{range for the plotted 'x' values, defaulting to the range of the finite values of 'x' (by default NULL)}
 \item{ylim}{range for the plotted 'y' values, defaulting to the range of the finite values of 'y' (by default NULL)}
 \item{title}{title of the graph (by default NULL and the title is automatically assigned)}
 \item{col.doc}{color for the documents-points (by default "blue")}
 \item{col.word}{color for words-points (by default "red")}
 \item{col.doc.ell}{color for the ellipses around documents-points (by default the same as col.doc)}
 \item{col.word.ell}{color for the ellipses around words-points (by default the same as col.word)}
 \item{cex}{text and symbol size is scaled by cex, in relation to size 1 (by default 1)}
}


\value{
Returns a LexCA-like map representing the selected points and their confidence ellipses
}

\details{

The method "multinomial" is used to generate the replicated tables. 
So, the active lexical table contained in the LexCA object (active table) is taken as a reference.

Then, replicated lexical tables are generated by repeating nbsample times the following process: N (the sum of active table elements) values are drawn from a multinomial distribution with theoretical frequencies equal to the values in the active table cells divided by N. A replicated table is built from each drawing.

The nbsample documents-rows and/or words-columns of the replicated tables are projected as supplementary documents (rows) and/or supplementary words (columns) on the graph computed from the active lexical table. 
Then, confidence ellipses are drawn around each active element from the nbsample supplementary points.\cr
The replicated samples with empty row-documents and/or word-columns with null frequency are dropped. \cr 
If over 10\% of the total of replicated samples are dropped, the execution is stopped. Information is given through a stop-message.\cr

The selDoc and selWord arguments allow for selecting the documents and/ or words.\cr
The syntax for these arguments is similar to the one used in plot.LexCA. \cr 
However they only concern the active elements and selecting the characteristic words is not allowed.

Some examples follow:
selDoc=c(1:5): the documents 1 to 5 are represented.\cr
selDoc=c("doc1","doc5"): documents with labels doc1 or doc5 are represented.\cr
selWord=c("word1","word3"): words with labels word1 or word3 are represented.\cr
selDoc/selWord = "coord 10": the 10 documents/words with the highest coordinates on the 2
 chosen axes are selected.\cr
selDoc/selWord="contrib 10": documents/words with a contribution to the inertia of any of both axes over 10\% of the axis inertia are selected.\cr
selDoc/selWord="cos2 0.85: the documents/words with cos2 over 0.85 (as summed on the 2 axes) are selected.\cr
selDoc ="meta 3": documents/words with a contribution over 3 times the average document/word contribution on any of both axes are selected.\cr
}

\references{
Husson F., Lê S., Pagès J. (2011). Exploratory Multivariate Analysis by Example Using R. Chapman & Hall/CRC. \doi{10.1201/b10345}.

Lebart, L., Piron, M., & Morineau, A. (2006). Statistique exploratoire multidimensionnelle. (Dunod, Ed.).

Lebart, L., Salem, A., & Berry, L. (1998). Exploring textual data. (Kluwer, Ed.). \doi{10.1007/978-94-017-1525-6}.
}

\author{Monica Bécue-Bertaut, Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{LexCA}}, \code{\link{print.LexCA}},  \code{\link{plot.LexCA}}, \code{\link{summary.LexCA}}}

\examples{
\dontrun{
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), remov.number=TRUE, Fmin=10, Dmin=10,  
  stop.word.tm=TRUE, context.quali=c("Gender","Age_Group","Education"), 
  context.quanti=c("Age"))
res.LexCA<-LexCA(res.TD, graph=FALSE,ncp=8)
ellipseLexCA(res.LexCA, selWord="meta 1",selDoc=NULL, col.word="brown")
ellipseLexCA(res.LexCA, selWord="contrib 10",selDoc=NULL, col.word="brown")
ellipseLexCA(res.LexCA, selWord=c("work","job","money","comfortable"), selDoc=NULL,
  col.word="brown")
ellipseLexCA(res.LexCA, selWord="cos2 0.2", selDoc=NULL, col.word="brown")
}
\dontrun{
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), var.agg="Gen_Age", Fmin=10, Dmin=10,
  remov.number=TRUE, stop.word.tm=TRUE)
res.LexCA<-LexCA(res.TD, graph=FALSE)
ellipseLexCA(res.LexCA, selWord=NULL, col.doc="black")
ellipseLexCA(res.LexCA, selWord="meta 3", selDoc=NULL, col.word="brown")
ellipseLexCA(res.LexCA, selWord="contrib 10", selDoc=NULL, col.word="brown")
ellipseLexCA(res.LexCA, selWord=c("work","job","money","comfortable"), selDoc=NULL,
       col.word="brown")
ellipseLexCA(res.LexCA, selWord="cos2 0.2", selDoc=NULL, col.word="brown")    
	}
}
\keyword{multivariate}

\name{LexCA}
\encoding{UTF-8}
\alias{LexCA}

\title{Correspondence Analysis of a Lexical Table from a TextData object (LexCA)}

\description{
Performs Correspondence Analysis on the working lexical table contained in TextData object. Supplementary documents, words, segments, contextual quantitative and qualitative variables can be considered if previously selected in TextData function.
}

\usage{LexCA(object, ncp=5, context.sup="ALL", doc.sup=NULL, word.sup=NULL, 
  segment=FALSE, graph=TRUE, axes=c(1, 2), lmd=3, lmw=3)}

\arguments{
 \item{object}{object of TextData class}
 \item{ncp}{number of dimensions kept in the results (by default 5)}
 \item{context.sup}{column index(es) or name(s) of the contextual qualitative or quantitative variables among those selected in TextData function (by default "ALL")}
 \item{doc.sup}{vector indicating the index(es) or name(s) of the supplementary documents (rows) (by default NULL)}
 \item{word.sup}{vector indicating the index(es) or name(s) of the supplementary words (columns) (by default NULL)}
 \item{segment}{if TRUE, the repeated segments identified by TextData function will be considered as supplementary columns (by default FALSE)} 
 \item{graph}{if TRUE, basic graphs are displayed; use plot.LexCA to obtain more graphs (by default TRUE)}
 \item{axes}{length-2 vector indicating the axes to plot (by default axes=c(1,2))}
 \item{lmd}{only the documents whose contribution is over lmd times the average-document-contribution are plotted (by default lmd=3)}
 \item{lmw}{only the words whose contribution is over lmw times the average-word-contribution are plotted (by default lmw=3)}
}


\value{
Returns a list including:
\item{eig}{matrix with the eigenvalues, the percentages of inertia and the cumulative percentages of inertia}
\item{row}{list of matrices with all the results for the documents
(coordinates, square cosines, contributions, inertia)}
\item{col}{list of matrices with all the results for the words (coordinates, square cosines, contributions, inertia)}
\item{row.sup}{if row.sup is non-NULL, list of matrices with all the results for the supplementary documents (coordinates, square cosines)}
\item{col.sup}{if col.sup is non-NULL, list of matrices with all the results for the supplementary words (coordinates, square cosines)}
\item{quanti.sup}{if quanti.sup is non-NULL, list of matrices containing the results for the supplementary quantitative variables (coordinates, square cosines)}
\item{quali.sup}{if quali.sup is non-NULL, list of matrices with all the results for the supplementary categorical variables; see section details}
\item{meta}{list of the documents/words whose contribution is over lmd/lmw times the average document/word contribution}
\item{VCr}{Cramer's V coefficient}
\item{Inertia}{total inertia}
\item{info}{information about the corpus}
\item{segment}{if segment is TRUE, list of matrices with the results for the repeated segments (coordinates, square cosines)}
\item{var.agg}{name of the aggregation variable in the case of an aggregate correspondence analysis}
\item{call}{a list with some statistics}
}




\details{
In the case of a direct CA, DocTerm is a non-aggregate table and:
 \enumerate{
 \item the contextual quantitative variables are considered as supplementary quantitative columns in CA. 
 \item the categories of the contextual qualitative variables are considered as supplementary columns in CA.}
 
 In the case of an aggregate CA, DocTerm is an aggregate table and:
 \enumerate{
 \item the contextual quantitative variables are considered as supplementary quantitative columns in CA; the value of an active aggregate-document for a variable is the mean of the values corresponding to
 the source-documents belonging to this aggregate-document.
 \item the categories of the contextual qualitative variables are threatened as supplementary rows in CA; these rows contain the frequency with which each the set of documents belonging to this category has used the different words. }
}

\references{
Benzécri, J, P. (1981). Pratique de l'analyse des donnees. Linguistique & lexicologie (Vol.3). (P. Dunod., Ed).

Husson F., Lê S., Pagès J. (2011). Exploratory Multivariate Analysis by Example Using R. Chapman & Hall/CRC. \doi{10.1201/b10345}.

Lebart, L., Salem, A., & Berry, L. (1998). Exploring textual data. (D. Kluwer, Ed.). \doi{10.1007/978-94-017-1525-6}.

Murtagh F. (2005). Correspondence Analysis and Data Coding with R and Java. Chapman & Hall/CRC.
}
\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Mónica Bécue-Bertaut, Josep-Anton 
  Sánchez-Espigares\cr}

\seealso{ \code{\link{TextData}}, \code{\link{print.LexCA}},  \code{\link{plot.LexCA}}, \code{\link{summary.LexCA}}, \code{\link{ellipseLexCA}}}

\examples{
data(open.question)
\dontrun{
### non-aggregate CA
res.TD<-TextData(open.question, var.text=c(9,10), Fmin=10, Dmin=10,
        remov.number=TRUE, stop.word.tm=TRUE)
res.LexCA<-LexCA(res.TD, lmd=0, lmw=1)
}

### aggregate CA
res.TD<-TextData(open.question, var.text=c(9,10), var.agg="Age_Group", Fmin=10, Dmin=10,
        remov.number=TRUE, stop.word.tm=TRUE)
res.LexCA<-LexCA(res.TD, lmd=0, lmw=1)
}

\keyword{multivariate}

\name{print.TextData}
\encoding{UTF-8}
\alias{print.TextData}

\title{Print TextData objects}

\description{
Print statistical results for documents, words and segments from TextData objects, in alphabetical and frequency order.
}

\usage{
\method{print}{TextData}(x, file = NULL, sep=";", dec=".", ...) 
}

\arguments{
  \item{x}{object of TextData class}
  \item{file}{connection, or character string giving the name of the file to print to (in csv format). If NULL (by default value), 
the results are not printed in a file}
  \item{sep}{character inserted between the objects to print (if file argument is non-NULL) (by default ";"\\t" for tab)}
 \item{dec}{the string to use for decimal points in numeric or complex columns: must be a single character (by default ".")}
 \item{\dots}{further arguments passed to or from other methods}
}

\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Monica Bécue-Bertaut, Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{TextData}}, \code{\link{plot.TextData}},  \code{\link{summary.TextData}}}


\examples{
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), remov.number=TRUE, Fmin=10, Dmin=10,  
stop.word.tm=TRUE, context.quali=c("Gender","Age_Group","Education"),
   context.quanti=c("Age"))
print(res.TD)
}

\keyword{print}

\name{LexHCca}
\encoding{UTF-8}
\alias{LexHCca}

\title{Hierarchical Clustering on Textual Correspondence Analysis Coordinates (LexHCca)}

\description{Agglomerative hierarchical clustering of documents or words issued from correspondence analysis coordinates}


\usage{LexHCca(x, cluster.CA="docs",  type="agnes", ncp=5, nb.clust="click", min=2, 
   max=NULL, kk=Inf, consol=FALSE, iter.max=500, graph=TRUE, description=TRUE, 
   proba=0.05, nb.desc=5, size.desc=80,  marg.doc = "before", seed=12345,...)}
   

\arguments{
\item{x}{object of LexCA class}
\item{cluster.CA}{if "rows" or "docs" cluster analysis is performed on documents; if "columns" or "words", cluster analysis is performed on words (by default "docs")}
\item{type}{type of cluster; "agnes" (Agglomerative), "diana" (Divisive) (by default agnes)}
\item{ncp}{number of dimensions used from LexCA object (by default 5)}
\item{nb.clust}{number of clusters. If 0 (or "click"), the tree is cut at the level the user clicks on.
If -1 (or "auto"), the tree is automatically cut at the suggested level. 
If a (positive) integer, the tree is cut with nb.clust clusters (by default "click")}
\item{min}{minimum number of clusters (by default 2)}
\item{max}{maximum number of clusters (by default NULL, then max is computed as the minimum between 10 and the number of documents divided by 2)}
\item{kk}{in case the user wants to perform a Kmeans clustering previously to the hierarchical clustering (preprocessing step), kk is an integer corresponding to the number of clusters of this previous partition. Further, the hierarchical tree is constructed starting from the nodes of this partition as terminal elements. This is very useful when the number of elements to be classified is very large. By default, the value is Inf and no Kmeans preprocessing is performed}
\item{consol}{if TRUE, a Kmeans consolidation step is performed after the hierarchical clustering (consolidation cannot be performed if kk is used and equals a number) (by default FALSE)}
\item{iter.max}{maximum number of iterations in the consolidation step (by default 500)}
\item{graph}{if TRUE, graphs are displayed (by default TRUE)}
\item{description}{if TRUE, description of the clusters of documents or words by the axes, the characteristic words in the case of clustering documents or the characteristic documents in the case of clustering words. The documents or words considered as paragon (para) or specific (dist) are identified. In the case of clustering documents, contextual variables also characterize the clusters. These variables have to be selected in LexCA (by default TRUE)}
\item{proba}{threshold on the p-value used in selecting the elements characterizing significantly the clusters (by default 0.05)}
\item{nb.desc}{number of edited paragons (para) and specific documents labels (dist) (by default 5)}
\item{size.desc}{text size of edited paragons (para) and specific documents (dist) when describing the clusters of documents (by default 80)}
\item{marg.doc}{if after/before, frequencies after/before TextData selection are used as document weighting by characterization if description=TRUE (by default "before")}
\item{seed}{Seed to obtain the same results in successive Kmeans (by default 12345)}
\item{...}{other arguments from other methods}
}

\value{
 Returns a list including:
 \item{data.clust}{the active lexical table used in LexCA plus a new column called Clust_ containing the partition}
 \item{coord.clust}{coordinates table issued from CA plus a new column called Clust_ containing the partition}
 \item{centers}{coordinates of the gravity centers of the clusters} 
 \item{clust.count}{counts of documents/words belonging to each cluster and contribution of the clusters to the variability decomposition}
 \item{clust.content}{list of the document/word labels according to the cluster they belong to}
 \item{call}{list of internal objects. \code{call$t} giving the results for the hierarchical tree. See the second reference for more details}
 \item{description}{$desc.axes for description of the clusters by the characteristic axes ($axes) and eta-squared between axes and clusters ($quanti.var).

 $des.cluster.doc for description of the clusters by their characteristic words ($word), supplementary words ($wordsup) and, 
 if contextual variables were considered in LexCA, description of the partition/clusters by qualitative ($qualisup) and quantitative ($quantisup) variables, paragons ($para) and specific words ($dist) of each cluster.

  $des.word.doc description of the clusters of words by their characteristic documents ($docs), paragons ($para) and 
  specific documents ($dist) of each cluster.}
  
 \item{type}{Type of cluster used (by default agnes).}
 \item{coef.hclust}{agglomerative coefficient (Divisive coefficient for diana), measuring the clustering structure of the dataset.}
 \item{t$tree}{tree object to use with hclust function}

Returns the hierarchical tree and the first CA map of the documents/words. The labels are colored according to the cluster.
}


\details{
LexHCca starts from the documents/words coordinates issued from correspondence analysis axes. 
Euclidean metric and Ward method are used. 

If the agglomerative clustering starts from many elements (documents or words), it is possible to previously perform a Kmeans partition with kk clusters to further build the tree from these (weighted) kk clusters. 
 
The object $para contains the distance between each document and the centroid of its class. 

The object $dist contains the distance between each document and the centroid of the farthest cluster.

The results include a thorough description of the clusters. Graphs are provided.

}



\references{ 

Bécue-Bertaut M. Textual Data Science with R. Chapman & Hall/CRC.  \doi{10.1201/9781315212661}.

Husson F., Lê S., Pagès J. (2017). Exploratory Multivariate Analysis by Example Using R. Chapman & Hall/CRC. \doi{10.1201/b21874}.

Lebart, L., Salem, A., & Berry, L. (1998). Exploring textual data. (D. Kluwer, Ed.). \doi{10.1007/978-94-017-1525-6}.
}

\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Monica Bécue-Bertaut, Josep-Anton Sánchez-Espigares}

\seealso{ \code{\link{LexCA}}, \code{\link{plot.LexHCca}}}

\examples{
data(open.question)	
res.TD<-TextData(open.question, var.text=c(9,10), Fmin=10, Dmin=10, stop.word.tm=TRUE,	
        context.quali=c("Gender","Age_Group","Education"), context.quanti=c("Age"))	
res.LexCA<-LexCA(res.TD, graph=FALSE, ncp=8)	
res.hcca<-LexHCca(res.LexCA, graph=FALSE, nb.clust=5)	
}


\keyword{multivariate}

\name{TextData}
\encoding{UTF-8}
\alias{TextData}

\title{Building textual and contextual tables (TextData)}

\description{
Creates a textual and contextual working-base (TextData format) from a source-base (data frame format).
}

\usage{
TextData(base, var.text=NULL, var.agg=NULL, context.quali=NULL, context.quanti= NULL,
 selDoc="ALL", lower=TRUE, remov.number=TRUE,lminword=1, Fmin=Dmin,Dmin=1, Fmax=Inf,
 stop.word.tm=FALSE, idiom="en", stop.word.user=NULL, segment=FALSE,
 sep.weak="default",
 sep.strong="\u005B()\u00BF?./:\u00A1!=;{}\u005D\u2026", seg.nfreq=10, seg.nfreq2=10,
 seg.nfreq3=10, graph=FALSE)
}
\arguments{
 \item{base}{source data frame with at least one textual column}
 \item{var.text}{vector with index(es) or name(s) of the selected textual column(s) (by default NULL)}
 \item{var.agg}{index or name of the aggregation categorical variable (by default NULL)}
 \item{context.quali}{vector with index(es) or name(s) of the selected categorical variable(s) (by default NULL)}
 \item{context.quanti}{vector with index(es) or name(s) of the selected quantitative variable(s) (by default NULL)}
 \item{selDoc}{vector with index(es) or name(s) of the selected source-documents (rows of the source-base) (by default "ALL")}
 \item{lower}{if TRUE, the corpus is converted into lowercase (by default TRUE)}
 \item{remov.number}{if TRUE, numbers are removed (by default TRUE)}  
 \item{lminword}{minimum length of a word to be selected (by default 1)}
 \item{Fmin}{minimum frequency of a word to be selected (by default Dmin)}
 \item{Dmin}{a word has to be used in at least Dmin source-documents to be selected (by default 1)}
 \item{Fmax}{maximum frequency of a word to be selected (by default Inf)}
 \item{stop.word.tm}{if TRUE, stoplist automatically provided in accordance with the idiom (by default FALSE)}
 \item{idiom}{declared idiom for the textual column(s) (by default English "en", see IETF language in package NLP)}
 \item{stop.word.user}{stoplist provided by the user}
 \item{segment}{if TRUE, the repeated segments are identified (by default FALSE)}
 \item{sep.weak}{string with the characters marking out the terms (by default punctuation characters, space and control). See details}
 \item{sep.strong}{string with the characters marking out the repeated segments (by default "[()??./:?!=+;{}-]\")}
 \item{seg.nfreq}{minimum frequency of a more-than-three-words-long repeated segment (by default 10)}
 \item{seg.nfreq2}{minimum frequency of a two-words-long repeated segment (by default 10)}
 \item{seg.nfreq3}{minimum frequency of a three-words-long repeated segment (by default 10)}
 \item{graph}{if TRUE, documents, words and repeated segments barcharts are displayed; use plot.TextData to use more options (by default FALSE)}
}


\value{
A list including:
\item{summGen}{general summary}
\item{summDoc}{document summary}
\item{indexW}{index of words}
\item{DocTerm}{working lexical table (non-aggregate or aggregate table depending on var.agg value); 
working-documents by words table in slam package compressed format}
\item{context}{contextual variables if context.quali or context.quanti are non-NULL; the structure greatly differs 
in accordance with the nature of DocTerm table (non-aggregate/ aggregate), see details}
\item{info}{information about the selection of words}
\item{var.agg}{a one-column data frame with the values of the aggregation variable; NULL if non-aggregate analysis}
\item{SourceTerm}{in the case of DocTerm being an aggregate analysis, the source-documents by words table is kept in 
this data structure, in slam package compressed format}
\item{indexS}{working-documents by repeated-segments table, in slam package compressed format}
\item{remov.docs}{vector with the names of the removed empty source-documents}
\item{VCr}{Cramer's V coefficient of document x term matrix}
\item{Inertia}{total inertia of document x term matrix}
}

\details{
Each row of the source-base is considered as a source-document.
TextData function builds the working-documents-by-words table, submitted to the analysis.

sep.weak contains the string with the characters marking out the terms (by default punctuation characters, space and control).
Backslash or double backslash are used to start an escape sequence defining special characters. Each special character must by separated the symbol | (or) in sep.weak and sep.strong.
The default is:
\verb{

sep.weak = ("[\%`:*$&#/^|<=>;'+@.,~?(){}|[[:space:]]|

\u2014|\u002D|\u00A1|\u0021|\u00BF|\u00AB|\u00BB|\u2026|\u0022|\u005D|\u0097")

}
Some special characters can be introduced as unicode characters. Back slash (escape contol) is not allowed.

Information related to context.quanti and context.quali arguments:
  \enumerate{
    \item If numeric, contextual variables can be included in both vectors. The function TextData converts the numeric variable into factor to include it in context.quali vector. This possibility is interesting in some cases. For example, when treating open-ended questions, we can be interested in computing the correlation between the contextual variable "Age" and the axes and, at the same time, to draw the trajectory of the different values of "Age" (year by year) on the CA maps. 
    \item In the case of one or several columns with textual data not selected in vector var.text, if the argument context.quali is equal to "ALL", these columns will be considered as categorical variables. 
  }


Non-aggregate table versus aggregate table.

If var.agg=NULL: 
\enumerate{
 \item The work-documents are the non-empty-source-documents.
 \item DocTerm: non-aggregate lexical table with:
 \tabular{ll}{
 \tab as many rows as non-empty source-documents\cr
 \tab as many columns as words are selected.
  }
\item context$quali: data frame crossing the non-empty source-documents (rows) and the categorical contextual-variables (columns).
\item context$quanti: data frame crossing the non-empty source-documents (rows) and the quantitative contextual-variables (columns).
Both contextual tables can be juxtaposed row-wise to DocTerm table. 
}

If var.agg is NON-NULL:
\enumerate{
 \item The work-documents are aggregate-documents, issued from aggregating the source-documents depending on the categories of the aggregation variable; the aggregate-documents inherit the names of the corresponding categories.
 \item DocTerm is an aggregate table with:
 \tabular{ll}{
 \tab as many rows as as categories the aggregation variable has\cr
 \tab as many columns as words are selected.
  }
\item context$quali$qualitable: juxtaposes as many supplementary aggregate tables as categorical contextual variables. Each table has: 
 \tabular{ll}{
 \tab as many rows as categories the contextual categorical variable has\cr
 \tab as many columns as selected words, i.e. as many columns as DocTerm has.
  }
\item context$quali$qualivar: names of categories of the supplementary categorical variables.
\item context$quanti: data frame crossing the working aggregate-documents (rows) and the quantitative contextual-variables (columns).
 The value for an active aggregate-document is the mean-value of the source-documents belonging to this aggregate-document.
}
}


\references{
Lebart, L., Salem, A., & Berry, L. (1998). Exploring textual data. (D. Kluwer, Ed.). \doi{10.1007/978-94-017-1525-6}.
}

\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Monica Bécue-Bertaut, Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{print.TextData}}, \code{\link{summary.TextData}},  \code{\link{plot.TextData}}}

\examples{

# Non aggregate analysis
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), remov.number=TRUE, Fmin=10, Dmin=10,  
 stop.word.tm=TRUE, context.quali=c("Gender","Age_Group","Education"), context.quanti=c("Age"))

# Aggregate analysis and repeated segments
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), var.agg="Gen_Age", remov.number=TRUE, 
 Fmin=10, Dmin=10, stop.word.tm=TRUE, context.quali=c("Gender","Age_Group","Education"), 
 context.quanti=c("Age"), segment=TRUE)
}

\keyword{multivariate}

\name{plot.LexChar}
\encoding{UTF-8}
\alias{plot.LexChar}

\title{Plot LexChar objects}

\description{
Draws the characteristic and anti-characteristic words of documents from a LexChar object.
}

\usage{
\method{plot}{LexChar}(x, char.negat=TRUE, col.char.posit="blue", col.char.negat="red",
col.lines="black", theme=theme_bw(), text.size=12, numr=1, numc=2, top=NULL, 
max.posit=15, max.negat=15, type=c("CharWord","quanti","quali"),sel.var.cat="ALL",
txt.var.cat=NULL, sel.words="ALL",...) 
}
\arguments{
  \item{x}{object of LexChar class}
  \item{char.negat}{if TRUE, the anti-characteristic words are plotted (by default TRUE)}
  \item{col.char.posit}{color for the characteristic words (by default "blue")}
  \item{col.char.negat}{color for the anti-characteristic words (by default "red")}
  \item{col.lines}{color for the lines of barplot (by default "black")}
  \item{theme}{used to modify the theme settings by ggplot2 package (by default theme_bw())}
  \item{text.size}{size of the font (by default 12)}
  \item{numr}{number of rows in each multiple graph (by default 1 row)}
  \item{numc}{number of columns in each multiple graph (by default 2 columns)}
  \item{top}{title of the graph (by default NULL)}
  \item{max.posit}{maximum number of characteristic words (by default 15)}
  \item{max.negat}{maximum number of anti-characteristic words (by default 15)}
  \item{type}{CharWord and draws the characteristic and anti-characteristic words; quanti draws characteristic words for all quantitative variables; quali draws only the words for one qualitative variable  (by default CharWord)}
  \item{sel.var.cat}{name of contextual quantitative and/or qualitative contextual variables}
  \item{txt.var.cat}{new names of each category or quantitative variable (by default NULL}
  \item{sel.words}{words selected to plot if its p-value is less than prob (by default ALL}
  \item{\dots}{further arguments passed to or from other methods...}
}

\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Monica Bécue-Bertaut, Josep-Anton Sánchez-Espigares}

\seealso{ \code{\link{LexChar}}, \code{\link{print.LexChar}}, \code{\link{summary.LexChar}}}


\examples{
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), var.agg="Gen_Edu", Fmin=10, Dmin=10,
        remov.number=TRUE, stop.word.tm=TRUE)
LD<-LexChar(res.TD,maxCharDoc = 0)
plot(LD)
}

\keyword{plot}

\name{Xplortext-package}
\encoding{UTF-8}
\alias{Xplortext-package}
\docType{package}
\title{Textual Analysis}
\description{
Provides a set of functions devoted to multivariate exploratory statistics on textual data.
Classical methods such as correspondence analysis and agglomerative hierarchical clustering are available.
Chronologically constrained agglomerative hierarchical clustering enriched with labelled-by-words trees is offered.
Given a division of the corpus into parts, their characteristic words and documents are identified.
Further, accessing to 'FactoMineR' functions is very easy. Two of them are relevant in textual domain.
MFA() addresses multiple lexical table allowing applications such as dealing with multilingual corpora as well as simultaneously analyzing both open-ended and closed questions in surveys. 
See \url{https://xplortext.unileon.es} for examples.
}
\details{
\tabular{ll}{
Package: \tab Xplortext\cr
Type: \tab Package\cr
Version: \tab 1.6\cr
Date: \tab 2026-05-14\cr
License: GPL (>=2.0)
}
}
\author{Ramón Alvarez-Esteban\cr
Maintainer: \email{ramon.alvarez@unileon.es}
}


\references{
Bécue, M. (2019). Textual Data Science with R. Chapman & Hall/CRC. \doi{10.1201/9781315212661}.

Bécue-Bertaut, M., Alvarez-Esteban, R. (2026).  Textual and Contextual Data Analysis: A Multivariate Statistical Approach using R (1st ed.). Chapman and Hall/CRC. \doi{10.1201/9781003397472}.

Husson F., Lê S., Pagès J. (2011). Exploratory Multivariate Analysis by Example Using R. Chapman & Hall/CRC. \doi{10.1201/b10345}.

Lebart, L., Salem, A., & Berry, L. (1998). Exploring textual data. (D. Kluwer, Ed.).
 \doi{10.1007/978-94-017-1525-6}.

A website \url{https://xplortext.unileon.es}

}

\name{print.LexChar}
\encoding{UTF-8}
\alias{print.LexChar}

\title{Print LexChar objects}

\description{
Prints characteristic words and documents from LexChar objects
}

\usage{
\method{print}{LexChar}(x, file = NULL, sep=";", dec=".",  ...) 
}

\arguments{
  \item{x}{object of LexChar class}
  \item{file}{a connection, or a character string giving the name of the file to print to (in csv format). If NULL (the default), the results are not printed in a file}
  \item{sep}{character to insert between the objects to print (if the argument file is non-NULL) (by default ";")}
  \item{dec}{decimal point (by default ".")}
  \item{\dots}{further arguments passed to or from other methods}
}

\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Mónica Bécue-Bertaut, Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{LexChar}}, \code{\link{plot.LexChar}}}


\examples{
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), var.agg="Gen_Edu", Fmin=10, Dmin=10,
        stop.word.tm=TRUE)
LD<-LexChar(res.TD, maxCharDoc = 0)
print(LD)
}

\keyword{print}

\name{plot.LexHCca}
\encoding{UTF-8}
\alias{plot.LexHCca}
\title{Plots for Hierarchical Clustering from LexHCca Objects}

\description{Plots graphs from LexHCca results: tree and CA maps with the documents or words colored in accordance with the cluster.}


\usage{
\method{plot}{LexHCca}(x, type=c("map", "tree", "phylo", "clado", "radial", "fan"), 
     plot=c("points", "labels", "centers"), selClust="ALL",
     selInd="ALL",axes=c(1, 2), theme=theme_bw(), palette=NULL, title=NULL,
     axis.title=NULL, axis.text=NULL, xlim=NULL, ylim=NULL, hvline=NULL, 
     points=NULL, labels=NULL,centers=NULL, traject=NULL, hull=NULL, 
     rotate=FALSE, branches=NULL, mar.value=NULL,...) 
}

\arguments{
  \item{x}{object of LexHCca class}
  \item{type}{type of graph. \code{"map"} plots the CA map where the individuals are colored in accordance with the cluster of belonging (by default); "tree" plots the dendrogram if hierarchical method without consolidation is performed from LexHCca; other options are "phylo", "clado", "radial", "fan". See details}
  \item{plot}{elements to plot for map graph: points, labels, centers, hull, hvline or traject; by default "ALL" and points, labels and centers are plotted. Also combinations are allowed, i.e: plot=c(points,centers); For no maps plot elements are: branches, labels, hull and hvline. See details}
  \item{selClust}{vector indexes with the numbers of the clusters to plot (by default "ALL")} 
  \item{selInd}{vector with the active documents/words to plot (indexes, names or rules; see details; by default "ALL"). You can also use the "transparent"" option defining the color for clusters and/or cases}
  \item{axes}{length-2 vector indicating the axes of the CA map to plot; by default (1,2)}
  \item{theme}{used to modify the theme settings by ggplot2 package of the CA map (by default theme_bw())}
  \item{palette}{color palette used to draw the clusters. As many numbers as clusters. See details}
  \item{title}{title of the map graph. If NULL or FALSE, a title is automatically defined (by default NULL).
  Other parameters can be chosem using for map in a list: text, color, size, family, face, just; For "tree" only "text" argument can be used. See details}
  \item{axis.title}{axis titles parameters can be used por map plots: text.x, text.y, color, size, family, face, just;  If text.x and text.y are NULL automatic texts are plotted (by default NULL). ; For tree only FALSE are allowed and height are removed. See details}
  \item{axis.text}{For maps, format of numbers can be chosen: color, size, family, face}
 \item{xlim}{For map, pair of values xlim=c(xmin,xmax). If a NA value, this limit is automatically calculated}
 \item{ylim}{For map, pair of values ylim=c(ymin,ymax). If a NA value, this limit is automatically calculated}
  \item{hvline}{For map, horizontal (intercept.y) and vertical line (intercept.x) added by default at (0,0) position in map. Parameters: intercept.y, intercept.x, linetype (by default "dashed"), color, linesize, alpha.t. 
For tree draws a line at level of the height chosen by the clusters selected. Parameters pos (position), linesize, linetype and color}
\item{points}{For maps: format of points. Parameters: size (if size=0 the points are no plotted), shape (by default 21), fill (if a color, the same for all the points, if color is NULL palette colors used for the clusters are applied; if more than one color use palette argument; only for shapes from 21 to 25 to fill the point), stroke (controls the edge of the point (by default 0 no edge), border (color of the border, same specifications than fill), alpha.t (by default 1). See geom_point() in ggplot2 library. See details}
\item{labels}{format of labels. For no maps: cex (value or vector with the length of cases, if 0 transparent) and color. For map plots, parameters: cex, size (if size=0 the labels are not plotted; by default 4), family, face, hjust, vjust, color.text, alpha.t.text, numbers(if TRUE the label will be replaced by the number of the cluster to which it belongs, by default FALSE), color.fill (color into the rectangle, by default FALSE is transparent), alpha.t.fill, groupLabels and labels will be added to each cluster in tree plots. For map: force (to do repulsive textual annotations and make it easier to read), max.overlaps (maximum number of overlapped points, by default 10, can be Inf),
 set.seed (by default a new seed for each plot draws different positions, for the same seed i.e: set.seed=1234)}
 \item{traject}{for map: draws trajectory arrows in accordance with the order of clusters or in the selInd order. Parameters: color (by default blue), linetype (by default 1 solid), space (by default 0 and no space is added from point to arrow, be careful with this value), size (width,by default 1), arrow.length (of the arrow, by default .3), arrow.type (by default "closed"), arrow.angle (by default 30), alpha.t. See geom_segment for details}
\item{centers}{draws the barycenter of the clusters. Parameters: size (by default 5), family, face, color (of the border, only one), fill, alpha.t, labels (string vector with the names of the clusters)}
 \item{hull}{draws a hull containing all the elements of each cluster. Parameters: type (ellipse, by default, hull), alpha.t, color, linetype (by default "dotted")). For tree, no null value, rect for example, draws a rectangle. See details.}
\item{rotate}{rotation degrees, TRUE or FALSE. Not allowed for map. By default 0 or FALSE.}
\item{branches}{color, linesize and linetype (integer (0-6), a name (0 = blank, 1 = solid, 2 = dashed, 3 = dotted, 4 = dotdash, 5 = longdash, 6 = twodash)}
\item{mar.value}{mar value of par function: bottom, left, top and right values for margins in cm (by default c(4,3,1,1) for dendrograms and c(1,1,1,1) for maps)}
 \item{...}{other arguments from other methods}
}


\details{

Parameter type="tree" shows the dendrogram
\preformatted{
 - if hierarchical cluster without consolidation is performed.
 - if hierarchical cluster with consolidation before the consolidation.
 - if kmeans the hierarchical tree with the output of kmeans.
}
You can make customer dendrograms by accessing the hclust format object located inside the object in
hclust format from object$call$t$tree

Selection of individuals (documents or words) to plot:

1. Using labels:
\preformatted{
selInd = c("doc1","doc5"): only the documents with labels doc1 and doc5 are plotted.
}

2. Using indexes:
\preformatted{
selInd = c(1:5): cases 1 to 5 are plotted.
   }


3. Using rules:
\preformatted{
 Rules are based on the coordinates (coord), the contribution (contrib or meta; 
 concerning only active elements) or the square cosine (cos2). }
 
 Somes examples hereafter:

\preformatted{
selInd="coord 10": only the 10 cases with the highest coordinates, as globally
   computed on the 2 axes, are plotted.
selInd="contrib 10": the cases with a contribution to the inertia, of any of 
   the 2 axes over 10 percent.
selInd="meta 3": the cases with a contribution over 3 times the average word/document 
   contribution on any of the two axes are plotted.
selInd="cos2 .85": the documents with a cos2 over 0.85, as summed on the 2 axes, 
   are plotted.
}

Parameters can be used in combination, e.g.: title=list("text"="CA", "color"="red").

See grDevices package (The R Graphics Devices and Support for Colours and Fonts).

palette, the color of the palette used to draw the points. 
   By default colors are chosen. If you want to define the colors for three clusters : palette=c("black","red","blue"); or you can use: palette= palette(rainbow(30)); or in black and white for example: palette=palette(gray(seq(0,.9,len=25))).

Family Fonts (family). Also see the extrafont package for a much better support of fonts: library(extrafont); font_import(). By default "family"='serif'.

Face fonts (face). Can be 'plain', 'bold', 'italic', 'bold.italic', 'symbol'. By default 'plain'.

alpha.t is the level of transparency for some objects. 0 value means full transparency and 1 opacity. By default 1.

Values for horizontal justification hjust, vertical vjust and both hvjust can be (c,centered or 0.5 if centered; l,left or 0 if left; r, right or 1 if right).

groupLabels, only for tree, can be NULL or FALSE and no labels are added to each cluster, TRUE for all the clusters numbers are used, "as.roman", "letters" or "LETTERS" for capital letters. For several lines in the same cluster or no labels:labels=list("groupLabels"=c(paste0("FirstLine,"\\n","SecondLine", "b", "").

\preformatted{
By default in:
* title: text="Clusters on the CA map"; color=black; size=18; familiy=serif; face=plain; 
      hjust=0.5.

* axis titles:  text.x=Dim x (\%), text.y=Dim y (\%), color=black, size=12, family=serif, 
      face=plain, just=centered.
  
* axis.text: color=black, size=8, family=serif, face=plain.

* hvline: intercept.x=0, intercept.y=0, linetype=dashed, color=gray, size=0.5, alpha.t=1.
  
* points: size=2, shape=21, border:automatic cluster color, fill:automatic cluster color, 
      stroke=0, border: automatic cluster color, alpha.t=1.

* labels: size=4, family=serif, face=plain, hjust=1, vjust=1, color.text=same of points, 
      alpha.t.text=1, numbers=FALSE, rect=FALSE, color.fill=transparent, alpha.t.fill=1,
      force=1, max.overlaps=10.
      
* traject: color=blue, linetype=solid, space=1, arrow.length=.3, size=1,
      arrow.type= c("closed",       "open"), arrow.angle=30, alpha.t=1.

* centers: size=5, family=serif, face=italic, color, fill=automatic cluster color,
      alpha.t=1, labels=automatic strig vector with the names of the clusters.

* hull: type=ellipse, alpha.t=0.1, color=black, linetype=dotted .
      For rectangles in tree, you can use some dendextend::rect.dendrogram arguments as
      which for select the cluster, border for the color, prop_k_height (value between 0 
      to 1, indicating what proportion of the height our rect will be between the height 
      needed for k and k+1 clustering), lower_rect (value of how low should the lower 
      part of the rect be), upper_rect (value to add (default is 0) to how high should
      the upper part of the rect be).
      }
}


\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Mónica Bécue-Bertaut, Josep-Anton Sánchez-Espigares}

\seealso{\code{\link{LexHCca}}}

\references{
The Xplortext web site provides several examples at <https://xplortext.unileon.es/?page_id=766>.
}

\examples{
data(open.question)
res.TD<-TextData(open.question,var.text=c(9,10), var.agg="Age_Group", Fmin=10, Dmin=10,
        stop.word.tm=TRUE)
res.LexCA<-LexCA(res.TD, graph=FALSE)
res.hcca<-LexHCca(res.LexCA, nb.clust=4, min=3, graph=FALSE)
plot(res.hcca, type="tree")
plot(res.hcca, type="map")
}

\keyword{plot}

\name{plot.TextData}
\encoding{UTF-8}
\alias{plot.TextData}

\title{Plot TextData objects}

\description{
Draws the barcharts of the longest documents, most frequent words and segments from a TextData object.
}

\usage{
\method{plot}{TextData}(x, ndoc=25, nword=25, nseg=25, sel=NULL, ordFreq=TRUE, 
 stop.word.tm=FALSE, idiom="en", stop.word.user=NULL, theme=theme_bw(), title=NULL,
 xtitle=NULL, col.fill="grey", col.lines="black", text.size=12, freq=NULL, vline=NULL, 
 interact=FALSE, round.dec = 4,...) 
}
\arguments{
  \item{x}{object of TextData class}
  \item{ndoc}{number of documents in the barchart (by default 25)}
  \item{nword}{number of words in the barchart  (by default 25)}
  \item{nseg}{number of segments in the barchart (by default 25)}
  \item{sel}{type of barchart (doc, word or seg for documents, words or repeated segments) (by default NULL and all he graphs are drawn)), see details}
   \item{ordFreq}{if ordFreq=TRUE, glossaries of words and repeated segments, are drawn in frequency order; if ordFreq=FALSE, glossaries are drown in alphabetic order (by default TRUE)}
  \item{stop.word.tm}{if TRUE, the tm stopwords (if the words are selected in TextData object) are not considered for the barchart (by default FALSE)}
  \item{idiom}{declared idiom for the textual column(s) (by default English "en", see IETF language in package NLP)}
  \item{stop.word.user}{the user's stopwords (if the words are selected in TextData object) are not considered for the barchart (by default NULL)}
  \item{theme}{theme settings (see ggplot2 package; by default theme_bw())}
  \item{title}{title of the graph (by default NULL and the title is automatically assigned)}
  \item{xtitle}{x title of the graph (by default NULL and the x title is automatically assigned)}
  \item{col.fill}{background color for the barChart bars (by default grey)}
  \item{col.lines}{lines color for the barChart bars (by default black)}
  \item{text.size}{text font size (by default 12)}
  \item{freq}{add frequencies to word and document barplots, see details (by default NULL)}
  \item{vline}{if "YES" or TRUE add vertical line to barplot, see details (by default NULL)}
  \item{interact}{if FALSE a ggplot graph, if TRUE an interactive plotly graph, see details (by default FALSE)}
  \item{round.dec}{number of decimals (by default 4)}
  \item{\dots}{further arguments passed to or from other methods...}
}



\details{
freq adds frequencies to barplot (by default NULL). If "YES" or TRUE displays the frequencies at the right of the bars at +5 position.
Numerical values display the frequencies at the right positions (positive values) or at the left (negative values).

vline adds two vertical line to word and document barplot (by default NULL). If TRUE a first vertical row line is added at mean level computed from the selected items from TextData,
and a second vertical blue line with the frequency mean of words/documents selected to plot in plot.TextData. If row and blue lines are the same, only blue line is shown.
If vline is a number, a line is show with this value.

Barchart selected in sel argument (doc, word and/or repeated segments) is in ggplot format. 
Barchart is used with geom_bar function of ggplot package. If it is only one element in sel argument the plot can
be saved in ggplot format:  newobject <- plot(TextDataObject,sel="word")

Selection of docs, words or segments can be done by numbers sel=list(type="doc", select=c(1,2:4,6)) or names sel= list(type="doc", select=c("M31_55", "M>55")).

If interact, rank for words/docs/segments from TextData selection are shown.

}

\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Mónica Bécue-Bertaut, Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{TextData}}, \code{\link{print.TextData}},  \code{\link{summary.TextData}}}

\examples{
# Non aggregate analysis

 data(open.question)
 res.TD<-TextData(open.question, var.text=c(9,10), remov.number=TRUE, Fmin=10, Dmin=10,  
 stop.word.tm=TRUE, context.quali=c("Gender","Age_Group","Education"), context.quanti=c("Age"))
 plot(res.TD)

# Aggregate analysis
 data(open.question)
 res.TD<-TextData(open.question, var.text=c(9,10), var.agg="Gen_Age", remov.number=TRUE, 
 Fmin=10, Dmin=10, stop.word.tm=TRUE, context.quali=c("Gender","Age_Group","Education"), 
 context.quanti=c("Age"), segment=TRUE)
 plot(res.TD)

}

\keyword{plot}

\name{summary.LexChar}
\encoding{UTF-8}
\alias{summary.LexChar}

\title{Summary LexChar object}

\description{
Summarizes LexChar objects
}

\usage{
\method{summary}{LexChar}(object, CharWord=TRUE, stats=TRUE, CharDoc=TRUE, Vocab=TRUE,
    file = NULL, ...)

}

\arguments{
  \item{object}{object of TextData class}
  \item{CharWord}{if TRUE characteristic words of all the documents are shown (by default TRUE)}
  \item{stats}{if TRUE association statistics of lexical table are shown (by default TRUE)}
\item{CharDoc}{if TRUE characteristic source-documents of all the aggregate-documents are shown (by default TRUE)}
\item{Vocab}{if TRUE characteristic quantitative and qualitative variables of the words. CharWord and stats are provide}

  \item{file}{a connection, or a character string naming the file to print to in csv format. If NULL (the default), the results are not printed in a file}
  \item{\dots}{further arguments passed to or from other methods,...}
}

\details{
Vocab$quali$CharWord provides the qualitative variables and their categories.
Vocab$quali$stats provides association statistics for vocabulary and qualitative variables.
Vocab$quanti$CharWord provides characteristic quantitative variables for each word. This summary.LexChart function provides the characteristic words for each quantitative variable.
Vocab$quali$stats provides statistics for vocabulary and quantitative variables.
}


\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Monica Bécue-Bertaut, Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{LexChar}}, \code{\link{print.LexChar}}, \code{\link{plot.LexChar}}}




\examples{
data(open.question)
res.TD<-TextData(open.question, var.text=c(9,10), var.agg="Gen_Edu", Fmin=10, Dmin=10, 
        remov.number=TRUE, stop.word.tm=TRUE)
res.LexChar <- LexChar(res.TD)
summary(res.LexChar)

}

\keyword{summary}

\name{LexCHCca}
\encoding{UTF-8} 
\alias{LexCHCca}

\title{Chronological Constrained Hierarchical Clustering on Correspondence Analysis Components (LexCHCca)}
\description{Chronological constrained agglomerative hierarchical clustering on a corpus of documents}

\usage{
LexCHCca (object, ncp=5, nb.clust=0, min=2, max=NULL, graph=TRUE, 
 cut.test=FALSE, alpha.test =0.05, description=FALSE, proba=0.05,
 nb.desc=5, size.desc=80,...)
}


\arguments{
  \item{object}{object of LexCA class}
  \item{ncp}{number of dimensions used from LexCA object (by default 5)}
  \item{nb.clust}{number of clusters only if no test (cut.test=FALSE). If 0 (or "click"), the tree is cut at the level the user clicks on. If -1 (or "auto"), the tree is automatically cut at the suggested level. If a (positive) integer, the tree is cut with nb.clust clusters (by default 0)}
  \item{min}{minimum number of clusters. Available only if cut.test=FALSE. (by default 2)}
  \item{max}{maximum number of clusters. Available only if cut.test=FALSE. (by default NULL; then max is computed as the minimum between 10 and the number of documents divided by 2)}
  \item{graph}{if TRUE, graphs are displayed (by default TRUE)}
  \item{cut.test}{if FALSE (by default), Legendre test is not performed when joining two nodes. This test is used to determine whether two clusters should be joined or not; see details}
  \item{alpha.test}{threshold on the p-value used in selecting aggregation clusters for Legendre test (by default 0.05)}
  \item{description}{if TRUE, description of the clusters by the characteristic words/documents, paragon (para), specific documents (dist) and contextual variables if these latter have been selected in the previous LexCA function (by default FALSE)}
  \item{proba}{threshold on the p-value used to describe the clusters (by default 0.05)}
  \item{nb.desc}{number of paragons (para) and specific documents (dist) that are edited when describing the clusters (by default 5)}
  \item{size.desc}{maximum of characters when editing the paragons (para) and specific documents (dist) to describe the clusters (by default 80)}
\item{...}{other arguments from other methods}
}

\value{
Returns a list including:
 \item{data.clust}{the active lexical table used in LexCA plus a new column called Clust_ containing the partition}
 \item{coord.clust}{coordinates table issued from CA plus a new column called weigths and another column called Clust_, corresponds to the partition}
 \item{centers}{coordinates of the gravity centers of the clusters}
 \item{description}{
 $des.word for description of the clusters of documents by their characteristic words, the paragons (des.doc$para) and 
  specific documents (des.doc$dist) of each cluster; see details}
 \item{call}{list of internal objects. \code{call$t} giving the results for the hierarchical tree}
 \item{dendro}{hclust object. This allows for using the dendrogram in other packages}
\item{phases}{details of the tracking of the agglomerative hierarchical process. In particular, the cut points (joining documents not allowed) can be identified}
\item{sum.squares}{sum of squares decomposition for documents and clusters}
}

\details{
LexCHCca starts from the document coordinates issued from a textual correspondence analysis. 
The hierarchical tree is built in such a way that only chronological contiguous nodes can be joined. 
The documents have to be ranked in their chronological order in the source-base (data frame format) before to apply the function (TextData format).

Legendre test allows to determine whether the fusion between two nodes based on their contiguity lead to a heterogenous new node (no homogeneity-between-clusters).
If Legendre test is applied (cut.test=TRUE), the number of clusters is the number obtained by the test and nb.clust has not effects.

If no Legendre test is applied (cut.test= FALSE), the number of clusters is determined either a priori or from the constrained hierarchical tree structure.  

The object $para contains the distance between each document and the centroid of its class.
 
The object $dist contains the distance between each document and the centroid of the farthest cluster.

The results of the description of the clusters and graphs are provided.

}


\references{
Bécue-Bertaut, M., Kostov, B., Morin, A., & Naro, G. (2014). Rhetorical Strategy in Forensic Speeches: Multidimensional Statistics-Based Methodology. Journal of Classification,31, 85-106. \doi{10.1007/s00357-014-9148-9}.

Husson F., Lê S., Pagès J. (2017). Exploratory Multivariate Analysis by Example Using R. Chapman & Hall/CRC. \doi{10.1201/b21874}.

Lebart L. (1978). Programme d'agrégation avec contraintes. Les Cahiers de l'Analyse des Données, 3, pp. 275--288.

Legendre, P. & Legendre, L. (1998), Numerical Ecology (2nd ed.), Amsterdam: Elsevier Science.

Murtagh F. (1985). Multidimensional Clustering Algorithms. Vienna: Physica-Verlag, COMPSTAT Lectures.

}

\author{Monica Bécue-Bertaut, Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Josep-Antón Sánchez-Espigares, 
Belchin Kostov\cr}

\seealso{\code{\link{plot.LexCHCca}}, \code{\link{LexCA}}}

\examples{
data(open.question)
res.TD<-TextData(open.question,var.text=c(9,10), var.agg="Age_Group", Fmin=10, Dmin=10, 
        stop.word.tm=TRUE)
res.LexCA<-LexCA(res.TD, graph=FALSE)
res.ccah<-LexCHCca(res.LexCA, nb.clust=4, min=3)
}
\keyword{multivariate}

\name{LexGalt}
\encoding{UTF-8}
\alias{LexGalt}

\title{Correspondence Analysis on a Simple or Multiple Generalized Aggregate Lexical Table (LexGalt)}

\description{
Performs an extension of correspondence analysis on either a simple or a multiple generalized aggregated lexical table. In the case of a multiple table, a multiple factor analysis approach is used
}

\usage{
LexGalt(object, context="ALL", conf.ellip =FALSE, nb.ellip = 100, graph=TRUE, 
        axes = c(1, 2), label.group=NULL)
}

\arguments{
 \item{object}{object or list of objects (s) of TextData class (see details)}
 \item{context}{column index(es) or name(s) of the contextual variables (either qualitative or quantitative) used to build the generalized aggregated lexical table(s). These variables must have been previously selected in TextData function (by default "ALL")}
 \item{conf.ellip}{computing confidence ellipses (available only in the case of a simple table) (by default FALSE)}
 \item{nb.ellip}{number of samples drawn to evaluate the stability of the points (by default 100) only if conf.ellip= TRUE }
 \item{graph}{if TRUE, all several graphs are displayed; use \code{\link{plot.LexGalt}} to obtain detailed graphs (by default TRUE)}
 \item{axes}{length-2 vector indicating the axes to plot (by default axes=c(1,2))}
 \item{label.group}{In the case of analyzing a multiple generalized aggregated lexical table, vector containing the name of the groups (by default, NULL and the group are named GROUP.1, GROUP.2 and so on)}
}


\value{
Returns a list including an object named SQL if the simple qualitative analysis is performed, SQN for simple quantitative analysis, MQL for multiple qualitative analysis or
 MQN for multiple quantitative analysis (see details):
  \item{eig}{eigenvalues, percentages of inertia and cumulative percentages of inertia}
  \item{word}{the results for the words (coordinates, square cosine, contributions)}
  \item{quali.var}{results for the categorical variables (coordinates of each categories of each variables, square cosines)}
  \item{quanti.var}{results for the quantitative variables (coordinates, correlation between variables and axes, square cosines)}
  \item{ellip}{coordinates for confidence ellipses (words and categories) are drawn}
  \item{group}{in the case of multiple analysis, results for the groups (coordinates, contributions and square cosines) (MQL or MQN)} 
Returns the factor maps. The plots may be improved using the plot.LexGalt function. 
}

\details{


The default "context" argument is "ALL" and may contain qualitative and/or quantitative variables (names or indexes). If both types of variables are included, two independent LexGalt analyses are performed, 
saving the results for the qualitative analysis into an object named SQL (or MQL in the multiple case) and for the quantitative analysis into the SQN object (or MQN in the multiple case). 

In the multiple case, each TextData object must be created from as many executions of the function TextData as there are tables. They are joined in a list in the call to LexGalt function:

LexGalt(list(object1,object2,object3),...).

The variable names of each object in the list must be the same as the name of the variables selected in object1.

}

\references{

Bécue-Bertaut M. and Pagès J. (2015). Correspondence analysis of textual data involving contextual information: CA-GALT on principal components. Advances in Data Analysis and Classification, vol.(9) 2: 125-142. 
\doi{10.1007/s11634-014-0171-9}

Bécue-Bertaut M., Pagès J. and Kostov B. (2014). Untangling the influence of several contextual variables on the respondents' lexical choices. A statistical approach. SORT - Statistics and Operations Research Transactions, vol.(38) 2: 285-302.

Kostov B. A. (2015). A principal component method to analyse disconnected frequency tables by means of contextual information. (Doctoral dissertation). Retrieved from \url{https://upcommons.upc.edu/handle/2117/95759}.

Kostov, B., Bécue-Bertaut, M., & Husson, F. (2015). Correspondence Analysis on Generalised Aggregated Lexical Tables (CA-GALT) in the FactoMineR Package. 
The R Journal, Vol.7, Num.1, 109-117. \doi{10.32614/RJ-2015-010}
}

\author{Belchin Kostov, Monica Bécue-Bertaut, Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{plot.LexGalt}}}

\examples{

data(open.question)

res.TD<-TextData(open.question,var.text=c(9,10), Fmin=10, Dmin=10,
 context.quali=c("Gender", "Age_Group", "Education"),
 remov.number=TRUE, stop.word.tm=TRUE)

# res.LexGalt <- LexGalt(res.TD, graph=FALSE, conf.ellip =FALSE)
# plot(res.LexGalt, selQualiVar="ALL")

}

\keyword{multivariate}

\name{print.LexCA}
\encoding{UTF-8}
\alias{print.LexCA}

\title{Print LexCA objects}

\description{
Prints the Textual Correspondence Analysis (CA) results from a LexCA object
}

\usage{
\method{print}{LexCA}(x, file = NULL, sep=";", ...) 
}

\arguments{
  \item{x}{object of LexCA class}
  \item{file}{a connection, or a character string giving the name of the file to print to (in csv format). If NULL (the default), the results are not printed in a file}
  \item{sep}{character to insert between the objects to print (if the argument file is non-NULL) (by default ";")}
  \item{\dots}{further arguments passed to or from other methods}
}


\author{Ramón Alvarez-Esteban \email{ramon.alvarez@unileon.es}, Mónica Bécue-Bertaut, Josep-Antón Sánchez-Espigares}

\seealso{ \code{\link{LexCA}}, \code{\link{plot.LexCA}}, \code{\link{summary.LexCA}}, \code{\link{TextData}}}


\examples{
data(open.question)
res.TD<-TextData(open.question,var.text=c(9,10), var.agg="Age_Group", Fmin=10, Dmin=10,
        remov.number=TRUE, stop.word.tm=TRUE)
res.LexCA<-LexCA(res.TD,lmd=0,lmw=1)
print(res.LexCA)
}

\keyword{print}
